/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hc.c
**  13 Jan 2004 : Hydra II communication CLI
**
*/


/*  defines
------------------------------------------------------------------------------*/
#define MODULE_NAME ("hc")
#define MODULE_VERSION ("1.0")


/*  headers
------------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <getopt.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <ctype.h>
#include <signal.h>

#include "hydra_common.h"
#include "hydra_com.h"
#include "hydra_sm.h"


/*  prototypes
------------------------------------------------------------------------------*/
static void help (void);


/*==============================================================================
**  help
**
*/
static void help (void)
{
    fprintf (stderr, "\n%s\n\n"
                    "%s [-hsv] [-f file]\n"
                    "\t-h\t\tprint this\n"
                    "\t-s\t\tsimulation mode (skip actual sending of commands)\n"
                    "\t-v\t\tverbose mode\n"
                    "\t-f file\t\tuse file for configuration (default is %s)\n\n",
                    version(), MODULE_NAME, HYDRA_COM_CONFIG);
}


/*==============================================================================
**  main
**
*/
int main (int argc, char *argv[])
{
    int s;
    char *cfg_file = NULL;
    bool simulation = false;
    VERBOSE = false;

    signal (SIGINT,  SIG_IGN);	// ignore Ctrl-C
    signal (SIGTSTP, SIG_IGN);	// ignore Ctrl-Z
    signal (SIGTERM, SIG_IGN);  // ignore kill

    while ((s = getopt (argc, argv, ":hsvf:")) != -1)
    {
        switch (s)
        {
            case 'h' :
                help();
                return OK;
            case 's' :
                simulation = true;
                break;
            case 'v' :
                VERBOSE = true;
                break;
            case 'f' :
                cfg_file = optarg;
                break;
            case '?' :
                fprintf (stderr, "FATAL: unrecognized switch: %c\n", optopt);
                help();
                return ERR;
            case ':' :
                fprintf (stderr, "FATAL: switch -%c requires argument\n", optopt);
                help();
                return ERR;
        }
    }

    if (argv[optind])
    {
        fprintf (stderr, "FATAL: extraneous argument: %s\n", argv[optind]);
        help();
        return ERR;
    }

    com_init (cfg_file ? cfg_file : HYDRA_COM_CONFIG, HYDRA_COM_END_CHAR, HYDRA_COM_WAIT, REAL_COMMUNICATION | IGNORE_LOCK);

    {
        char cmdline [256] = "\0";
        int i;

        for (i = 0; i != argc; ++i)
        {
            strcat (cmdline, argv[i]);
            strcat (cmdline, " ");
        }

        hc_log (0, cmdline);
    }

    read_history (".hc.history");

    /* test
    {
        char *reply = 0;
        singlecast ("2RW", &reply);
        if (reply) printf ("{%s}\n", reply);
    }

    //sm_init();
    //init_xyz();
    */

    for (;;)
    {
        if (get_cl (0) == ERR) continue;

        if (strcasecmp (CL, "quit") == 0 || strcasecmp (CL, "exit") == 0 || strcasecmp (CL, "bye") == 0)
        {
            get_cl ("Are you sure you want to quit? (y/n): ");

            while (strncasecmp (CL, "y", 1) && strncasecmp (CL, "n", 1))
            {
                get_cl ("Please enter y or n: ");
            }

            if (strncasecmp (CL, "y", 1) == 0)
            {
                com_end();
                hc_log (2, "Bye!");
                break;
            }
        }
        else if (strcasecmp (CL, "version") == 0)
        {
            printf ("%s\n", version());
        }
        else if (*CL)
        {
            char *reply = 0;

            if (isdigit (*CL))
            {
                singlecast (CL, &reply);
            }
            else
            {
                broadcast (CL, &reply);
            }

            if (reply) printf ("{%s}\n", reply);
        }
    }

    write_history (".hc.history");

    return OK;
}
