/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_config.h
**  3 Feb 2004 : Hydra II config.
**
*/


#ifndef HYDRA_CONFIG_H
#define HYDRA_CONFIG_H 1

#ifdef __cplusplus
    extern "C" {
#endif


/*  defines
------------------------------------------------------------------------------*/
#define ENC_CONST_X (2.5)
#define ENC_CONST_Y (2.5)
#define ENC_CONST_Z (1.0)

#define ENC_SENSE_X0 (1)
#define ENC_SENSE_X1 (-1)   // X1 encoder is installed backwards
#define ENC_SENSE_Y (1)
#define ENC_SENSE_Z (1)

#define P_CONST_XY (4000.0)
#define V_CONST_XY (64424.0)
#define A_CONST_XY (15.82)

// Z motor values are half the X and Y
#define P_CONST_Z (2000.0)
#define V_CONST_Z (32212.0)
#define A_CONST_Z (7.91)

// <volatile>
// WARNING: CENTER and MODEL values shuld not be changed!
// Real measurement should be done form the right hand side. 
// If real measurement is greater than MODEL, add the difference to REAL
// If real measurement is less than MODEL, subtract the difference from REAL
// Note that X is opposite of Y in this regard.
#define CENTER_X (320000)
#define MODEL_CENTER_TO_RHS_X0 (12.604 * 2.54 * 10000)
#define MODEL_CENTER_TO_RHS_X1 (12.604 * 2.54 * 10000)
#define REAL_CENTER_TO_RHS_X0  (12.604 * 2.54 * 10000 + 6303.6 - 335.0)         // -335u orthogonality correction (17 Aug 2004)
#define REAL_CENTER_TO_RHS_X1  (12.604 * 2.54 * 10000 + 14469.0 + 335.0)        //  335u orthogonality correction (17 Aug 2004)
// add 736.6 to above line to remove non-orthogonality (same as chamber tests)

// WARNING: CENTER and MODEL values shuld not be changed!
// Real measurement should be done form bottom.
// If real measurement is greater than MODEL, subtract the difference to REAL.
// If real measurement is less than MODEL, add the difference from REAL.
// Note that Y is opposite of X in this regard.
#define CENTER_Y (220000)
#define MODEL_CENTER_TO_BOTTOM_Y (9.869 * 2.54 * 10000)
#define REAL_CENTER_TO_BOTTOM_Y  (9.869 * 2.54 * 10000 + 17087.8)
// last mechanical mod resulted in moving up by 5511.8 u, so
// subtracting this from above line takes us back to chamber test
// conditions (undesirable)

// Stow circle diameter = 16.875"
// Stow radius = 214312.5
// Park radius = Stow radius + 1500 = 215812.5
//
// Hydra I: 225000
// Hydra II X stow position: 9.17" = 232900 
#define P_MAX_X (233000)
// possible but too close to primary limit:
//#define P_MAX_X (236474 - 736.6)    // 12.375 - 3.065 (in)
                                      // 736.6 was introduced on 2 Aug 2004
                                      // when resetting origin (see note above)
//
// Hydra I: 247500
#define P_MAX_Y (235414 - 5511.8)    // 11.9 - 2.188 (in) - 11270 u (correction!)

// Hydra I: -245000
#define P_MIN_X (-273812)   // 12.458 - 1.615 - 0.063 (in)
// Hydra I: -225050
#define P_MIN_Y (-215313 - 1000)   // 9.355 - 2.188 (in) - 11000 u + 30000 u (correction) + 14271 u (correction)
                                   // - 1000 to reach stow position of farthest button down (Hydra I)
//#define P_MIN_Y (-215313)   // 9.355 - 2.188 (in) - 11000 u + 30000 u (correction) + 14271 u (correction)

// Hydra I: -60000 to 30000
// Z units: 2.5 microns 
#define P_MIN_Z -3460
#define P_MAX_Z 0

#define SW_LIM_OFFSET (1600)
// </volatile>

/// Hydra II can move with V = 50 cm/s and A = 100 cm/s/s for
/// temperatures above 20 F. For 20 F and below V should be reduced
/// to 40 cm/s. The V and A are cnstrained by the power supply.
/// During scientific tests we decided to use V = 40 cm/s and
/// A = 50 cm/s/s as our maximum values (used for moving without a
/// fiber), to be at a more comfortable level belwo the peak output
/// of the power supply while maintaining practically the same
/// performance in terms of fiber placement speed, and increasing
/// the smoothness of the moves; besides, these values are good at
/// all tempertures.  
#define V_MAX_XY (40)       // cm/s
#define A_MAX_XY (50)       // rev/s^2
        
#define V_MAX_Z (40)       // cm/s
#define A_MAX_Z (50)       // rev/s^2

/// The SAFE tag denotes operations with a fiber. We decided to move at 
/// exactly the same speed as Hydra I when handling fibers. Hydra I has
/// a pitch of 5 rev/inch, and its linear encoder resolution is 2032, 
/// which means every motor step is 2.5 microns. With V = 40 rev/s and 
/// A = 40 rev/s/s we get V = 20.32 cms/s and A = 20.32 cm/s/s. 
#define V_SAFE_XY (20.32)   // rev/s (cm/s)
#define A_SAFE_XY (20.32)   // rev/s^2
//#define V_SAFE_XY (9.18)  // rev/s (cm/s)
//#define A_SAFE_XY (10.0)  // rev/s^2
        
#define V_SAFE_Z (4)   // rev/s (cm/s)
#define A_SAFE_Z (50)   // rev/s^2
        
// The minimum size when decomposing vectors
#define V_MIN_XY (0.125)    // rev/s (cm/s)
#define A_MIN_XY (0.127)    // rev/s^2

#define V_MIN_Z (0.125)    // rev/s (cm/s)
#define A_MIN_Z (0.127)    // rev/s^2

#define Z_PLATE  (P_MIN_Z)
#define Z_PARK   (P_MIN_Z + 1460)
#define Z_CRUISE (P_MAX_Z)

// Acceptable mismatch between target and current position in XY
// just after the estimated time for the move has elapsed. System
// will settle shortly after. 
#define ERR_MARGIN_XY (50)
#define ERR_MARGIN_Z  (5)

// Hydra 1 units are 2.5 microns each
#define HYDRA_ONE_SCALE (2.5)

/* DEACTIVATED */
/*
#define TOO_COLD (20)
#define V_MAX_XY_TOO_COLD (40)
*/


/*  headers
------------------------------------------------------------------------------*/


/*  typedefs
------------------------------------------------------------------------------*/


/*  globals 
------------------------------------------------------------------------------*/


/*  prototypes
------------------------------------------------------------------------------*/


/* 
------------------------------------------------------------------------------*/
#ifdef HYDRA_CONFIG_H_TEST
int main (void)
{
    return 0;
}
#endif

#ifdef __cplusplus
    } 
#endif

#endif /* HYDRA_CONFIG_H */
