/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_log.c
**  12 Feb 2004 : Thread-safe logging module.
**
*/


/*  headers
------------------------------------------------------------------------------*/
#include "hydra_log_private.h"


/*  globals & statics
------------------------------------------------------------------------------*/
//static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;


/*==============================================================================
**  log_init
**
*/
int log_init (Log_Unit *lu)
{
    // <not thread safe as a function>
    char file_name [512];
    struct tm *tp;
    time_t t;
    time (&t);
    tp = localtime (&t);
    strftime (file_name, 255, "%d-%m-%Y__%X.", tp);
    // </not thread safe as a function>

    if ( ! (isalnum (lu->tag[0])) )
    {
        sprintf (lu->tag, "%d", getpid());
    }
    
    strcat (file_name, lu->tag);
    strcat (file_name, ".log");
    
    if ((lu->file = fopen (file_name, "w")) == NULL)
    {
        fprintf (stderr, "Error: cannot open log file %s.\n", file_name);
        return ERR;
    }

    return OK;
} 


/*==============================================================================
**  log_it
**
*/
int log_it (Log_Unit *lu, int pipe, char *msg)
{
    //if (pthread_mutex_lock (&mutex) == 0)
    {
        // <not thread safe as a function>
        char ts [256];
        struct tm *tp;
        time_t t;
        time (&t);
        tp = localtime (&t);
        strftime (ts, 255, "%d%b%Y %T #", tp);
        // </not thread safe as a function>

        bool needs_newline = (strrchr (msg, '\n') == NULL) ? true : false;
        
        switch (pipe)
        {
            case 1 :
                if (needs_newline) printf ("%s %s %s\n", lu->tag, ts, msg);
                else printf ("%s %s %s", lu->tag, ts, msg);
                break;
            case 2 :
                if (needs_newline) fprintf (stderr, "%s %s %s\n", lu->tag, ts, msg);
                else fprintf (stderr, "%s %s %s", lu->tag, ts, msg);
                break;
            default :
                break;
        }

        if (needs_newline) fprintf (lu->file, "%s %s %s\n", lu->tag, ts, msg);
        else fprintf (lu->file, "%s %s %s", lu->tag, ts, msg);

        fflush (0); 
    }

    //return pthread_mutex_unlock (&mutex);

    return OK;
}
