/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_models.c
**  11 Jun 2004 : Hydra II models.
**
*/


/*  headers
------------------------------------------------------------------------------*/
#include "hydra_models_private.h"


/*  globals & statics
------------------------------------------------------------------------------*/


/*==============================================================================
** model_geometry
**
*/
Point model_geometry (Point *p)
{
    Point mp = *p;

    mp.x = (long) (mp.x * cos (ROTATION) + mp.y * sin (ROTATION) + OFFSET_X);
    mp.y = (long) (mp.y * cos (ROTATION) - mp.x * sin (ROTATION) + OFFSET_Y - mp.x * tan (NONORTHO));
    return mp;
}


/*==============================================================================
** model_temperature
**
*/
Point model_temperature (Point *p, double temp_in_f)
{
    Point mp = *p;
    double temp_in_c = (temp_in_f - 32.0) * 5.0 / 9.0;

    // WARNING: denominator will become zero if temp_in_c = 46112.3
    // but we do sanity check on temperature somewhere else so we
    // are safe, for now :-)
    mp.y = (long) (mp.y / (1.0 + (2.17 * 0.00001 * (29.4 - temp_in_c))));

    return mp;
}
