/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_sleep.c
**  12 Feb 2004 : High resolution (10 ms) sleep module.
**                Independent from SIGALRM, so may be mixed with alarm().
**
*/


/*  headers
------------------------------------------------------------------------------*/
#include "hydra_sleep_private.h"


/*  globals & statics
------------------------------------------------------------------------------*/


/*==============================================================================
**  hydra_sleep
**
*/
int hydra_sleep (Hydra_Time *tp)
{
    return sleep_s_ms (tp->s, tp->ms);
}


/*==============================================================================
**  sleep_s_ms  
**  As an after thought I decided to log when sleep_s_ms fails, which is
**  not necessarily an error (e.g., for very short moves when the time
**  it takes to finish the move is under 10 ms). So I used sleep_s_ms
**  as a wrapper for __sleep_s_ms__, which does the real work.
*/
static int __sleep_s_ms__ (const unsigned long s, const unsigned long ms)
{
	struct timespec req, rem = {0, 0};

	if (ms > 999) return ERR;
	if (s == 0 && ms < 10) return ERR;

	req.tv_sec = s;
	req.tv_nsec = ms * 1000 * 1000;

	while (nanosleep (&req, &rem) == -1)
	{
		req = rem;
	}

	return OK;
}
// Wrapper:
int sleep_s_ms (const unsigned long s, const unsigned long ms)
{
    if (__sleep_s_ms__ (s, ms) == ERR)
    {
        fprintf (stderr, "WARNING: sleep_s_ms failed");
        return ERR;
    }
    else
    {
	    return OK;
    }
}
