/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  hydra_sm_private.h
**  3 Feb 2004 : Hydra II Smart Motor private interface.
**
*/


#ifndef HYDRA_SM_PRIVATE_H
#define HYDRA_SM_PRIVATE_H 1

#ifdef __cplusplus
    extern "C" {
#endif


/*  defines
------------------------------------------------------------------------------*/
#define HYDRA_SM_BUF_SIZE (1024)

// used in axes_update() and axes_reset
#define EXCEPT_Z (true)
#define ALL_AXES (false)


/*  headers
------------------------------------------------------------------------------*/
#include <stdio.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <stdbool.h>
#include <getopt.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
#include <sys/wait.h>

#include "hydra_common.h"
#include "hydra_com.h"
#include "hydra_config.h"
#include "hydra_log.h"
#include "hydra_models.h"
#include "hydra_sleep.h"
#include "smart_motor.h"
#include "smart_motor_protected.h"
#include "hydra_sm.h"
#include "hydra_sm_protected.h"


/*  typedefs
------------------------------------------------------------------------------*/


/*  globals 
------------------------------------------------------------------------------*/


/*  prototypes
------------------------------------------------------------------------------*/
static int abs_enc_state_machine (int state);
static int axes_pv (bool except_z);
static int axes_update (bool except_z);
static void set_defaults (void);
static int set_a (Axis *ax, unsigned long a);
static int set_p (Axis *ax, long p, bool validate);
static int set_v (Axis *ax, long v);
static int validate_a (Axis *ax, unsigned long *accel);
static int validate_p (Axis *ax, long *pos, bool is_absolute);
static int validate_v (Axis *ax, long *vel);
static int which_direction (void);


/* 
------------------------------------------------------------------------------*/
#ifdef HYDRA_SM_PRIVATE_H_TEST
int main (void)
{
    return 0;
}
#endif

#ifdef __cplusplus
    } 
#endif

#endif /* HYDRA_SM_PRIVATE_H */
