/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  smart_motor.h
**  3 Feb 2004 : Instrument-neutral Smart Motor interface.
**
*/


#ifndef SMART_MOTOR_H
#define SMART_MOTOR_H 1

#ifdef __cplusplus
    extern "C" {
#endif

        
/*  defines
------------------------------------------------------------------------------*/


/*  headers
------------------------------------------------------------------------------*/
#include <stdbool.h>


/*  typedefs
------------------------------------------------------------------------------*/
typedef struct {
    char name [16];
    short enc_sense;
    long enc_index[2];
    long enc_p;
    long calib_dist_model;
    long calib_dist_real;
    long origin;
    long p;
    long v;
    long p_err;
    bool com_fail;
    unsigned int status;
    bool fault;
    bool excess_current;
    bool servo;
    bool excess_heat;
    bool excess_p_err;
    bool wraparound;
    bool index_available;
    bool lim_neg;
    bool lim_pos;
    bool moving; 

    char CMD [16];
    char *cmd; 
} Smart_Motor;

typedef struct {
    char name [16];
    Smart_Motor *sm;
    unsigned short dim;
    unsigned long A;
    unsigned long A_MIN;
    unsigned long A_MAX;
    unsigned long A_SAFE;
    long P;
    /// allow relative mode to work properly when models applied.
    long P_VIRTUAL, P_VIRTUAL_NEW;
    long P_MAX;
    long P_MIN;
    long V;
    long V_MIN;
    long V_MAX;
    long V_SAFE;
    bool fault;
    bool asleep;
    bool moving;
    double P_CONST;
    double A_CONST;
    double V_CONST;
    double ENC_CONST;
    short enc_dir;
    long alignment_err;
    long CENTER;
    // Z uses SLEEP and WAKE; X & Y use SLEEP1 and WAKE1
    char *CMD_SLEEP;
    char *CMD_WAKEUP;
} Axis;


/*  globals 
------------------------------------------------------------------------------*/


/*  prototypes
------------------------------------------------------------------------------*/


/* 
------------------------------------------------------------------------------*/
#ifdef SMART_MOTOR_H_TEST
int main (void)
{
    return 0;
}
#endif

#ifdef __cplusplus
    } 
#endif

#endif /* SMART_MOTOR_H */
