/*
    Copyright  2004 NOAO

    Behzad Abareshi <behzad@noao.edu>
    National Optical Astronomy Observatories
    950 N Cherry Ave, Tucson AZ 85719, USA
------------------------------------------------------------------------------*/


/*
**  smart_motor_protected.h
**  25 Jun 2004 : Instrument-neutral Smart Motor protected interface.
**
*/


#ifndef SMART_MOTOR_PROTECTED_H
#define SMART_MOTOR_PROTECTED_H

#ifdef __cplusplus
    extern "C" {
#endif


/*  defines
------------------------------------------------------------------------------*/
// G stands for global
#define CMD_G_AMPS              ("0AMPS=1023")
#define CMD_G_BRKSRV            ("0BRKSRV")
#define CMD_G_CLR_ALL           ("0ZS")
#define CMD_G_DECELERATE        ("0X")
#define CMD_G_ENC_EXTERNAL      ("0ENC1")
#define CMD_G_GO                ("0G")
#define CMD_G_HALT              ("0S")
#define CMD_G_OFF               ("0OFF")
#define CMD_G_P_MODE            ("0MP")
#define CMD_G_P_ERR             ("0E=2000")
#define CMD_G_RESET             ("0Z")
#define CMD_G_ENABLE_ISR        ("0F=64")
#define CMD_G_DISABLE_ISR       ("0F=0")
#define CMD_G_ENABLE_LIM_NEG    ("0UDM")
#define CMD_G_DISABLE_LIM_NEG   ("0UDI")
#define CMD_G_ENABLE_LIM_SOFT   ("0SLE")
#define CMD_G_DISABLE_LIM_SOFT  ("0SLD")
#define CMD_G_CLR_LIM_NEG       ("0Zl")
#define CMD_G_CLR_LIM_POS       ("0Zr")
#define CMD_G_SET_ORIGIN        ("0O=0")

#define CMD_A                   ("A")
#define CMD_P_ABS               ("P")
#define CMD_P_REL               ("D")
#define CMD_V                   ("V")

#define CMD_ZERO_A              ("A=0")
#define CMD_ZERO_P_ABS          ("P=0")
#define CMD_ZERO_P_REL          ("D=0")
#define CMD_ZERO_V              ("V=0")

#define CMD_G_ZERO_A            ("0A=0")
#define CMD_G_ZERO_P_ABS        ("0P=0")
#define CMD_G_ZERO_P_REL        ("0D=0")
#define CMD_G_ZERO_V            ("0V=0")

#define CMD_REPORT_P            ("RP")
#define CMD_REPORT_P_ERR        ("RPE")
#define CMD_REPORT_STATUS       ("RW")
#define CMD_REPORT_V            ("RV")

#define CMD_CLR_LIM_POS         ("Zr")
#define CMD_CLR_LIM_NEG         ("Zl")
#define CMD_DISABLE_LIM_NEG     ("UDI")
#define CMD_DISABLE_LIM_POS     ("UCI")
#define CMD_DISABLE_LIM_SOFT    ("SLD")

// used in wakeup_toggle()
#define ASLEEP (1)


/*  headers
------------------------------------------------------------------------------*/
#include "smart_motor.h"


/*  typedefs
------------------------------------------------------------------------------*/


/*  globals 
------------------------------------------------------------------------------*/
enum {
    EXCESS_CURRENT = 0x100,
    SERVO = 0x80,
    EXCESS_HEAT = 0x40,
    EXCESS_P_ERR = 0x20,
    WRAPAROUND = 0x10,
    INDEX_AVAILABLE = 0x8,
    LIM_NEG = 0x4,
    LIM_POS = 0x2,
    MOVING = 0x1
};


/*  prototypes
------------------------------------------------------------------------------*/
char* send_sm_cmd (Smart_Motor *sm, char *cmd);
char* send_sm_val (Smart_Motor *sm, char *var, long val);
int axis_pv (Axis *ax);
int axis_status (Axis *ax);
int report_long (Axis *ax);
int report_short (Axis *ax);
void send_axis_cmd (Axis *ax, char *cmd);
void send_axis_val (Axis *ax, char *var, long val);
int wakeup_toggle (Axis *ax);


/* 
------------------------------------------------------------------------------*/
#ifdef SMART_MOTOR_PROTECTED_H_TEST
int main (void)
{
    return 0;
}
#endif

#ifdef __cplusplus
    } 
#endif

#endif /* SMART_MOTOR_PROTECTED_H */
