
/*
		
		benchVar.c

   This file has the functions to create variables for the gwc
   client that relate to the bench spectrograph.

*/

#include <tk.h>
#include <assert.h>
#include <gwc/gwc.h>

#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "kaboom.h"
#include "nesslib.h"
#include "concen.h"
#include "xref.h"

int BenchCable = 0;
static int benchCableUpdate (gwcEventArgs args);

/* local prototypes */
void doBenchSetup (gwcConnectInfo info);
static int benchCableUpdate (gwcEventArgs args);
int benchHydraCableMismatch (void);

/* Setup the Hydra send database */

void doBenchSetup (gwcConnectInfo info)
{

    /* subscribe to the rotator */

    if (gwcStreamSubscribe (info, "bsa.main") != TCL_OK)
    {
        fputs ("WARNING: Subscribe to bench info failed\n", stderr);
        return;
    }

    if (gwcAddEvent (info, "bsa.main.fibername",
                     EV_CHANGED, benchCableUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for bench status failed\n",
               stderr);
        return;
    }
}

/* callback handler for rotator */

static int benchCableUpdate (gwcEventArgs args)
{

    char **value = (char **) args.value;

    fprintf (stderr, "New bench cable: %s\n", value[0]);

    if ((BenchCable = getCableIdentifier (value[0])) == CurrentCable)
    {
        fputs ("Hey! Bench and Hydra now agree on current cable! :) \n",
               stderr);
    }
    else
    {
        fputs ("WARNING: Bench and Hydra do not agree on current cable!!!\n",
               stderr);
    }

    return TCL_OK;
}

int benchHydraCableMismatch (void)
{
    return (BenchCable != CurrentCable);
}
