
/*
		
		fieldVar.c

   This file has the functions to create variables for the gwc
   client that relate to the hydra field currently configured 

*/

#include <stdlib.h>
#include <tk.h>
#include <assert.h>

#include "gwc/gwc.h"
#include "miscMath.h"
#include "motor.h"
#include "plate.h"
#include "kaboom.h"
#include "irafIO.h"
#include "slitVar.h"

/* hydra fiber data */
static gwcVid fieldInfo[6];
static char *FieldCoordBundle[6];

/* local function prototypes */
void bundleInitField (void);
void doFieldSetup (gwcConnectInfo info);
void setVarField (gwcConnectInfo info);
void setVarCable (gwcConnectInfo info);
void setVarCableSize (gwcConnectInfo info);
static void packFieldCoord (char *bundle[]);

/* function to create a static array of strings to 
   hand to GWC */

void bundleInitField (void)
{
    int i;

    for (i = 0; i < FIELD_BUNDLE_SIZE; i++)
    {

        FieldCoordBundle[i] = (char *) malloc (MAX_VALUE_SIZE);
    }
}

/* Setup the Hydra send database */

void doFieldSetup (gwcConnectInfo info)
{

    fieldInfo[0] = gwcNewLocalVariable (info,
                                        "hydra.field.fldname", ENG_STRING,
                                        "FLDNAME", 1, SCOPE_PUBLIC);

    fieldInfo[1] = gwcNewLocalVariable (info,
                                        "hydra.field.fldtime", ENG_STRING,
                                        "FLDTIME", 1, SCOPE_PUBLIC);

    fieldInfo[2] = gwcNewLocalVariable (info,
                                        "hydra.field.fldlst", ENG_STRING,
                                        "FLDLST", 1, SCOPE_PUBLIC);

    fieldInfo[3] = gwcNewLocalVariable (info,
                                        "hydra.field.fldcoord", ENG_STRING,
                                        "FLDCOORD", 3, SCOPE_PUBLIC);

    fieldInfo[4] = gwcNewLocalVariable (info,
                                        "hydra.field.cable", ENG_STRING,
                                        "FLDCABLE", 1, SCOPE_PUBLIC);

    fieldInfo[5] = gwcNewLocalVariable (info,
                                        "hydra.field.cablesize", ENG_INT,
                                        "CABLESIZE", 1, SCOPE_PUBLIC);

    if (gwcStreamSubscribe (info, "hydra.field") != TCL_OK)
    {
        fputs ("WARNING: Subscribe to our own field info failed\n", stderr);
        return;
    }
}

void setVarField (gwcConnectInfo info)
{

    char *temp;
    int temp2;

    /* write to the gwc variable */

    temp = packFieldName ();
    gwcSetVarStringValue (fieldInfo[0], 1, &temp);

    temp = packFieldTime ();
    gwcSetVarStringValue (fieldInfo[1], 1, &temp);

    temp = packFieldLST ();
    gwcSetVarStringValue (fieldInfo[2], 1, &temp);

    packFieldCoord (FieldCoordBundle);

    gwcSetVarStringValue (fieldInfo[3], 3, FieldCoordBundle);

    temp = packCable ();
    gwcSetVarStringValue (fieldInfo[4], 1, &temp);

    temp2 = packCableSize ();
    gwcSetVarIntValue (fieldInfo[5], 1, &temp2);
}

void setVarCable (gwcConnectInfo info)
{

    char *temp;

    /* write to the gwc variable */

    temp = packCable ();
    gwcSetVarStringValue (fieldInfo[4], 1, &temp);
}

void setVarCableSize (gwcConnectInfo info)
{

    int temp;

    /* write to the gwc variable */

    temp = packCableSize ();

    /* fprintf( stderr, "cablesize = %d\n", temp ); */

    gwcSetVarIntValue (fieldInfo[5], 1, &temp);
}

static void packFieldCoord (char *bundle[])
{

    printFieldCenter (bundle[0], bundle[1], bundle[2]);

    /* fill in dummy coords if necessary */

    if (*bundle[2] == '\0')
    {
        sprintf (bundle[2], "00 00 00.000");
    }
    if (*bundle[3] == '\0')
    {
        sprintf (bundle[3], "00 00 00.00");
    }

}
