
/*
		
		fopsVar.c

   This file has the functions to create variables for the gwc
   client that relate to the locations of the fops.  It also
   has the functions to pack up and ship off the report for
   the fops.

*/

#include <tk.h>
#include <assert.h>

#include "gwc/gwc.h"
#include "miscMath.h"
#include "motor.h"
#include "plate.h"
#include "kaboom.h"
#include "nesslib.h"
#include "xref.h"
#include "fopsVar.h"

/* hydra fiber data */
static gwcVid fopInfo[NUM_FOPS];

extern int tellWhoeverCares (struct fiber_dbase *fib);

/* local prototypes */
void doFOPsetup (gwcConnectInfo info);
void setVarFOPall (fibDB fibers[]);
void setVarFOP (gwcConnectInfo info, struct fiber_dbase *fib);
void packAfop (struct fiber_dbase *fop, float bundle[]);

/* Setup the Hydra send database */

void doFOPsetup (gwcConnectInfo info)
{
    char i;
    char varName[32];
    char fitsName[32];

    /* HYDRA variables */

    for (i = 'A'; (int) (i - 'A') < NUM_FOPS; i++)
    {

        sprintf (varName, "hydra.guider.fop%c", i);
        sprintf (fitsName, "FOP%c", i);

        fopInfo[(int) (i - 'A')] = gwcNewLocalVariable (info,
                                                        varName, ENG_FLOAT,
                                                        fitsName,
                                                        RECORD_SIZE_FOPS,
                                                        SCOPE_PUBLIC);

        if (gwcStreamSubscribe (info, varName) != TCL_OK)
        {
            fputs ("WARNING: Subscribe to fops info failed\n", stderr);
            return;
        }
    }
}

void setVarFOPall (fibDB fibers[])
{
    int i;

    for (i = 0; i < MAXFIBER; i++)
    {
        if (fiberIsFop (i))
        {
            tellWhoeverCares (&fibers[i]);
        }
    }
}

void setVarFOP (gwcConnectInfo info, struct fiber_dbase *fib)
{

    float bundle[RECORD_SIZE_FOPS];

    int fopOrdinal;

    if ((fopOrdinal = fibToFop (fib)) < 0)
    {
        fputs ("GWC: attempt to set fop with non fop\n", stderr);
    }
    /* put the values we need into an array */

    packAfop (fib, bundle);

    /* write to the gwc variable */

    gwcSetVarFloatValue (fopInfo[fopOrdinal], RECORD_SIZE_FOPS, bundle);

    assert (info != (gwcConnectInfo) NULL);

    gwcPutStream (info, "hydra", STREAM_CHANGED);
}

void packAfop (struct fiber_dbase *fop, float bundle[])
{

    /* coordinates of the fiber in the x/y coord system */

    bundle[0] = fop->CurGeometry->button.x;
    bundle[1] = fop->CurGeometry->button.y;

    /* distance from origin to button */

    bundle[2] = fop->CurGeometry->d2;

    /* angle between fiber 0 pivot and this fiber's pivot */

    bundle[3] = fop->CurGeometry->theta;

    /* angle between fiber 0 pivot and this fiber's button */

    bundle[4] = fop->CurGeometry->phi;

    /* angle between this fiber's radial line and the tube
       at the button */

    bundle[5] = fop->CurGeometry->chi;

    /* status of the fop */

    if (fiberIsAssignedFop (fop->fiber_id, Stars, FiBase))
    {

        if (fiberIsAssignedSky (fop->fiber_id, Stars, FiBase))
        {
            bundle[6] = FOP_ASSIGNED_SKY;
        }
        else
        {
            bundle[6] = FOP_ASSIGNED_OBJ;
        }
    }
    else
    {
        bundle[6] = FOP_RANDOM_SKY;
    }

/*
   fprintf( stderr, "fopsBundle[0] %f\n", bundle[0] );
   fprintf( stderr, "fopsBundle[1] %f\n", bundle[1] );
   fprintf( stderr, "fopsBundle[2] %f\n", bundle[2] );
   fprintf( stderr, "fopsBundle[3] %f\n", bundle[3] );
   fprintf( stderr, "fopsBundle[4] %f\n", bundle[4] );
   fprintf( stderr, "fopsBundle[5] %f\n", bundle[5] );
   fprintf( stderr, "fopsBundle[6] %f\n\n", bundle[6] );
*/
}
