
/* 
 * hydraTkAppInit.c --
 * 
 */

#include <tk.h>
#include <assert.h>

#include "gwc/gwc.h"
#include "miscMath.h"
#include "motor.h"
#include "plate.h"
#include "kaboom.h"
#include "concen.h"
#include "fopsVar.h"
#include "tkInterface.h"

/* a global from the main hydra program */

extern int noGWC;

/***************************************/

/*
 * The following variable is a special hack that allows applications
 * to be linked using the procedure "main" from the Tk library.  The
 * variable generates a reference to "main", which causes main to
 * be brought in from the library (and all of Tk and Tcl with it).
 */
extern int main ();
int *tclDummyMathPtr = (int *) matherr;
int *tclDummyMainPtr = (int *) main;

int gwcAppInit (Tcl_Interp * interp);
extern void bundleInit (void);
extern void bundleInitField (void);
extern void doSlitSetup (gwcConnectInfo info);
extern void doFieldSetup (gwcConnectInfo info);
extern void doRotatorSetup (gwcConnectInfo info);
extern void doBenchSetup (gwcConnectInfo info);
extern void doThermocoupleSetup (gwcConnectInfo info);
extern int doRotatorInitialValue (gwcConnectInfo info);
extern int doRotatorInitialOffset (gwcConnectInfo info);
extern int doRotatorInitialEncoderstate (gwcConnectInfo info);
extern int doRotatorInitialAmpStatus (gwcConnectInfo info);
extern int doThermocoupleInitialValue (gwcConnectInfo info);
extern void setVarFOPall (fibDB * fibers);
extern void setVarSCS (gwcConnectInfo info, int slitPosition);
extern void setVarSlit (gwcConnectInfo info, int slitPosition);
extern void setVarField (gwcConnectInfo info);
extern void setVarCable (gwcConnectInfo info);
extern void setVarCableSize (gwcConnectInfo info);

/* local prototypes */
int Tcl_AppInit (Tcl_Interp * interp);
static int cmdSetup (ClientData data, Tcl_Interp * interp, int argc,
                     char *argv[]);
static void gwcHydraConnect (gwcConnectInfo info);
static int doConnection (Tcl_Interp * interp, gwcConnectInfo * hostInfo);
static int doSubscribe (gwcConnectInfo info);
static int createVariables (gwcConnectInfo info);
int initVariables (void);
static int makeGWCConnections (Tcl_Interp * interp);
int testTclInterface (Tcl_Interp * interp);
int gwcAppInit (Tcl_Interp * interp);
static int streamFunc (gwcEventArgs args);
int cmdSend (ClientData data, Tcl_Interp * interp, int argc, char *argv[]);
int cmdSendAll (ClientData data, Tcl_Interp * interp, int argc, char *argv[]);
int tellWhoeverCares (struct fiber_dbase *fib);
int sendIRAFstuff (void);
int sendFieldStuff (void);
int sendCableStuff (void);
int sendAll (ClientData data);

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *	
 *	This particular file is for hydra/gwc stuff.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int Tcl_AppInit (Tcl_Interp * interp)
{
    Tk_Window main;

    main = Tk_MainWindow (interp);

    /*
     * Call the init procedures for included packages.  Each call should
     * look like this:
     *
     * if (Mod_Init(interp) == TCL_ERROR) {
     *     return TCL_ERROR;
     * }
     *
     * where "Mod" is the name of the module.
     */
    if (Tcl_Init (interp) == TCL_ERROR)
    {
        return TCL_ERROR;
    }
    if (main && Tk_Init (interp) == TCL_ERROR)
    {
        return TCL_ERROR;
    }

    /* Call the Generic WIYN Client Init */
    if (noGWC == FALSE)
    {
        if (gwcAppInit (interp) == TCL_ERROR)
        {
            return TCL_ERROR;
        }
    }
    /*
     * Specify a user-specific startup file to invoke if the application
     * is run interactively.  Typically the startup file is "~/.apprc"
     * where "app" is the name of the application.  If this line is deleted
     * then no user-specific startup file will be run under any conditions.
     */

    /*tcl_RcFileName = "~/.wishrc"; */
    return TCL_OK;
}

static Tcl_CmdProc cmdSetup;

/* In file prototypes */
static int doConnection (Tcl_Interp * interp, gwcConnectInfo * hostInfo);
static int makeGWCConnections (Tcl_Interp * interp);
static int createVariables (gwcConnectInfo info);
static gwcInfoFunc streamFunc;
static void gwcHydraConnect (gwcConnectInfo info);

int cmdSend (ClientData data, Tcl_Interp * interp, int argc, char *argv[]);

int cmdSendAll (ClientData data, Tcl_Interp * interp, int argc, char *argv[]);

/* In file data */
gwcConnectInfo hostInfo;

 /*ARGSUSED*/
    static int cmdSetup (ClientData data, Tcl_Interp * interp,
                         int argc, char *argv[])
{
    gwcConnectInfo info = (gwcConnectInfo) data;

    assert (info != (gwcConnectInfo) NULL);

    return TCL_OK;

}

static void gwcHydraConnect (gwcConnectInfo info)
{
    gwcPrintStatus (info, "Final Hydra Connection established.");
}

/* Create a connection */
static int doConnection (Tcl_Interp * interp, gwcConnectInfo * hostInfo)
{
    return gwcInit (interp, "wiyn", "hydra",
                    (gwcConnectFunc *) gwcHydraConnect, NULL,
                    hostInfo, CONNECT_YES);
}

static int doSubscribe (gwcConnectInfo info)
{
    /* First make subscriptions */
    if (gwcStreamSubscribe (hostInfo, "hydra.all") != TCL_OK)
    {
        fprintf (stderr, "Subscribe failed\n");
        return TCL_ERROR;
    }
    return TCL_OK;
}

static int createVariables (gwcConnectInfo info)
{
    assert (info != (gwcConnectInfo) NULL);

    bundleInit ();              /* used by doSlitSetup */
    bundleInitField ();

    /* create the various variables */

    doFOPsetup (info);
    doSlitSetup (info);
    doFieldSetup (info);
    doRotatorSetup (info);
    doBenchSetup (info);
    doThermocoupleSetup (info);

    doRotatorInitialValue (info);
    doRotatorInitialOffset (info);
    doRotatorInitialEncoderstate (info);
    doRotatorInitialAmpStatus (info);

    doThermocoupleInitialValue (info);

    setVarFOPall (FiBase);

    /* doMiscSetup( info ); */

    return TCL_OK;
}

int initVariables (void)        /* uses local global */
{
    if (noGWC == TRUE)
        return TCL_OK;

    /* doRotatorInitialValue( hostInfo ); */

    return TCL_OK;
}

static int makeGWCConnections (Tcl_Interp * interp)
{
    /* Make a connection to the wiyn system */
    if (doConnection (interp, &hostInfo) == TCL_ERROR)
    {
        /* Messages provided by library */
        noGWC = TRUE;
        return TCL_ERROR;
    }

    /* Subscribe to hydra data */
    if (doSubscribe (hostInfo) == TCL_ERROR)
    {
        /* Messages provided by library */
        noGWC = TRUE;
        return TCL_ERROR;
    }

    return TCL_OK;
}

int testTclInterface (Tcl_Interp * interp)
{
    Tcl_CreateCommand (interp, "setup", cmdSetup, (ClientData) hostInfo,
                       (Tcl_CmdDeleteProc *) NULL);
    return TCL_OK;
}

int gwcAppInit (Tcl_Interp * interp)
{

    /* Initialize the interpreter */

    if (gwcTclInit (interp) == TCL_ERROR)
    {
        return TCL_ERROR;
    }

    /* Test the C interface */

    if (noGWC == FALSE)
    {

        if (makeGWCConnections (interp) == TCL_ERROR)
        {
            return TCL_ERROR;
        }

        /* create the variables */

        if (createVariables (hostInfo) == TCL_ERROR)
        {
            return TCL_ERROR;
        }
    }

    return TCL_OK;
}

static int streamFunc (gwcEventArgs args)
{
    fprintf (stderr, "\nStream name: %s\n", args.attribute);
    return TCL_OK;
}

/* 
 * this stuff used to be in hydra.c
 * 
 */

/* A command to send just changed items */
 /*ARGSUSED*/
    int cmdSend (ClientData data, Tcl_Interp * interp, int argc, char *argv[])
{
    gwcConnectInfo info = (gwcConnectInfo) data;

    assert (info != (gwcConnectInfo) NULL);

    gwcPutStream (info, "hydra", STREAM_CHANGED);

    return TCL_OK;
}

/* A command to send all stream data */
 /*ARGSUSED*/
    int cmdSendAll (ClientData data, Tcl_Interp * interp,
                    int argc, char *argv[])
{
    gwcConnectInfo info = (gwcConnectInfo) data;

    assert (info != (gwcConnectInfo) NULL);

    gwcPutStream (info, "hydra.guider", STREAM_ALL);
    gwcPutStream (info, "hydra.assign", STREAM_ALL);
    gwcPutStream (info, "hydra.field", STREAM_ALL);
    gwcPutStream (info, "hydra.gripper", STREAM_ALL);
    gwcPutStream (info, "hydra.misc", STREAM_ALL);

    return TCL_OK;
}

/* A command to send a fops */

int tellWhoeverCares (struct fiber_dbase *fib)
{

    /* uses global "hostInfo" */

    if (noGWC)
        return TCL_OK;

    setVarFOP (hostInfo, fib);

    gwcPutStream (hostInfo, "hydra.guider", STREAM_CHANGED);

    return TCL_OK;
}

/* A command to send a slit record */

int sendIRAFstuff (void)
{

    int i;
    int maxSlit;

    maxSlit = getMaxSlitInCable (CurrentCable);

    if (noGWC)
        return TCL_OK;

    setVarSCS (hostInfo, 1);

    for (i = 2; i <= maxSlit; i++)
    {

        /* fprintf( stderr, "sending slit %3d\n", i ); */

        /* uses global "hostInfo" */

        setVarSlit (hostInfo, i);
    }

    setVarSCS (hostInfo, (maxSlit + 1));

    gwcPutStream (hostInfo, "hydra.assign", STREAM_CHANGED);

    while (Tk_DoOneEvent (TK_DONT_WAIT) != 0)
    {
        /* next line is just for testing */
        /* fputs( "Tick...\n", stderr ); */
    }

    setVarField (hostInfo);

    gwcPutStream (hostInfo, "hydra.field", STREAM_CHANGED);

    /* Tk_DoOneEvent( TK_DONT_WAIT ); */

    while (Tk_DoOneEvent (TK_DONT_WAIT) != 0)
    {
        /* next line is just for testing */
        /* fputs( "Tick...\n", stderr ); */
    }

    return TCL_OK;
}

int sendFieldStuff (void)
{
    /* uses global "hostInfo" */

    if (noGWC)
        return TCL_OK;

    setVarField (hostInfo);

    gwcPutStream (hostInfo, "hydra.field", STREAM_CHANGED);

    return TCL_OK;
}

int sendCableStuff (void)
{
    /* uses global "hostInfo" */

    if (noGWC)
        return TCL_OK;

    setVarCable (hostInfo);
    setVarCableSize (hostInfo);

    gwcPutStream (hostInfo, "hydra.field", STREAM_CHANGED);

    return TCL_OK;
}

/* A non-tcl command to send changes */
 /*ARGSUSED*/ int sendAll (ClientData data)
{
    gwcConnectInfo info = (gwcConnectInfo) data;

    assert (info != (gwcConnectInfo) NULL);

    gwcPutStream (info, "hydra.guider", STREAM_ALL);
    gwcPutStream (info, "hydra.assign", STREAM_ALL);
    gwcPutStream (info, "hydra.field", STREAM_ALL);
    gwcPutStream (info, "hydra.gripper", STREAM_ALL);
    gwcPutStream (info, "hydra.misc", STREAM_ALL);

    return TCL_OK;
}
