/* Generic WIYN Client 
 * File: Init
 *
 * Created: Kim Gillies
 * Date: 19 July 94 
 *
 * CAUTION: The TCL interface will only support one connection because
 *  it works off SIGIO just like the WIYN message system.
 */
#include <stdio.h>
#include <signal.h>
#include <sys/file.h>   /* Required for fcntl */
#include <assert.h>
#include <tcl.h>
#include <msg/msg.h>
#include <gwc/gwc.h>

/* External function for gwcInit */
int gwcSetupHandlers(gwcConnectInfo info);

/* In file prototypes */
static Tcl_AsyncHandler intHandler;
static void IntProc(int);
static int IntHandlerProc (ClientData clientData, 
                             Tcl_Interp *interp, int code);

int gwcSetupHandlers(gwcConnectInfo info)
{
  int sock;
  assert(info != (gwcConnectInfo)NULL);

  sock = gwcGetClientSocket(info);

  intHandler = Tcl_AsyncCreate(IntHandlerProc, (ClientData)info);
  /* Set the SIGIO handler */
  signal(SIGIO, IntProc);

  /* allow sigio signals to this process */
  if (fcntl(sock, F_SETOWN, getpid()) == -1) {
    fprintf(stderr, "SETOWN failed \n");
    return TCL_ERROR;
  }

  /* non-blocking, SIGIO enabled */
  if (fcntl(sock, F_SETFL, O_NDELAY|FASYNC) == -1) {
    fprintf(stderr, "F_SETFL failed \n");
    return TCL_ERROR;
  }

  return TCL_OK;
}

/*ARGSUSED*/
static void IntProc(int ignored)
{
  Tcl_AsyncMark(intHandler);
}

static int IntHandlerProc(ClientData clientData, Tcl_Interp *interp, int code)
{
  int error;
  gwcConnectInfo intInfo;

  /* Set the global connection info */
  intInfo = (gwcConnectInfo)clientData;

  /* Call the msg system */
  error = gwcGetProcessEDM(intInfo);

  signal(SIGIO, IntProc);
  if (error != 0) fprintf(stderr, "Handler Error; %d\n", error);
  return code;
}
