#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <signal.h>
#include <errno.h>

#include <tcl.h>
#include <tk.h>
#include "gwc.h"

extern int main();
extern int matherr();

int *tclDummyMathPtr =  (int *) matherr;
int *tclDummyMainPtr =  (int *) main;

/* In file prototypes */

        int Tcl_AppInit ( Tcl_Interp *interp );
        int gwcAppInit  ( Tcl_Interp *interp );
static  int GetPosition ( gwcEventArgs args );
        int SetPID      ( void );


/* In file data */

static gwcConnectInfo   hostInfo;
static int              Armed;
static int              hydraPid;


/* -------------------------------------------------------------------------- */

int Tcl_AppInit(Tcl_Interp *interp)
{
    Tk_Window main;

    Armed = 1;

    main = Tk_MainWindow(interp);

    if (Tcl_Init(interp) == TCL_ERROR)
        return TCL_ERROR;

    if (main && Tk_Init(interp) == TCL_ERROR)
        return TCL_ERROR;

    /* Call the Generic WIYN Client Init */

    if (gwcAppInit(interp) == TCL_ERROR)
        return TCL_ERROR;

    tcl_RcFileName = "~/.wishrc";

    return TCL_OK;
}


int gwcAppInit(Tcl_Interp *interp)
{

    extern int cmdClient(ClientData data, Tcl_Interp *interp, int argc, char *argv[]);

    /* Initialize the interpreter */

    if (gwcTclInit(interp) == TCL_ERROR) {
        fprintf(stderr,"Could not initialize interpreter\n");
        return TCL_ERROR;
        }

    fprintf(stderr,"Interpreter active\n");

    /* Make a connection to the system */

    if (gwcInit(interp, "wiyn", &hostInfo, CONNECT_YES) == TCL_ERROR) {
        fprintf(stderr,"Could not attach to router\n");
        return TCL_ERROR;
        }

    fprintf(stderr,"Connected to router\n");

    if (SetPID() == TCL_ERROR)
        return TCL_ERROR;
    
    if (gwcStreamSubscribe(hostInfo,"tcs.telescope") != TCL_OK) {
        fprintf(stderr,"Could not subscribe to TCS telescope data\n");
        return TCL_ERROR;
        }

    fprintf(stderr,"Listening to tcs.telescope\n");

    if (gwcAddEvent(hostInfo,
                    "tcs.telescope",
                    "mnir",
                    EV_ALWAYS,
                    GetPosition,
                    NULL,
                    (ClientData)hostInfo) != TCL_OK) {
        fprintf(stderr,"TCS add event failed\n");
        return TCL_ERROR;
        }

    fprintf(stderr,"Monitor ready for action\n");

    return TCL_OK;
}


/* -------------------------------------------------------------------------- */

/* callback to obtain the current rotator position */

static int GetPosition(gwcEventArgs args)
{
    float *rpos;

    rpos = (float *) args.value;

    fprintf(stderr,"EVENT: %s: %f\n",args.attribute,*rpos);

    if (*rpos != 0.0) {
        if (Armed) {
            kill(hydraPid,SIGUSR1);
            Armed = 0;
            }
        }
    else
        Armed = 1;

    return TCL_OK;
}


/* get Hydra's PID from the environment */

int SetPID(void)
{
    char tstr[16];

    if (getenv("hydra.pid")) {
        strcpy(tstr,getenv("hydra.pid"));
        hydraPid = atoi(tstr);
        return TCL_OK;
        }
    else {
        fprintf(stderr,"Unable to locate Hydra PID\n");
        return TCL_ERROR;
        }
}








