
/*
		
		rotatorVar.c

   This file has the functions to create variables for the gwc
   client that relate to the instrument rotator .  It also
   has the callback for the rotator moving.

*/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/scrollbar.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include "xvFibSim_ui.h"

#include <assert.h>
#include <gwc/gwc.h>
#include <miscMath.h>
#include <motor.h>
#include <plate.h>
#include <kaboom.h>
#include <nesslib.h>

#define ROTATOR_LIMIT  0.0175   /* one degree */

/* hydra fiber data */

static double RotatorAngle = 0.0;

static int rotatorUpdate (gwcEventArgs args);
static int offsetUpdate (gwcEventArgs args);
static int trackerUpdate (gwcEventArgs args);

/* Setup the Hydra send database */

void doRotatorSetup (gwcConnectInfo info)
{

    /* subscribe to the rotator */

    if (gwcStreamSubscribe (info, "tcs.mnir") != TCL_OK)
    {
        fputs ("WARNING: Subscribe to tcs info failed\n", stderr);
        return;
    }

    /* subscribe to the rotator position */

    if (gwcAddEvent (info, "tcs.mnir.plantpos",
                     EV_CHANGED, rotatorUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for rotator angle failed\n",
               stderr);
        return;
    }

    /* subscribe to the rotator position */

    if (gwcAddEvent (info, "tcs.mnir.tracker",
                     EV_CHANGED, trackerUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for rotator angle failed\n",
               stderr);
        return;
    }

    /* subscribe to the rotator position */

    if (gwcAddEvent (info, "tcs.mnir.offset",
                     EV_CHANGED, offsetUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for rotator angle failed\n",
               stderr);
        return;
    }
}

doRotatorInitialValue (gwcConnectInfo info)
{
    valueType type;
    void *value;
    int count;

    sleep (10);

    if (gwcGetAttrValue (info, "tcs.mnir.plantpos", &type, &count, &value) !=
        TCL_OK)
    {

        /*
           fputs( "WARNING: Couldn't get rotator value!\n", stderr );
           fputs( "         Please manually verify rotator position.\n", stderr );
         */
    }
    else
    {
        if (count != 0)
        {
            RotatorAngle = *(double *) value;
        }
        else
        {
            /*fputs( "WARNING: Zero count for initial rotator value!\n", stderr ); */
        }
    }

    if (gwcGetAttrValue (info, "tcs.mnir.offset", &type, &count, &value) !=
        TCL_OK)
    {

        fputs ("WARNING: Couldn't get rotator offset value!\n", stderr);
        fputs ("         Please manually verify rotator position.\n", stderr);
    }
    else
    {
        if (count != 0)
        {
            /* fprintf ( stderr, "Initial rotator offset: %f\n", *(double*)value ); */
        }
        else
        {
            /* fputs( "WARNING: Zero count for initial rotator offset!\n", stderr ); */
        }
    }

    return (TCL_OK);
}

/* callback handler for rotator */

static int rotatorUpdate (gwcEventArgs args)
{

    char temp[32];

    double *value;

    value = (double *) args.value;

    RotatorAngle = value[0];

    /* fprintf ( stderr, "New rotator angle of %f\n", RotatorAngle ); */

    sprintf (temp, "%7.2f", (RotatorAngle / DEG2RAD));

    /* use the info here */

    return TCL_OK;
}

int rotatorNotZero (void)
{
    return (abs (RotatorAngle) > ROTATOR_LIMIT);
}

static int trackerUpdate (gwcEventArgs args)
{

    char status[8];

    int *value;

    value = (int *) args.value;

    if (value[0] == 0)
        sprintf (status, "OFF");
    else
        sprintf (status, "ON");

    /* use the info here */

    return TCL_OK;
}

static int offsetUpdate (gwcEventArgs args)
{
    char temp[32];

    double *value;

    value = (double *) args.value;

    fputs ("offset Update \n", stderr);

    sprintf (temp, "%8.3f", (value[0] / DEG2RAD));

    /* use the info here */

    return TCL_OK;
}

/* function to create a static array of strings to 
   hand to GWC */
