
/*
		
		slitVar.c

   This file has the functions to create variables for the gwc
   client that relate to the assignment of objects to fibers. 
   It also has the functions to pack up and ship off the report for
   the assignments.

*/

#include <string.h>
#include <stdlib.h>
#include <tk.h>
#include <assert.h>

#include "gwc/gwc.h"
#include "miscMath.h"
#include "motor.h"
#include "plate.h"
#include "kaboom.h"
#include "concen.h"
#include "nessIO.h"

/* hydra fiber data */

#define RECORD_SIZE_SLIT 5

static gwcVid slitInfo[MAX_SLITVARS_WE_COULD_EVER_NEED + 1];

static char *SlitBundle[RECORD_SIZE_SLIT];

/* local function prototypes */
void bundleInit (void);
void doSlitSetup (gwcConnectInfo info);
void setVarSlit (gwcConnectInfo info, int slitPosition);
void setVarSCS (gwcConnectInfo info, int slitPosition);
static void packASlit (int slitPosition, char *bundle[]);
static void packAnSCS (int slitPosition, char *bundle[]);

/* function to create a static array of strings to 
   hand to GWC */

void bundleInit (void)
{
    int i;

    for (i = 0; i < RECORD_SIZE_SLIT; i++)
    {

        SlitBundle[i] = (char *) malloc (MAX_VALUE_SIZE);
    }
}

/* Setup the Hydra send database */

void doSlitSetup (gwcConnectInfo info)
{
    int i;
    char fitsName[32];

    if (gwcStreamSubscribe (info, "hydra.assign") != TCL_OK)
    {
        fputs ("WARNING: Subscribe to our own slit info failed\n", stderr);
        return;
    }

    /* HYDRA variables */

    for (i = 0; i <= MAX_SLITVARS_WE_COULD_EVER_NEED; i++)
    {

        sprintf (fitsName, "hydra.assign.SLFIB%d", i);

        slitInfo[i] = gwcNewLocalVariable (info,
                                           fitsName, ENG_STRING,
                                           fitsName, RECORD_SIZE_SLIT,
                                           SCOPE_PUBLIC);

    }

}

void setVarSlit (gwcConnectInfo info, int slitPosition)
{

    /* put the values we need into an array */

    packASlit (slitPosition, SlitBundle);

    /* write to the gwc variable */

    gwcSetVarStringValue (slitInfo[slitPosition],
                          RECORD_SIZE_SLIT, SlitBundle);
}

void setVarSCS (gwcConnectInfo info, int slitPosition)
{

    /* put the values we need into an array */

    packAnSCS (slitPosition, SlitBundle);

    /* write to the gwc variable */

    gwcSetVarStringValue (slitInfo[slitPosition],
                          RECORD_SIZE_SLIT, SlitBundle);
}

static void packASlit (int slitPosition, char *bundle[])
{

    char dotIrafLine[64];

    char tmpString[64];

    int i;
    int j;

    printIRAFline (slitPosition, NULL, dotIrafLine);

    sscanf (dotIrafLine, "%s%s%[^$]", bundle[0], bundle[1], bundle[4]);

/*
   *bundle[2] = '\0';
   *bundle[3] = '\0';
*/

    printCoordPair (slitPosition, bundle[2], bundle[3]);

    /* the following is a major hack, but I don't feel like
       re-writing the whole thing to allow for colons */

    for (j = 2; j <= 3; j++)
    {

        for (i = 1; bundle[j][i] != '\0'; i++)
        {

            if ((bundle[j][i] == ' ') && (bundle[j][i + 1] != '\0'))
            {
                bundle[j][i] = ':';
            }
        }
    }

    /* clean up white space in a couple of fields */

    while (*bundle[3] == ' ')
    {
        strcpy (tmpString, (bundle[3] + 1));
        strcpy (bundle[3], tmpString);
    }

    while (*bundle[4] == ' ')
    {
        strcpy (tmpString, (bundle[4] + 1));
        strcpy (bundle[4], tmpString);
    }

    /* fill in dummy coords if necessary */

    if (*bundle[2] == '\0')
    {
        sprintf (bundle[2], "00:00:00.00");
    }
    if (*bundle[3] == '\0')
    {
        sprintf (bundle[3], "00:00:00.0");
    }

    /* fprintf( stderr, "%s'\n%s'\n%s'\n%s'\n%s'\n", bundle[0], bundle[1], bundle[2], bundle[3], bundle[4] ); */
}

static void packAnSCS (int slitPosition, char *bundle[])
{

    char dotIrafLine[64];

    printSCSline (slitPosition, NULL, dotIrafLine);

    sscanf (dotIrafLine, "%s%s%[^$]", bundle[0], bundle[1], bundle[4]);

    sprintf (bundle[2], "00:00:00.00");
    sprintf (bundle[3], "00:00:00.0");

    /* fprintf( stderr, "%s'\n%s'\n%s'\n%s'\n%s'\n", bundle[0], bundle[1], bundle[2], bundle[3], bundle[4] ); */
}
