
/*
		
		thermVar.c

   This file has the functions to create variables for the gwc
   client that relate to the instrument temperature.

*/

#include <tk.h>
#include <assert.h>

#include "gwc/gwc.h"
#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "kaboom.h"
#include "nesslib.h"
#include "concen.h"
#include "xref.h"

static double InstrumentTemperature = 0;
static int TimeOut;

/* local prototypes */
void doThermocoupleSetup (gwcConnectInfo info);
static int hydraThermocoupleUpdate (gwcEventArgs args);
double getInstrumentTemperature (void);
int doThermocoupleInitialValue (gwcConnectInfo info);
static void rotatorTimeoutFunction (void);

void doThermocoupleSetup (gwcConnectInfo info)
{

    /* subscribe to the temperature */

    if (gwcStreamSubscribe (info, "oss.thermocouple.hydra") != TCL_OK)
    {
        fputs ("WARNING: Subscribe to thermocouple info failed\n", stderr);
        return;
    }

    if (gwcAddEvent (info, "oss.thermocouple.hydra",
                     EV_CHANGED, hydraThermocoupleUpdate, NULL,
                     (ClientData) info) != TCL_OK)
    {

        fputs ("WARNING: Callback registration for thermocouple failed\n",
               stderr);
        return;
    }
}

/* callback handler for thermocouple */

static int hydraThermocoupleUpdate (gwcEventArgs args)
{

    double *value = (double *) args.value;

    /* fprintf ( stderr, "New thermocouple Value : %f\n", *value ); */

    return TCL_OK;
}

double getInstrumentTemperature (void)
{
    return (InstrumentTemperature);
}

int doThermocoupleInitialValue (gwcConnectInfo info)
{
    valueType type;
    void *value;
    int count = 0;

    Tk_TimerToken theTimer;

    TimeOut = 0;

    theTimer = Tk_CreateTimerHandler (5000,     /* milliseconds */
                                      (void (*)()) rotatorTimeoutFunction,
                                      (ClientData *) NULL);

    while (count == 0)
    {

        if (gwcGetAttrValue (info,
                             "oss.thermocouple.hydra", &type, &count,
                             &value) != TCL_OK)
        {

            fputs
                ("WARNING: Couldn't get hydra thermocouple initial value!\n",
                 stderr);
        }
        else
        {
            if (count != 0)
            {
                InstrumentTemperature = *(double *) value;
                fprintf (stderr, "Initial thermocouple reading : %f\n",
                         (1.8 * InstrumentTemperature) + 32);
            }
        }

        Tk_DoOneEvent (0);

        if (TimeOut)
        {

            fputs
                ("ERROR: Timed out trying to get hydra thermocouple value!\n",
                 stderr);
            break;
        }
    }

    Tk_DeleteTimerHandler (theTimer);

    return (TimeOut);
}

static void rotatorTimeoutFunction (void)
{
    TimeOut = 1;
}
