
/*

	complications.c

   A set of functions for collecting the effects of
   moving a fiber

*/

#include <stdio.h>
#include <stdlib.h>

#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "kaboom.h"
#include "fiblist.h"
#include "occult.h"
#include "complications.h"

/***************** local prototypes *******************/

/***************** end local prototypes *******************/

/***************** begin public functions *******************/

complications *getComplications (fibDB * targetFiber, fibDB otherFibers[])
{
    /* 
       This code is based on the function "DestOK" from 
       kaboomlib.  Instead of simply returning a boolean,
       however, it creates a list of fibers that are in
       the way of the target destination.  This list,
       along with the list of occulting fibers and 
       fibers that will be occulted make up the complete
       set of complications.
     */

    int i;
    int statusCode;

    complications *newComplications =
        (complications *) malloc (sizeof (complications));

    newComplications->inTheWay = listAlloc (NULL);

    /* see who's occulting us */

    newComplications->oppressors = occultCopy (targetFiber->occultedBy);
    /* newComplications->oppressors = targetFiber->occultedBy; */

    /* see of the range is OK */

    newComplications->coordCode = coordStatus (targetFiber, NO_RESTRICT);

    /* clear the pending occultation list in case it was set previously and not recorded */

    occultClearPending (targetFiber);

    /* loop through all other fibers and check for collisions */

    for (i = 0; i < MAXFIBER; i++)
    {
        if (i != targetFiber->fiber_id)
        {                       /* don't do current fiber! */

            statusCode = pairHits (targetFiber, &otherFibers[i]);

            if (statusCode != 0)
            {

                /* fprintf( stderr, "foo %d\n", i ); */

                /* add this fiber to the "in the way" list */

                listAdd (newComplications->inTheWay, &otherFibers[i]);

            }
        }
    }

    /*
       Now that we've gone through all the fibers, the 
       pending occultations are available, so link
       them in
     */

    newComplications->futureVictims = occultCopy (targetFiber->pendWeOccult);
    /* newComplications->futureVictims = targetFiber->pendWeOccult; */

    /* return a pointer to the newly malloc'd complications structure */

    /* printComplications( targetFiber, newComplications ); */

    return (newComplications);
}

void freeComplications (complications * theComplications)
{
    /* first free the "in the way" list */

    listFree (theComplications->inTheWay);

    free ((char *) theComplications);
}

void printComplications (fibDB * theFiber, complications * theComplications)
{
    fprintf (stderr, "For fiber %d, complications are: \n",
             theFiber->fiber_id);

    fputs ("  Fibers In the way:\n", stderr);
    listThelist (theComplications->inTheWay);

    fputs ("  Fibers Oppressing:\n", stderr);
    occultList2 (theComplications->oppressors);

    fputs ("  Future Victims   :\n", stderr);
    occultList2 (theComplications->futureVictims);

}

/***************** end public functions *******************/

/***************** begin local functions *******************/

/***************** end local functions *******************/
