
/*

		occult.c

   functions for creating and managing occultations of one
   fiber by another fiber.

*/

#include <stdio.h>
#include <stdlib.h>

#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "kaboom.h"
#include "fiblist.h"

/***************** local prototypes *******************/

static void appendListEntry (fiblist * oldEntries, fiblist * newEntry);

/***************** end local prototypes *******************/

/***************** begin public functions *******************/

/*
	listAlloc

   Create a listEntry and return a pointer to it
   after linking in the argument.
*/

fiblist *listAlloc (fibDB * theFiber)
{
    fiblist *newEntry;

    /* create the new one */

    newEntry = (fiblist *) malloc ((unsigned) sizeof (fiblist));

    newEntry->next = NULL;
    newEntry->prev = NULL;

    newEntry->fiber = theFiber;

    return (newEntry);
}

/*
	listFree

   Completely free the list pointed to by the argument.

*/

void listFree (fiblist * theList)
{
    fiblist *target;
    fiblist *next;

    target = theList;

    while (target != NULL)
    {

        next = target->next;
        free (target);
        target = next;
    }

}

/*
	listAdd

   Add the specified fiber to the specified list.
   Allocate the required memory.
*/

void listAdd (fiblist * theList, fibDB * theFiber)
{
    appendListEntry (theList, listAlloc (theFiber));
}

/*
	listRemove

   Remove the specified fiber from the specified list.
*/

void listRemove (fiblist * theList, fibDB * theFiber)
{
    fiblist *tmpPtr = theList;

    while (tmpPtr != NULL)
    {

        if (tmpPtr->fiber == theFiber)
        {

            tmpPtr->prev->next = tmpPtr->next;

            tmpPtr->next->prev = tmpPtr->prev;

            free ((char *) tmpPtr);

            return;
        }
        else
        {
            tmpPtr = tmpPtr->next;
        }
    }
}

/*
	listThelist

   Print the id's of the fibers in the specified list,
   one per line.
*/

void listThelist (fiblist * theList)
{
    fiblist *nextItem;

    nextItem = theList;

    while (nextItem != NULL)
    {

        if (nextItem->fiber == NULL)
        {

            /* skip the header record */

            /* fputs("     skipping header\n", stderr ); */

            nextItem = nextItem->next;
            continue;
        }

        fprintf (stderr, "    %3d\n", nextItem->fiber->fiber_id);

        nextItem = nextItem->next;
    }
}

/***************** end public functions *******************/

/***************** begin private functions *******************/

void appendListEntry (fiblist * oldEntries, fiblist * newEntry)
{

    fiblist *tailPtr;

    /* start at the beginning of the list */

    tailPtr = oldEntries;

    /* find the end of the list */

    while (tailPtr->next != NULL)
    {
        tailPtr = tailPtr->next;
    }

    /* now link in the new entry */

    /* set the forward link */

    tailPtr->next = newEntry;

    /* set the backward link */

    tailPtr->next->prev = tailPtr;
}
