#include <math.h>

/**
 *
 *     - - - - -
 *      a t m s
 *     - - - - -
 *
 *
 *  internal routine used by refro
 *
 *  model atmosphere for the stratosphere
 *
 *
 *  given:
 *    rt    dp    height of tropopause from centre of the earth (metre)
 *    tt    dp    temperature at the tropopause (deg k)
 *    dnt   dp    refractive index at the tropopause
 *    gmdr  dp    constant of the atmospheric model = g*md/r
 *    r     dp    current distance from the centre of the earth (metre)
 *
 *  returned:
 *    dn    dp    refractive index at r
 *    dndr  dp    rate the refractive index is changing at r
 *
 *  This routine is a version of the atmosstr routine (C.Hohenkerk,
 *  hmnao), with trivial modifications.
 *
 *  original version by c.hohenkerk, hmnao, august 1984
 *  starlink version by p.t.wallace, july 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_atms (double rt, double tt, double dnt, double gmdr, double r, double *dn,
          double *dndr)
{

    double b, c;

    b = gmdr / tt;
    c = exp (-b * (r - rt));
    *dn = 1e0 + (dnt - 1e0) * c;
    *dndr = -b * (dnt - 1e0) * c;
}
