#include <math.h>

/**
 *
 *     - - - - -
 *      a t m t
 *     - - - - -
 *
 *
 *  internal routine used by refro
 *
 *  model atmosphere for the troposphere
 *
 *
 *  given:
 *    rt    dp    height of tropopause from centre of the earth (metre)
 *    r0    dp    height of observer from centre of the earth (metre)
 *    t0    dp    temperature at the observer (deg k)
 *    a   dp(10)  constants defined at the observer
 *    r     dp    current distance from the centre of the earth (metre)
 *
 *  returned:
 *    t     dp    temperature at r (deg k)
 *    dn    dp    refractive index at r
 *    dndr  dp    rate the refractive index is changing at r
 *
 *  This routine is a version of the atmostro routine (C.Hohenkerk,
 *  hmnao), with trivial modifications.
 *
 *  original version by c.hohenkerk, hmnao, august 1984
 *  starlink version by p.t.wallace, july 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_atmt (double r0, double t0, double *a, double r, double *t, double *dn,
          double *dndr)
{
    double tt0, tt01, tt02;

    *t = t0 - a[0] * (r - r0);
    tt0 = *t / t0;
    tt01 = pow (tt0, a[2]);
    tt02 = pow (tt0, a[3]);
    *dn = 1e0 + (a[6] * tt01 - a[7] * tt02) * tt0;
    *dndr = -a[8] * tt01 + a[9] * tt02;
}
