
/**
 *
 *     - - - - - -
 *      c a l d j
 *     - - - - - -
 *
 *
 *  gregorian calendar to modified julian date
 *
 *  (includes century default feature:  use sla_cldj for years
 *   before 100AD.)
 *
 *
 *  given:
 *     iy,im,id     int    year, month, day in gregorian calendar
 *
 *  returned:
 *     djm          dp     modified julian date (jd-2400000.5) for 0 hrs
 *     j            int    status:
 *                           0 = ok
 *                           1 = bad year   (mjd not computed)
 *                           2 = bad month  (mjd not computed)
 *                           3 = bad day    (mjd computed)
 *
 *  Acceptable years are 00-49, interpreted as 2000-2049,
 *                       50-99,     "       "  1950-1999,
 *                       100 upwards, interpreted literally.
 *
 *  called:  sla_cldj
 *
 *
 *  p.t.wallace   starlink   november 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_cldj (int iy, int im, int id, double *djm, int *j);

sla_caldj (int iy, int im, int id, double *djm, int *j)
{

    int ny;

    /* default century if appropriate */
    if (iy >= 0 && iy <= 49)
    {
        ny = iy + 2000;
    }
    else if (iy >= 50 && iy <= 99)
    {
        ny = iy + 1900;
    }
    else
    {
        ny = iy;
    }

    /* modified julian date */
    sla_cldj (ny, im, id, djm, j);
}
