
/**
 *
 *     - - - - - -
 *      c a l y d
 *     - - - - - -
 *
 *
 *  calendar to year and day in year
 *
 *
 *  given:
 *     iy,im,id   int    year, month, day in gregorian calendar
 *                       (year may optionally omit the century)
 *  returned:
 *     ny         int    year ad
 *     nd         int    day in year (1 = january 1st)
 *     j          int    status:
 *                        -1 = ok, but outside range 1900/3/1-2100/2/28
 *                         0 = ok, and within above range
 *                         2 = bad month  (day not computed)
 *                         3 = bad day    (year,day both computed)
 *
 *  j=0 means that the date is within the range where the gregorian
 *  rule concerning century leap years can be neglected.
 *
 *
 *  p.t.wallace   starlink   april 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_calyd (int iy, int im, int id, int *ny, int *nd, int *j)
{

    int i, lj, lny, lnd;        /* temp - local variables */

    /* month lengths in days */
    /* keep 1 based system */
    static int mtab[13] = {
        0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
    };

    /* default century if appropriate */
    if (iy >= 0 && iy <= 49)
    {
        lny = iy + 2000;
    }
    else if (iy >= 50 && iy <= 99)
    {
        lny = iy + 1900;
    }
    else
    {
        lny = iy;
    }

    /* preset status */
    lj = 0;

    /* validate month */
    if (im >= 1 && im <= 12)
    {

        /* allow for leap year */
        if ((lny % 4) == 0)
        {
            mtab[2] = 29;
        }
        else
        {
            mtab[2] = 28;
        }
        if ((iy % 100) == 0 && (iy % 400) != 0)
            mtab[2] = 28;

        /* validate day */
        if (id < 1 || id > mtab[im])
            lj = 3;

        /* day in year */
        lnd = id;
        for (i = 1; i <= im - 1; i++)
        {
            lnd += mtab[i];
        }

        /* bad month */
    }
    else
    {
        lj = 2;
    }

    /* flag dates outside the range 1900 march 1 to 2100 february 28 */
    if (lj == 0 &&
        (lny < 1900 || lny > 2100 || (lny == 1900 &&
                                      lnd < 60) || (lny == 2100 && lnd > 59)))
        lj = -1;

    *ny = lny;
    *nd = lnd;
    *j = lj;
}
