#include <math.h>

/**
 *
 *     - - - - - -
 *      c c 6 2 s
 *     - - - - - -
 *
 *
 *  conversion of position & velocity in cartesian coordinates
 *  to spherical coordinates
 *
 *  (double precision)
 *
 *
 *  given:
 *     v     r(6)   cartesian position & velocity vector
 *
 *  returned:
 *     a     r      longitude (radians)
 *     b     r      latitude (radians)
 *     r     r      radial coordinate
 *     ad    r      longitude derivative (radians per unit time)
 *     bd    r      latitude derivative (radians per unit time)
 *     rd    r      radial derivative
 *
 *
 *  p.t.wallace   starlink   november 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_cc62s (double *v, double *a, double *b, double *r, double *ad, double *bd,
           double *rd)
                                                /* variables were REAL */
{

    /* variables were REAL */
    double x, y, z, xd, yd, zd, rxy2, rxy, r2, xyp;
    double temp;

    /* components of vector */
    x = v[0];
    y = v[1];
    z = v[2];
    xd = v[3];
    yd = v[4];
    zd = v[5];

    /* component of r in xy plane squared (with crude precaution */
    /* against polar problems and daft units) */
    rxy2 = x * x + y * y;
    temp = (xd * xd + yd * yd + zd * zd) / 1e30;
    rxy2 = (rxy2 > temp ? rxy2 : temp);

    /* other useful functions */
    rxy = sqrt (rxy2);
    r2 = rxy2 + z * z;
    xyp = x * xd + y * yd;

    /* position in spherical coordinates */
    *a = atan2 (y, x);
    *b = atan2 (z, rxy);
    *r = sqrt (r2);

    /* velocity in spherical coordinates */
    *ad = (x * yd - y * xd) / rxy2;
    *bd = (zd * rxy2 - z * xyp) / (r2 * rxy);
    *rd = (xyp + z * zd) / *r;
}
