#include <math.h>

/**
 *
 *     - - - - - -
 *      c d 2 t f
 *     - - - - - -
 *
 *
 *  break an interval in days into h,m,s fields for output
 *
 *  (single precision)
 *
 *
 *  given:
 *     ndp       int      number of decimal places of seconds
 *     days      real     interval in days
 *
 *  returned:
 *     sign      char     '+' or '-'
 *     ihmsf     int(4)   hours, minutes, seconds, fraction
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_cd2tf (int ndp, double days, char *sign, int *ihmsf)
                                 /* variable was REAL */
{

    /* days to seconds */
    /* define was REAL */
#define d2s  (double)86400.0
    double rs, rm, rh, a, ah, am, as, af;       /* variables were REAL */

    /* handle sign */
    *sign = '+';
    if (days < 0.0)
        *sign = '-';

    /* field units in terms of least significant figure */
    /* rs = pow (10, max(ndp, 0)) */
    rs = 1.0;
    while (ndp-- > 0)
        rs *= 10.0;
    rm = rs * 60.0;
    rh = rm * 60.0;

    /* round interval and express in smallest units required */
    a = rint (rs * d2s * fabs (days));

    /* separate into fields */
    ah = floor (a / rh);
    a = a - ah * rh;
    am = floor (a / rm);
    a = a - am * rm;
    as = floor (a / rs);
    af = a - as * rs;

    /* return results */
    ihmsf[0] = (int) ah;
    ihmsf[1] = (int) am;
    ihmsf[2] = (int) as;
    ihmsf[3] = (int) af;
}
