
/**
 *
 *     - - - - -
 *      c l d j
 *     - - - - -
 *
 *
 *  gregorian calendar to modified julian date
 *
 *
 *  given:
 *     iy,im,id     int    year, month, day in gregorian calendar
 *
 *  returned:
 *     djm          dp     modified julian date (jd-2400000.5) for 0 hrs
 *     j            int    status:
 *                           0 = ok
 *                           1 = bad year   (mjd not computed)
 *                           2 = bad month  (mjd not computed)
 *                           3 = bad day    (mjd computed)
 *
 *  The year must be -4699 (i.e. 4700bc) or later.
 *
 *  The algorithm is derived from that of Hatcher 1984
 *  (qjras 25, 53-55).
 *
 *
 *  p.t.wallace   starlink   december 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_cldj (int iy, int im, int id, double *djm, int *j)
{

    /* month lengths in days */
    /* keep 1 based calculation */
    static int mtab[13] = {
        0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
    };
    int cj;

    /* preset status */
    cj = 0;

    /* validate year */
    if (iy < (-4699))
    {
        cj = 1;
    }
    else
    {

        /* validate month */
        if (im >= 1 && im <= 12)
        {

            /* allow for leap year */
            if ((iy % 4) == 0)
            {
                mtab[2] = 29;
            }
            else
            {
                mtab[2] = 28;
            }
            if ((iy % 100) == 0 && (iy % 400) != 0)
                mtab[2] = 28;

            /* validate day */
            if (id < 1 || id > mtab[im])
                cj = 3;

            /* modified julian date */
            *djm = (double) ((1461 * (iy - (12 - im) / 10 + 4712)) / 4
                             + (306 * ((im + 9) % 12) + 5) / 10
                             - (3 * ((iy - (12 - im) / 10 + 4900) / 100)) / 4
                             + id - 2399904);

            /* bad month */
        }
        else
        {
            cj = 2;
        }
    }
    *j = cj;
}
