#include <math.h>

/**
 *
 *     - - - - - -
 *      c s 2 c 6
 *     - - - - - -
 *
 *
 *  conversion of position & velocity in spherical coordinates
 *  to cartesian coordinates
 *
 *  (single precision)
 *
 *
 *  given:
 *     a     r      longitude (radians)
 *     b     r      latitude (radians)
 *     r     r      radial coordinate
 *     ad    r      longitude derivative (radians per unit time)
 *     bd    r      latitude derivative (radians per unit time)
 *     rd    r      radial derivative
 *
 *  returned:
 *     v     r(6)   cartesian position & velocity vector
 *
 *
 *  p.t.wallace   starlink   november 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_cs2c6 (double a, double b, double r, double ad, double bd, double rd,
           double *v)
                                                 /* variables were REAL */
{
    /* variables were REAL */
    double sa, ca, sb, cb, rcb, x, y, rbd, w;

    /* useful functions */
    sa = sin (a);
    ca = cos (a);
    sb = sin (b);
    cb = cos (b);
    rcb = r * cb;
    x = rcb * ca;
    y = rcb * sa;
    rbd = r * bd;
    /* cbrd = cb * rd; */
    w = rbd * sb - cb * rd;

    /* position */
    v[0] = x;
    v[1] = y;
    v[2] = r * sb;

    /* velocity */
    v[3] = -y * ad - w * ca;
    v[4] = x * ad - w * sa;
    v[5] = rbd * cb + sb * rd;
}
