
/**
 *
 *     - - - - - -
 *      d a f 2 r
 *     - - - - - -
 *
 *
 *  convert degrees, arcminutes, arcseconds to radians
 *
 *  (double precision)
 *
 *
 *  given:
 *     ideg        int       degrees
 *     iamin       int       arcminutes
 *     asec        dp        arcseconds
 *
 *  returned:
 *     rad         dp        angle in radians
 *     j           int       status:  0 = ok
 *                                    1 = ideg outside range 0-359
 *                                    2 = iamin outside range 0-59
 *                                    3 = asec outside range 0-59.999...
 *
 *  notes:
 *     1)  The result is computed even if any of the range checks
 *         fail.
 *     2)  The sign must be dealt with outside this routine.
 *
 *
 *  p.t.wallace   starlink   july 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_daf2r (int ideg, int iamin, double asec, double *rad, int *j)
{

    /* arc seconds to radians */
#define as2r  (double)0.4848136811095359949e-05

    /* preset status */
    *j = 0;

    /* validate arcsec, arcmin, deg */
    if (asec < 0e0 || asec >= 60e0)
        *j = 3;
    if (iamin < 0 || iamin > 59)
        *j = 2;
    if (ideg < 0 || ideg > 359)
        *j = 1;

    /* compute angle */
    *rad = as2r * (60e0 * (60e0 * (double) ideg + (double) iamin) + asec);
}
