#include <math.h>

/**
 *
 *     - - - - - -
 *      d c c 2 s
 *     - - - - - -
 *
 *
 *  direction cosines to spherical coordinates (double precision)
 *
 *
 *  given:
 *     v     dp(3)   x,y,z vector
 *
 *  returned:
 *     a,b   dp      spherical coordinates in radians
 *
 *
 *  The spherical coordinates are longitude (+ve anticlockwise
 *  looking from the +ve latitude pole) and latitude.  The
 *  cartesian coordinates are right handed, with the x axis
 *  at zero longitude and latitude, and the z axis at the
 *  +ve latitude pole.
 *
 *
 *  p.t.wallace   starlink   july 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_dcc2s (double *v, double *a, double *b)
{

    double x, y;

    x = v[0];
    y = v[1];
    if (x == 0e0 && y == 0e0)
    {
        *a = 0e0;
    }
    else
    {
        *a = atan2 (y, x);
    }
    *b = atan2 (v[2], sqrt (x * x + y * y));
}
