#include <math.h>

/**
 *
 *     - - - - - -
 *      d c s 2 c
 *     - - - - - -
 *
 *
 *  spherical coordinates to direction cosines (double precision)
 *
 *
 *  given:
 *     a,b       dp      spherical coordinates in radians
 *                        (ra,dec), (long,lat) etc
 *
 *  returned:
 *     v         dp(3)   x,y,z unit vector
 *
 *
 *  The spherical coordinates are longitude (+ve anticlockwise
 *  looking from the +ve latitude pole) and latitude.  The
 *  cartesian coordinates are right handed, with the x axis
 *  at zero longitude and latitude, and the z axis at the
 *  +ve latitude pole.
 *
 *
 *  p.t.wallace   starlink   october 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_dcs2c (double a, double b, double *v)
{
    double cosb;

    cosb = cos (b);
    v[0] = cos (a) * cosb;
    v[1] = sin (a) * cosb;
    v[2] = sin (b);
}
