
/**
 *
 *     - - - - - -
 *      d i m x v
 *     - - - - - -
 *
 *
 *  performs the 3-d backward unitary transformation:
 *     vector vb = (inverse of matrix dm) * vector va
 *
 *  (double precision)
 *
 *  (n.b.  The matrix must be unitary, as this routine assumes that
 *   the inverse and transpose are identical)
 *
 *
 *  given:
 *     dm       dp(3,3)    matrix
 *     va       dp(3)      vector
 *
 *  returned:
 *     vb       dp(3)      result vector
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_dimxv (double (*dm)[3], double *va, double *vb)
{

    int i, j;
    double w, vw[3];

    /* inverse of matrix dm * vector va -> vector vw */
    for (j = 0; j <= 2; j++)
    {
        w = 0e0;
        for (i = 0; i <= 2; i++)
        {
            w += dm[i][j] * va[i];
        }
        vw[j] = w;
    }

    /* vector vw -> vector vb */
    for (j = 0; j <= 2; j++)
    {
        vb[j] = vw[j];
    }
}
