#include <math.h>

/**
 *
 *     - - - - -
 *      d j c l
 *     - - - - -
 *
 *
 *  modified julian date to gregorian year, month, day,
 *  and fraction of a day.
 *
 *
 *  given:
 *     djm      dp     modified julian date (JD-2400000.5)
 *
 *  returned:
 *     iy       int    year ad
 *     im       int    month
 *     id       int    day
 *     fd       dp     fraction of day
 *     j        int    status:
 *                      -1 = unacceptable date (before 4701bc march 1)
 *
 *  The algorithm is derived from that of Hatcher 1984
 *  (qjras 25, 53-55).
 *
 *  p.t.wallace   starlink   february 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_djcl (double djm, int *iy, int *im, int *id, double *fd, int *j)
{

    double f, d;
    int jd, n4, nd10;

    /* check if date is acceptable */
    if (djm <= (-2395522e0) || djm >= 1e9)
    {
        *j = -1;
    }
    else
    {

        /* separate day and fraction */
        /* f = mod(djm, 1d0) */
        f = modf (djm, &d);
        if (f < 0e0)
            f = f + 1e0;
        d = (int) rint (djm - f);

        /* express day in gregorian calendar */
        jd = (int) d + 2400001;
        n4 = 4 * (jd + ((6 * ((4 * jd - 17918) / 146097)) / 4 + 1) / 2 - 37);
        nd10 = 10 * (((n4 - 237) % 1461) / 4) + 5;
        *iy = n4 / 1461 - 4712;
        *im = ((nd10 / 306 + 2) % 12) + 1;
        *id = (nd10 % 306) / 10 + 1;
        *fd = f;
        *j = 0;
    }
}
