
/**
 *
 *     - - - - -
 *      d m x m
 *     - - - - -
 *
 *
 *  product of two 3x3 matrices:
 *      matrix c  =  matrix a  x  matrix b
 *
 *  (double precision)
 *
 *
 *  given:
 *      a      dp(3,3)        matrix
 *      b      dp(3,3)        matrix
 *
 *  returned:
 *      c      dp(3,3)        matrix result
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_dmxm (double (*a)[3], double (*b)[3], double (*c)[3])
{

    int i, j, k;
    double w, wm[3][3];

    /* multiply into scratch matrix */
    for (i = 0; i < 3; i++)
    {
        for (j = 0; j < 3; j++)
        {
            w = 0e0;
            for (k = 0; k < 3; k++)
            {
                w += a[i][k] * b[k][j];
            }
            wm[i][j] = w;
        }
    }

    /* return the result */
    for (j = 0; j < 3; j++)
    {
        for (i = 0; i < 3; i++)
        {
            c[i][j] = wm[i][j];
        }
    }
}
