
/**
 *
 *     - - - - -
 *      d m x v
 *     - - - - -
 *
 *
 *  performs the 3-d forward unitary transformation:
 *     vector vb = matrix dm * vector va
 *
 *  (double precision)
 *
 *
 *  given:
 *     dm       dp(3,3)    matrix
 *     va       dp(3)      vector
 *
 *  returned:
 *     vb       dp(3)      result vector
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_dmxv (double (*dm)[3], double *va, double *vb)
{

    int i, j;
    double w, vw[3];

    /* matrix dm * vector va -> vector vw */
    for (j = 0; j < 3; j++)
    {
        w = 0e0;
        for (i = 0; i < 3; i++)
        {
            w += dm[j][i] * va[i];
        }
        vw[j] = w;
    }

    /* vector vw -> vector vb */
    for (j = 0; j < 3; j++)
    {
        vb[j] = vw[j];
    }
}
