#include <math.h>

/**
 *
 *     - - - - - -
 *      d s 2 c 6
 *     - - - - - -
 *
 *
 *  conversion of position & velocity in spherical coordinates
 *  to cartesian coordinates
 *
 *  (double precision)
 *
 *
 *  given:
 *     a     dp      longitude (radians)
 *     b     dp      latitude (radians)
 *     r     dp      radial coordinate
 *     ad    dp      longitude derivative (radians per unit time)
 *     bd    dp      latitude derivative (radians per unit time)
 *     rd    dp      radial derivative
 *
 *  returned:
 *     v     dp(6)   cartesian position & velocity vector
 *
 *
 *  p.t.wallace   starlink   september 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_ds2c6 (double a, double b, double r, double ad, double bd, double rd,
           double *v)
{

    double sa, ca, sb, cb, rcb, x, y, rbd, w;

    /* useful functions */
    sa = sin (a);
    ca = cos (a);
    sb = sin (b);
    cb = cos (b);
    rcb = r * cb;
    x = rcb * ca;
    y = rcb * sa;
    rbd = r * bd;
    /* cbrd = cb * rd; */
    w = rbd * sb - cb * rd;

    /* position */
    v[0] = x;
    v[1] = y;
    v[2] = r * sb;

    /* velocity */
    v[3] = -y * ad - w * ca;
    v[4] = x * ad - w * sa;
    v[5] = rbd * cb + sb * rd;
}
