#include <math.h>

/**
 *
 *     - - - - -
 *      d s e p
 *     - - - - -
 *
 *
 *  angle between two points on a sphere
 *
 *  (double precision)
 *
 *
 *  given:
 *     a1,b1    dp    spherical coordinates of one point
 *     a2,b2    dp    spherical coordinates of the other point
 *
 *  (the spherical coordinates are ra,dec, long,lat etc, in radians.)
 *
 *  The result is the angle, in radians, between the two points.  It
 *  is always positive.
 *
 *  called:  sla_dcs2c
 *
 *
 *  p.t.wallace   starlink   april 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);

double sla_dsep (double a1, double b1, double a2, double b2)
{
    int i;
    double v1[3], v2[3], w;
    double value;

    /* convert coordinates from spherical to cartesian */
    sla_dcs2c (a1, b1, v1);
    sla_dcs2c (a2, b2, v2);

    /* modulus squared of half the difference vector */
    w = 0e0;
    for (i = 0; i <= 2; i++)
    {
        w += (v1[i] - v2[i]) * (v1[i] - v2[i]);
    }
    w /= 4e0;

    /* angle between the vectors */
    value = 2e0 * atan2 (sqrt (w), (1.0 - w) < 0 ? 0 : sqrt (1.0 - w));
    return (value);
}
