
/**
 *
 *     - - - - - -
 *      d t f 2 r
 *     - - - - - -
 *
 *
 *  convert hours, minutes, seconds to radians
 *
 *  (double precision)
 *
 *
 *  given:
 *     ihour       int       hours
 *     imin        int       minutes
 *     sec         dp        seconds
 *
 *  returned:
 *     rad         dp        angle in radians
 *     j           int       status:  0 = ok
 *                                    1 = ihour outside range 0-23
 *                                    2 = imin outside range 0-59
 *                                    3 = sec outside range 0-59.999...
 *
 *  called:
 *     sla_dtf2d
 *
 *  notes:
 *
 *     1)  The result is computed even if any of the range checks
 *         fail.
 *     2)  The sign must be dealt with outside this routine.
 *
 *
 *  p.t.wallace   starlink   july 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dtf2d (int ihour, int imin, double sec, double *days, int *j);

sla_dtf2r (int ihour, int imin, double sec, double *rad, int *j)
{

    double turns;

    /* turns to radians */
#define t2r  (double)6.283185307179586476925287e0

    /* convert to turns then radians */
    sla_dtf2d (ihour, imin, sec, &turns, j);
    *rad = t2r * turns;
}
