#include <math.h>

/**
 *
 *     - - - - - -
 *      d t p 2 s
 *     - - - - - -
 *
 *
 *  transform tangent plane coordinates into spherical
 *
 *  (double precision)
 *
 *
 *  given:
 *     xi,eta      dp   tangent plane rectangular coordinates
 *     raz,decz    dp   spherical coordinates of tangent point
 *
 *  returned:
 *     ra,dec      dp   spherical coordinates
 *
 *  called:        sla_dranrm
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_dtp2s (double xi, double eta, double raz, double decz, double *ra,
           double *dec)
{

    double sla_dranrm (double angle);
    double sdecz, cdecz, denom, radif;

    sdecz = sin (decz);
    cdecz = cos (decz);
    denom = cdecz - eta * sdecz;
    radif = atan2 (xi, denom);
    *ra = sla_dranrm (radif + raz);
    *dec = atan2 (sdecz + eta * cdecz, sqrt (xi * xi + denom * denom));
}
