#include <math.h>

/**
 *
 *     - - - -
 *      d v n
 *     - - - -
 *
 *
 *  normalises a 3-vector also giving the modulus
 *
 *  (double precision)
 *
 *
 *  given:
 *     v       dp(3)      vector
 *
 *  returned:
 *     uv      dp(3)      unit vector in direction of v
 *     vm      dp         modulus of v
 *
 *
 *  If the modulus of v is zero, uv is set to zero as well
 *
 *
 *  p.t.wallace   starlink   november 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_dvn (double *v, double *uv, double *vm)
{

    int i;
    double w1, w2;

    /* modulus */
    w1 = 0e0;
    for (i = 0; i <= 2; i++)
    {
        w2 = v[i];
        w1 += w2 * w2;
    }
    w1 = sqrt (w1);
    *vm = w1;

    /* normalise the vector */
    if (w1 <= 0e0)
        w1 = 1e0;
    for (i = 0; i <= 2; i++)
    {
        uv[i] = v[i] / w1;
    }
}
