
/**
 *
 *     - - - - - -
 *      e c l e q
 *     - - - - - -
 *
 *
 *  transformation from ecliptic coordinates to
 *  j2000.0 equatorial coordinates
 *
 *
 *  (double precision)
 *
 *
 *  given:
 *     dl,db       dp      ecliptic longitude and latitude
 *                           (mean of date, iau 1980 theory, radians)
 *     date        dp      TDB (loosely ET) as modified julian date
 *                                              (JD-2400000.5)
 *  returned:
 *     dr,dd       dp      j2000.0 mean ra,dec (radians)
 *
 *
 *  called:
 *     sla_dcs2c, sla_ecmat, sla_dimxv, sla_prec, sla_epj, sla_dcc2s,
 *     sla_dranrm, sla_drange
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);
extern int sla_ecmat (double date, double (*rmat)[3]);
extern int sla_dimxv (double (*dm)[3], double *va, double *vb);
extern int sla_prec (double ep0, double ep1, double (*rmatp)[3]);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_ecleq (double dl, double db, double date, double *dr, double *dd)
{

    double sla_epj (double date), sla_dranrm (double angle),
        sla_drange (double angle);
    double rmat[3][3], v1[3], v2[3];

    /* spherical to cartesian */
    sla_dcs2c (dl, db, v1);

    /* ecliptic to equatorial */
    sla_ecmat (date, rmat);
    sla_dimxv (rmat, v1, v2);

    /* mean of date to j2000 */
    sla_prec (2000e0, sla_epj (date), rmat);
    sla_dimxv (rmat, v2, v1);

    /* cartesian to spherical */
    sla_dcc2s (v1, dr, dd);

    /* express in conventional ranges */
    *dr = sla_dranrm (*dr);
    *dd = sla_drange (*dd);
}
