#include <ctype.h>

/**
 *
 *     - - - - -
 *      e p c o
 *     - - - - -
 *
 *
 *  convert an epoch into the appropriate form - 'b' or 'j'
 *
 *
 *  given:
 *     k0    char    form of result:  'b'=Besselian, 'j'=Julian
 *     k     char    form of given epoch:  'b' or 'j'
 *     e     dp      epoch
 *
 *  called:  sla_epb, sla_epj2d, sla_epj, sla_epb2d
 *
 *  The result is the given epoch converted as required;
 *  the change, if any, will be small.
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

double sla_epco (char k0, char k, double e)
{
    double sla_epb (double date), sla_epj2d (double epj),
        sla_epj (double date), sla_epb2d (double epb);
    double value;

    char lk0 = k0;
    char lk = k;

    /* allow for upper or lower case */
    if (isupper (lk0))
        lk0 = tolower (lk0);
    if (isupper (lk))
        lk = tolower (lk);

    if (lk == lk0)
    {
        value = e;
    }
    else if (lk0 == 'b')
    {
        value = sla_epb (sla_epj2d (e));
    }
    else
    {
        value = sla_epj (sla_epb2d (e));
    }
    return (value);
}
