
/**
 *
 *     - - - - - -
 *      e q e c l
 *     - - - - - -
 *
 *
 *  transformation from j2000.0 equatorial coordinates to
 *  ecliptic coordinates
 *
 *
 *  (double precision)
 *
 *
 *  given:
 *     dr,dd       dp      j2000.0 mean ra,dec (radians)
 *     date        dp      TDB (loosely ET) as modified Julian date
 *                                              (JD-2400000.5)
 *  returned:
 *     dl,db       dp      ecliptic longitude and latitude
 *                         (mean of date, IAU 1980 theory, radians)
 *
 *
 *  called:
 *     sla_dcs2c, sla_prec, sla_epj, sla_dmxv, sla_ecmat, sla_dcc2s,
 *     sla_dranrm, sla_drange
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_dcs2c (double a, double b, double *v);
extern int sla_prec (double ep0, double ep1, double (*rmatp)[3]);
extern int sla_dmxv (double (*dm)[3], double *va, double *vb);
extern int sla_ecmat (double date, double (*rmat)[3]);
extern int sla_dcc2s (double *v, double *a, double *b);

sla_eqecl (double dr, double dd, double date, double *dl, double *db)
{
    double sla_epj (double date), sla_dranrm (double angle),
        sla_drange (double angle);
    double rmat[3][3], v1[3], v2[3];

    /* spherical to cartesian */
    sla_dcs2c (dr, dd, v1);

    /* mean j2000 to mean of date */
    sla_prec (2000e0, sla_epj (date), rmat);
    sla_dmxv (rmat, v1, v2);

    /* equatorial to ecliptic */
    sla_ecmat (date, rmat);
    sla_dmxv (rmat, v2, v1);

    /* cartesian to spherical */
    sla_dcc2s (v1, dl, db);

    /* express in conventional ranges */
    *dl = sla_dranrm (*dl);
    *db = sla_drange (*db);
}
