#include <math.h>

/**
 *
 *     - - - - -
 *      g m s t
 *     - - - - -
 *
 *
 *  conversion from universal time to sidereal time
 *
 *  (double precision)
 *
 *  The IAU 1982 expression (see page s15 of 1984 Astronomical
 *  Almanac) is used, but rearranged to reduce rounding errors.
 *
 *
 *  given:
 *
 *    ut1    dp     universal time (strictly UT1) expressed as
 *                  modified Julian date (JD-2400000.5)
 *
 *
 *  The result is the Greenwich mean sidereal time (double
 *  precision, radians).
 *
 *
 *  called:  sla_dranrm
 *
 *
 *  p.t.wallace   starlink   september 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

double sla_gmst (double ut1)
{

    double sla_dranrm (double angle);

#define d2pi  (double)6.283185307179586476925287e0
#define s2r   (double)0.7272205216643039849e-4
    double tu;
    double junk, temp;

    /* Julian centuries from fundamental epoch j2000 to this UT */
    tu = (ut1 - 51544.5e0) / 36525e0;

    /* GMST at this UT */
    temp = modf (ut1, &junk);   /* get integer part, with sign */
    return (sla_dranrm (temp * d2pi +
                        (24110.54841e0 +
                         (8640184.812866e0 +
                          (0.093104e0 - 6.2e-6 * tu) * tu) * tu) * s2r));
}
