#define TRUE 1
#define FALSE 0
#include <math.h>

/**
 *
 *     - - - - - - -
 *      g r e s i d
 *     - - - - - - -
 *
 *
 *  generate pseudo-random normal deviate ( = 'gaussian residual')
 *
 *  (single precision)
 *
 *
 *  given:
 *
 *     s      real     standard deviation
 *
 *
 *  called:  sla_random
 *
 *
 *  The results of many calls to this routine will be
 *  normally distributed with mean zero and standard deviation s.
 *
 *  The algorithm is from D.J.Best, Australian Computer Journal,
 *  vol 11, no 2, May 1979.  It is method 6, described as 'the
 *  combined ratio of uniform variates and envelope rejection
 *  method'.
 *
 *
 *  p.t.wallace   starlink   february 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

/* function was REAL */
double sla_gresid (double s)
                                /* variables was REAL */
{

    /* functions, variables were REAL */
    double sla_random (double *seed);
    double u, usq, v, vsq, x, xsq;

    static double seed = 1.234567;
    int done = FALSE;

    while (done == FALSE)
    {
        u = sla_random (&seed);
        usq = u * u;
        v = sla_random (&seed) - 0.5;
        vsq = v * v;
        if (usq + vsq <= u)
        {
            x = 1.732050807 * v / u;
            xsq = x * x;
            u = sla_random (&seed);
            usq = u * u;
            if (0.86 - xsq / 8.6 >= usq)
            {
                done = TRUE;
            }
            else
            {
                v = 3.0 + xsq;
                vsq = v * v;
                if (log (vsq * 0.1030450794 / u) >= 0.5 * xsq)
                    done = TRUE;
            }
        }
    }
    return (x * s);
}
