
/**
 *
 *     - - - - -
 *      i m x v
 *     - - - - -
 *
 *
 *  performs the 3-d backward unitary transformation:
 *     vector vb = (inverse of matrix rm) * vector va
 *
 *  ([was] single precision)
 *
 *  (n.b.  the matrix must be unitary, as this routine assumes that
 *   the inverse and transpose are identical)
 *
 *
 *  given:
 *     rm       real(3,3)    matrix
 *     va       real(3)      vector
 *
 *  returned:
 *     vb       real(3)      result vector
 *
 *
 *  The same vector can be specified for both va and vb.
 *
 *
 *  p.t.wallace   starlink   november 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_imxv (double (*rm)[3], double *va, double *vb)
    /* variables were REAL */
{

    int i, j;

    /* variables were REAL */
    double w, vw[3];

    /* inverse of matrix rm * vector va -> vector vw */
    for (j = 0; j < 3; j++)
    {
        w = 0.0;
        for (i = 0; i < 3; i++)
        {
            w += rm[i][j] * va[i];
        }
        vw[j] = w;
    }

    /* vector vw -> vector vb */
    for (j = 0; j < 3; j++)
    {
        vb[j] = vw[j];
    }
}
