
/**
 *
 *     - - - -
 *      k b j
 *     - - - -
 *
 *
 *  select epoch prefix 'b' or 'j'
 *
 *
 *  given:
 *     jb     int     sla_dbjin prefix status:  0=none, 1='b', 2='j'
 *     e      dp      epoch - Besselian or Julian
 *
 *  returned:
 *     k      char    'b' or 'j'
 *     j      int     status:  0=ok
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_kbj (int jb, double e, char *k, int *j)
{

    /* preset status */
    *j = 0;

    /* if prefix given expressly, use it */
    if (jb == 1)
    {
        *k = 'b';
    }
    else if (jb == 2)
    {
        *k = 'j';

        /* if no prefix, examine the epoch */
    }
    else if (jb == 0)
    {

        /*
         * if epoch is pre-1984.0, assume besselian;  otherwise
         * julian 
         */
        if (e < 1984e0)
        {
            *k = 'b';
        }
        else
        {
            *k = 'j';
        }

        /* if illegal prefix, return error status */
    }
    else
    {
        *k = ' ';
        *j = 1;
    }
}
