
/**
 *
 *     - - - -
 *      m x v
 *     - - - -
 *
 *
 *  performs the 3-d forward unitary transformation:
 *     vector vb = matrix rm * vector va
 *
 *  (was single precision)
 *
 *
 *  given:	now all dp
 *     rm       real(3,3)    matrix
 *     va       real(3)      vector
 *
 *  returned:
 *     vb       real(3)      result vector
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_mxv (double (*rm)[3], double *va, double *vb)
                                        /* variables were REAL */
{

    int i, j;
    double w, vw[3];            /* variables were REAL */

    /* matrix rm * vector va -> vector vw */
    for (j = 0; j < 3; j++)
    {
        w = 0.0;
        for (i = 0; i < 3; i++)
        {
            w += rm[j][i] * va[i];
        }
        vw[j] = w;
    }

    /* vector vw -> vector vb */
    for (j = 0; j < 3; j++)
    {
        vb[j] = vw[j];
    }
}
