
/**
 *
 *     - - - -
 *      n u t
 *     - - - -
 *
 *
 *  form the matrix of nutation for a given date (IAU 1980 theory)
 *
 *  (double precision)
 *
 *
 *  references:
 *     Final report of the IAU working group on nutation,
 *      chairman P.K.Seidelmann, 1980.
 *     Kaplan,G.H., 1981, USNO circular no. 163, pa3-6.
 *
 *
 *  given:
 *
 *     date   dp         TDB (loosely ET) as modified Julian date
 *                                           (=JD-2400000.5)
 *
 *  returned:
 *
 *     rmatn  dp(3,3)    nutation matrix
 *
 *
 *  the matrix is in the sense   v(true)  =  rmatn * v(mean) .
 *
 *
 *  called:   sla_nutc, sla_nutm
 *
 *
 *  p.t.wallace   starlink   july 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_nutc (double date, double *dpsi, double *deps, double *eps0);
extern int sla_nutm (double dpsi, double deps, double eps0,
                     double (*rmatn)[3]);

sla_nut (double date, double (*rmatn)[3])
{

    double dpsi, deps, eps0;

    /* removed double eps;  eps not used */

    /* nutation components and mean obliquity */
    sla_nutc (date, &dpsi, &deps, &eps0);

    /* rotation matrix */
    sla_nutm (dpsi, deps, eps0, rmatn);
}
