#include <math.h>

/**
 *
 *     - - - - -
 *      n u t m
 *     - - - - -
 *
 *
 *  form the matrix of nutation
 *
 *  (double precision)
 *
 *
 *  given:
 *     dpsi,deps    dp    nutation in longitude & obliquity
 *     eps0         dp    mean obliquity
 *
 *  returned:
 *     rmatn  dp(3,3)    nutation matrix
 *
 *
 *  the matrix is in the sense   v(true)  =  rmatn * v(mean) .
 *
 *
 *  p.t.wallace   starlink   july 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_nutm (double dpsi, double deps, double eps0, double (*rmatn)[3])
{

    double eps, sdpsi, cdpsi, seps0, ceps0, seps, ceps;

    /* true obliquity */
    eps = eps0 + deps;

    /* rotation matrix */
    sdpsi = sin (dpsi);
    cdpsi = cos (dpsi);
    seps0 = sin (eps0);
    ceps0 = cos (eps0);
    seps = sin (eps);
    ceps = cos (eps);
    rmatn[0][0] = cdpsi;
    rmatn[0][1] = -sdpsi * ceps0;
    rmatn[0][2] = -sdpsi * seps0;
    rmatn[1][0] = sdpsi * ceps;
    rmatn[1][1] = cdpsi * ceps0 * ceps + seps0 * seps;
    rmatn[1][2] = cdpsi * seps0 * ceps - ceps0 * seps;
    rmatn[2][0] = sdpsi * seps;
    rmatn[2][1] = cdpsi * ceps0 * seps - seps0 * ceps;
    rmatn[2][2] = cdpsi * seps0 * seps + ceps0 * ceps;
}
