
/*******************************************************************

		plate.c

   This file contains the functions for determining the
   locations of features on the focal plane plate.

   Included functions are:

   *  PlateZ()

   ( * indicates function is an extern )

*******************************************************************/

#include <math.h>

#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "kaboom.h"

/*
      PlateZ

   Find the distance to the plate at the current gripper location.
   If a step falls within a button radius of the target location,
   return zero to indicate an inditerminate result.
*/

PlateZ (fib, currentLoc, step)
struct fiber_dbase *fib;
struct plate_coords *currentLoc;
int *step;

{
    long z;

    double diff1;
    double diff2;
    double diff3;
    double targetRadius;

    double buttRadius = (double) (fib->ButtonDia / 2);

    targetRadius = sqrt (pow ((double) currentLoc->x, 2.0) +
                         pow ((double) currentLoc->y, 2.0));

    /* See if we are too close to a step */

    diff1 = fabs ((double) STEP1 - targetRadius);
    diff2 = fabs ((double) STEP2 - targetRadius);

    diff3 = fabs ((double) STOW - targetRadius);

    if (diff3 < STOWSLOP)
    {
        z = LEVEL4;             /*  We are at stow circle and over a "hole" */
        *step = 4;
    }

    else
    {
        if ((diff1 > (buttRadius + STEPMARGIN)) &&
            (diff2 > (buttRadius + STEPMARGIN)))
        {

            /* figure out which level we are at */

            if (targetRadius < STEP1)
            {
                z = LEVEL1;
                *step = STEP1ORD;
            }
            else
            {
                if (targetRadius > STEP2)
                {
                    z = LEVEL3;
                    *step = STEP3ORD;
                }

                else
                {
                    z = LEVEL2;
                    *step = STEP2ORD;
                }
            }
        }

        else
        {                       /*  too close to step! */
            z = 0;
            *step = 0;
        }

    }
    return (z);
}
