
/**
 *
 *     - - - - - - -
 *      p r e n u t
 *     - - - - - - -
 *
 *
 *  form the matrix of precession and nutation (IAU1976/FK5)
 *
 *  (double precision)
 *
 *
 *  given:
 *
 *     epoch   dp         Julian epoch for mean coordinates
 *     date    dp         modified Julian date (JD-2400000.5)
 *                        for true coordinates
 *
 *
 *  returned:
 *
 *     rmatpn  dp(3,3)    combined precession/nutation matrix
 *
 *
 *  called:  sla_prec, sla_epj, sla_nut, sla_dmxm
 *
 *
 *  notes:
 *
 *  1)  The epoch and date are TDB (loosely ET).
 *
 *  2)  The matrix is in the sense   v(true)  =  rmatpn * v(mean) .
 *
 *
 *  p.t.wallace   starlink   april 1987
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_prec (double ep0, double ep1, double (*rmatp)[3]);
extern int sla_nut (double date, double (*rmatn)[3]);
extern int sla_dmxm (double (*a)[3], double (*b)[3], double (*c)[3]);

sla_prenut (double epoch, double date, double (*rmatpn)[3])
{

    double rmatp[3][3], rmatn[3][3];
    double sla_epj (double date);

    /* precession */
    sla_prec (epoch, sla_epj (date), rmatp);

    /* nutation */
    sla_nut (date, rmatn);

    /* combine the matrices:  pn = n x p */
    sla_dmxm (rmatn, rmatp, rmatpn);
}
