#include <math.h>

/**
 *
 *     - - - - - -
 *      p v o b s
 *     - - - - - -
 *
 *
 *  position and velocity of an observing station
 *
 *  (double precision)
 *
 *
 *  given:
 *     p     dp     latitude (geodetic, radians)
 *     h     dp     height above reference spheroid (geodetic, metres)
 *     stl   dp     local apparent sidereal time (radians)
 *
 *  returned:
 *     pv    dp(6)  position/velocity 6-vector (au, au/s, geocentric
 *                                                            apparent)
 *
 *  called:  sla_geoc
 *
 *  IAU 1976 constants are used.
 *
 *
 *  p.t.wallace   starlink   october 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

extern int sla_geoc (double p, double h, double *r, double *z);

sla_pvobs (double p, double h, double stl, double *pv)
{

    double r, z, s, c, v;

    /* seconds to radians */
#define ds2r  (double)0.7272205216643039849e-04

    /* geodetic to geocentric conversion */
    sla_geoc (p, h, &r, &z);

    /* functions of st */
    s = sin (stl);
    c = cos (stl);

    /* speed */
    v = ds2r * r;

    /* position */
    pv[0] = r * c;
    pv[1] = r * s;
    pv[2] = z;

    /* velocity */
    pv[3] = -v * s;
    pv[4] = v * c;
    pv[5] = 0e0;
}
