#include <math.h>

/**
 *
 *     - - - - - - -
 *      r v e r o t
 *     - - - - - - -
 *
 *
 *  velocity component in a given direction due to earth rotation
 *
 *  (was single precision)
 *
 *
 *  given:
 *     phi     real    latitude of observing station (geodetic)
 *     ra,da   real    apparent ra,dec
 *     st      real    local apparent sidereal time
 *
 *  phi, ra, dec and st are all in radians.
 *
 *  result:
 *     component of earth rotation in direction ra,da (km/s)
 *
 *  sign convention:
 *     The result is +ve when the observatory is receding from the
 *     given point on the sky.
 *
 *  accuracy:
 *     The simple algorithm used assumes a spherical earth and
 *     an observing station at sea level.  For actual observing
 *     sites, the error is unlikely to be greater than 0.0005 km/s.
 *     For applications requiring greater precision, see the
 *     routine sla_pvobs.
 *
 *
 *  p t wallace   starlink   april 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

/* function was REAL */
double sla_rverot (double phi, double ra, double da, double st)
                                 /* variables were REAL */
{

    /* Sidereal speed of earth equator, adjusted to compensate for
     * the simple algorithm used.  (The true value is 0.4651.)
     * ( variables were REAL )
     */

#define espeed  (double) 0.4655

    return (espeed * cos (phi) * sin (st - ra) * cos (da));
}
