
/**
 *
 *     - - - - - - -
 *      r v g a l c
 *     - - - - - - -
 *
 *
 *  velocity component in a given direction due to the rotation
 *  of the galaxy
 *
 *  (was single precision)
 *
 *
 *  given:
 *     r2000,d2000   real    j2000.0 mean ra,dec (radians)
 *
 *  result:
 *     component of lsr motion in direction r2000,d2000 (km/s)
 *
 *  sign convention:
 *     The result is +ve when the lsr is receding from the
 *     given point on the sky.
 *
 *  called:
 *     sla_cs2c, sla_vdv
 *
 *
 *  p t wallace   starlink   april 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

/* function was REAL */

extern int sla_cs2c (double a, double b, double *v);

double sla_rvgalc (double r2000, double d2000)
                                 /* variables were REAL */
{

    /* variables were REAL */
    double vb[3];
    double sla_vdv (double *va, double *vb);

    /*
     * lsr velocity due to galactic rotation
     *
     * speed = 250 km/s
     *
     * apex  = l2,b2  90deg, 0deg
     * = ra,dec  21 12 01.1  +48 19 47  j2000.0
     *
     * this is expressed in the form of a j2000.0 x,y,z vector:
     *
     * va(0) = x = -speed*cos(ra)*cos(dec)
     * va(1) = y = -speed*sin(ra)*cos(dec)
     * va(2) = z = -speed*sin(dec)
     */
    static double va[] = { -123.52737, 111.20740, -186.74556 };

    /* convert given j2000 ra,dec to x,y,z */
    sla_cs2c (r2000, d2000, vb);

    /* compute dot product with lsr motion vector */
    return (sla_vdv (va, vb));
}
