
/**
 *
 *     - - - - -
 *      r v l g
 *     - - - - -
 *
 *
 *  velocity component in a given direction due to the combination
 *  of the rotation of the galaxy and the motion of the galaxy
 *  relative to the mean motion of the local group.
 *
 *  (was single precision)
 *
 *
 *  given:
 *     r2000,d2000   real    j2000.0 mean ra,dec (radians)
 *
 *  result:
 *     component of solar motion in direction r2000,d2000 (km/s)
 *
 *  sign convention:
 *     The result is +ve when the sun is receding from the
 *     given point on the sky.
 *
 *  reference:
 *     IAU Trans 1976, 168, p201.
 *
 *  called:
 *     sla_cs2c, sla_vdv
 *
 *
 *  p t wallace   starlink   june 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

/* function was REAL */

extern int sla_cs2c (double a, double b, double *v);

double sla_rvlg (double r2000, double d2000)
                                 /* variables were REAL */
{

    /* variables were REAL */
    double vb[3];
    double sla_vdv (double *va, double *vb);

    /*
     * solar velocity due to galactic rotation and translation
     *
     * speed = 300 km/s
     *
     * apex  = l2,b2  90deg, 0deg
     * = ra,dec  21 12 01.1  +48 19 47  j2000.0
     *
     * this is expressed in the form of a j2000.0 x,y,z vector:
     *
     * va(1) = x = -speed*cos(ra)*cos(dec)
     * va(2) = y = -speed*sin(ra)*cos(dec)
     * va(3) = z = -speed*sin(dec)
     */
    static double va[] = { -148.23284, 133.44888, -224.09467 };

    /* convert given j2000 ra,dec to x,y,z */
    sla_cs2c (r2000, d2000, vb);

    /* compute dot product with solar motion vector */
    return (sla_vdv (va, vb));
}
