
/**
 *
 *     - - - - - -
 *      r v l s r
 *     - - - - - -
 *
 *
 *  velocity component in a given direction due to the sun's
 *  motion with respect to the local standard of rest
 *
 *  (single precision)
 *
 *
 *  given:
 *     r2000,d2000   real    j2000.0 mean ra,dec (radians)
 *
 *  result:
 *     component of solar motion in direction r2000,d2000 (km/s)
 *
 *  sign convention:
 *     the result is +ve when the sun is receding from the
 *     given point on the sky.
 *
 *  called:
 *     sla_cs2c, sla_vdv
 *
 *
 *  p t wallace   starlink   april 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

/* function was REAL */

extern int sla_cs2c (double a, double b, double *v);

double sla_rvlsr (double r2000, double d2000)
                                 /* variables were REAL */
{

    /* variables were REAL */
    double vb[3];
    double sla_vdv (double *va, double *vb);

    /*
     * solar motion
     *
     * speed = 20 km/s
     *
     * apex = 18hrs, +30deg, 1900.0
     * = 18 03 50.2, +30 00 17, j2000.0
     *
     * this is expressed in the form of a j2000.0 x,y,z vector:
     *
     * va(1) = x = -speed*cos(ra)*cos(dec)
     * va(2) = y = -speed*sin(ra)*cos(dec)
     * va(3) = z = -speed*sin(dec)
     */
    static double va[] = { -0.28997, 17.31727, -10.00141 };

    /* convert given j2000 ra,dec to x,y,z */
    sla_cs2c (r2000, d2000, vb);

    /* compute dot product with solar motion vector */
    return (sla_vdv (va, vb));
}
