#include <math.h>

/**
 *
 *     - - - - -
 *      s 2 t p
 *     - - - - -
 *
 *
 *  projection of spherical coordinates onto tangent plane
 *  ('gnomonic' projection - 'standard coordinates')
 *
 *  (was single precision)
 *
 *
 *  given:
 *     ra,dec      real  spherical coordinates of point to be projected
 *     raz,decz    real  spherical coordinates of tangent point
 *
 *  returned:
 *     xi,eta      real  rectangular coordinates on tangent plane
 *     j           int   status:   0 = ok, star on tangent plane
 *                                 1 = error, star too far from axis
 *                                 2 = error, antistar too far from axis
 *                                 3 = error, antistar on tangent plane
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_s2tp (double ra, double dec, double raz, double decz, double *xi,
          double *eta, int *j)
    /* variables were REAL */
{

    /* variables were REAL */
    double sdecz, sdec, cdecz, cdec, radif, sradif, cradif, denom;

#define tiny  (double)1e-6

    /* trig functions */
    sdecz = sin (decz);
    sdec = sin (dec);
    cdecz = cos (decz);
    cdec = cos (dec);
    radif = ra - raz;
    sradif = sin (radif);
    cradif = cos (radif);

    /* reciprocal of star vector length to tangent plane */
    denom = sdec * sdecz + cdec * cdecz * cradif;

    /* handle vectors too far from axis */
    if (denom > tiny)
    {
        *j = 0;
    }
    else if (denom >= 0.0)
    {
        *j = 1;
        denom = tiny;
    }
    else if (denom > -tiny)
    {
        *j = 2;
        denom = -tiny;
    }
    else
    {
        *j = 3;
    }

    /* compute tangent plane coordinates (even in dubious cases) */
    *xi = cdec * sradif / denom;
    *eta = (sdec * cdecz - cdec * sdecz * cradif) / denom;
}
