#include <math.h>

/**
 *
 *     - - - -
 *      s e p
 *     - - - -
 *
 *
 *  angle between two points on a sphere
 *
 *  (was single precision)
 *
 *
 *  given:
 *     a1,b1    real    spherical coordinates of one point
 *     a2,b2    real    spherical coordinates of the other point
 *
 *  (The spherical coordinates are ra,dec, long,lat etc, in radians.)
 *
 *  The result is the angle, in radians, between the two points.  It
 *  is always positive.
 *
 *  called:  sla_cs2c
 *
 *
 *  p.t.wallace   starlink   april 1985
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

/* function was REAL */

extern int sla_cs2c (double a, double b, double *v);

double sla_sep (double a1, double b1, double a2, double b2)
                                 /* variables were REAL */
{

    int i;

    /* variables were REAL */
    double v1[3], v2[3], w;
    double temp;

    /* convert coordinates from spherical to cartesian */
    sla_cs2c (a1, b1, v1);
    sla_cs2c (a2, b2, v2);

    /* modulus squared of half the difference vector */
    w = 0.0;
    for (i = 0; i < 3; i++)
    {
        temp = v1[i] - v2[i];
        w += temp * temp;
    }
    w /= 4.0;

    /* angle between the vectors */
    temp = 1.0 - w;
    if (temp < 0.0)
        temp = 0.0;
    return (2.0 * atan2 (sqrt (w), sqrt (temp)));
}
