#include <math.h>

/**
 *
 *     - - - - -
 *      t p 2 s
 *     - - - - -
 *
 *
 *  transform tangent plane coordinates into spherical
 *
 *  (was single precision)
 *
 *  given:
 *     xi,eta      real  tangent plane rectangular coordinates
 *     raz,decz    real  spherical coordinates of tangent point
 *
 *  returned:
 *     ra,dec      real  spherical coordinates
 *
 *  called:        sla_ranorm
 *
 *
 *  p.t.wallace   starlink   march 1986
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_tp2s (double xi, double eta, double raz, double decz, double *ra,
          double *dec)
                                         /* variables were REAL */
{

    double sla_ranorm (double angle);
    double sdecz, cdecz, denom, radif;  /* variables were REAL */

    sdecz = sin (decz);
    cdecz = cos (decz);
    denom = cdecz - eta * sdecz;
    radif = atan2 (xi, denom);
    *ra = sla_ranorm (radif + raz);
    *dec = atan2 (sdecz + eta * cdecz, sqrt (xi * xi + denom * denom));
}
