#include <math.h>

/**
 *
 *     - - -
 *      v n
 *     - - -
 *
 *
 *  normalises a 3-vector also giving the modulus
 *
 *  (was single precision)
 *
 *
 *  given:
 *     v       real(3)      vector
 *
 *  returned:
 *     uv      real(3)      unit vector in direction of v
 *     vm      real         modulus of v
 *
 *
 *  If the modulus of v is zero, uv is set to zero as well
 *
 *
 *  p.t.wallace   starlink   november 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_vn (double *v, double *uv, double *vm)
    /* variables were REAL */
{

    int i;

    /* variables were REAL */
    double w1, w2;

    /* modulus */
    w1 = 0.0;
    for (i = 0; i < 3; i++)
    {
        w2 = v[i];
        w1 += w2 * w2;
    }
    w1 = sqrt (w1);
    *vm = w1;

    /* normalise the vector */
    if (w1 <= 0.0)
        w1 = 1.0;
    for (i = 0; i < 3; i++)
    {
        uv[i] = v[i] / w1;
    }
}
