
/*
	    test.c

   A set of routines for testing the output routines for the
   stdBus/wiyn interface.
*/

#include <stdio.h>
#include "motor.h"

#define STDBUS_ORD 	0

#define ON		1
#define OFF		0

#define TRUE		1
#define FALSE		0

FILE *devOut;

int verbose = 0;
int quiet = 0;

struct axis_characteristics stdBus[1];

main ()
{

    devOut = stdout;

    setbuf (devOut, (char *) NULL);

    stdBus[0].unitID = 7;

    sleep (5);

    stdTHlamps (STDBUS_ORD, ON);
    sleep (1);
    stdTHlamps (STDBUS_ORD, OFF);
    sleep (1);

    stdCUlamps (STDBUS_ORD, ON);
    sleep (1);
    stdCUlamps (STDBUS_ORD, OFF);
    sleep (1);

    stdQZlamps (STDBUS_ORD, ON);
    sleep (1);
    stdQZlamps (STDBUS_ORD, OFF);
    sleep (1);

    stdLampStatus (STDBUS_ORD);
    sleep (6);

    stdVacSwitch (STDBUS_ORD, ON);
    sleep (1);
    stdVacSwitch (STDBUS_ORD, OFF);
    sleep (1);

    vacStatus (STDBUS_ORD);
    sleep (6);

    stdGripLED (STDBUS_ORD, ON);
    sleep (1);
    stdGripLED (STDBUS_ORD, OFF);
    sleep (1);

    stdGripLEDstatus (STDBUS_ORD);
    sleep (4);

    stdScreen (STDBUS_ORD, ON);
    sleep (1);
    stdScreen (STDBUS_ORD, OFF);
    sleep (1);

    stdScreenStatus (STDBUS_ORD);
    sleep (6);

    stdTvStatus (STDBUS_ORD);
    sleep (1);

    stdHingeStatus (STDBUS_ORD);
    sleep (6);

    stdACindexers (STDBUS_ORD, ON);
    sleep (1);
    stdACindexers (STDBUS_ORD, OFF);
    sleep (1);

    stdACdrives (STDBUS_ORD, ON);
    sleep (1);
    stdACdrives (STDBUS_ORD, OFF);
    sleep (1);

    stdACcalib (STDBUS_ORD, ON);
    sleep (1);
    stdACcalib (STDBUS_ORD, OFF);
    sleep (1);

    stdACpier (STDBUS_ORD, ON);
    sleep (1);
    stdACpier (STDBUS_ORD, OFF);
    sleep (1);

    stdACvalve (STDBUS_ORD, ON);
    sleep (1);
    stdACvalve (STDBUS_ORD, OFF);
    sleep (1);
}

/*
      send_it

   Send a command.  All commands call this routine.  Only error or
   user messages use stdio calls directly.
*/

send_it (theString)
char *theString;

{
    if (quiet)
        return (FALSE);

    else
    {

        if (verbose)
            (void) fprintf (stderr, "->%s\n", theString);

        if (fprintf (devOut, "%s\n", theString) == EOF)
        {
            fsync (fileno (devOut));
            return (TRUE);
        }
        else
            return (FALSE);
    }
}

/*
		GetTriggers
   
   Obtain the current trigger bit status from the
   axis specified in the first argument.  Put the
   result in the structure pointed to by the second argument.
   If the report does not return in the specified
   amount of time, a timer will expire and return
   the user to the parser after a cleanup routine is
   called by the time out signal handler.
   ( See death.c )
*/

void GetTriggers (axis, response, maxTime)
int axis;
int response[];
struct itimerval *maxTime;

{
    int i;
    int numBits;
    char buff[64];              /* response line */

    /*  This is kludgy--should be replaced by an entry
       in the global limit data structure! */

    if (axis == Z_ord)
        numBits = ZNUMBITS;
    else
        numBits = XNUMBITS;

    /* send the request */

    if (trigger_status (axis))
        labtyd (COMMUNICATION, "Trigger request transmission failure");

    getString (buff, maxTime);

    if (verbose)
        (void) log_fprintf (stderr, "axis %d reports triggers of %s \n", axis,
                            buff);

    for (i = 0; i < numBits; i++)

        switch (buff[i])
        {

            case '0':
                response[i] = FALSE;
                break;
            case '1':
                response[i] = TRUE;
                break;
            default:
                (void) log_fprintf (stderr, "bit string received was: %s \n",
                                    buff);
                Ping ();
                labtyd (2, "Other than zero or one in trigger bit");
                break;
        }
}

void getString (respPtr, maxWait)
char *respPtr;
struct itimerval *maxWait;

{
    if (setitimer (ITIMER_REAL, maxWait, (struct itimerval *) NULL))
        labtyd (SYSTEM_CALL, "Watchdog timer failed to set");

    if (fgets (respPtr, 128, responses) != respPtr)
        labtyd (FILE_SYS, "fgets returned EOF or other error.");

    if (setitimer (ITIMER_REAL, &ClearTimer, (struct itimerval *) NULL))
        labtyd (SYSTEM_CALL, "Watchdog timer failed to de-activate");
}
