#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <sys/termios.h>

#include <math.h>

#include "miscMath.h"
#include "nesslib.h"
#include "sla.h"

/*** Globals ***/

int Verbose = FALSE;

/***************/

void getABnew (double temp, double w, double *a, double *b);

void precessIt (aCoord * coordIn, aCoord * coordOut,
                struct exposinfo *pExposure);
void unprecessIt (aCoord * coordIn, aCoord * coordOut,
                  struct exposinfo *pExposure);

void refractIt (aCoord * coordIn, aCoord * coordOut,
                struct exposinfo *pExposure, double a, double b);
void unrefractIt (aCoord * coordIn, aCoord * coordOut,
                  struct exposinfo *pExposure, double wave);
void refractAtTime (aCoord * coordIn, double siderialNow, double a, double b,
                    double *refRA, double *refDC, double *airmass);

void projectIt (aCoord * coordIn, point * coordOut,
                aCoord * fieldCenterCoord);
void deprojectIt (point * coordIn, aCoord * coordOut,
                  aCoord * fieldCenterCoord);

extern double FieldRotationalOffset;

calcPlateCoords (struct starinfo **theStars, struct exposinfo *exposure)
{

    struct starinfo *pTheStar;

    double waveA;
    double waveB;

    double camWaveA;
    double camWaveB;

    int i;

    /* Compute the A and B coefficients for refraction corrections */

    getABnew (exposure->tempadopt, exposure->wave, &waveA, &waveB);

    getABnew (exposure->tempadopt, exposure->guidewave, &camWaveA, &camWaveB);

    for (i = 0; (i < MAXSTARS) && (theStars[i] != NULL); i++)
    {

        if (Verbose)
        {
            fprintf (stderr, "% 3d\n", i);
        }

        pTheStar = theStars[i];

        pTheStar->rawCoord.ra = pTheStar->ra;
        pTheStar->rawCoord.dec = pTheStar->dc;

        if (Verbose)
        {
            fprintf (stderr, "raw    %f, %f\n",
                     pTheStar->rawCoord.ra, pTheStar->rawCoord.dec);
        }

        precessIt (&pTheStar->rawCoord, &pTheStar->precessedCoord, exposure);

        if (Verbose)
        {
            fprintf (stderr, "prec   %f, %f\n",
                     pTheStar->precessedCoord.ra,
                     pTheStar->precessedCoord.dec);
        }

        /* process the coordinate for the camera wavelength */

        refractIt (&pTheStar->precessedCoord,
                   &pTheStar->camRefractedCoord,
                   exposure, camWaveA, camWaveB);

        if (Verbose)
        {
            fprintf (stderr, "ref    %f, %f\n",
                     pTheStar->camRefractedCoord.ra,
                     pTheStar->camRefractedCoord.dec);
        }

        projectIt (&pTheStar->camRefractedCoord,
                   &pTheStar->camProjectedCoord,
                   &theStars[0]->camRefractedCoord);

        if (Verbose)
        {
            fprintf (stderr, "proj   %f, %f\n",
                     pTheStar->camProjectedCoord.x,
                     pTheStar->camProjectedCoord.y);
        }

        correctIt (&pTheStar->camProjectedCoord,
                   &pTheStar->camCorrectedCoord, NORMAL_ASTROMETRY);

        if (Verbose)
        {
            fprintf (stderr, "cor    %f, %f\n",
                     pTheStar->camCorrectedCoord.x,
                     pTheStar->camCorrectedCoord.y);
        }

        flattenIt (&pTheStar->camCorrectedCoord,
                   &pTheStar->camFlattenedCoord, NORMAL_ASTROMETRY);

        if (Verbose)
        {
            fprintf (stderr, "flat   %f, %f\n",
                     pTheStar->camFlattenedCoord.x,
                     pTheStar->camFlattenedCoord.y);
        }

        rotateIt (&pTheStar->camFlattenedCoord,
                  &pTheStar->camRotatedCoord, FieldRotationalOffset);

        if (Verbose)
        {
            fprintf (stderr, "rot    %f, %f\n",
                     pTheStar->camRotatedCoord.x,
                     pTheStar->camRotatedCoord.y);
        }

        unWarpIt (&pTheStar->camRotatedCoord,
                  &pTheStar->camUnWarpedCoord, NORMAL_ASTROMETRY);

        if (Verbose)
        {
            fprintf (stderr, "warp   %f, %f\n",
                     pTheStar->camUnWarpedCoord.x,
                     pTheStar->camUnWarpedCoord.y);
        }

        encoderIt (&pTheStar->camUnWarpedCoord,
                   &pTheStar->camEncoderCoord, NORMAL_ASTROMETRY);

        if (Verbose)
        {
            fprintf (stderr, "enc    %f, %f\n\n",
                     pTheStar->camEncoderCoord.x,
                     pTheStar->camEncoderCoord.y);
        }

        /* process the coordinate for the spectrograph wavelength */

        refractIt (&pTheStar->precessedCoord,
                   &pTheStar->specRefractedCoord, exposure, waveA, waveB);

        if (Verbose)
        {
            fprintf (stderr, "ref    %f, %f\n",
                     pTheStar->specRefractedCoord.ra,
                     pTheStar->specRefractedCoord.dec);
        }

        projectIt (&pTheStar->specRefractedCoord,
                   &pTheStar->specProjectedCoord,
                   &theStars[0]->camRefractedCoord);

        if (Verbose)
        {
            fprintf (stderr, "proj   %f, %f\n",
                     pTheStar->specProjectedCoord.x,
                     pTheStar->specProjectedCoord.y);
        }

        correctIt (&pTheStar->specProjectedCoord,
                   &pTheStar->specCorrectedCoord, NORMAL_ASTROMETRY);

        if (Verbose)
        {
            fprintf (stderr, "cor    %f, %f\n",
                     pTheStar->specCorrectedCoord.x,
                     pTheStar->specCorrectedCoord.y);
        }

        flattenIt (&pTheStar->specCorrectedCoord,
                   &pTheStar->specFlattenedCoord, NORMAL_ASTROMETRY);

        if (Verbose)
        {
            fprintf (stderr, "flat   %f, %f\n",
                     pTheStar->specFlattenedCoord.x,
                     pTheStar->specFlattenedCoord.y);
        }

        rotateIt (&pTheStar->specFlattenedCoord,
                  &pTheStar->specRotatedCoord, FieldRotationalOffset);

        if (Verbose)
        {
            fprintf (stderr, "rot    %f, %f\n",
                     pTheStar->specRotatedCoord.x,
                     pTheStar->specRotatedCoord.y);
        }

        unWarpIt (&pTheStar->specRotatedCoord,
                  &pTheStar->specUnWarpedCoord, NORMAL_ASTROMETRY);

        if (Verbose)
        {
            fprintf (stderr, "warp   %f, %f\n",
                     pTheStar->specUnWarpedCoord.x,
                     pTheStar->specUnWarpedCoord.y);
        }

        encoderIt (&pTheStar->specUnWarpedCoord,
                   &pTheStar->specEncoderCoord, NORMAL_ASTROMETRY);

        if (Verbose)
        {
            fprintf (stderr, "enc    %f, %f\n\n",
                     pTheStar->specEncoderCoord.x,
                     pTheStar->specEncoderCoord.y);
        }
    }
}

calcAstrometricCoords (struct starinfo **theStars, struct exposinfo *exposure)
{

    struct starinfo *pTheStar;

    int i;

    double camWaveA;
    double camWaveB;

    char nameMod[128];
    char *tailPtr;

    getABnew (exposure->tempadopt, exposure->guidewave, &camWaveA, &camWaveB);

    pTheStar = theStars[0];

    /* process the center coordinate for the spectrograph wavelength */

    if (Verbose)
    {
        fprintf (stderr, "raw    %f, %f\n",
                 pTheStar->rawCoord.ra, pTheStar->rawCoord.dec);
    }

    precessIt (&theStars[0]->rawCoord, &theStars[0]->precessedCoord,
               exposure);

    if (Verbose)
    {
        fprintf (stderr, "prec   %f, %f\n",
                 pTheStar->precessedCoord.ra, pTheStar->precessedCoord.dec);
    }

    refractIt (&theStars[0]->precessedCoord,
               &theStars[0]->camRefractedCoord, exposure, camWaveA, camWaveB);

    if (Verbose)
    {
        fprintf (stderr, "ref    %f, %f\n",
                 pTheStar->camRefractedCoord.ra,
                 pTheStar->camRefractedCoord.dec);
    }

    /* now process the list */

    for (i = 0; (i < MAXSTARS) && (theStars[i] != NULL); i++)
    {

        if (Verbose)
        {
            fprintf (stderr, "%d\n", i);
        }

        pTheStar = theStars[i];

        switch (pTheStar->objectType)
        {

                /* only do objects that have been flagged as updated/new
                   this preserves the accuracy/integrity of the initial
                   input coordinates. */

            case 'r':
            case 'R':
            case 't':
            case 'T':
            case 'b':
            case 'B':

                /* process the coordinate for the camera wavelength */

                if (Verbose)
                {
                    fprintf (stderr, "enc    %f, %f\n\n",
                             pTheStar->camEncoderCoord.x,
                             pTheStar->camEncoderCoord.y);
                }

                encoderIt (&pTheStar->camEncoderCoord,
                           &pTheStar->camUnWarpedCoord, REVERSE_ASTROMETRY);

                if (Verbose)
                {
                    fprintf (stderr, "warp   %f, %f\n",
                             pTheStar->camUnWarpedCoord.x,
                             pTheStar->camUnWarpedCoord.y);
                }

                unWarpIt (&pTheStar->camUnWarpedCoord,
                          &pTheStar->camRotatedCoord, REVERSE_ASTROMETRY);

                if (Verbose)
                {
                    fprintf (stderr, "rot    %f, %f\n",
                             pTheStar->camRotatedCoord.x,
                             pTheStar->camRotatedCoord.y);
                }

                rotateIt (&pTheStar->camRotatedCoord,
                          &pTheStar->camFlattenedCoord,
                          -FieldRotationalOffset);

                if (Verbose)
                {
                    fprintf (stderr, "Flat    %f, %f\n",
                             pTheStar->camFlattenedCoord.x,
                             pTheStar->camFlattenedCoord.y);
                }

                flattenIt (&pTheStar->camFlattenedCoord,
                           &pTheStar->camCorrectedCoord, REVERSE_ASTROMETRY);

                if (Verbose)
                {
                    fprintf (stderr, "Corr   %f, %f\n",
                             pTheStar->camCorrectedCoord.x,
                             pTheStar->camCorrectedCoord.y);
                }

                correctIt (&pTheStar->camCorrectedCoord,
                           &pTheStar->camProjectedCoord, REVERSE_ASTROMETRY);

                if (Verbose)
                {
                    fprintf (stderr, "proj    %f, %f\n",
                             pTheStar->camProjectedCoord.x,
                             pTheStar->camProjectedCoord.y);
                }

                deprojectIt (&pTheStar->camProjectedCoord,
                             &pTheStar->camRefractedCoord,
                             &theStars[0]->camRefractedCoord);

                if (Verbose)
                {
                    fprintf (stderr, "ref    %f, %f\n",
                             pTheStar->camRefractedCoord.ra,
                             pTheStar->camRefractedCoord.dec);
                }

                unrefractIt (&pTheStar->camRefractedCoord,
                             &pTheStar->precessedCoord,
                             exposure, exposure->guidewave);

                if (Verbose)
                {
                    fprintf (stderr, "prec   %f, %f\n",
                             pTheStar->precessedCoord.ra,
                             pTheStar->precessedCoord.dec);
                }

                unprecessIt (&pTheStar->precessedCoord,
                             &pTheStar->rawCoord, exposure);

                if (Verbose)
                {
                    fprintf (stderr, "raw    %f, %f\n",
                             pTheStar->rawCoord.ra, pTheStar->rawCoord.dec);
                }

                switch (pTheStar->objectType)
                {

                        /* Now that we've done the reverse astrometry, 
                           update the type of object we had to be of
                           a "regular" input type */

                    case 'r':
                    case 'R':

                        /* "random" position.  Becomes sky */

                        pTheStar->objectType = 'S';
                        strcpy (pTheStar->name, "Random Sky");
                        break;

                    case 'T':
                    case 't':
                        /* tweaked object */

                        pTheStar->objectType = 'O';

                        tailPtr =
                            &pTheStar->name[strlen (pTheStar->name) - 1];

                        while (*tailPtr == ' ')
                            tailPtr--;

                        *++tailPtr = '\0';

                        sprintf (nameMod, "%s(t)", pTheStar->name);
                        sprintf (pTheStar->name, "%-20s", nameMod);

                        break;

                    case 'b':
                    case 'B':
                        /* tweaked fops */

                        pTheStar->objectType = 'F';

                        tailPtr =
                            &pTheStar->name[strlen (pTheStar->name) - 1];

                        while (*tailPtr == ' ')
                            tailPtr--;
                        *++tailPtr = '\0';

                        sprintf (nameMod, "%s(t)", pTheStar->name);
                        sprintf (pTheStar->name, "%-20s", nameMod);
                        break;

                    default:
                        break;
                }

                break;

            default:
                continue;
                break;
        }

    }
}

/*  *  *  *  *  getab.new  *  *  *  *  *  *  */

void getABnew (double temp, double w, double *a, double *b)
{
    double t0;
    double wavelength;
    double latitude;

    /* check on OK wavelength w: */

    if (w < 3000)
        w = 3000.0;

    if (w > 10000)
        w = 10000.0;

    /* determine coefficients using the new & improved routines */

    t0 = FtoK (temp);

    /* turn wavelength into microns */

    wavelength = w / 10000.0;

    /* turn latitude into radians */

    latitude = KPNO_LAT * 3600.0 * AS2R;

    /* call slalib */

    sla_refco (KPNO_ALT, t0, KPNO_ATM_PRES, KPNO_HUMIDITY,
               wavelength, latitude, LAPSE_RATE, REFRACT_PREC, a, b);

}

void refractIt (aCoord * coordIn, aCoord * coordOut,
                struct exposinfo *pExposure, double a, double b)
{
    int i;

    double division;
    double stStart;
    double stNow;

    double refRA;
    double refDC;

    double airmass;

    double sumRA = 0;
    double sumDC = 0;
    double sumWeights = 0;

    division = pExposure->exptime / REFRACT_PTS;

    /* sidereal time at beginning of exposure */

    stStart = pExposure->st - (pExposure->exptime / 2.0) + (division / 2.0);

    /* calculate the average refraction for spectrograph wavelength */

    for (i = 1; i <= REFRACT_PTS; i++)
    {

        /* calculate sidereal time for this point. */

        stNow = stStart + (i - 1) * division;

        /* generate the refracted coord for the given s.t. */

        refractAtTime (coordIn, stNow, a, b, &refRA, &refDC, &airmass);

        /* save it to the average */

        sumRA += (refRA / airmass);

        sumDC += (refDC / airmass);

        sumWeights += (1.0 / airmass);
    }

    coordOut->ra = coordIn->ra + (sumRA / sumWeights);

    coordOut->dec = coordIn->dec + (sumDC / sumWeights);

}

void unrefractIt (aCoord * coordIn, aCoord * coordOut,
                  struct exposinfo *pExposure, double wave)
{
    double HAold;

    double z;
    double cz;
    double zr;

    double t0;
    double wavelength;
    double latitude;

    double x;

    double az;

    double sinHa;
    double cosHa;

    double newHa;

    double realLatitude = KPNO_LAT * M_PI / 180.0;

    double rst = pExposure->st * 15.0 * M_PI / 180.0;

    latitude = KPNO_LAT * 3600.0 * AS2R;

    /* check on OK wavelength  and convert to microns */

    if (wave < 3000)
        wave = 3000.0;

    if (wave > 10000)
        wave = 10000.0;

    wavelength = wave / 10000.0;

    /* convert temp to deg K */

    t0 = FtoK (pExposure->tempadopt);

    /* turn latitude into radians */

    latitude = KPNO_LAT * 3600.0 * AS2R;

    HAold = rst - coordIn->ra;

    /* see to it that ha is between  - pi and  + pi */

    if (HAold >= M_PI)
        HAold -= TWOPI;

    if (HAold < -(M_PI))
        HAold += TWOPI;

    /* cosine of the zenith distance */

    cz = sin (coordIn->dec) * sin (realLatitude) +
        cos (coordIn->dec) * cos (realLatitude) * cos (HAold);

    z = acos (cz);

    /* call slalib to de-refract z */

    sla_refro (z, KPNO_ALT, t0, KPNO_ATM_PRES, KPNO_HUMIDITY,
               wavelength, latitude, LAPSE_RATE, REFRACT_PREC, &zr);

    /* x is the tan of the azimuth: */

    x = sin (HAold) /
        (cos (HAold) * sin (realLatitude) -
         tan (coordIn->dec) * cos (realLatitude));

    if (HAold >= 0)
    {

        if (x >= 0)
            az = M_PI + atan (x);
        else
            az = TWOPI + atan (x);
    }
    else
    {

        if (x >= 0)
            az = atan (x);
        else
            az = M_PI + atan (x);
    }

    /* Az should be between 0 and twopi at this point. */

    /* find new declination: */

    coordOut->dec = asin (sin (realLatitude) * cos (z + zr) +
                          cos (realLatitude) * sin (z + zr) * cos (az));

    /* find new hour angle and RA  */

    sinHa = -sin (az) * sin (z + zr) / cos (coordOut->dec);

    cosHa = (cos (z + zr) - sin (coordOut->dec) * sin (realLatitude)) /
        (cos (coordOut->dec) * cos (realLatitude));

    newHa = atan2 (sinHa, cosHa);

    coordOut->ra = coordIn->ra + (HAold - newHa);

    if (coordOut->ra > M_PI)
        coordOut->ra -= TWOPI;

    if (coordOut->ra < -(M_PI))
        coordOut->ra += TWOPI;

}

void refractAtTime (aCoord * coordIn, double siderialNow, double a, double b,
                    double *refRA, double *refDC, double *airmass)
{

    /*
       INPUT:
       pTheStar is pointer to a star data structure
       siderialNow is sidereal time in decimal hrs
       a and b are the refraction coefficients for r = A * tanz  +  Btan^3z
       OUTPUT:
       refRA is a pointer to the correction for refraction in RA in RADIANS
       corrected for declination
       refDC is correction for refraction in declination in RADIANS
       airmass is what it sounds like...
     */

    double HAold;

    double newHa;
    double newDec;

    double z;
    double cz;
    double zr;

    double x;

    double az;

    double sinHa;
    double cosHa;

    double realLatitude = KPNO_LAT * M_PI / 180.0;

    double rst = siderialNow * 15.0 * M_PI / 180.0;

    HAold = rst - coordIn->ra;

    /* see to it that ha is between  - pi and  + pi */

    if (HAold >= M_PI)
        HAold -= TWOPI;

    if (HAold < -(M_PI))
        HAold += TWOPI;

    /* cosine of the zenith distance */

    cz = sin (coordIn->dec) * sin (realLatitude) +
        cos (coordIn->dec) * cos (realLatitude) * cos (HAold);

    *airmass = 1.0 / cz;

    z = acos (cz);

    /* call slalib to refract z */

    sla_refz (z, a, b, &zr);

    /* x is the tan of the azimuth: */

    x = sin (HAold) /
        (cos (HAold) * sin (realLatitude) -
         tan (coordIn->dec) * cos (realLatitude));

    if (HAold >= 0)
    {

        if (x >= 0)
            az = M_PI + atan (x);
        else
            az = TWOPI + atan (x);
    }
    else
    {

        if (x >= 0)
            az = atan (x);
        else
            az = M_PI + atan (x);
    }

    /* Az should be between 0 and twopi at this point. */

    /* find new declination: */

    newDec = asin (sin (realLatitude) * cos (zr) +
                   cos (realLatitude) * sin (zr) * cos (az));

    *refDC = newDec - coordIn->dec;

    /* find new hour angle */

    sinHa = -sin (az) * sin (zr) / cos (newDec);

    cosHa = (cos (zr) - sin (newDec) * sin (realLatitude)) /
        (cos (newDec) * cos (realLatitude));

    newHa = atan2 (sinHa, cosHa);

    *refRA = HAold - newHa;

    if (*refRA > M_PI)
        *refRA -= TWOPI;

    if (*refRA < -(M_PI))
        *refRA += TWOPI;

}

/*
		projectIt

	Find the x,y focal plane coordinates of the object pointed to be
	the argument.  
	Save the results in the starinfo structure.
*/

void projectIt (aCoord * coordIn, point * coordOut, aCoord * fieldCenterCoord)
{
    int flag;

    /* call slalib to project to a tangent plane */

    sla_s2tp (coordIn->ra, coordIn->dec,
              fieldCenterCoord->ra, fieldCenterCoord->dec,
              &coordOut->x, &coordOut->y, &flag);

}

/*
		deprojectIt

	Find the x,y focal plane coordinates of the object pointed to be
	the argument.  
	Save the results in the starinfo structure.
*/

void deprojectIt (point * coordIn, aCoord * coordOut,
                  aCoord * fieldCenterCoord)
{
    /* call slalib to de-project from a tangent plane */

    sla_tp2s (coordIn->x, coordIn->y,
              fieldCenterCoord->ra, fieldCenterCoord->dec,
              &coordOut->ra, &coordOut->dec);

}

#ifdef IGNORE

/*

		flattenIt

	Deproject the x,y coordinates of the objects in the curved
	focal plane onto a flat surface (stepped plate or otherwise...)
	I have also retained the "original" logic of "changeCoords"
        which converts to inches, fixes the signs if need be and
        converts to motor steps at the end, although these logically
        could be grouped together somewhere else

	Written by Phil Massey, Nov. 1990
*/

void flattenIt (pointIn, pointOut)
point *pointIn;
point *pointOut;

{
    double zed;
    double dprime;
    double theta;
    double xprime = 0;
    double yprime = 0;

    double radius = RADIUS * ENCODER_SCALE;

    /* tired of writing big words */

    double x = pointIn->x;
    double y = pointIn->y;

    /* let x and y be the original coordinates in the curved 
       focal plane in mm.  Then d^2 = x^2 +y^2 is the distance from the
       center of the field to the point x,y ON THE CURVED SURFACE        
       Here we are only interested in the projection of this onto a flat
       plate, xprime and yprime.
       (no matter what height we are talking about about; the
       correction for the "steps" are <0.1 arcsec and can be added
       in a separate subroutine if needed)
     */

    if (x != 0 || y != 0)
    {

        /* do a whole bunch of stuff if both x and y aren't zero */

        zed = sqrt ((x * x) + (y * y));

        theta = zed / radius;

        dprime = radius * sin (theta);

        /* This is now the deprojected distance.  If x and y are not 0,
           Therefore yprime= (y/x) * xprime and
           xprime**2 = dprime**2 - yprime**2
           But what if there are zeros? */

        if (x == 0)
        {
            xprime = 0;
            yprime = dprime;
        }

        else if (y == 0)
        {
            xprime = dprime;
            yprime = 0;
        }

        else
        {
            xprime = sqrt ((dprime * dprime) / (1. + ((y * y) / (x * x))));
            yprime = xprime * (y / x);
            if (yprime < 0.0)
                yprime = -yprime;
        }

        /* OK, now correct the signs  */

        if (x < 0.0)
            xprime = -xprime;

        if (y < 0.0)
            yprime = -yprime;

    }                           /* end the bracket here begun with both x and y not being 0 */

    /* the following have nothing to do with the geometry, but are included
       for ease in simply replacing "fixCoords" with this routine. */

    /* fix signs using global fudges */

    xprime *= EWSIGN;
    yprime *= NSSIGN;

    /* store the motor step xprime and y prime into the data structure */

    pointOut->x = xprime;
    pointOut->y = yprime;
}

/*

		flexIt

	The above routine, but with the ability to un-flatten, too!

	Written by Lee Oct, '92
*/

void flexIt (pointIn, pointOut, flag)
point *pointIn;
point *pointOut;
int flag;

{
    double zed;
    double dprime;
    double theta;
    double xprime = 0;
    double yprime = 0;

    double radius = RADIUS * ENCODER_SCALE;

    /* tired of writing big words */

    double x = pointIn->x;
    double y = pointIn->y;

    if (x != 0 || y != 0)
    {

        /* do a whole bunch of stuff if both x and y aren't zero */

        zed = sqrt ((x * x) + (y * y));

        /* the following two cases are exact inverses
           including the error from the original routine by Phil */

        if (flag == NORMAL_ASTROMETRY)
        {
            theta = asin (zed / radius);
            dprime = theta * radius;
        }
        else
        {
            theta = zed / radius;
            dprime = radius * sin (theta);
        }

        if (x == 0)
        {
            xprime = 0;
            yprime = dprime;
        }

        else if (y == 0)
        {
            xprime = dprime;
            yprime = 0;
        }

        else
        {
            xprime = sqrt ((dprime * dprime) / (1. + ((y * y) / (x * x))));
            yprime = xprime * (y / x);
            if (yprime < 0.0)
                yprime = -yprime;
        }

        /* OK, now correct the signs  */

        if (x < 0.0)
            xprime = -xprime;

        if (y < 0.0)
            yprime = -yprime;

    }                           /* end the bracket here begun with both x and y not being 0 */

    /* fix signs using global fudges */

    xprime *= EWSIGN;
    yprime *= NSSIGN;

    /* store the motor step xprime and y prime into the data structure */

    pointOut->x = xprime;
    pointOut->y = yprime;
}
#endif

/*
		precessIt

	Use the function sla_preces to precess the coordinates of the object
	pointed to by the argument. 
*/

void precessIt (aCoord * coordIn, aCoord * coordOut,
                struct exposinfo *pExposure)
{
    /* slalib modifies the value in place, so copy it to the
       destination first */

    *coordOut = *coordIn;

    sla_preces ("fk4", pExposure->epoch, pExposure->current,
                &coordOut->ra, &coordOut->dec);
}

/*
		unprecessIt

	Use the function sla_preces to precess the coordinates of the object
	pointed to by the argument. 
*/

void unprecessIt (aCoord * coordIn, aCoord * coordOut,
                  struct exposinfo *pExposure)
{
    /* slalib modifies the value in place, so copy it to the
       destination first */

    *coordOut = *coordIn;

    sla_preces ("fk4", pExposure->current, pExposure->epoch,
                &coordOut->ra, &coordOut->dec);
}

/*

		correctIt

	Correct the specified point for optical distortion from the
	camera.

	Written by Lee Jun, '94
	Modified by Lee Mar, '95
*/

void correctIt (point * pIn, point * pOut, int flag)
{
    double f;

    if ((pIn->x == 0) && (pIn->y == 0))
    {

        pOut->x = 0;
        pOut->y = 0;

    }
    else
    {

        /* compute the pincushion term */

        if (flag == NORMAL_ASTROMETRY)
        {

            f = 1.0 + (WIYN_C * ((pIn->x * pIn->x) + (pIn->y * pIn->y)));

            if (Verbose)
                fprintf (stderr, "pincushion f = %f\n", f);

            pOut->x = pIn->x * f;
            pOut->y = pIn->y * f;

            /* Correct for plate scale */

            pOut->x /= (WIYN_SCALE * AS2R);
            pOut->y /= (WIYN_SCALE * AS2R);
        }
        else
        {

            /* Correct for plate scale */

            pIn->x *= (WIYN_SCALE * AS2R);
            pIn->y *= (WIYN_SCALE * AS2R);

            f = 1.0 + (WIYN_C * ((pIn->x * pIn->x) + (pIn->y * pIn->y)));

            if (Verbose)
                fprintf (stderr, "pincushion rev. f = %f\n", f);

            pOut->x = pIn->x / f;
            pOut->y = pIn->y / f;

        }
    }
}

/*

		unCorrectIt

	Remove the correction for the specified point for
	optical distortion from the camera.

	Written by Lee Jun, '94
*/

void unCorrectIt (point * pIn, point * pOut)
{
    double d;
    double dp;

    *pOut = *pIn;

    if ((pIn->x == 0) && (pIn->y == 0))
    {

        pOut->x = 0;
        pOut->y = 0;

    }
    else
    {

        /* find the angle from zenith */

        d = sqrt ((pOut->x * pOut->x) + (pOut->y * pOut->y));

/*
      if( Verbose )
	 fprintf( stderr, "d = %f\n", d );
*/

        /* compute the magic scale factors */

        dp = (A_CORR * (d)) +
            (B_CORR * (d * d)) +
            (C_CORR * (d * d * d)) + (D_CORR * (d * d * d * d));

/*
      if( Verbose )
	 fprintf( stderr, "dp= %f\n", dp );
*/

        pOut->x *= (d / dp);
        pOut->y *= (d / dp);

        pOut->x *= MIN2R;
        pOut->y *= MIN2R;
    }
}

/*

		flattenIt

	Correct for field curvature.
*/

void flattenIt (point * pointIn, point * pointOut, int flag)
{
    double zed;
    double theta;
    double h;
    double xprime = 0;
    double yprime = 0;

    /* temp variables for succinctness */

    double x = pointIn->x;
    double y = pointIn->y;

    zed = sqrt ((x * x) + (y * y));

    theta = zed / WIYN_RADIUS;

    h = WIYN_RADIUS * sin (theta);

    if (flag == NORMAL_ASTROMETRY)
    {

        if (zed > 0.0)
        {

            xprime = x * (h / zed);
            yprime = y * (h / zed);

        }
    }
    else
    {                           /* reverse astrometry */

        if (zed > 0.0)
        {

            xprime = x / (h / zed);
            yprime = y / (h / zed);

        }
    }

    /* store the motor step xprime and y prime into the data structure */

    pointOut->x = xprime;
    pointOut->y = yprime;

}

/*

		rotateIt

	Correct for field rotation.
*/

void rotateIt (point * pIn, point * pOut, double angle)
{
    double correctAngle = angle - (M_PI / 2);

    if (Verbose)
        fprintf (stderr, "rotational offset = %f\n", (angle * (180 / M_PI)));

    pOut->x = pIn->y * cos (correctAngle) + pIn->x * sin (correctAngle);
    pOut->y = -pIn->x * cos (correctAngle) + pIn->y * sin (correctAngle);

}

/*

		unWarpIt

	Correct for plate warp .
*/

void unWarpIt (point * pIn, point * pOut, int flag)
{
    double dIn;
    double dOut;

    double reff;
    double temp;

    dIn = sqrt ((pIn->x * pIn->x) + (pIn->y * pIn->y));

    if (dIn < 151.9325)
    {

        reff = 5679.881;
    }
    else
    {

        temp = dIn * 400;
        reff = 5670.799 + temp * (2.988726E-4 - (temp * 2.458933E-9));
    }

    dOut = reff * asin (dIn / WIYN_RADIUS);

    pOut->x = 0;
    pOut->y = 0;

    if (flag == NORMAL_ASTROMETRY)
    {
        if (dIn > 0)
        {

            pOut->x = pIn->x * (dOut / dIn);
            pOut->y = pIn->y * (dOut / dIn);
        }
    }
    else
    {
        if (dIn > 0)
        {

            pOut->x = pIn->x * (dIn / dOut);
            pOut->y = pIn->y * (dIn / dOut);
        }
    }
}

/*
	encoderIt

   Convert a value in arcmin to encoder steps

*/

void encoderIt (point * pIn, point * pOut, int flag)
{
    /* convert to the encoder scale */

    *pOut = *pIn;

    if (flag == NORMAL_ASTROMETRY)
    {

        pOut->x *= 400;
        pOut->y *= 400;
    }
    else
    {
        pOut->x /= 400;
        pOut->y /= 400;
    }

}
