
/***********************    COMPLAMP CANVAS THINGIE  *************************

     Use this file as a template for drawing into canvases.

     The following functions are included:
     (all can be customized)

      void complampWinInit( canvas );	Initialization of the canvas window.

      void complampSetCanvas( char *args );	String based control of window contents.

      void complampRenderCanvas( a, b, c );	Main rendering function. 

*****************************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include "xvFibSim_ui.h"

#include <math.h>

/**** defines for canvas characteristics ****/

#define  WIN_SIZE 75

#define CANVAS_LABEL		"COMPLAMPS"
#define CANVAS_LABEL_SIZE	9
#define CANVAS_LABEL_X		12
#define CANVAS_LABEL_Y		11

/**** put defines for misc dimensions here. ****/

#define  CIR_SIZE 56
#define  SPOT_SIZE 12
#define  SPOT_RADIUS 18

/**** put defines for colors here. ****/

#define WHITE   0
#define BLACK   1
#define RED     2
#define TURQ    3

/**** local structures ****/

struct genericWinInfo
{

    Display *display;
    XID canvasXid;              /* stashed id of canvas for convenience */

    int canvasSizeX;            /* size of drawing canvas */
    int canvasSizeY;
    int windowSizeX;            /* size of visable window */
    int windowSizeY;
    int windowOffsetX;          /* Location of window on canvas */
    int windowOffsetY;

    GC GCwhite;                 /* pre-defined graphic contexts */
    GC GCblack;
    GC GCred;
    GC GCturq;
    GC GCmisc1;
    GC GCmisc2;

    double scaleFactor;         /* scale to get from hydra scale to canvas scale */

};

static struct genericWinInfo TheWinInfo;

/********local prototypes*******/

/* these get renamed for each use of this file */

void complampRenderCanvas ();
void complampSetCanvas ();

/*****   initialization function  ******/

/*
   the following is the initialization for a generic window.
   it uses the basic GC structure defined in this file.
   The GC values here can be changed without hurting any other canvas
*/

void complampWinInit (canvas)
Xv_opaque *canvas;              /* main program supplies this so that we can ignore type */

{

    Cms cms;
    XFontStruct *font;
    XGCValues gc_val;

    static unsigned long *Colors;       /* color table */

    /* make sure the defines above match the colors here! */

    static Xv_singlecolor cms_colors[] = {
        {255, 255, 255},
        {0, 0, 0},
        {255, 0, 0},
        {0, 255, 255},
        {1, 1, 1},
    };

    cms = xv_create (NULL, CMS,
                     CMS_SIZE,
                     (sizeof (cms_colors) / sizeof (Xv_singlecolor)),
                     CMS_TYPE, XV_STATIC_CMS, CMS_COLORS, cms_colors, NULL);

    TheWinInfo.display = (Display *) xv_get ((int) canvas, XV_DISPLAY);

    TheWinInfo.canvasXid =
        (XID) xv_get (canvas_paint_window ((int) canvas), XV_XID);

    xv_set ((int) canvas, WIN_CMS, cms, NULL);

    /* get the colormap from the canvas now that
       the cms has been installed */

    Colors = (unsigned long *) xv_get ((int) canvas, WIN_X_COLOR_INDICES);

    /* set the font */

    if (!(font = XLoadQueryFont (TheWinInfo.display, "fixed")))
    {
        (void) fputs ("Failure to load fixed font\n", stderr);
        exit (1);
    }

    gc_val.font = font->fid;

         /***** create the GCs *****/

    /* at first they're all the same. */

    TheWinInfo.GCwhite = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCblack = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCred = XCreateGC (TheWinInfo.display,
                                  TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCturq = XCreateGC (TheWinInfo.display,
                                   TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCmisc1 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCmisc2 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    /* now set the colors */

    XSetForeground (TheWinInfo.display, TheWinInfo.GCwhite, Colors[WHITE]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCblack, Colors[BLACK]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCred, Colors[RED]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCturq, Colors[TURQ]);

         /***** custom drawing modes go here *****/

}

/* routine to operate canvas from a string arguemnt
   Customize this to suit your needs.*/

void complampSetCanvas (args)
char *args;

{
    int lampFlags[9];
    int count;

    count = sscanf (args, "%d %d %d %d %d %d %d %d",
                    &lampFlags[1],
                    &lampFlags[2],
                    &lampFlags[3],
                    &lampFlags[4],
                    &lampFlags[5],
                    &lampFlags[6], &lampFlags[7], &lampFlags[8]);

    if (count >= 8)
        complampRenderCanvas (lampFlags, 1);    /* flag means new data, not refresh */
}

/* the routine to draw stuff.  Args will need to be customized */

void complampRenderCanvas (lampFlags, flag)
int lampFlags[];
int flag;                       /* non-zero flag means new data, not refresh */

{
    int x1;
    int y1;
    int i;

    double angle;
    char labelString[8];

    GC spotGC;
    GC idGC;

    static int internalFlags[9];

    if (flag == 1)
        for (i = 1; i <= 8; i++)
            internalFlags[i] = lampFlags[i];

    XFillArc (TheWinInfo.display,
              TheWinInfo.canvasXid,
              TheWinInfo.GCblack,
              (int) ((WIN_SIZE / 2) - (CIR_SIZE / 2)),
              (int) ((WIN_SIZE / 2) - (CIR_SIZE / 2)),
              (int) (CIR_SIZE),
              (int) (CIR_SIZE), 0, 23040 /* full circle */ );

    for (i = 1; i <= 8; i++)
    {

        angle = (i - 1) * (M_PI / 4);

        sprintf (labelString, "%d", i);

        if (internalFlags[i] == 1)
        {
            /* The lamp is on */
            spotGC = TheWinInfo.GCturq;
            idGC = TheWinInfo.GCblack;
        }
        else
        {
            /* The lamp is off */
            spotGC = TheWinInfo.GCblack;
            idGC = TheWinInfo.GCwhite;
        }

        x1 = (int) ((SPOT_RADIUS * cos (angle))
                    - (SPOT_SIZE / 2) + (WIN_SIZE / 2));

        y1 = (int) (-(SPOT_RADIUS * sin (angle))
                    - (SPOT_SIZE / 2) + (WIN_SIZE / 2));

        XFillArc (TheWinInfo.display,
                  TheWinInfo.canvasXid,
                  spotGC,
                  x1, y1,
                  (int) (SPOT_SIZE),
                  (int) (SPOT_SIZE), 0, 23040 /* full circle */ );

        XDrawString (TheWinInfo.display,
                     TheWinInfo.canvasXid,
                     idGC, (x1 + 4), (y1 + 9), labelString, 1);

    }

    XDrawString (TheWinInfo.display,
                 TheWinInfo.canvasXid,
                 TheWinInfo.GCblack,
                 CANVAS_LABEL_X, CANVAS_LABEL_Y,
                 CANVAS_LABEL, CANVAS_LABEL_SIZE);
}
