
/*
  gc.c  defines and functions to initialize the graphic contexts
  for the hydra simulator.
*/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include "xvFibSim_ui.h"

#include "miscMath.h"
#include "xFibSim.h"
#include "inits.h"
#include "gc.h"

/* indices into color table renders specified colors. */

#define WHITE   0
#define RED     1
#define GREEN   2
#define BLUE    3
#define BLACK   4
#define ORANGE  5
#define YELLOW  6
#define FOO     7

/** additional Globals not created by Guide **/

unsigned long *Colors;          /* color table */

void gcInit (theWinInfo, window)
struct winInfo *theWinInfo;
xvFibSim_window1_objects *window;

{

    Cms cms;
    Xv_cmsdata cms_data;
    XFontStruct *font;
    XGCValues gc_val;

    static Xv_singlecolor cms_colors[] = {
        {255, 255, 255},
        {255, 0, 0},
        {0, 175, 0},
        {0, 0, 255},
        {0, 0, 0},
        {225, 175, 55},
        {255, 255, 0},
        {1, 1, 1},
    };

    cms = xv_create (NULL, CMS, CMS_SIZE, 8,
                     /* CMS_TYPE,    use_dynamic? XV_DYNAMIC_CMS : XV_STATIC_CMS, */
                     CMS_TYPE, XV_STATIC_CMS, CMS_COLORS, cms_colors, NULL);

    theWinInfo->display = (Display *) xv_get (window->window1, XV_DISPLAY);

    theWinInfo->canvasXid =
        (XID) xv_get (canvas_paint_window (window->canvas1), XV_XID);

    xv_set (window->canvas1, WIN_CMS, cms, NULL);

    /* get the colormap from the canvas now that
       the cms has been installed */

    Colors = (unsigned long *) xv_get (window->canvas1, WIN_X_COLOR_INDICES);

    /* set the font */

    if (!(font = XLoadQueryFont (theWinInfo->display, "fixed")))
    {
        fputs ("Failure to load fixed font\n", stderr);
        exit (1);
    }

    gc_val.font = font->fid;

    /* create the GCs */

    /* gc to draw a fiber normally */

    theWinInfo->fiberGCnormal = XCreateGC (theWinInfo->display,
                                           theWinInfo->canvasXid, GCFont,
                                           &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->fiberGCnormal, Colors[BLACK]);

    /* gc to highlight a fiber */

    theWinInfo->fiberGCselect = XCreateGC (theWinInfo->display,
                                           theWinInfo->canvasXid, GCFont,
                                           &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->fiberGCselect, Colors[RED]);

    /* gc to erase a fiber */

    theWinInfo->fiberGCerase = XCreateGC (theWinInfo->display,
                                          theWinInfo->canvasXid, GCFont,
                                          &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->fiberGCerase, Colors[WHITE]);

    /* gc to draw a star */

    theWinInfo->starGCnormal = XCreateGC (theWinInfo->display,
                                          theWinInfo->canvasXid, GCFont,
                                          &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->starGCnormal, Colors[GREEN]);

    /* gc to select a star */

    theWinInfo->starGCselect = XCreateGC (theWinInfo->display,
                                          theWinInfo->canvasXid, GCFont,
                                          &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->starGCselect, Colors[RED]);

    /* gc to draw a sky location */

    theWinInfo->skyGCnormal = XCreateGC (theWinInfo->display,
                                         theWinInfo->canvasXid, GCFont,
                                         &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->skyGCnormal, Colors[BLUE]);

    /* gc to draw an acquisition star */

    theWinInfo->acqGCnormal = XCreateGC (theWinInfo->display,
                                         theWinInfo->canvasXid, GCFont,
                                         &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->acqGCnormal, Colors[GREEN]);

    /* gc to draw a locked fiber */

    theWinInfo->fiberGClocked = XCreateGC (theWinInfo->display,
                                           theWinInfo->canvasXid, GCFont,
                                           &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->fiberGClocked, Colors[YELLOW]);

    /* gc to draw an occulted fiber */

    theWinInfo->fiberGCoccult = XCreateGC (theWinInfo->display,
                                           theWinInfo->canvasXid, GCFont,
                                           &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->fiberGCoccult, Colors[ORANGE]);

    /* gc to draw a fops fiber */

    theWinInfo->fopsGCnormal = XCreateGC (theWinInfo->display,
                                          theWinInfo->canvasXid, GCFont,
                                          &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->fopsGCnormal, Colors[GREEN]);
    /* gc to drag a fiber */

    /* gc to draw the steps on the plate */

    theWinInfo->plateGCzones = XCreateGC (theWinInfo->display,
                                          theWinInfo->canvasXid, GCFont,
                                          &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->plateGCzones, Colors[ORANGE]);

    /* gc to drag a fiber */

    gc_val.function = GXxor;

    theWinInfo->fiberGCdrag = XCreateGC (theWinInfo->display,
                                         theWinInfo->canvasXid,
                                         (GCFont | GCFunction), &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->fiberGCdrag, Colors[ORANGE]);

    /* gc to draw the target for coord readback */

    gc_val.function = GXxor;

    theWinInfo->crosshairGC = XCreateGC (theWinInfo->display,
                                         theWinInfo->canvasXid,
                                         (GCFont | GCFunction), &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->crosshairGC, Colors[RED]);

    /* gc to draw the target for calibration screen */

    gc_val.function = GXxor;

    theWinInfo->calScreenGC = XCreateGC (theWinInfo->display,
                                         theWinInfo->canvasXid,
                                         (GCFont | GCFunction), &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->calScreenGC, Colors[ORANGE]);

    XSetLineAttributes (theWinInfo->display,
                        theWinInfo->calScreenGC,
                        2, LineSolid, CapButt, JoinMiter);

}

/*
   the following is the initialization for the cardinal direction window.
   it uses the same GC structure and most values as the main canvas.
   The GC values here can be changed without hurting the main canvas
*/

void cardinalWinInit (theWinInfo, window)
struct winInfo *theWinInfo;
xvFibSim_window1_objects *window;

{

    Cms cms;
    Xv_cmsdata cms_data;
    XFontStruct *font;
    XGCValues gc_val;

    static Xv_singlecolor cms_colors[] = {
        {255, 255, 255},
        {255, 0, 0},
        {0, 175, 0},
        {0, 0, 255},
        {0, 0, 0},
        {225, 175, 55},
        {255, 255, 0},
        {1, 1, 1},
    };

    cms = xv_create (NULL, CMS, CMS_SIZE, 8,
                     /* CMS_TYPE,    use_dynamic? XV_DYNAMIC_CMS : XV_STATIC_CMS, */
                     CMS_TYPE, XV_STATIC_CMS, CMS_COLORS, cms_colors, NULL);

    theWinInfo->display = (Display *) xv_get (window->window1, XV_DISPLAY);

    theWinInfo->canvasXid =
        (XID) xv_get (canvas_paint_window (window->cardinalCanvas), XV_XID);

    xv_set (window->cardinalCanvas, WIN_CMS, cms, NULL);

    /* get the colormap from the canvas now that
       the cms has been installed */

    Colors = (unsigned long *) xv_get (window->cardinalCanvas,
                                       WIN_X_COLOR_INDICES);

    /* set the font */

    if (!(font = XLoadQueryFont (theWinInfo->display, "fixed")))
    {
        fputs ("Failure to load fixed font\n", stderr);
        exit (1);
    }

    gc_val.font = font->fid;

    /* create the GCs */

    /* gc to draw a fiber normally */

    theWinInfo->fiberGCnormal = XCreateGC (theWinInfo->display,
                                           theWinInfo->canvasXid, GCFont,
                                           &gc_val);

/*
	 XSetArcMode( theWinInfo->display,
	    theWinInfo->fiberGCnormal, ArcPieSlice );

	 XSetLineAttributes( theWinInfo->display,
	    theWinInfo->fiberGCnormal,
	    2,
	    LineSolid,
	    CapButt,
	    JoinMiter );
*/

    XSetForeground (theWinInfo->display,
                    theWinInfo->fiberGCnormal, Colors[BLACK]);

    /* gc to highlight a fiber */

    theWinInfo->fiberGCselect = XCreateGC (theWinInfo->display,
                                           theWinInfo->canvasXid, GCFont,
                                           &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->fiberGCselect, Colors[RED]);

    /* gc to erase a fiber */

    theWinInfo->fiberGCerase = XCreateGC (theWinInfo->display,
                                          theWinInfo->canvasXid, GCFont,
                                          &gc_val);

    XSetForeground (theWinInfo->display,
                    theWinInfo->fiberGCerase, Colors[GREEN]);

}
