
/*
		idIt.c

   Routines for taking a plate coordinate and turning it into
   a button or object id.
*/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include "xvFibSim_ui.h"

#include "miscNumbers.h"
#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "kaboom.h"
#include "nesslib.h"
#include "nessIO.h"
#include "xref.h"
#include "xFibSim.h"
#include "inits.h"
#include "gc.h"
#include "idIt.h"

extern FILE *stuffIn;
extern int BlockedByParent;

/**** globals ****/

struct CurrentFiberInfo CurrentFiber;
struct CurrentObjectInfo CurrentObject;
struct CurrentCrosshairInfo CurrentCrosshair;

/****************/

void clickToCoord ();
void selectFiber ();
void selectObject ();

void clickToFiber (event, xRefFlag)
Event *event;
int xRefFlag;                   /* causes any assigned object to select too */

{
    int button;

    char fiberMessage[32];
    char objectMessage[32];

    double hydraX;
    double hydraY;

    int whereClicked;

    int dx = HydraWinInfo.canvasSizeX / 2;
    int dy = HydraWinInfo.canvasSizeY / 2;

    hydraX = (double) (event->ie_locx - dx) / HydraWinInfo.scaleFactor;
    hydraY = (double) -(event->ie_locy - dy) / HydraWinInfo.scaleFactor;

    button = coordToButton ((int) hydraX, (int) hydraY, &whereClicked);

    /*fprintf( stderr, "click at %5f %5f is %d, %d\n",
       hydraX, hydraY, button, whereClicked ); */

    /* we assume that a confirmation click for a move has already been
       ruled out */

    selectFiber (button, SELECT, xRefFlag);     /* draws it */
}

void clickToObject (event, xRefFlag)
Event *event;
int xRefFlag;                   /* causes any assigned fiber to select too */

{
    int object;

    char message[64];

    double hydraX;
    double hydraY;

    int dx = HydraWinInfo.canvasSizeX / 2;
    int dy = HydraWinInfo.canvasSizeY / 2;

    hydraX = (double) (event->ie_locx - dx) / HydraWinInfo.scaleFactor;
    hydraY = (double) -(event->ie_locy - dy) / HydraWinInfo.scaleFactor;

    /*fprintf( stderr, "hydra: %d, %d \n", (int)hydraX, (int)hydraY ); */

    object = coordToObject ((int) hydraX, (int) hydraY);

    selectObject (object, SELECT, xRefFlag);
}

void setIDmessages (button, object)
int button;
int object;

{
    char fiberMessage[32];
    char objectMessage[48];

    /* see if there is anything to cross reference
       and decide what is unknown */

    /* set up the messages */

    if (button >= 0)
        sprintf (fiberMessage, "Fib: %3d", button);
    else
        sprintf (fiberMessage, "unknown");

    if (object >= 0)
    {
        sprintf (objectMessage, "User #%3d : %-31s",
                 Stars[object]->IDnumber, Stars[object]->name);
    }
    else
        sprintf (objectMessage, "unknown");

    /* move the messages to the window */

    if (button != LEAVE_ALONE)
        xv_set (XvFibSim_window1->fiberField, PANEL_VALUE, fiberMessage,
                NULL);
    if (object != LEAVE_ALONE)
        xv_set (XvFibSim_window1->objectField, PANEL_VALUE, objectMessage,
                NULL);
}

void clickToPark (event)
Event *event;

{
    xItem theList;

    clickToFiber (event, XREF);

    if (CurrentFiber.status == INACTIVE)
        return;

    /* now set up the action and verify with the user */

    sprintf (PendingCommandDescr, "Park button #%d", CurrentFiber.id);

    if (xRefFiber (CurrentFiber.id, &theList, Stars, FiBase))
    {

        if (theList.object != VOIDENTRY)
        {
            sprintf (PendingCommandDescr2,
                     "Which is object #%d; \"%s\"",
                     Stars[theList.object]->internalID,
                     Stars[theList.object]->name);
        }
        else
        {
            PendingCommandDescr2[0] = '\0';
        }
    }
    else
    {
        PendingCommandDescr2[0] = '\0';
    }

    sprintf (PendingCommand, "park %d\n", CurrentFiber.id);

    activateConfirm ();

}

void clickToView (event)
Event *event;

{
    clickToObject (event, XREF);

    if (CurrentObject.status != INACTIVE)
    {
        sprintf (PendingCommand, "view o %d\n",
                 Stars[CurrentObject.id]->IDnumber);
    }
    else
    {

        clickToFiber (event, XREF);

        if (CurrentFiber.status == INACTIVE)
            return;
        else
            sprintf (PendingCommand, "view %d\n", CurrentFiber.id);
    }

    dispatchIt (PendingCommand);

}

void clickToSky (event)
Event *event;

{
    int tempId = CurrentFiber.id;
    xItem theList;

    if (isButton (event))
    {

        clickToFiber (event, XREF);

        if (tempId == CurrentFiber.id)
        {

            /* already selected, so assume it is the sky position */

            clickToCoord (event, 0);
        }
    }
    else
    {
        clickToCoord (event, 0);
    }

    /* if we have a location and a fiber, verify the intent */

    if ((CurrentFiber.status != INACTIVE) &&
        (CurrentCrosshair.status != INACTIVE))
    {

        /* set up the action and verify with the user */

        sprintf (PendingCommandDescr, "Move button #%d to X = %d, Y = %d",
                 CurrentFiber.id,
                 (int) CurrentCrosshair.location.x,
                 (int) CurrentCrosshair.location.y);

        /* get the object assigned to this fiber (if any) */

        if (xRefFiber (CurrentFiber.id, &theList, Stars, FiBase))
        {

            if (theList.object != VOIDENTRY)
            {
                sprintf (PendingCommandDescr2,
                         "instead of to object #%d; \"%s\"",
                         Stars[theList.object]->IDnumber,
                         Stars[theList.object]->name);
            }
            else
            {
                PendingCommandDescr2[0] = '\0';
            }
        }
        else
        {
            PendingCommandDescr2[0] = '\0';
        }

        sprintf (PendingCommand, "move %d %d %d\n",
                 CurrentFiber.id,
                 (int) CurrentCrosshair.location.x,
                 (int) CurrentCrosshair.location.y);

        activateConfirm ();

        selectFiber (CURRENT, DE_SELECT, XREF);
    }
}

void clickToAssign (event)
Event *event;

{

    if (isObject (event))
        clickToObject (event, NO_XREF);

    if (isButton (event))
        clickToFiber (event, NO_XREF);

    /* if we have a location and a fiber, verify the intent */

    if ((CurrentFiber.status != INACTIVE) &&
        (CurrentObject.status != INACTIVE))
    {

        /* set up the action and verify with the user */

        sprintf (PendingCommandDescr, "Assign button #%d ", CurrentFiber.id);

        sprintf (PendingCommandDescr2,
                 "to object #%d; \"%s\"",
                 Stars[CurrentObject.id]->IDnumber,
                 Stars[CurrentObject.id]->name);

        sprintf (PendingCommand, "assign %d %d \n",
                 Stars[CurrentObject.id]->IDnumber, CurrentFiber.id);

        activateConfirm ();

        selectFiber (CURRENT, DE_SELECT, XREF);
    }
}

void clickToCoord (event, flag)
Event *event;
int flag;

{
    char temp[16];

    struct aPoint coords;

    int dx = HydraWinInfo.canvasSizeX / 2;
    int dy = HydraWinInfo.canvasSizeY / 2;

    int hydraX = (double) (event->ie_locx - dx) / HydraWinInfo.scaleFactor;
    int hydraY = (double) -(event->ie_locy - dy) / HydraWinInfo.scaleFactor;

    coords.x = hydraX;
    coords.y = hydraY;

    switch (CurrentCrosshair.status)
    {

        case SELECTED:

            /* erase it */

            drawCrosshair (&CurrentCrosshair.location, 15, 0);

            break;

        case INACTIVE:
        default:
            break;
    }

    /* now draw the new selection */

    if (flag == 0)
    {

        drawCrosshair (&coords, 15, 0);

        CurrentCrosshair.location = coords;
        CurrentCrosshair.status = SELECTED;

        sprintf (temp, "%6d", hydraX);
        xv_set (XvFibSim_window1->textfield1, PANEL_VALUE, temp, NULL);

        sprintf (temp, "%6d", hydraY);
        xv_set (XvFibSim_window1->textfield4, PANEL_VALUE, temp, NULL);

    }
    else
    {

        CurrentCrosshair.status = INACTIVE;

        sprintf (temp, " ");
        xv_set (XvFibSim_window1->textfield1, PANEL_VALUE, temp, NULL);

        sprintf (temp, " ");
        xv_set (XvFibSim_window1->textfield4, PANEL_VALUE, temp, NULL);
    }

}

isButton (event)
Event *event;

{
    int button;

    char message[32];

    double hydraX;
    double hydraY;

    int whereClicked;

    int dx = HydraWinInfo.canvasSizeX / 2;
    int dy = HydraWinInfo.canvasSizeY / 2;

    hydraX = (double) (event->ie_locx - dx) / HydraWinInfo.scaleFactor;
    hydraY = (double) -(event->ie_locy - dy) / HydraWinInfo.scaleFactor;

    button = coordToButton ((int) hydraX, (int) hydraY, &whereClicked);

    return (button >= 0);

}

isObject (event)
Event *event;

{
    int object;

    char message[64];

    double hydraX;
    double hydraY;

    int dx = HydraWinInfo.canvasSizeX / 2;
    int dy = HydraWinInfo.canvasSizeY / 2;

    hydraX = (double) (event->ie_locx - dx) / HydraWinInfo.scaleFactor;
    hydraY = (double) -(event->ie_locy - dy) / HydraWinInfo.scaleFactor;

    /*fprintf( stderr, "hydra: %d, %d \n", (int)hydraX, (int)hydraY ); */

    object = coordToObject ((int) hydraX, (int) hydraY);

    return (object >= 0);

}

/*
   clickIsConfirm()

   See if the latest mouse down is within a reasonable distance to
   be taken as a confirmation of a drag of a fiber to a new location.

   Uses global:  CurrentFiber
*/

int clickIsConfirm (event)
Event *event;

{
    int id;
    double dx, dy;
    double r;
    double d1, d2;
    double hydraX;
    double hydraY;

    /* be sure there is actually something to confirm! */

    if (CurrentFiber.status != DRAG_PENDING)
        return (FALSE);

    /* do the math */

    hydraX = (double) (event->ie_locx - dx) / HydraWinInfo.scaleFactor;
    hydraY = (double) -(event->ie_locy - dy) / HydraWinInfo.scaleFactor;

    dx = hydraX - CurrentFiber.requestedX;
    dy = hydraY - CurrentFiber.requestedY;

    r = (FiBase[id].ButtonDia / 2) + BUTTON_BUTTON_MARGIN;

    if (((dx * dx) + (dy * dy)) < (r * r))
        return (TRUE);
    else
        return (FALSE);
}

#ifdef IGNORE

/* 
   dragFiber()

   Using the geometry structure of the current fiber, clear the drag
   location image of the fiber and re-draw it at the new location
   that resulted from the drag.
   
   If fiber is not already being drug at time of call, set up for 
   future drags, and update the status of the CurrentFiber.
*/

dragFiber (event)
Event *event;

{
    int button;
    int whereClicked;

    double hydraX;
    double hydraY;

    int id = CurrentFiber.id;

    long dx = HydraWinInfo.canvasSizeX / 2;
    long dy = HydraWinInfo.canvasSizeY / 2;

    hydraX = (double) (event->ie_locx - dx) / HydraWinInfo.scaleFactor;
    hydraY = (double) -(event->ie_locy - dy) / HydraWinInfo.scaleFactor;

    if (CurrentFiber.status != DRAGGING)
    {

        button = coordToButton ((int) hydraX, (int) hydraY, &whereClicked);

        if (whereClicked != BUTTON_CLICKED)
            return;

        /* We haven't done any dragging yet, so set up the temporary
           geometry structure */

        CurrentFiber.geo->pivot.x = FiBase[id].PivotPos.x;
        CurrentFiber.geo->pivot.y = FiBase[id].PivotPos.y;

        CurrentFiber.geo->button.x = hydraX;
        CurrentFiber.geo->button.y = hydraY;

        MagicNumbers (CurrentFiber.geo);

        /* draw the tube once to make it visible */

        drawFiber (&FiBase[id], CurrentFiber.geo, FIBER_DRAG);

        CurrentFiber.status = DRAGGING;
    }

    else
    {

        /*
           draw the tube a second time where it is now.
           Because of the draw mode of the GC, this will
           clear it, leaving the original pixels below it 
           intact.
         */

        drawFiber (&FiBase[id], CurrentFiber.geo, FIBER_DRAG);

        /* set up the new location */

        CurrentFiber.geo->button.x = hydraX;
        CurrentFiber.geo->button.y = hydraY;

        MagicNumbers (CurrentFiber.geo);

        /* draw the tube at the new location */

        drawFiber (&FiBase[id], CurrentFiber.geo, FIBER_DRAG);

    }
}

#endif

int coordToButton (x, y, whereClicked)
int x, y;
int *whereClicked;

{
    int id;
    double dx, dy;
    double r;
    double d1, d2;
    double segLength;
    int segment;
    struct aPoint mousePt;
    struct aLine *lineSeg;

    /* check for click on a button */

    for (id = 0; id < MAXFIBER; id++)
    {

        dx = FiBase[id].CurPos.x - x;
        dy = FiBase[id].CurPos.y - y;

        r = (FiBase[id].ButtonDia / 2) + BUTTON_BUTTON_MARGIN;

        if (((dx * dx) + (dy * dy)) < (r * r))
        {
            *whereClicked = BUTTON_CLICKED;
            return (id);
        }
    }

    /* now check for a click on a tube */

    mousePt.x = x;
    mousePt.y = y;

    for (id = 0; id < MAXFIBER; id++)
    {

        for (segment = 0; segment < NUM_TUBE_SEGS; segment++)
        {

            lineSeg = FiBase[id].CurGeometry->segments[segment];

            r = (double) pointLineSeparation (&mousePt, lineSeg);

            if (r < TUBE_ZONE)
            {                   /* we are close to *some* line */

                segLength = distance (lineSeg->p1, lineSeg->p2);

                d1 = distance (&mousePt, lineSeg->p1);
                d2 = distance (&mousePt, lineSeg->p2);

                /* find out if target point is between endpoints of segment */

                if ((d1 < segLength) && (d2 < segLength))
                {
                    *whereClicked = TUBE_CLICKED;
                    return (id);
                }
            }
        }
    }

    /* if we made it to here, we didn't find it... */

    *whereClicked = NOT_FOUND;
    return (NOT_FOUND);
}

int coordToObject (x, y)
int x, y;

{
    int id = 1;
    double dx, dy;
    double r;

    /* check for click on an object */

    while (Stars[id] != NULL)
    {

        dx = Stars[id]->camEncoderCoord.x - x;
        dy = Stars[id]->camEncoderCoord.y - y;

        r = OBJECT_ZONE;

        if (((dx * dx) + (dy * dy)) < (r * r))
        {
            return (id);
        }

        id++;
    }

    return (NOT_FOUND);
}

void selectFiber (button, mode, xRefToo)
int button;
int mode;
int xRefToo;

{
    xItem theList;

    switch (CurrentFiber.status)
    {

        case DRAGGING:
        case DRAG_PENDING:
            /*
               user has selected a new fiber without confirming a previous move.
               If we get here, there is an image of the pending move. 
               Drawing it once more will clear it because of the draw
               mode of the GC.
             */

            drawFiber (&FiBase[CurrentFiber.id], CurrentFiber.geo,
                       FIBER_DRAG);

            /* fiber is *also* selected, so... */

            /* fputs( "drag abandoned\n", stderr ); */

             /*FALLTHROUGH*/;

        case SELECTED:

            /* Put it back to the normal color */

            drawFiber (&FiBase[CurrentFiber.id],
                       FiBase[CurrentFiber.id].CurGeometry,
                       getFiberType (CurrentFiber.id));

            break;

        case INACTIVE:
        default:
            break;
    }

    if ((button >= 0) && (xRefToo))
    {

        if (xRefFiber (button, &theList, Stars, FiBase))
        {
            selectObject (theList.object, mode, NO_XREF);
        }
    }

    if ((button >= 0) && (mode == SELECT))
    {

        CurrentFiber.id = button;
        CurrentFiber.status = SELECTED;

        drawFiber (&FiBase[CurrentFiber.id],
                   FiBase[CurrentFiber.id].CurGeometry, FIBER_SELECT);

        setIDmessages (button, LEAVE_ALONE);
    }
    else
    {

        CurrentFiber.id = NO_FIBER;
        CurrentFiber.status = INACTIVE;

        setIDmessages (CLEAR, LEAVE_ALONE);
    }
}

void selectObject (object, mode, xRefToo)
int object;
int mode;
int xRefToo;

{
    xItem theList;

    /* clean up the old selection, if one exists */

    switch (CurrentObject.status)
    {

        case SELECTED:

            /* Put it back to the normal color */

            drawStar (CurrentObject.id, OBJ_DEFAULT_MODE);

            break;

        case INACTIVE:
        default:
            break;
    }

    if ((object >= 0) && (xRefToo))
    {

        if (xRefObject (object, &theList, Stars, FiBase))
        {
            selectFiber (theList.fiber, mode, NO_XREF);
        }
    }

    /* now draw the new selection */

    if ((object >= 0) && (mode == SELECT))
    {

        CurrentObject.id = object;
        CurrentObject.status = SELECTED;

        drawStar (object, OBJ_SELECTED_MODE);

        setIDmessages (LEAVE_ALONE, object);
    }
    else
    {

        CurrentObject.id = NO_FIBER;
        CurrentObject.status = INACTIVE;

        setIDmessages (LEAVE_ALONE, CLEAR);
    }
}

activateConfirm ()
{
    int x;
    int y;

    /* prepare the location of the pop-up box */

    x = (int) xv_get (XvFibSim_window1->window1, XV_X) - 100;

    if (x < 0)
        x = 0;

    y = (int) xv_get (XvFibSim_window1->window1, XV_Y) + 10;

    if (y < 0)
        y = 0;

    xv_set (XvFibSim_confirmPopup->confirmText,
            PANEL_VALUE, PendingCommandDescr, NULL);

    xv_set (XvFibSim_confirmPopup->confirmText2,
            PANEL_VALUE, PendingCommandDescr2, NULL);

    xv_set (XvFibSim_confirmPopup->confirmPopup, XV_X, x, XV_Y, y, NULL);

    xv_set (XvFibSim_confirmPopup->confirmPopup, XV_SHOW, TRUE, NULL);

}

updateLabels (stars, exposure, filename)
struct starinfo *stars[];
struct exposinfo *exposure;
char *filename;

{
    int ra[4];
    int dec[4];
    char raSign;
    char decSign;

    char temp1[16];
    char temp2[16];
    char temp3[32];

    xv_set (XvFibSim_window1->fileNameItem, PANEL_VALUE, filename, NULL);

    xv_set (XvFibSim_window1->internalNameItem,
            PANEL_VALUE, exposure->expName, NULL);

    sprintf (temp1, "%7.2f", exposure->epoch);

    xv_set (XvFibSim_window1->epochItem, PANEL_VALUE, temp1, NULL);

    sprintf (temp3, "%9.3f (incl. mech. offset)",
             (FieldRotationalOffset * (180 / M_PI)));

    xv_set (XvFibSim_window1->fieldRotItem, PANEL_VALUE, temp3, NULL);

    /* convert ra and dec into x:x:x type fields */

    sla_dr2tf (1, stars[0]->rawCoord.ra, &raSign, ra);
    sla_dr2af (3, stars[0]->rawCoord.dec, &decSign, dec);

    if (decSign == '+')
        decSign = ' ';

    sprintf (temp1, "%2d:%02d:%02d.%1d", ra[0], ra[1], ra[2], ra[3]);

    /* 
       spisprintf( temp2, "%c%02d:%02d:%02d.%03d ",
       decSign, dec[0], dec[1], dec[2], dec[3] );
     */

    sprintf (temp2, "%c%02d:%02d:%02d", decSign, dec[0], dec[1], dec[2]);

    xv_set (XvFibSim_window1->RAitem, PANEL_VALUE, temp1, NULL);

    xv_set (XvFibSim_window1->DECitem, PANEL_VALUE, temp2, NULL);
}
