
/***********************    LED CANVAS THINGIE  *************************

     Use this file as a template for drawing into canvases.

     The following functions are included:
     (all can be customized)

      void ledWinInit( canvas );	Initialization of the canvas window.

      void ledSetCanvas( char *args );	String based control of window contents.

      void ledRenderCanvas( int mode );	Main rendering function. 

*****************************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include "xvFibSim_ui.h"

#include <math.h>

/**** defines for canvas characteristics ****/

#define  WIN_SIZE 75

#define CANVAS_LABEL		"GRIP LED"
#define CANVAS_LABEL_SIZE	8
#define CANVAS_LABEL_X		10
#define CANVAS_LABEL_Y		11

/**** put defines for misc dimensions here. ****/

#define  LED_RADIUS 10          /* radius of the dome of the led */
#define  LED_R1     15.         /* radius to start of "rays" */
#define  LED_R2     30.         /* radius to end of "rays" */

/**** put defines for colors here. ****/

#define WHITE   0
#define BLACK   1
#define RED     2
#define TURQ    3

/**** local structures ****/

struct genericWinInfo
{

    Display *display;
    XID canvasXid;              /* stashed id of canvas for convenience */

    int canvasSizeX;            /* size of drawing canvas */
    int canvasSizeY;
    int windowSizeX;            /* size of visable window */
    int windowSizeY;
    int windowOffsetX;          /* Location of window on canvas */
    int windowOffsetY;

    GC GCwhite;                 /* pre-defined graphic contexts */
    GC GCblack;
    GC GCred;
    GC GCturq;
    GC GCmisc1;
    GC GCmisc2;

    double scaleFactor;         /* scale to get from hydra scale to canvas scale */

};

static struct genericWinInfo TheWinInfo;

/********local prototypes*******/

/* these get renamed for each use of this file */

void ledRenderCanvas ();
void ledSetCanvas ();

/*****   initialization function  ******/

/*
   the following is the initialization for a generic window.
   it uses the basic GC structure defined in this file.
   The GC values here can be changed without hurting any other canvas
*/

void ledWinInit (canvas)
Xv_opaque *canvas;              /* main program supplies this so that we can ignore type */

{

    Cms cms;
    XFontStruct *font;
    XGCValues gc_val;

    static unsigned long *Colors;       /* color table */

    /* make sure the defines above match the colors here! */

    static Xv_singlecolor cms_colors[] = {
        {255, 255, 255},
        {0, 0, 0},
        {255, 0, 0},
        {0, 255, 255},
        {1, 1, 1},
    };

    cms = xv_create (NULL, CMS,
                     CMS_SIZE,
                     (sizeof (cms_colors) / sizeof (Xv_singlecolor)),
                     CMS_TYPE, XV_STATIC_CMS, CMS_COLORS, cms_colors, NULL);

    TheWinInfo.display = (Display *) xv_get ((int) canvas, XV_DISPLAY);

    TheWinInfo.canvasXid =
        (XID) xv_get (canvas_paint_window ((int) canvas), XV_XID);

    xv_set ((int) canvas, WIN_CMS, cms, NULL);

    /* get the colormap from the canvas now that
       the cms has been installed */

    Colors = (unsigned long *) xv_get ((int) canvas, WIN_X_COLOR_INDICES);

    /* set the font */

    if (!(font = XLoadQueryFont (TheWinInfo.display, "fixed")))
    {
        (void) fputs ("Failure to load fixed font\n", stderr);
        exit (1);
    }

    gc_val.font = font->fid;

         /***** create the GCs *****/

    /* at first they're all the same. */

    TheWinInfo.GCwhite = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCblack = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCred = XCreateGC (TheWinInfo.display,
                                  TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCturq = XCreateGC (TheWinInfo.display,
                                   TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCmisc1 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCmisc2 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    /* now set the colors */

    XSetForeground (TheWinInfo.display, TheWinInfo.GCwhite, Colors[WHITE]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCblack, Colors[BLACK]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCred, Colors[RED]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCturq, Colors[TURQ]);

         /***** custom drawing modes go here *****/

    XSetLineAttributes (TheWinInfo.display,
                        TheWinInfo.GCblack, 2, LineSolid, CapButt, JoinMiter);

}

/* routine to operate canvas from a string arguemnt
   Customize this to suit your needs.*/

void ledSetCanvas (args)
char *args;

{
    if (strcmp (args, "on") == 0)
    {

        ledRenderCanvas (1, 1);
    }

    if (strcmp (args, "off") == 0)
    {

        ledRenderCanvas (0, 1);
    }
}

/* the routine to draw stuff.  Args will need to be customized */

void ledRenderCanvas (mode, flag)
int mode;
int flag;                       /* 0 = refresh, 1=new data */

{

    int x1;
    int y1;
    int x2;
    int y2;

    double angle;

    GC modeGC;

    static int internalMode;

    if (flag == 1)
        internalMode = mode;

    if (internalMode == 0)
        modeGC = TheWinInfo.GCblack;
    else
        modeGC = TheWinInfo.GCred;

    XClearWindow (TheWinInfo.display, TheWinInfo.canvasXid);

    XFillRectangle (TheWinInfo.display,
                    TheWinInfo.canvasXid,
                    modeGC,
                    (int) ((WIN_SIZE / 2) - LED_RADIUS),
                    (int) ((WIN_SIZE / 2)),
                    (int) ((LED_RADIUS * 2) + 1), (int) (LED_RADIUS));

    XDrawRectangle (TheWinInfo.display,
                    TheWinInfo.canvasXid,
                    TheWinInfo.GCblack,
                    (int) ((WIN_SIZE / 2) - LED_RADIUS),
                    (int) ((WIN_SIZE / 2)),
                    (int) ((LED_RADIUS * 2) + 1), (int) (LED_RADIUS));

    XFillArc (TheWinInfo.display,
              TheWinInfo.canvasXid,
              modeGC,
              (int) ((WIN_SIZE / 2) - LED_RADIUS),
              (int) ((WIN_SIZE / 2) - LED_RADIUS),
              (int) (LED_RADIUS * 2),
              (int) (LED_RADIUS * 2), 0, 23040 /* full circle */ );

    XDrawArc (TheWinInfo.display,
              TheWinInfo.canvasXid,
              TheWinInfo.GCblack,
              (int) ((WIN_SIZE / 2) - LED_RADIUS),
              (int) ((WIN_SIZE / 2) - LED_RADIUS),
              (int) (LED_RADIUS * 2),
              (int) (LED_RADIUS * 2), 0, 11520 /* half circle */ );

    XDrawLine (TheWinInfo.display,
               TheWinInfo.canvasXid,
               modeGC,
               (int) ((WIN_SIZE / 2) - (2 * LED_RADIUS / 3)),
               (int) ((WIN_SIZE / 2) + LED_RADIUS),
               (int) ((WIN_SIZE / 2) - (2 * LED_RADIUS / 3)),
               (int) ((WIN_SIZE / 2) + (2 * LED_RADIUS)));

    XDrawLine (TheWinInfo.display,
               TheWinInfo.canvasXid,
               TheWinInfo.GCblack,
               (int) ((WIN_SIZE / 2) + (2 * LED_RADIUS / 3)),
               (int) ((WIN_SIZE / 2) + (LED_RADIUS)),
               (int) ((WIN_SIZE / 2) + (2 * LED_RADIUS / 3)),
               (int) ((WIN_SIZE / 2) + (2 * LED_RADIUS)));

    /* draw the rays if it's on. */

    if (mode == 1)
    {

        for (angle = 0; angle <= M_PI; angle += (M_PI / 8))
        {

            x1 = (int) ((LED_R1 * cos (angle)) + (WIN_SIZE / 2));
            y1 = (int) (-(LED_R1 * sin (angle)) + (WIN_SIZE / 2));

            x2 = (int) ((LED_R2 * cos (angle)) + (WIN_SIZE / 2));
            y2 = (int) (-(LED_R2 * sin (angle)) + (WIN_SIZE / 2));

            XDrawLine (TheWinInfo.display,
                       TheWinInfo.canvasXid, modeGC, x1, y1, x2, y2);
        }

    }

    /* label the leads on the LED */

    XDrawString (TheWinInfo.display,
                 TheWinInfo.canvasXid,
                 modeGC,
                 (int) ((WIN_SIZE / 2) - (2 * LED_RADIUS / 3) - 10),
                 (int) ((WIN_SIZE / 2) + (2 * LED_RADIUS) + 5), "+", 1);

    XDrawString (TheWinInfo.display,
                 TheWinInfo.canvasXid,
                 TheWinInfo.GCblack,
                 (int) ((WIN_SIZE / 2) + (2 * LED_RADIUS / 3) + 5),
                 (int) ((WIN_SIZE / 2) + (2 * LED_RADIUS) + 5), "-", 1);

    XDrawString (TheWinInfo.display,
                 TheWinInfo.canvasXid,
                 TheWinInfo.GCblack,
                 CANVAS_LABEL_X, CANVAS_LABEL_Y,
                 CANVAS_LABEL, CANVAS_LABEL_SIZE);
}
