
/* Generic WIYN Client: myTclAppInit.c

 *  Module name:
        myTclAppInit.c

 *  Function:
        Template application application init.

 *  Description:
        Customize this file in your application.  Tcl_AppInit is called by
        the TCL library when an application is initialized. (See TCL/Tk
        documentation.)
        Add your own commands etc. to make this "yours."

 *  Language:
      C

 *  Support: Kim Gillies, NOAO

 *

 *  History:
      12 Dec 94: Version 2 - goes into CVS

 *  Rcs Id:  $Id: myTclAppInit.c,v 1.1.1.1 2004/08/13 20:46:05 behzad Exp $

 */
#include <tcl.h>
#include <assert.h>
#include <gwc/gwc.h>

#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "buttons.h"
#include "kaboom.h"

#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif

extern int noGWC;

/* In file prototypes */
static int gwcAppInit (Tcl_Interp * interp);

/* globals */

gwcConnectInfo hostInfo;

/*
 * The following variable is a special hack that allows applications
 * to be linked using the procedure "main" from the Tk library.  The
 * variable generates a reference to "main", which causes main to
 * be brought in from the library (and all of Tk and Tcl with it).
 */

extern int main ();
extern int matherr ();
int *tclDummyMathPtr = (int *) matherr;
int *tclDummyMainPtr = (int *) main;

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int Tcl_AppInit (Tcl_Interp * interp)
{
    /*
     * Call the init procedures for included packages.  Each call should
     * look like this:
     *
     * if (Mod_Init(interp) == TCL_ERROR) {
     *     return TCL_ERROR;
     * }
     *
     * where "Mod" is the name of the module.
     */

    if (Tcl_Init (interp) == TCL_ERROR)
    {
        return TCL_ERROR;
    }

    if (noGWC)
        return TCL_OK;

    /* Call the Generic WIYN Client Init */
    if (gwcAppInit (interp) == TCL_ERROR)
    {
        return TCL_ERROR;
    }
    /*
     * Specify a user-specific startup file to invoke if the application
     * is run interactively.  Typically the startup file is "~/.apprc"
     * where "app" is the name of the application.  If this line is deleted
     * then no user-specific startup file will be run under any conditions.
     */

    /*tcl_RcFileName = "~/.wishrc"; */
    return TCL_OK;
}

static int gwcAppInit (Tcl_Interp * interp)
{
    /* Initialize the interpreter */
    if (gwcTclInit (interp) == TCL_ERROR)
    {
        return TCL_ERROR;
    }

    /* Initialize gwc if desired */
    if (gwcInit (interp, "wiyn", "hydra", NULL, NULL,
                 &hostInfo, CONNECT_YES) == TCL_ERROR)
    {
        return TCL_ERROR;
    }

    gwcLibPassRestartParameters (10, 10000);

    if (createVariables (hostInfo) == TCL_ERROR)
    {
        return TCL_ERROR;
    }

    if (doRotatorInitialValue (hostInfo) == TCL_ERROR)
    {
        return TCL_ERROR;
    }

    return TCL_OK;
}

/*******************  hydra simulator additions *****************/

static int createVariables (gwcConnectInfo info)
{
    assert (info != (gwcConnectInfo) NULL);

    /* create the various variables */

    doRotatorSetup (info);
    doBenchSetup (info);

    return TCL_OK;
}

int initVariables (void)        /* uses local global */
{
    if (noGWC == TRUE)
        return;

    doRotatorInitialValue (hostInfo);
}

static int streamFunc (gwcEventArgs args)
{
    fprintf (stderr, "\nStream name: %s\n", args.attribute);
    return TCL_OK;
}

/* A non-tcl command to send changes */
 /*ARGSUSED*/ int sendAll (ClientData data)
{
    gwcConnectInfo info = (gwcConnectInfo) data;

    assert (info != (gwcConnectInfo) NULL);

    gwcPutStream (info, "hydra.guider", STREAM_ALL);
    gwcPutStream (info, "hydra.assign", STREAM_ALL);
    gwcPutStream (info, "hydra.field", STREAM_ALL);
    gwcPutStream (info, "hydra.gripper", STREAM_ALL);
    gwcPutStream (info, "hydra.misc", STREAM_ALL);

    return TCL_OK;
}
