
/***********************    GENERIC CANVAS THINGIE  *************************

     Use this file as a template for drawing into canvases.

     The following functions are included:
     (all can be customized)

      void plateWinInit( canvas );	Initialization of the canvas window.

      void plateSetCanvas( char *args );	String based control of window contents.

      void plateRenderCanvas( a, b, c );	Main rendering function. 

*****************************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include "xvFibSim_ui.h"

extern double sin ();
extern double cos ();
extern double atan ();

/**** defines for canvas characteristics ****/

#define  WIN_SIZE 75

#define CANVAS_LABEL		"PLATE"
#define CANVAS_LABEL_SIZE	5
#define CANVAS_LABEL_X		20
#define CANVAS_LABEL_Y		11

/**** put misc defines here. ****/

#define WARP_PLATE	0
#define FLATTEN_PLATE	1

#define  NUM_POINTS  51

#define  ARC_RADIUS  100.0
#define  ARC_HIDDEN  75.0

/**** put defines for colors here. ****/

#define WHITE   0
#define BLACK   1
#define RED     2
#define TURQ    3
#define ORANGE  4

/**** local structures ****/

struct genericWinInfo
{

    Display *display;
    XID canvasXid;              /* stashed id of canvas for convenience */

    int canvasSizeX;            /* size of drawing canvas */
    int canvasSizeY;
    int windowSizeX;            /* size of visable window */
    int windowSizeY;
    int windowOffsetX;          /* Location of window on canvas */
    int windowOffsetY;

    GC GCwhite;                 /* pre-defined graphic contexts */
    GC GCblack;
    GC GCred;
    GC GCturq;
    GC GCmisc1;
    GC GCmisc2;

    double scaleFactor;         /* scale to get from hydra scale to canvas scale */

};

struct CurrentPlateInfo
{

    int status;                 /* see mode #defines above */
    double warpage;             /* last warpage drawn */
    GC gc;
    GC gcText;
    XPoint oldPoints[NUM_POINTS + 1];
    XPoint newPoints[NUM_POINTS + 1];
};

/******* local globals *******/

static struct genericWinInfo TheWinInfo;

static struct CurrentPlateInfo CurrentPlate;

/********local prototypes*******/

/* these get renamed for each use of this file */

void plateRenderCanvas ();
void plateSetCanvas ();
void animatePlate ();
void drawPlateAtWarpage ();
void plateInit ();

/*****   initialization function  ******/

/*
   the following is the initialization for a generic window.
   it uses the basic GC structure defined in this file.
   The GC values here can be changed without hurting any other canvas
*/

void plateWinInit (canvas)
Xv_opaque *canvas;              /* main program supplies this so that we can ignore type */

{

    Cms cms;
    XFontStruct *font;
    XGCValues gc_val;

    static unsigned long *Colors;       /* color table */

    /* make sure the defines above match the colors here! */

    static Xv_singlecolor cms_colors[] = {
        {255, 255, 255},
        {0, 0, 0},
        {255, 0, 0},
        {0, 255, 255},
        {225, 175, 55},
        {1, 1, 1},
    };

    cms = xv_create (NULL, CMS,
                     CMS_SIZE,
                     (sizeof (cms_colors) / sizeof (Xv_singlecolor)),
                     CMS_TYPE, XV_STATIC_CMS, CMS_COLORS, cms_colors, NULL);

    TheWinInfo.display = (Display *) xv_get ((int) canvas, XV_DISPLAY);

    TheWinInfo.canvasXid =
        (XID) xv_get (canvas_paint_window ((int) canvas), XV_XID);

    xv_set ((int) canvas, WIN_CMS, cms, NULL);

    /* get the colormap from the canvas now that
       the cms has been installed */

    Colors = (unsigned long *) xv_get ((int) canvas, WIN_X_COLOR_INDICES);

    /* set the font */

    if (!(font = XLoadQueryFont (TheWinInfo.display, "fixed")))
    {
        (void) fputs ("Failure to load fixed font\n", stderr);
        exit (1);
    }

    gc_val.font = font->fid;

         /***** create the GCs *****/

    /* at first they're all the same. */

    TheWinInfo.GCwhite = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCblack = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCred = XCreateGC (TheWinInfo.display,
                                  TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCturq = XCreateGC (TheWinInfo.display,
                                   TheWinInfo.canvasXid, GCFont, &gc_val);

    /* now set the colors */

    XSetForeground (TheWinInfo.display, TheWinInfo.GCwhite, Colors[WHITE]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCblack, Colors[BLACK]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCred, Colors[RED]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCturq, Colors[TURQ]);

         /***** custom GC's go here *****/

    /* gc to draw the plate curvature */

    gc_val.function = GXxor;

    TheWinInfo.GCmisc1 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid,
                                    (GCFont | GCFunction), &gc_val);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCmisc1, Colors[ORANGE]);

    XSetLineAttributes (TheWinInfo.display,
                        TheWinInfo.GCmisc1, 2, LineSolid, CapButt, JoinMiter);

         /***** misc. initialization go here *****/

    plateInit (&CurrentPlate);

}

/* routine to operate canvas from a string arguemnt
   Customize this to suit your needs.*/

void plateSetCanvas (args)
char *args;

{
    if (strcmp (args, "flat") == 0)
    {

        animatePlate (FLATTEN_PLATE);
    }

    if (strcmp (args, "warp") == 0)
    {

        animatePlate (WARP_PLATE);
    }
}

/* the routine to draw stuff.  Args will need to be customized */

void plateRenderCanvas ()
{
    XClearWindow (TheWinInfo.display, TheWinInfo.canvasXid);

    plateLabel (&CurrentPlate);

    plateActivate (&CurrentPlate);
}

/******* plate specific functions  *******/

/* NOTE, some of this code is historical, and more elaborate than necessary */

void plateInit (plateInfo)
struct CurrentPlateInfo *plateInfo;

{
    plateInfo->status = 5;
    plateInfo->gc = TheWinInfo.GCmisc1;
    plateInfo->gcText = TheWinInfo.GCblack;

    plateInfo->warpage = 0.0;

    computePlatePoints (plateInfo->oldPoints, plateInfo->warpage);
    computePlatePoints (plateInfo->newPoints, plateInfo->warpage);
}

void plateActivate (plateInfo)
struct CurrentPlateInfo *plateInfo;

{

    /* this will turn the bits off if they're on and visa-versa */

    XDrawLines (TheWinInfo.display,
                TheWinInfo.canvasXid,
                plateInfo->gc,
                plateInfo->oldPoints, (NUM_POINTS), CoordModeOrigin);
}

void plateRefresh (plateInfo)
struct CurrentPlateInfo *plateInfo;

{
    /* this makes sure the bits are at the correct scale */

    computePlatePoints (plateInfo->oldPoints, plateInfo->warpage);

    /* this will turn the bits back on */

    XDrawLines (TheWinInfo.display,
                TheWinInfo.canvasXid,
                plateInfo->gc,
                plateInfo->oldPoints, (NUM_POINTS), CoordModeOrigin);
}

void plateLabel ()
{
    XDrawString (TheWinInfo.display,
                 TheWinInfo.canvasXid,
                 TheWinInfo.GCblack,
                 CANVAS_LABEL_X, CANVAS_LABEL_Y,
                 CANVAS_LABEL, CANVAS_LABEL_SIZE);

}

/*
		animatePlate

   Show the plate moving from it's current position to it's
   new position which is specified in the argument.
*/

void animatePlate (mode)
int mode;

{
    double warpage;

    if (mode == WARP_PLATE)
    {

        for (warpage = 0.0; warpage <= 1.0; warpage += 0.01)
        {

            drawPlateAtWarpage (&CurrentPlate, warpage);
        }
    }
    else
    {                           /* mode is to flatten the plate */

        for (warpage = 1.0; warpage >= 0.0; warpage -= 0.01)
        {

            drawPlateAtWarpage (&CurrentPlate, warpage);

        }
    }
    plateLabel ();
}

/*
		drawPlateAtWarpage

   Draw the focal plane plate as a series of segments using
   the "warpage" argument to show the degree the plate is
   warped.  "Warpage" can be from 0.0 to 1.0
*/

void drawPlateAtWarpage (plateInfo, warpage)
struct CurrentPlateInfo *plateInfo;
double warpage;

{
    int i;

    /* compute the newPoints */

    computePlatePoints (plateInfo->newPoints, warpage);

    /* be sure that it's active for drawing... */

    /* draw the new points */

    XDrawLines (TheWinInfo.display,
                TheWinInfo.canvasXid,
                plateInfo->gc,
                plateInfo->newPoints, (NUM_POINTS), CoordModeOrigin);

    /* erase the old points */

    XDrawLines (TheWinInfo.display,
                TheWinInfo.canvasXid,
                plateInfo->gc,
                plateInfo->oldPoints, (NUM_POINTS), CoordModeOrigin);

    /* save the warpage for possible future refresh */

    plateInfo->warpage = warpage;

    /* make the new points the future old points */

    for (i = 0; i < NUM_POINTS; i++)
        plateInfo->oldPoints[i] = plateInfo->newPoints[i];
}

/* load up a set of endpoints for the plate segments */

computePlatePoints (pointSet, warpage)
XPoint pointSet[];
double warpage;

{
    int i;                      /* looping variable */
    int j;                      /* point index */
    int k;                      /* point index */

    double phi;

    double dx = (double) WIN_SIZE / 2.0;
    double dy = (double) WIN_SIZE / 2.0;

    for (i = 0; i <= (NUM_POINTS / 2 + 1); i++)
    {

        phi = atan ((double) i / ARC_HIDDEN);

        /* because of symmetry, we start in the middle of the
           plate and work outwards in two directions at once */

        j = (NUM_POINTS / 2 + 1) + i;
        k = (NUM_POINTS / 2 + 1) - i;

        pointSet[j].x =
            (warpage * ((ARC_RADIUS * (double) cos (phi)) - ARC_HIDDEN)) + dx;

        pointSet[k].x = pointSet[j].x;

        pointSet[j].y = (ARC_RADIUS * (double) sin (phi)) + dy;

        pointSet[k].y = -(ARC_RADIUS * (double) sin (phi)) + dy;

    }
}
