
/***********************    ROTATOR CANVAS THINGIE  *************************

     Use this file as a template for drawing into canvases.

     The following functions are included:
     (all can be customized)

      void rotatorWinInit( canvas );	Initialization of the canvas window.

      void rotatorSetCanvas( char *args );	String based control of window contents.

      void rotatorRenderCanvas( a, b, c );	Main rendering function. 

*****************************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include "xvFibSim_ui.h"

#include <math.h>

/**** defines for canvas characteristics ****/

#define  WIN_SIZE 75

#define CANVAS_LABEL		"ROTATOR"
#define CANVAS_LABEL_SIZE	7
#define CANVAS_LABEL_X		12
#define CANVAS_LABEL_Y		11

/**** put defines for misc dimensions here. ****/

#define  CIR_SIZE 38
#define  BOX_SIZE 60

/**** put defines for colors here. ****/

#define WHITE   0
#define BLACK   1
#define RED     2
#define TURQ    3

/**** local structures ****/

struct genericWinInfo
{

    Display *display;
    XID canvasXid;              /* stashed id of canvas for convenience */

    int canvasSizeX;            /* size of drawing canvas */
    int canvasSizeY;
    int windowSizeX;            /* size of visable window */
    int windowSizeY;
    int windowOffsetX;          /* Location of window on canvas */
    int windowOffsetY;

    GC GCwhite;                 /* pre-defined graphic contexts */
    GC GCblack;
    GC GCred;
    GC GCturq;
    GC GCmisc1;
    GC GCmisc2;

    double scaleFactor;         /* scale to get from hydra scale to canvas scale */

};

static struct genericWinInfo TheWinInfo;

/********local prototypes*******/

/* these get renamed for each use of this file */

void rotatorRenderCanvas ();
void rotatorSetCanvas ();

/*****   initialization function  ******/

/*
   the following is the initialization for a generic window.
   it uses the basic GC structure defined in this file.
   The GC values here can be changed without hurting any other canvas
*/

void rotatorWinInit (canvas)
Xv_opaque *canvas;              /* main program supplies this so that we can ignore type */

{

    Cms cms;
    XFontStruct *font;
    XGCValues gc_val;

    static unsigned long *Colors;       /* color table */

    /* make sure the defines above match the colors here! */

    static Xv_singlecolor cms_colors[] = {
        {255, 255, 255},
        {0, 0, 0},
        {255, 0, 0},
        {0, 255, 255},
        {1, 1, 1},
    };

    cms = xv_create (NULL, CMS,
                     CMS_SIZE,
                     (sizeof (cms_colors) / sizeof (Xv_singlecolor)),
                     CMS_TYPE, XV_STATIC_CMS, CMS_COLORS, cms_colors, NULL);

    TheWinInfo.display = (Display *) xv_get ((int) canvas, XV_DISPLAY);

    TheWinInfo.canvasXid =
        (XID) xv_get (canvas_paint_window ((int) canvas), XV_XID);

    xv_set ((int) canvas, WIN_CMS, cms, NULL);

    /* get the colormap from the canvas now that
       the cms has been installed */

    Colors = (unsigned long *) xv_get ((int) canvas, WIN_X_COLOR_INDICES);

    /* set the font */

    if (!(font = XLoadQueryFont (TheWinInfo.display, "fixed")))
    {
        (void) fputs ("Failure to load fixed font\n", stderr);
        exit (1);
    }

    gc_val.font = font->fid;

         /***** create the GCs *****/

    /* at first they're all the same. */

    TheWinInfo.GCwhite = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCblack = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCred = XCreateGC (TheWinInfo.display,
                                  TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCturq = XCreateGC (TheWinInfo.display,
                                   TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCmisc1 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCmisc2 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    /* now set the colors */

    XSetForeground (TheWinInfo.display, TheWinInfo.GCwhite, Colors[WHITE]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCblack, Colors[BLACK]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCred, Colors[RED]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCturq, Colors[TURQ]);

         /***** custom drawing modes go here *****/

}

/* routine to operate canvas from a string arguemnt
   Customize this to suit your needs.*/

void rotatorSetCanvas (args)
char *args;

{
    double tmpAngle;

    if (sscanf (args, "%lf", &tmpAngle) >= 1)
    {
        tmpAngle *= (M_PI / 180.);
        rotatorRenderCanvas (tmpAngle, 1);
    }

}

void rotatorRenderCanvas (angle, flag)
double angle;
int flag;

{

    static double internalAngle = 0;

    XPoint arrowPoints[4];      /* vertices of triangle */
    XPoint boxPoints[5];        /* vertices of triangle */

    double x1;
    double y1;

    double x2;
    double y2;

    double x3;
    double y3;

    double x4;
    double y4;

    /* quick hack to make it go in the opposite direction */

    angle *= -1;

    if (flag != 0)
        internalAngle = angle;

    /* first fill up the arrow points array */

    x1 = (CIR_SIZE / 2.0) * cos (internalAngle + (M_PI / 2));
    y1 = -(CIR_SIZE / 2.0) * sin (internalAngle + (M_PI / 2));

    x2 = (CIR_SIZE / 2.0) * cos (internalAngle + (5 * M_PI / 4));
    y2 = -(CIR_SIZE / 2.0) * sin (internalAngle + (5 * M_PI / 4));

    x3 = (CIR_SIZE / 2.0) * cos (internalAngle - (M_PI / 4));
    y3 = -(CIR_SIZE / 2.0) * sin (internalAngle - (M_PI / 4));

    /* point 0 is the direction hydra is oriented  */

    arrowPoints[0].x = (int) (x1 + WIN_SIZE / 2);
    arrowPoints[0].y = (int) (y1 + WIN_SIZE / 2);

    /* point 1 is one of the lower corners */

    arrowPoints[1].x = (int) (x2 + WIN_SIZE / 2);
    arrowPoints[1].y = (int) (y2 + WIN_SIZE / 2);

    /* point 2 is the other of the lower corners */

    arrowPoints[2].x = (int) (x3 + WIN_SIZE / 2);
    arrowPoints[2].y = (int) (y3 + WIN_SIZE / 2);

    /* final point is where we stared */

    arrowPoints[3] = arrowPoints[0];

    /* now fill up the box points array */

    x1 = (BOX_SIZE / 2.0) * cos (internalAngle + (M_PI / 4));
    y1 = -(BOX_SIZE / 2.0) * sin (internalAngle + (M_PI / 4));

    x2 = (BOX_SIZE / 2.0) * cos (internalAngle + (3 * M_PI / 4));
    y2 = -(BOX_SIZE / 2.0) * sin (internalAngle + (3 * M_PI / 4));

    x3 = (BOX_SIZE / 2.0) * cos (internalAngle + (5 * M_PI / 4));
    y3 = -(BOX_SIZE / 2.0) * sin (internalAngle + (5 * M_PI / 4));

    x4 = (BOX_SIZE / 2.0) * cos (internalAngle + (7 * M_PI / 4));
    y4 = -(BOX_SIZE / 2.0) * sin (internalAngle + (7 * M_PI / 4));

    boxPoints[0].x = (int) (x1 + WIN_SIZE / 2);
    boxPoints[0].y = (int) (y1 + WIN_SIZE / 2);

    boxPoints[1].x = (int) (x2 + WIN_SIZE / 2);
    boxPoints[1].y = (int) (y2 + WIN_SIZE / 2);

    boxPoints[2].x = (int) (x3 + WIN_SIZE / 2);
    boxPoints[2].y = (int) (y3 + WIN_SIZE / 2);

    boxPoints[3].x = (int) (x4 + WIN_SIZE / 2);
    boxPoints[3].y = (int) (y4 + WIN_SIZE / 2);

    /* final point is where we stared */

    boxPoints[4] = boxPoints[0];

    XClearWindow (TheWinInfo.display, TheWinInfo.canvasXid);

    XFillPolygon (TheWinInfo.display, TheWinInfo.canvasXid, TheWinInfo.GCblack, boxPoints, 5,   /* triangle needs 4 points */
                  Convex, CoordModeOrigin);

    XFillArc (TheWinInfo.display,
              TheWinInfo.canvasXid,
              TheWinInfo.GCwhite,
              (int) ((WIN_SIZE / 2) - (CIR_SIZE / 2)),
              (int) ((WIN_SIZE / 2) - (CIR_SIZE / 2)),
              (int) (CIR_SIZE), (int) (CIR_SIZE),
              0, 23040 /* full circle */ );

    XFillPolygon (TheWinInfo.display, TheWinInfo.canvasXid, TheWinInfo.GCblack, arrowPoints, 4, /* triangle needs 4 points */
                  Convex, CoordModeOrigin);

    XDrawString (TheWinInfo.display,
                 TheWinInfo.canvasXid,
                 TheWinInfo.GCblack,
                 CANVAS_LABEL_X, CANVAS_LABEL_Y,
                 CANVAS_LABEL, CANVAS_LABEL_SIZE);

}
