
/*

		rotatorCmd.c

   This file contains the functions that issue commands to
   the wiyn tcs for operating the MNIR port.  This includes
   setting the offset angle, the absolute position, and 
   turning tracking on and off.  Note that all status
   requests are in "rotatorVar.c"

*/

#include <tcl.h>
#include <assert.h>
#include <gwc/gwc.h>
#include <miscMath.h>
#include <motor.h>
#include <plate.h>
#include <kaboom.h>

/****   Some defines just to save looking through the code ****/

#define CMD_ROT_BRAKE	"mnir brake"
#define CMD_ROT_TRACK	"mnir tracker"
#define CMD_ROT_OFFSET	"mnir offset set"
#define CMD_ROT_POS	"mnir position set"

#define ARG_ENABLE	"enable"
#define ARG_DISABLE	"disable"

#define ROTATOR_OFF	0
#define ROTATOR_ON	1

#define BRAKE_OFF	0
#define BRAKE_ON	1

/* declarations of globals */

extern gwcConnectInfo hostInfo;

extern int noGWC;

/*
   A command to stop tracking and move to the position where
   hydra can move fibers.
*/

int rotatorStopAndZero ()
{

    int error = TCL_OK;

    /* stop the tracking */

    if ((error = rotatorSetTracking (ROTATOR_OFF)) != TCL_OK)
        return (error);

    /* zero the offset */

    if ((error = rotatorSetOffset (0.0)) != TCL_OK)
        return (error);

    /* send to zero angle */

    if ((error = rotatorSetAbsAngle (0.0)) != TCL_OK)
        return (error);

    return (error);
}

int rotatorSetTracking (int mode)
{
    char cliString[128];

    if (noGWC)
        return TCL_OK;

    if (mode == ROTATOR_ON)
        sprintf (cliString, "%s %s\n", CMD_ROT_TRACK, ARG_ENABLE);
    else
        sprintf (cliString, "%s %s\n", CMD_ROT_TRACK, ARG_DISABLE);

    /* fputs( cliString, stderr ); */

    return gwcSendTcsCommand (hostInfo, cliString);     /* hostInfo is a global */
}

int rotatorSetOffset (double offsetAngle)
{
    char cliString[128];

    if (noGWC)
        return TCL_OK;

    sprintf (cliString, "%s %10.5fd", CMD_ROT_OFFSET, offsetAngle);

    /* fputs( cliString, stderr ); */

    return gwcSendTcsCommand (hostInfo, cliString);     /* hostInfo is a global */
}

int rotatorSetAbsAngle (double absoluteAngle)
{
    char cliString[128];

    if (noGWC)
        return TCL_OK;

    sprintf (cliString, "%s %10.5fd", CMD_ROT_POS, absoluteAngle);

    /* fputs( cliString, stderr ); */

    return gwcSendTcsCommand (hostInfo, cliString);     /* hostInfo is a global */
}

int rotatorSetBrakes (int flag)
{
    char cliString[128];

    if (noGWC)
        return TCL_OK;

    if (flag == BRAKE_ON)
        sprintf (cliString, "%s %d\n", CMD_ROT_BRAKE, BRAKE_ON);
    else
        sprintf (cliString, "%s %d\n", CMD_ROT_BRAKE, BRAKE_OFF);

    /* fputs( cliString, stderr ); */

    return gwcSendTcsCommand (hostInfo, cliString);     /* hostInfo is a global */
}
