
/***********************    SCREEN CANVAS THINGIE  *************************

     Use this file as a template for drawing into canvases.

     The following functions are included:
     (all can be customized)

      void screenWinInit( canvas );	Initialization of the canvas window.

      void screenSetCanvas( char *args );	String based control of window contents.

      void screenRenderCanvas( a, b, c );	Main rendering function. 

*****************************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include "xvFibSim_ui.h"

#include <math.h>

#include "plate.h"

/**** defines for canvas characteristics ****/

#define  WIN_SIZE 75

#define CANVAS_LABEL		"SCREEN"
#define CANVAS_LABEL_SIZE	6
#define CANVAS_LABEL_X		15
#define CANVAS_LABEL_Y		11

/**** put misc defines here. ****/

#define  SCREEN_CIR 28

#define NUM_SCREEN_SEGS 13

#define INACTIVE     0
#define ACTIVE       5

#define OPEN_SCREEN	0
#define CLOSE_SCREEN	1

/**** put defines for colors here. ****/

#define WHITE   0
#define BLACK   1
#define RED     2
#define TURQ    3
#define ORANGE  4

/**** local structures ****/

struct genericWinInfo
{

    Display *display;
    XID canvasXid;              /* stashed id of canvas for convenience */

    int canvasSizeX;            /* size of drawing canvas */
    int canvasSizeY;
    int windowSizeX;            /* size of visable window */
    int windowSizeY;
    int windowOffsetX;          /* Location of window on canvas */
    int windowOffsetY;

    GC GCwhite;                 /* pre-defined graphic contexts */
    GC GCblack;
    GC GCred;
    GC GCturq;
    GC GCmisc1;
    GC GCmisc2;

    double scaleFactor;         /* scale to get from hydra scale to canvas scale */

};

struct CurrentCalScreenInfo
{

    int status;                 /* see mode #defines above */
    double radius;              /* last radius drawn */
    GC gc;
    XPoint oldPoints[NUM_SCREEN_SEGS];
    XPoint newPoints[NUM_SCREEN_SEGS];
};

/********local globals*******/

static struct genericWinInfo TheWinInfo;

static struct CurrentCalScreenInfo CurrentCalScreen;

/********local prototypes*******/

/* these get renamed for each use of this file */

void screenRenderCanvas ();
void screenSetCanvas ();

/*****   initialization function  ******/

/*
   the following is the initialization for a generic window.
   it uses the basic GC structure defined in this file.
   The GC values here can be changed without hurting any other canvas
*/

void screenWinInit (canvas)
Xv_opaque *canvas;              /* main program supplies this so that we can ignore type */

{

    Cms cms;
    XFontStruct *font;
    XGCValues gc_val;

    static unsigned long *Colors;       /* color table */

    /* make sure the defines above match the colors here! */

    static Xv_singlecolor cms_colors[] = {
        {255, 255, 255},
        {0, 0, 0},
        {255, 0, 0},
        {0, 255, 255},
        {225, 175, 55},
        {1, 1, 1},
    };

    cms = xv_create (NULL, CMS,
                     CMS_SIZE,
                     (sizeof (cms_colors) / sizeof (Xv_singlecolor)),
                     CMS_TYPE, XV_STATIC_CMS, CMS_COLORS, cms_colors, NULL);

    TheWinInfo.display = (Display *) xv_get ((int) canvas, XV_DISPLAY);

    TheWinInfo.canvasXid =
        (XID) xv_get (canvas_paint_window ((int) canvas), XV_XID);

    xv_set ((int) canvas, WIN_CMS, cms, NULL);

    /* get the colormap from the canvas now that
       the cms has been installed */

    Colors = (unsigned long *) xv_get ((int) canvas, WIN_X_COLOR_INDICES);

    /* set the font */

    if (!(font = XLoadQueryFont (TheWinInfo.display, "fixed")))
    {
        (void) fputs ("Failure to load fixed font\n", stderr);
        exit (1);
    }

    gc_val.font = font->fid;

         /***** create the GCs *****/

    /* at first they're all the same. */

    TheWinInfo.GCwhite = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCblack = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCred = XCreateGC (TheWinInfo.display,
                                  TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCturq = XCreateGC (TheWinInfo.display,
                                   TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCmisc1 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    TheWinInfo.GCmisc2 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid, GCFont, &gc_val);

    /* now set the colors */

    XSetForeground (TheWinInfo.display, TheWinInfo.GCwhite, Colors[WHITE]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCblack, Colors[BLACK]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCred, Colors[RED]);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCturq, Colors[TURQ]);

         /***** custom GC's go here *****/

    /* gc to draw the cal screen edges */

    gc_val.function = GXxor;

    TheWinInfo.GCmisc1 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid,
                                    (GCFont | GCFunction), &gc_val);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCmisc1, Colors[RED]);

    XSetLineAttributes (TheWinInfo.display,
                        TheWinInfo.GCmisc1, 2, LineSolid, CapButt, JoinMiter);

    /* gc to draw the cal screen fill */

    gc_val.function = GXxor;

    TheWinInfo.GCmisc2 = XCreateGC (TheWinInfo.display,
                                    TheWinInfo.canvasXid,
                                    (GCFont | GCFunction), &gc_val);

    XSetForeground (TheWinInfo.display, TheWinInfo.GCmisc2, Colors[BLACK]);

    XSetLineAttributes (TheWinInfo.display,
                        TheWinInfo.GCmisc2, 2, LineSolid, CapButt, JoinMiter);

         /***** misc. initialization go here *****/

    TheWinInfo.scaleFactor = 1; /* historical, obsolete, leave it at one */
    TheWinInfo.canvasSizeX = WIN_SIZE;  /* size of drawing canvas */
    TheWinInfo.canvasSizeY = WIN_SIZE;

    calScreenInit (&CurrentCalScreen);

}

/* routine to operate canvas from a string arguemnt
   Customize this to suit your needs.*/

void screenSetCanvas (args)
char *args;

{
    if (strcmp (args, "in") == 0)
    {

        animateScreen (CLOSE_SCREEN);
    }

    if (strcmp (args, "out") == 0)
    {

        animateScreen (OPEN_SCREEN);
    }
}

/* the routine to draw stuff.  Args will need to be customized */

void screenRenderCanvas (position, flag)
int position;
int flag;                       /* new value or refresh */

{
    if (flag == 0)
    {
        XClearWindow (TheWinInfo.display, TheWinInfo.canvasXid);
        calScreenActivate (&CurrentCalScreen);
    }
    else
    {
        animateScreen (position);
    }

    XDrawString (TheWinInfo.display,
                 TheWinInfo.canvasXid,
                 TheWinInfo.GCblack,
                 CANVAS_LABEL_X, CANVAS_LABEL_Y,
                 CANVAS_LABEL, CANVAS_LABEL_SIZE);
}

calScreenInit (screenInfo)
struct CurrentCalScreenInfo *screenInfo;

{
    screenInfo->status = ACTIVE;
    screenInfo->gc = TheWinInfo.GCmisc1;

    computeScreenPoints (screenInfo->oldPoints, (double) SCREEN_CIR);
    computeScreenPoints (screenInfo->newPoints, (double) SCREEN_CIR);
}

calScreenActivate (screenInfo)
struct CurrentCalScreenInfo *screenInfo;

{

    /* screenInfo->status = ACTIVE; */

    /* this will turn the on */

    XFillPolygon (TheWinInfo.display,
                  TheWinInfo.canvasXid,
                  TheWinInfo.GCmisc2,
                  screenInfo->oldPoints,
                  (NUM_SCREEN_SEGS), Nonconvex, CoordModeOrigin);

    XDrawLines (TheWinInfo.display,
                TheWinInfo.canvasXid,
                screenInfo->gc,
                screenInfo->oldPoints, (NUM_SCREEN_SEGS), CoordModeOrigin);
}

#ifdef IGNORE
calScreenRefresh (screenInfo)
struct CurrentCalScreenInfo *screenInfo;

{
    static int currentStatus;

    if (screenInfo->status == ACTIVE)
    {

        /* this makes sure the bits are at the correct scale */

        computeScreenPoints (screenInfo->oldPoints, screenInfo->radius);

        /* this will turn the bits back on */

        XDrawLines (TheWinInfo.display,
                    TheWinInfo.canvasXid,
                    screenInfo->gc,
                    screenInfo->oldPoints,
                    (NUM_SCREEN_SEGS), CoordModeOrigin);
    }
}

#endif

/*
		animateScreen

   Show the screen moving from it's current position to it's
   new position which is specified in the argument.
*/

void animateScreen (mode)
int mode;

{
    double phi;
    double radius;

    if (mode == OPEN_SCREEN)
    {

        for (phi = 0.0; phi <= (M_PI / 2); phi += (M_PI / 150.0))
        {

            radius = SCREEN_CIR - (SCREEN_CIR * cos (phi));

            drawScreenAtRadius (&CurrentCalScreen, radius);
        }

        /* now erase the segments altogether */

        /* calScreenActivate( &CurrentCalScreen, INACTIVE ); */

    }
    else
    {                           /* mode is to close the screen */

        /* turn on the segments */

/*
      calScreenActivate( &CurrentCalScreen, ACTIVE );
*/

        for (phi = 0.0; phi <= (M_PI / 2); phi += (M_PI / 50.0))
        {

            radius = SCREEN_CIR * cos (phi);

            drawScreenAtRadius (&CurrentCalScreen, radius);

        }

        /* makes sure it's all the way closed */

        drawScreenAtRadius (&CurrentCalScreen, 0.0);
    }
}

/*
		drawScreenAtRadius

   Draw the screen as a series of triangles about the plate
   The argument describes how far from the center to place
   the tips of the screen segments
*/

void drawScreenAtRadius (screenInfo, radius)
struct CurrentCalScreenInfo *screenInfo;
double radius;

{

    int i;

    /* compute the newPoints */

    computeScreenPoints (screenInfo->newPoints, radius);

    /* be sure that it's active for drawing... */

/*
   if( screenInfo->status == INACTIVE )
      return;
*/

    /* draw the new points */

    XDrawLines (TheWinInfo.display,
                TheWinInfo.canvasXid,
                screenInfo->gc,
                screenInfo->newPoints, (NUM_SCREEN_SEGS), CoordModeOrigin);

    XFillPolygon (TheWinInfo.display,
                  TheWinInfo.canvasXid,
                  TheWinInfo.GCmisc2,
                  screenInfo->newPoints,
                  (NUM_SCREEN_SEGS), Nonconvex, CoordModeOrigin);

    /* erase the old points */

    XDrawLines (TheWinInfo.display,
                TheWinInfo.canvasXid,
                screenInfo->gc,
                screenInfo->oldPoints, (NUM_SCREEN_SEGS), CoordModeOrigin);

    XFillPolygon (TheWinInfo.display,
                  TheWinInfo.canvasXid,
                  TheWinInfo.GCmisc2,
                  screenInfo->oldPoints,
                  (NUM_SCREEN_SEGS), Nonconvex, CoordModeOrigin);

    /* save the radius for possible future refresh */

    screenInfo->radius = radius;

    /* make the new points the future old points */

    for (i = 0; i < NUM_SCREEN_SEGS; i++)
        screenInfo->oldPoints[i] = screenInfo->newPoints[i];
}

/* load up a set of vertices for the triangles */

computeScreenPoints (pointSet, radius)
XPoint pointSet[];
double radius;

{
    int i;
    double theta;

    double dx = TheWinInfo.canvasSizeX / 2;
    double dy = TheWinInfo.canvasSizeY / 2;

    for (i = 0; i < NUM_SCREEN_SEGS; i++)
    {

        theta = i * (M_PI / 6.0);

        if ((i % 2) == 0)
        {

            /* these are the points along the plate */

            pointSet[i].x = (int) (((double) SCREEN_CIR * cos (theta) *
                                    TheWinInfo.scaleFactor) + dx + 0.5);

            pointSet[i].y = (int) (-((double) SCREEN_CIR * sin (theta) *
                                     TheWinInfo.scaleFactor) + dy + 0.5);
        }
        else
        {                       /* i % 2 != 0 */

            /* these are the tips of the screen segments */

            pointSet[i].x = (int) ((radius * cos (theta) *
                                    TheWinInfo.scaleFactor) + dx + 0.5);

            pointSet[i].y = (int) (-(radius * sin (theta) *
                                     TheWinInfo.scaleFactor) + dy + 0.5);
        }
    }
}
