
#include <stdio.h>
#include <strings.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gutil.h>
#include <math.h>
#include <signal.h>
#include <sys/socket.h>
#include "xvFibSim_ui.h"

#include "buttonHooks.h"

#include "miscNumbers.h"
#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "kaboom.h"
#include "translate.h"
#include "archive.h"
#include "log.h"
#include "xFibSim.h"
#include "nesslib.h"
#include "nessIO.h"
#include "coordConvert.h"
#include "concen.h"
#include "xref.h"
#include "findIt.h"
#include "statusIO.h"
#include "fibwindow.h"
#include "inits.h"
#include "idIt.h"

/* #define FIBFILENAME "SimLocations" */

#define DATAFILE        "DATAFILE"      /* database file for sys config. */
#define SIMFILE         "SIMFILE"       /* database file for sim. config. */
#define ECCEN_FILE  "concentricities"
#define STATUSFILE      "STATUSFILE"    /* database file for sim. config. */
#define SIMSTATUS       "SIMSTATUS"     /* database file for sim. config. */

/****************   local functions   ****************/

void initialize ();             /* misc initialization */
void execStream ();

void screenRefresh ();          /* redraw the screen and all buttons */
void update ();                 /* (eventually) smart picture updater */
void restore ();                /* force a cold load of the database */
void draw ();                   /* draw a specific button */
void loadTargets ();            /* load a star file containing targets */
void target ();                 /* draw a target */
void targMode ();               /* flag for whether to draw targets */
void setBlock ();               /* load the cross reference structure */
void setStats ();               /* set the statistics fields */
void setCable ();               /* set the cable name field */
void showButton ();             /* highlight the specified button */
void showObject ();             /* highlight the specified object */
void plateSetCanvas ();         /* plate location graphics */
void rotatorSetCanvas ();       /* rotator position graphics */
void screenSetCanvas ();        /* rotator position graphics */
void ledSetCanvas ();           /* led status graphics */
void complampSetCanvas ();      /* led status graphics */
void gripperSetCanvas ();       /* led status graphics */
void testFunction ();           /* used for programming tests */
void sync ();                   /* force an update */
void quit ();                   /* exit the program */

int checkForUpdate ();          /* see if the archive file has changed */

#ifdef IGNORE
void signalUpdate ();           /*  SIGUSR1 handler.  Updates window */
void signalAlarm ();            /*  SIGALRM handler.  Checks database */
#endif

/****************       globals      ****************/

FILE *positionArchive = NULL;   /*  file with current locations */
char positionArchiveName[128];  /*  name of file with current locations */

FILE *CurrentLog;               /* log of user input and system response */

struct fiber_dbase FiBase[MAXFIBER];

struct starinfo *Stars[MAXSTARS];
struct exposinfo Exposure;      /* exposure information */

long lastUpdate;                /* last time archive file was updated */

struct plate_coords screenLabelPos = { LABEL_X, LABEL_Y };

char *screenLabel;

int BackgroundFlag;

int faith = TRUE;

int noGWC = FALSE;

char aString[128];

int NextMove;                   /* archive.c expects this */

/* the following are used to translate the targets into plate coords. */

int dummyFlag;

double dummy;

struct templateItem statusList[] = {

    /*    keyname         type     where it goes   pdflt  deflt?  req?  found? */

    {"HOUR ANGLE", DOUBLE, (char *) &dummy, NULL, FALSE, TRUE, TRUE},
    {"RIGHT ASCENSION", DOUBLE, (char *) &dummy, NULL, FALSE, TRUE, TRUE},
    {"DECLINATION", DOUBLE, (char *) &dummy, NULL, FALSE, TRUE, TRUE},
    {"AMBIENT TEMP", DOUBLE, (char *) &dummy, NULL, FALSE, TRUE, TRUE},
    {"ROTATION ANGLE", DOUBLE, (char *) &RotationAngle, NULL, FALSE, TRUE,
     TRUE},

    /* keyname           type     where it goes             deflt?  req?   found? */
    /* flags */
    {"ROTMODE", INT, (char *) &Rotflag, NULL, FALSE, TRUE, TRUE},
    {"CONCENMODE", INT, (char *) &EccenFlag, NULL, FALSE, TRUE, TRUE},
    {"THERMMODE", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"WARPMODE", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"FLEXMODE", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"GRIPMODE", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"SCREENMODE", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"CABLE", CHAR, &CurrentCable, NULL, FALSE, TRUE, TRUE},
    {"INGRIPPER", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"LASTX", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"LASTY", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"XOFFSET", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"YOFFSET", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},
    {"NEXTMOVE", INT, (char *) &dummyFlag, NULL, FALSE, TRUE, TRUE},

    /* the next line is required and used to mark the end of the structure */
    {"", END, NULL, NULL, FALSE, FALSE, FALSE}
};

/* this structure is used to map command names to function pointers */

struct Acommand
{
    char *name;
    void (*routine) ();
};

struct Acommand FunctTable[] = {

    {"gfoo", quit},
    {"refresh", screenRefresh},
    {"update", update},
    {"restore", restore},
    {"draw", draw},
    {"field", loadTargets},
    {"targets", targMode},
    {"sync", sync},
    {"setblock", setBlock},
    {"setstats", setStats},
    {"setcable", setCable},
    {"showfib", showButton},
    {"showobj", showObject},
    {"showscreen", screenSetCanvas},
    {"showplate", plateSetCanvas},
    {"showrot", rotatorSetCanvas},
    {"showled", ledSetCanvas},
    {"showcomp", complampSetCanvas},
    {"showgrip", gripperSetCanvas},
    {"pushled", ledButtonSet},
    {"pushcomp", complampButtonSet},
    {"pushgrip", gripperButtonSet},
    {"quit", quit}
};

#ifdef IGNORE

struct sigvec oldVec;           /* stash for sigvec call */

struct itimerval scanTimer = {
    {0, 500000},
    {0, 500000}
};

struct itimerval dummy;

struct sigvec User1Vec = {
    signalUpdate,
    0,
    SV_INTERRUPT
};

struct sigvec AlarmVec = {
    signalAlarm,
    0,
    0
};

#endif

/**** globals ****/

FILE *stuffIn;

extern char *getenv ();

int BlockedByParent = TRUE;

/*************************************************************/

/*                                                           */

/*                     dBaseInit                             */

/*                                                           */

/*************************************************************/

dBaseInit (fileName)
char *fileName;

{
    char *buttonFile;

    FILE *concenFile;

    int theSocket;
    struct sockaddr name;

    fd_set selectMask;
    fd_set blankMask;
    fd_set readableFDs;

    int selectWidth = ulimit (4, 0);    /* the man gives the magic #s */

    int syncFlag = 0;           /* mode flags */
    int manualFlag = 0;

    int i;                      /* counter */

    /* fputs( "Initializing database\n", stderr ); */

    strcpy (positionArchiveName, fileName);

   /*****************       initialization     ******************/

    screenLabel = positionArchiveName;

    positionArchive = openArchive (positionArchiveName, READ_ONLY);

    /* use the appropriate status file */

    if (strcmp (positionArchiveName, getenv (SIMFILE)) == 0)
    {
        if (LoadStatusFile (SIMSTATUS, statusList) != 0)
            fputs ("WARNING:  Graphics could not obtain simulator status\n",
                   stderr);
    }
    if (strcmp (positionArchiveName, getenv (DATAFILE)) == 0)
    {
        if (LoadStatusFile (STATUSFILE, statusList) != 0)
            fputs ("WARNING:  Graphics could not obtain system status\n",
                   stderr);
    }
    else
    {
    }

    /* load database for concentricity correction */

    concenFile = eccenOpen (ECCEN_FILE);

    if (concenFile != NULL)
        eccenLoad (concenFile, FiBase);
    else
    {
        fputs ("ERROR:  Graphics couldn't open concentricities file.\n",
               stderr);
        exit (1);
    }

    initDbase ();

    restoreFromArchive (positionArchive, FiBase, ALL);

    theSocket = socket (PF_UNIX, SOCK_STREAM, 0);

    if (theSocket < 0)
    {
        perror (NULL);
    }

    name.sa_family = AF_UNIX;

    strcpy (name.sa_data, SOCK_NAME);

    if (connect (theSocket, &name, sizeof (name.sa_data)) != 0)
        perror ("connect:");

    stuffIn = fdopen (theSocket, "r+");

    setbuf (stuffIn, 0);

    if (stuffIn == NULL)
        fputs ("couldn't open input", stderr);

}

/*************************************************************
		local functions
*************************************************************/

/*
		initialize

   Perform the initialization of various structures/variables.
*/

void initialize ()
{
}

/*
		execStream

   process commands from the specified input stream until
   EOF is encountered.  Commands are delimited by a newline.
   fields within commands are delimited by spaces.  The first
   field is the command itself.  A search is made of the
   structure of known commands, and if a match is found,
   execution transfers to the corresponding function for
   processing of the arguments (if any).
*/

void execStream (cmdStream)
FILE *cmdStream;

{
    int noExit = 1;             /* flag not to exit */
    int i;

    char buff[256];             /* command line */
    char command[64];           /* command word */
    char args[256];             /* command args */

    bzero (buff, sizeof (buff));
    bzero (command, sizeof (command));
    bzero (args, sizeof (args));

    /* try to get a line, and flag loop to break if EOF */

    noExit = (fgets (buff, sizeof (buff), cmdStream) != NULL);

    buff[strlen (buff) - 1] = '\0';

    /*  ignore empty lines */

    if (strlen (buff) != 0)
    {

        /* fprintf( stderr, " graphics command: %s \n", buff ); */

        /* break line into two pieces, the command and the arguments */

        sscanf (buff, "%s %128c", command, args);

        /* if no prompt, do show the input string to show the progress
           of scripts that are executing */

        /* Loop thru command list looking for match and call the 
           appropriate functions from the table */

        for (i = 0; i < (sizeof (FunctTable) / sizeof (FunctTable[0])); i++)
            if (strcmp (command, FunctTable[i].name) == 0)
            {

                FunctTable[i].routine (args);
            }
    }
}

/*
		screenRefresh
   
   Re-draw the entire graphics window.

*/

void screenRefresh (args)
char *args;

{
    refresh ();
}

/*
		update

   Find out which fibers have moved and update only those that have 
   changed.

*/

 /*ARGSUSED*/ void update (args)
char *args;

{
    int lineStatus = BAD_LINE;

    /*
       turn of the signals so that we don't 
       try to read the file while it is closed.
     */

    while (lineStatus == BAD_LINE)
    {

        closeArchive (positionArchive);

        positionArchive = openArchive (positionArchiveName, READ_ONLY);

        lineStatus = restoreFromArchive (positionArchive, FiBase, RECENT);
    }

    screenUpdate ();
}

void updateOnNotify ()
{
    int lineStatus = BAD_LINE;

    /* fputs( "Notifier called function for update \n", stderr ); */

    while (lineStatus == BAD_LINE)
    {

        closeArchive (positionArchive);

        positionArchive = openArchive (positionArchiveName, READ_ONLY);

        lineStatus = restoreFromArchive (positionArchive, FiBase, RECENT);
    }

    screenRefresh ((char *) NULL);
}

/*
		restore

   Restore the fiber database from the position file.

*/

 /*ARGSUSED*/ void restore (args)
char *args;

{
    int oldSigMask;

    int lineStatus = BAD_LINE;

    /*
       turn of the signals so that we don't 
       try to read the file while it is closed.
     */

    oldSigMask = sigblock (SIGALRM | SIGUSR1);

    closeArchive (positionArchive);

    positionArchive = openArchive (positionArchiveName, READ_ONLY);

    lineStatus = restoreFromArchive (positionArchive, FiBase, ALL);

    screenRefresh ((char *) NULL);

    /* turn the signals back on */

    sigsetmask (oldSigMask);
}

/*
		draw

   Draw the specified button with the specified coordinates.

*/

 /*ARGSUSED*/ void draw (args)
char *args;

{
    int button;
    struct plate_coords target;
    struct fiber_dbase *fib;

    if ((sscanf (args, "%d %ld %ld",
                 &button,
                 &target.x,
                 &target.y) >= 3) && (button >= 0) && (button < MAXFIBER))
    {

        drawFiber (button, ERASE);

        fib = &FiBase[button];
    }
    else
    {
        fputs ("Bad button id", stderr);
    }
}

/*
		setblock

   Set the cross reference structure from the command line.

*/

 /*ARGSUSED*/ void setBlock (args)
char *args;

{
    if (sscanf (args, "%d", &BlockedByParent) < 1)
    {
        fprintf (stderr, "WARNING: bad argument to setblock \"%s\"\n", args);
        BlockedByParent = FALSE;
    }

    /* fprintf( stderr, "setblock %d\n", BlockedByParent ); */
}

/*
		target

   Draw a target with the specified coordinates.

*/

 /*ARGSUSED*/ void target (args)
char *args;

{
}

/*
		sync

   Turn polling of archive file status on or off.

*/

 /*ARGSUSED*/ void sync (args)
char *args;

{
}

void targMode (args)
char *args;

{
    char cleanArg[32];

    sscanf (args, "%s", cleanArg);

    if (strcmp (cleanArg, "on") == 0)
        BackgroundFlag = TRUE;
    else
    {
        if (strcmp (cleanArg, "off") == 0)
            BackgroundFlag = FALSE;
        else
            fputs ("Do WHAT with the targets???...", stderr);
    }
    update ((char *) NULL);
}

/*
		loadTargets

   Load in the file with the target coordinates.
*/

void loadTargets (args)
char *args;

{
    FILE *starFile;
    char *starPath;
    char starFileName[64];
    char message[16];

    struct plate_coords target;

    int i;

    /* clean up any old file */

    for (i = 0; i < MAXSTARS; i++)
    {
        if (Stars[i] != NULL)
        {
            free ((char *) Stars[i]);
            Stars[i] = NULL;
        }
    }

    sprintf (message, "unknown");
    CurrentObject.id = NO_FIBER;
    CurrentObject.status = INACTIVE;
    xv_set (XvFibSim_window1->objectField, PANEL_VALUE, message, NULL);

    starFile = openCoordFile (args, "r");

    if (starFile == 0)
    {
        fputs ("Hydra data file not found.\n", stderr);
    }

    else
    {
        if (LoadHydraFile (starFile, Stars, &Exposure) != FILE_OK)
        {

            fprintf (stderr,
                     "ERROR: File \"%s\"\nNot readable as Hydra file\n",
                     starFileName);

            /* clean it up */
            for (i = 0; i < MAXSTARS; i++)
            {
                if (Stars[i] != NULL)
                {
                    free ((char *) Stars[i]);
                    Stars[i] = NULL;
                }
            }
        }
        else
        {

            /* fputs( "Targets loaded OK\n", stderr ); */

            /* update information at top of window */

            updateLabels (Stars, &Exposure, args);

            userRefresh ();

        }

        fclose (starFile);
    }
}

void setStats (args)
char *args;

{

    char item1[32];
    char item2[32];
    char item3[32];

    sscanf (args, "%s %s %s", item1, item2, item3);

    xv_set (XvFibSim_window1->objsAssignedItem, PANEL_VALUE, item1, NULL);
    xv_set (XvFibSim_window1->skysAssignedItem, PANEL_VALUE, item2, NULL);
    xv_set (XvFibSim_window1->fopsAssignedItem, PANEL_VALUE, item3, NULL);
}

/* 
		setCable

   Word to set the value of the cable text field
*/

void setCable (args)
char *args;

{
    char cableName[16];

    /* cleans up spaces */

    sscanf (args, "%s", cableName);

    CurrentCable = getCableIdentifier (cableName);

    /* fprintf( stderr ,"%s, %c\n", cableName, CurrentCable ); */

    if (CurrentCable != NULL)
    {
        xv_set (XvFibSim_window1->hydraCableField, PANEL_VALUE,
                getNameForCable (CurrentCable), NULL);
    }
    else
    {
        xv_set (XvFibSim_window1->hydraCableField, PANEL_VALUE, "??", NULL);
    }

    refresh ();
}

/*
		showButton

   Highlight the specified button as though it had been clicked
   with the mouse.  Cross-reference it if it is assigned to an
   object
*/

void showButton (args)
char *args;

{
    int button;
    int i;

    if (sscanf (args, "%d", &button) > 0)
    {

        /* now draw it, leaving it on */

        if (BlockedByParent)
            selectFiber (button, SELECT, NO_XREF);      /* draws it */
        else
            selectFiber (button, SELECT, XREF); /* draws it */
    }
    else
    {
        fputs ("graphics: button id out of range\n", stderr);
    }

}

/*
		showObject

   Highlight the specified object as though it had been clicked
   with the mouse.  Cross-reference it if it is assigned to an
   button
*/

void showObject (args)
char *args;

{
    int object;

    if ((sscanf (args, "%d", &object) > 0) &&
        (object > 0) && (object <= MAXSTARS) && (Stars[object] != NULL))
    {

        if (BlockedByParent)
            selectObject (object, SELECT, NO_XREF);
        else
            selectObject (object, SELECT, XREF);
    }
}

/*
		quit

   Exit the program.

*/

 /*ARGSUSED*/ void quit (args)
char *args;

{
    fputs ("Grapics exiting\n", stderr);
    exit (0);
}

/*		
		checkForUpdate

   See if the specified file has been updated since the
   specified time.

*/

int checkForUpdate (lastUpdate, buttonFile)
long *lastUpdate;
FILE *buttonFile;

{
    struct stat statBuf;

    if (fstat (fileno (buttonFile), &statBuf))
        fputs ("Couldn't get files stats for archive", stderr);;

    if (*lastUpdate != (long) statBuf.st_mtime)
    {

        *lastUpdate = (long) statBuf.st_mtime;

        return (TRUE);

    }
    else
        return (FALSE);

}

#ifdef IGNORE

/*
		 signalUpdate

   Handler for SIGUSR1.  Update the database and then re-draw
   the graphics window.  

*/

void signalUpdate (sig, code, scp)      /*  SIGUSR1 handler.  Updates window */
 int sig, code;
struct sigcontext *scp;
{
    /* sprintf( stdout, "Update Signal Received\n" ); */

    update ((char *) NULL);
}

/*
		 signalAlarm

   Handler for SIGALRM.  Check the database for update on
   a timer expire.

*/

void signalAlarm (sig, code, scp)       /*  SIGALRM handler.  Checks database */
 int sig, code;
struct sigcontext *scp;
{
    /* fputs( "Timer Signal Received", stderr ); */

    if (checkForUpdate (&lastUpdate, positionArchive))
    {
        update ((char *) NULL);
    }
}

#endif

/*
      labtyd

   Life's A Bitch, Then You Die--labtyd

*/

void labtyd (error, msg)
int error;
char *msg;
{
    (void) fprintf (stderr, "%s\n", msg);
    exit (error);

}
