
/*                      xvmain.c

 *  Module name:
      xvmain.c

 *  Function:
      This file is the xview-based main for test programs.

 *  Description:

 *  Language:
      C

 *  Support: Kim Gillies, NOAO

 *  History:
      30-Jun-1994 - KG - Added wflman header.

 */
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/defaults.h>
#include <stdlib.h>
#include <tcl.h>

#define PROGRAMPROMPT "hydra>"

char *initCmd =
    "if [file exists [info library]/init.tcl] {source [info library]/init.tcl}";

/* Tcp variables */
static int quitFlag = FALSE;
static char *fileName = NULL;
static Notify_client inclient;  /* XView client for cli */

static Tcl_Interp *interp;      /* Interpreter for this application. */
char *tcl_RcFileName = NULL;    /* Name of a user-specific startup script
                                 * to source if the application is being run
                                 * interactively (e.g. "~/.wishrc").  Set
                                 * by Tcl_AppInit.  NULL means don't source
                                 * anything ever. */
static Tcl_DString command;     /* Used to assemble lines of terminal input
                                 * into Tcl commands. */
static int tty;                 /* Non-zero means standard input is a
                                 * terminal-like device.  Zero means it's
                                 * a file. */
static char errorExitCmd[] = "exit 1";

/*
 * Forward declarations for procedures defined later in this file:
 */
static void Prompt (Tcl_Interp * interp, int partial);
static Notify_value StdinProc (Notify_client cl, int fd);

/* In file function prototypes */
static int cmdEcho (ClientData * clientData, Tcl_Interp * interp,
                    int argc, char *argv[]);
static int cmdQuit (ClientData * clientData, Tcl_Interp * interp,
                    int argc, char *argv[]);

/*
 * Special main for tcl/xview WIYN interface.
 */
int gwcMain ()
{
    char *msg;

    /*
     * Initialize XView.
     */

    /* xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL); */

    /* Create a notifier client */
    inclient = (Notify_client) xv_unique_key ();

    interp = Tcl_CreateInterp ();

#ifdef TCL_MEM_DEBUG
    Tcl_InitMemory (interp);
#endif

    if (tclStockInit (interp) != TCL_OK)
    {
        fprintf (stderr, "Could not initialize interpreter.\n");
        exit (-1);
    }

#ifdef IGNORE
    /*
     * Turn control over to XView.
     */
    /*notify_start(); */

    msg = Tcl_GetVar (interp, "errorInfo", TCL_GLOBAL_ONLY);
    if (msg == NULL)
    {
        msg = interp->result;
    }

    Tcl_Eval (interp, errorExitCmd);
#endif

}

int tclStockInit (Tcl_Interp * interp)
{
    int result;

    /* Some standard tcl built-ins */
    Tcl_CreateCommand (interp, "echo", (Tcl_CmdProc *) cmdEcho,
                       (ClientData) "echo", (Tcl_CmdDeleteProc *) NULL);
    Tcl_CreateCommand (interp, "quit", (Tcl_CmdProc *) cmdQuit,
                       (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);

    /* Source the tcl library */
    /*result = Tcl_Eval(interp, initCmd);
       if (result != TCL_OK) {
       printf("%s\n", interp->result);
       exit(1);
       } */

    tty = 0;                    /*isatty(0); */
    Tcl_SetVar (interp, "tcl_interactive",
                ((fileName == NULL) && tty) ? "1" : "0", TCL_GLOBAL_ONLY);

    /*
     * Invoke application-specific initialization.
     */
    if (Tcl_AppInit (interp) != TCL_OK)
    {
        fprintf (stderr, "Tcl_AppInit failed: %s\n", interp->result);
    }
    /* Setup the xview notifier - 0 is the fd */

    /*
       (void)notify_set_input_func(inclient, (Notify_func)StdinProc, 0);
     */

    if (tty)
    {
        Prompt (interp, 0);
    }

    return TCL_OK;
}

static void Prompt (Tcl_Interp * interp, int partial)
{
    char *promptCmd;
    int code;

    promptCmd = Tcl_GetVar (interp,
                            partial ? "tcl_prompt2" : "tcl_prompt1",
                            TCL_GLOBAL_ONLY);
    if (promptCmd == NULL)
    {
      defaultPrompt:
        if (!partial)
        {
            fputs ("% ", stdout);
        }
    }
    else
    {
        code = Tcl_Eval (interp, promptCmd);
        if (code != TCL_OK)
        {
            Tcl_AddErrorInfo (interp, "\n    (script that generates prompt)");
            fprintf (stderr, "%s\n", interp->result);
            goto defaultPrompt;
        }
    }
    fflush (stdout);
    Tcl_DStringInit (&command);
}

/* XView support for a command line aside the GUI */
 /*ARGSUSED*/ static Notify_value StdinProc (Notify_client cl, int fd)
{
#define BUFFER_SIZE 4000
    char input[BUFFER_SIZE + 1];
    static int gotPartial = 0;
    char *cmd;
    int code = TCL_OK;
    int count;

    count = read (fd, input, BUFFER_SIZE);
    if (count <= 0)
    {
        if (!gotPartial)
        {
            if (tty)
            {
                Tcl_Eval (interp, "exit");
                exit (1);
            }
            else
            {
                (void) notify_set_input_func (inclient, NOTIFY_FUNC_NULL, 0);
                printf ("Error: the other part!\n");
            }
            /* Not sure about this... */
            return NOTIFY_DONE;
        }
        else
        {
            count = 0;
        }
    }
    cmd = Tcl_DStringAppend (&command, input, count);
    if (count != 0)
    {
        if ((input[count - 1] != '\n') && (input[count - 1] != ';'))
        {
            gotPartial = 1;
            goto prompt;
        }
        if (!Tcl_CommandComplete (cmd))
        {
            gotPartial = 1;
            goto prompt;
        }
    }

    gotPartial = 0;
    /*
     * Disable the stdin file handler while evaluating the command;
     * otherwise if the command re-enters the event loop we might
     * process commands from stdin before the current command is
     * finished.  Among other things, this will trash the text of the
     * command being evaluated.
     */
    (void) notify_set_input_func (inclient, NOTIFY_FUNC_NULL, 0);
    code = Tcl_Eval (interp, cmd);
    (void) notify_set_input_func (inclient, (Notify_func) StdinProc, 0);
    Tcl_DStringFree (&command);
    if (*interp->result != 0)
    {
        if ((code != TCL_OK) || (tty))
        {
            printf ("%s\n", interp->result);
        }
    }

    /* Output the prompt */
  prompt:
    if (tty)
    {
        Prompt (interp, gotPartial);
    }

    return NOTIFY_DONE;
}

 /*ARGSUSED*/
    static int cmdEcho (ClientData * clientData, Tcl_Interp * interp,
                        int argc, char *argv[])
{
    int i;

    for (i = 1;; i++)
    {
        if (argv[i] == NULL)
        {
            if (i != argc)
            {
              echoError:
                sprintf (interp->result,
                         "argument list wasn't properly NULL-terminated in \"%s\" command",
                         argv[0]);
            }
            break;
        }
        if (i >= argc)
        {
            goto echoError;
        }
        fputs (argv[i], stdout);
        if (i < (argc - 1))
        {
            printf (" ");
        }
    }
    printf ("\n");
    return TCL_OK;
}

/* A formal quit command */
 /*ARGSUSED*/
    static int cmdQuit (ClientData * clientData, Tcl_Interp * interp,
                        int argc, char *argv[])
{
    if (argc != 1)
    {
        Tcl_AppendResult (interp, "wrong # args: should be \"", argv[0],
                          (char *) NULL);
        return TCL_ERROR;
    }
    quitFlag = 1;
    return TCL_OK;
}
