
/*
		xref.h

   Functions available in xref.c
*/

#define ECCENTRIC_FILE 	"concentricities"

#define FIBER_KNOWN  1
#define OBJECT_KNOWN 2
#define SLIT_KNOWN   3
#define FOP_KNOWN    4
#define ACQ_KNOWN    5

#define NONE_FOUND   0
#define ONE_FOUND    1
#define MULTI_FOUND  2

#define FIRST_ACQ   	 801
#define FIRST_ACQ_CHAR   'I'

#define FIRST_FOP   	 901
#define FIRST_FOP_CHAR   'A'

#define PROBE 	 	 "P"

#ifndef RED
#define RED 	1               /* cable ordinals */
#define BLUE    2
#define AAAA    3
#define BBBB    4
#define CCCC    5
#define DDDD    6
#define EEEE    7
#define FFFF    8
#endif

#define RED_OFFSET   100
#define BLUE_OFFSET  200
#define AAAA_OFFSET  300
#define BBBB_OFFSET  400
#define CCCC_OFFSET  500
#define DDDD_OFFSET  600
#define EEEE_OFFSET  700
#define FFFF_OFFSET  800

#define INTACT  0
#define BROKEN  1

#define OBJ_DEAD_ZONE 9

#define VOIDENTRY   -1

#define WRONG_FOOT  -2

struct XrefItem
{

    int fiber;
    int object;
    int slit;
    char cable;
};

typedef struct XrefItem xItem;

extern int eccenFlag;           /*  Flag to apply eccentricity correction */

int fiberBroken (int id);
char slitToProbeID (int id);

void xRefClear (fibDB fibers[]);
int xRefFiber (int theFiber, xItem * theList, aStar * stars[],
               fibDB fibers[]);
int xRefObject (int theObject, xItem * theList, aStar * stars[],
                fibDB fibers[]);
int xRefSlit (int theSlit, xItem * theList, aStar * stars[], fibDB fibers[]);
int xRefProbe (char theProbe, xItem * theList, aStar * stars[],
               fibDB fibers[]);

int fiberAtObject (int fiber, double tolerance);

int userToInternalID (aStar * stars[], int targetID, int *internalID);

int fibToFop (fibDB * fib);
int fiberIsFop (int fiber);

int fiberIsAssignedFop (int fiber, aStar * stars[], fibDB fibers[]);

int fiberIsSky (int fiber, aStar * stars[], fibDB fibers[]);
int fiberIsAssignedSky (int fiber, aStar * stars[], fibDB fibers[]);
int fiberIsRandomSky (int fiber, aStar * stars[], fibDB fibers[]);

int fiberInCable (int fiber, char cableID);

int fiberLocked (int id);
int fiberParked (int id);
