#include <stdio.h>

#include "miscNumbers.h"
#include "miscMath.h"
#include "errors.h"
#include "motor.h"
#include "buttons.h"
#include "nesslib.h"
#include "nessIO.h"
#include "ness.h"
#include "cardinalBump.h"
#include "log.h"

#define NORTHTOKEN 'N'
#define SOUTHTOKEN 'S'
#define EASTTOKEN  'E'
#define WESTTOKEN  'W'
#define VOIDTOKEN  'V'

extern double FieldRotationalOffset;

double dAStoSteps (double arcSec);
extern void rotate (point *, double);
extern int offsetToXY (long int dx, long int dy, double vel, double accel);

/*
		cardinalBump
   
   Nudge the gripper by the specified amount in the specified
   direction, taking into account the cardinal direction
*/
void cardinalBump (char *args)
{
    int error = 0;
    int xyError = 0;

    double dn;

    struct aPoint theVector;

    char direction;

    /* break off the direction character and the magnitude of the move */

    if (sscanf (args, "%lf %c", &dn, &direction) >= 2)
    {

        /* set up the initial vector */

        switch (direction)
        {

            case 'n':
            case 'N':
                theVector.x = dAStoSteps (dn);
                theVector.y = 0;
                break;

            case 's':
            case 'S':
                theVector.x = -dAStoSteps (dn);
                theVector.y = 0;
                break;

            case 'e':
            case 'E':
                theVector.y = dAStoSteps (dn);
                theVector.x = 0;
                break;

            case 'w':
            case 'W':
                theVector.y = -dAStoSteps (dn);
                theVector.x = 0;
                break;

            default:
                error = TRUE;
                log_fputs ("Invalid direction\n", stderr);
                break;
        }

        /* convert from cardinal direction to plate x,y */

        rotate (&theVector, -(FieldRotationalOffset - (M_PI / 2)));

        /* handle cases where the field is north of zenith */

        if (Stars[0]->rawCoord.dec < KPNO_LAT)
        {
            theVector.y *= -1;
        }

        /* now try to move the stages */

        if (!error)
        {

            if ((xyError = offsetToXY ((long) theVector.x, (long) theVector.y,
                                       BUMP_VEL, BUMP_ACCEL)) != 0)
            {

                log_fputs ("ERROR: Can't bump beyond limits...\n", stderr);
            }
        }
    }

    else
    {
        log_fputs ("ERROR:  Bad arguments to 'bump'\n", stderr);
    }
}

double dAStoSteps (double arcSec)
{
    return (((arcSec / WIYN_SCALE) / 25.4) * MOTOR_RESOLUTION);
}
