
/**************************************************************************

		globals.c

   This file contains the declarations for all the global variables.

**************************************************************************/

#include <stdio.h>
#include <sys/time.h>
#include <setjmp.h>

#ifdef __CRUISE__
#include <ncurses.h>
#endif

#include "errors.h"
#include "death.h"
#include "miscMath.h"
#include "init.h"
#include "motor.h"
#include "flex.h"
#include "buttons.h"
#include "kaboom.h"
#include "translate.h"
#include "detrot.h"
#include "nesslib.h"
#include "coordConvert.h"
#include "concen.h"
#include "xref.h"
#include "archive.h"
#include "reports.h"
#include "statusIO.h"
#include "parser.h"
#include "fiborg.h"
#include "log.h"
#include "prompter.h"
#include "version.h"

#define PE PhysicalEnvironment
#define DE DefaultEnvironment

struct exposinfo Exposure;

struct starinfo *Stars[MAXSTARS];

#ifdef SIMULATE                 /*  for the simulator, force these flags true */
int direct = 1;                 /*  don't launch communications */
int faith = 1;                  /*  operate on faith WRT system location. */
int quiet = 1;                  /*  don't print motor level output */
#else /*  normal initial values */
int direct = 0;                 /*  send commands directly to stdout */
int faith = 0;                  /*  operate on faith WRT system location. Implied by */
int quiet = 0;                  /*  generate output for motors */
#endif

int NextMove = 0;               /*  Serial number of the move */

int simulate = 0;               /*  Don't launch fibcomm */

int graphic = 0;                /*  flag to launch graphics */

int verbose = 0;                /*  show all output on user terminal. */

int remote = 1;                 /*  flag to use downloaded sequences */
int remoteTemp = 1;             /*  Used by "unstow" and labtyd to restore status */

int progeny = 0;                /*  flag that we are someone else's child */

int supress = 0;                /*  flag to suppress prompts */

int noGWC = TRUE;               /*  flag to bypass GWC calls */

int ForceOn = TRUE;
int ForceOff = FALSE;

FILE *responses = NULL;         /*  input stream for responses from indexers */

FILE *positionArchive = NULL;   /*  file for archiving each move */

FILE *graphCmds;

FILE *nullFile;

#ifdef __CRUISE__
WINDOW *win;                    /* for curses */
#endif

struct axis_characteristics axis[4];

struct itimerval masterTimer;

int NoFieldLoaded = 1;          /* flag to indicate that no field has been loaded.
                                   used by "saveit" to prevent the generation of
                                   a field with no header, by forcing a prompt.
                                   Cleared after the first setup or load */

int InCurses = 0;               /* flag to let other routines know to use curses
                                   string functions and not just printf's or puts's */

int GripPos;                    /* Current Gripper status */

int MoveInProgress;             /* flag to error handling */

int Configuring;                /* flag to handle undoable moves differently
                                   and to do special abort on ctrl-c in configure  */

int currentButton = VOIDBUTTON; /* button we are positioned over */

int rememberedButton = VOIDBUTTON;      /* button we are going to tweak */

int commPid;                    /* process ID for communication program */

int graphPid;                   /* process ID for the graphics program */

int RestrictedMoves;            /* Force additional error checking */

int Sloppiness = 0;             /* how far off a button can be and still count
                                   as assigned */

/* double Theta = 0;*//*  Global for coord system rotation */

/* int Rotflag = 0;*//*  Flag to rotate input coords by global "Theta" */

double Hangle = 0;              /*  Global for flexure correction */

int thermFlag = 0;              /*  Flag to apply flexure correction function */

int warpFlag = 0;               /*  Flag to apply flexure correction function */

int flexFlag = 0;               /*  Flag to apply flexure correction function */

int PromptHw = 0;               /* makes simulator ask gripper/rack status */

jmp_buf Env;                    /*  environment for error recovery */

jmp_buf StreamEnv;              /*  environment for error recovery to stream */

struct plate_coords LastXY;     /* saved when can't drop button, void otherwise */

struct plate_coords OffsetXY;   /* Offset from home to plate center */

struct rotStats RotData;

struct envData PhysicalEnvironment = {
    0.0,                        /* RA */
    32.0,                       /* DEC */
    0.0,                        /* ha */
    38.0,                       /* temp */
    0.0,                        /* not used */
    0.0                         /* not used */
};

struct envData DefaultEnvironment = {
    0.0,                        /* RA */
    32.0,                       /* DEC */
    0.0,                        /* ha */
    38.0,                       /* temp */
    0.0,                        /* not used */
    0.0                         /* not used */
};

struct templateItem statusList[] = {

    /* note there may be other programs with a structure that must match this one */

    /*    keyname         type     where it goes   loc. of dflt.       deflt? req?  found? */

    /* numeric values */
    {"HOUR ANGLE", DOUBLE, (char *) &(PE.ha), NULL, FALSE, TRUE, TRUE},
    {"RIGHT ASCENSION", DOUBLE, (char *) &(PE.ra), NULL, FALSE, TRUE, TRUE},
    {"DECLINATION", DOUBLE, (char *) &(PE.dec), NULL, FALSE, TRUE, TRUE},
    {"AMBIENT TEMP", DOUBLE, (char *) &(PE.temp), NULL, FALSE, TRUE, TRUE},

    {"ROTATION ANGLE", DOUBLE, (char *) &(RotData.aveAngle),
     NULL, FALSE, TRUE, TRUE},

    /* keyname           type     where it goes                      deflt?  req?   found? */
    /* flags */
    {"ROTMODE", INT, (char *) &Rotflag, (char *) &ForceOn, FALSE, TRUE, TRUE},
    {"CONCENMODE", INT, (char *) &EccenFlag, (char *) &ForceOn, TRUE, TRUE,
     TRUE},
    {"THERMMODE", INT, (char *) &thermFlag, (char *) &ForceOn, TRUE, TRUE,
     TRUE},
    {"WARPMODE", INT, (char *) &warpFlag, (char *) &ForceOn, TRUE, TRUE,
     TRUE},
    {"FLEXMODE", INT, (char *) &flexFlag, (char *) &ForceOn, TRUE, TRUE,
     TRUE},
    {"GRIPMODE", INT, (char *) &GripperParked, NULL, FALSE, TRUE, TRUE},
    {"SCREENMODE", INT, (char *) &ScreenStatus, NULL, FALSE, TRUE, TRUE},
    {"CABLE", CHAR, &CurrentCable, NULL, FALSE, TRUE, TRUE},
    {"INGRIPPER", INT, (char *) &FiberInTransit.id, NULL, TRUE, TRUE, TRUE},
    {"LASTX", INT, (char *) &LastXY.x, NULL, FALSE, TRUE, TRUE},
    {"LASTY", INT, (char *) &LastXY.y, NULL, FALSE, TRUE, TRUE},
    {"NEXTMOVE", INT, (char *) &NextMove, NULL, FALSE, TRUE, TRUE},
    {"XOFFSET", INT, (char *) &OffsetXY.x, NULL, FALSE, TRUE, TRUE},
    {"YOFFSET", INT, (char *) &OffsetXY.y, NULL, FALSE, TRUE, TRUE},

    /* the next line is required and used to mark the end of the structure */
    {"", END, NULL, NULL, FALSE, FALSE, FALSE}
};

struct snapShot FiberInTransit = {
    NULL,
    0
};

struct snapShot FiberInTrouble = {
    NULL,
    0
};

char *ExecEnvItems[] = {        /* environment items with paths in them */
    "FIBFIELDS",
    "FIBSCRIPTS",
    NULL
};
