
/******************************************************************

		init.c

   System Initialization routines.
   Included functions are:

   *  load_limits()
   *  void createAxisList(axisList)

   ( * indicates that function is declared as an extern )

*******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>

#include "errors.h"
#include "death.h"
#include "miscMath.h"
#include "motor.h"
#include "init.h"
#include "fiborg.h"
#include "log.h"

/*
	Create an array with the keywords and order we expect
	to see when reading the limits file.
*/

struct keyrec fieldKeys[] = {

    {"UNIT_ID_X", INT, (double *) &axis[X_ord].unitID},

    {"MIN_ACCEL_X", DOUBLE, &axis[X_ord].accel.min},
    {"MAX_ACCEL_X", DOUBLE, &axis[X_ord].accel.max},
    {"DEFLT_ACCEL_X", DOUBLE, &axis[X_ord].accel.dflt},

    {"MIN_VEL_X", DOUBLE, &axis[X_ord].velocity.min},
    {"MAX_VEL_X", DOUBLE, &axis[X_ord].velocity.max},
    {"DEFLT_VEL_X", DOUBLE, &axis[X_ord].velocity.dflt},

    {"MIN_POS_X", LONG, (double *) &axis[X_ord].position.min},
    {"MAX_POS_X", LONG, (double *) &axis[X_ord].position.max},

    {"UNIT_ID_Y", INT, (double *) &axis[Y_ord].unitID},

    {"MIN_ACCEL_Y", DOUBLE, &axis[Y_ord].accel.min},
    {"MAX_ACCEL_Y", DOUBLE, &axis[Y_ord].accel.max},
    {"DEFLT_ACCEL_Y", DOUBLE, &axis[Y_ord].accel.dflt},

    {"MIN_VEL_Y", DOUBLE, &axis[Y_ord].velocity.min},
    {"MAX_VEL_Y", DOUBLE, &axis[Y_ord].velocity.max},
    {"DEFLT_VEL_Y", DOUBLE, &axis[Y_ord].velocity.dflt},

    {"MIN_POS_Y", LONG, (double *) &axis[Y_ord].position.min},
    {"MAX_POS_Y", LONG, (double *) &axis[Y_ord].position.max},

    {"UNIT_ID_Z", INT, (double *) &axis[Z_ord].unitID},

    {"MIN_ACCEL_Z", DOUBLE, &axis[Z_ord].accel.min},
    {"MAX_ACCEL_Z", DOUBLE, &axis[Z_ord].accel.max},
    {"DEFLT_ACCEL_Z", DOUBLE, &axis[Z_ord].accel.dflt},

    {"MIN_VEL_Z", DOUBLE, &axis[Z_ord].velocity.min},
    {"MAX_VEL_Z", DOUBLE, &axis[Z_ord].velocity.max},
    {"DEFLT_VEL_Z", DOUBLE, &axis[Z_ord].velocity.dflt},

    {"MIN_POS_Z", LONG, (double *) &axis[Z_ord].position.min},
    {"MAX_POS_Z", LONG, (double *) &axis[Z_ord].position.max},

    {"UNIT_ID_STD", INT, (double *) &axis[STD_ord].unitID}

};

/*
		load_limits

   This function reads the file with the data for loading the
   position, velocity, and acceleration limits for each
   axis.

   All entries are checked for order of appearance, and a
   tally is kept of the number of entries found. An error in
   either item causes the entire fiborg program to terminate
   and an error message is printed.
*/

void load_limits (void)
{
    char log_buf[256];
    char *limitPath;
    char limitFileName[128];
    FILE *limits;

    int keycount = 0;
    char theline[128];
    char keyword[32];
    struct keyrec *KeyPtr;
    double theArg;

    theline[0] = '\n';

    if ((limitPath = getenv (FIBINITS)) != NULL)
        (void) strcpy (limitFileName, limitPath);

    (void) strcat (limitFileName, "/");
    (void) strcat (limitFileName, LIMIT_FILE);

    limits = fopen (limitFileName, "r");

    if (limits == NULL)
        labtyd (1, "Limit parameter file not found.");

    /* read all lines and process them */

    while (fgets (theline, sizeof (theline), limits) != NULL)
    {

        KeyPtr = &fieldKeys[keycount];

        if (theline[0] == '#' || theline[0] == '\n')
            continue;           /* ignore comments/blank lines */

        if (sscanf (theline, " %s %lf\n", keyword, &theArg) < 2)
        {
            sprintf (log_buf, "Bad limit file entry: %s\n", keyword);
            log_fputs (log_buf, stderr);
            labtyd (INIT_DATA, "Bad entry in limit file");
        }

        /* be sure we get the keyword we expect */

        if (strcmp (keyword, KeyPtr->theKey) == 0)
        {

            /*
               Take the argument and convert to the
               correct type  then save in limit
               data structure.
             */

            switch (KeyPtr->typeord)
            {

                case INT:
                    *(int *) KeyPtr->fldPtr = (int) theArg;
                    break;

                case LONG:
                    *(long *) KeyPtr->fldPtr = (long) theArg;
                    break;

                case FLOAT:
                    *(float *) KeyPtr->fldPtr = (float) theArg;
                    break;

                case DOUBLE:
                    *KeyPtr->fldPtr = theArg;
                    break;

            }

            keycount++;
        }

        else
        {                       /* Keywords didn't match */
            sprintf (log_buf, "Loading limits. Expected: %s ",
                     KeyPtr->theKey);
            log_fputs (log_buf, stderr);
            sprintf (log_buf, "Found %s instead.\n", keyword);
            log_fputs (log_buf, stderr);

            labtyd (INIT_DATA, "Bad keyword--Exiting.");
        }

    }

    fclose (limits);

    /*
       if( ( KeyPtr -(struct keyrec *)&fieldKeys[0] ) !=
       ( sizeof(fieldKeys) / sizeof( struct keyrec ) ) )
       labtyd( INIT_DATA, "Wrong number of keywords--Exiting.");
     */
}
