
/****************************************************************
		parser.c

   This file contains all of the routines that deal with
   operator input.  This input may originate from stdin,
   or from a file.  
***************************************************************/
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <signal.h>
#include <setjmp.h>
#include <tk.h>

#ifdef __CRUISE__
#include <ncurses.h>
extern WINDOW *win;             /* for curses */
#endif

#define LOCK_PATH "/tmp/hydraRunning"

#include "miscNumbers.h"
#include "miscMath.h"
#include "errors.h"
#include "death.h"
#include "drives.h"
#include "motor.h"
#include "detrot.h"
#include "flex.h"
#include "reports.h"
#include "stdBus.h"
#include "kaboom.h"
#include "buttons.h"
#include "archive.h"
#include "fiborg.h"
#include "nesslib.h"
#include "nessIO.h"
#include "ness.h"
#include "coordConvert.h"
#include "concen.h"
#include "xref.h"
#include "statusIO.h"
#include "prompter.h"
#include "setup.h"
#include "cardinalBump.h"
#include "parser.h"
#include "findIt.h"
#include "log.h"
#include "death.h"
#include "restrictions.h"
#include "version.h"
#include "tkInterface.h"
#include "proto.h"

#define BADBUTT "ERROR: Missing or bad button id.\n"
#define BADARG  "ERROR: Missing or bad argument.\n"
#define NORTHTOKEN 'N'
#define SOUTHTOKEN 'S'
#define EASTTOKEN  'E'
#define WESTTOKEN  'W'
#define VOIDTOKEN  'V'

#define DEFAULT_VIEW_TYPE "fiber"

#define UP_ORD    2             /* ordinals for cursor keys */
#define RIGHT_ORD 3
#define DOWN_ORD  0
#define LEFT_ORD  1
#define VOID_ORD  4

#define BIGNUDGE    150
#define SMALLNUDGE  3

#define ESCAPE '\033'

#define NOTOKEN         0       /* tokens for arguments to std bus stuff */
#define TOK_LITERAL     1       /* tokens for arguments to std bus stuff */
#define TOK_ON        101
#define TOK_OFF       102
#define TOK_STATUS    103
#define TOK_TEST      104
#define TOK_FLAT      105
#define TOK_WARP      106
#define TOK_ILLUM     107
#define TOK_IN        108
#define TOK_OUT       109
#define TOK_INDEXERS  110
#define TOK_DRIVES    111
#define TOK_CAL       112
#define TOK_VAC       113
#define TOK_VALVE     114
#define TOK_INIT      115

#define TOK_FIELD	201
#define TOK_FLATS	202
#define TOK_COMPS	203
#define TOK_QUARTZ	204
#define TOK_OBSERVE	205

#define TOK_TRACKING	301
#define TOK_POSITION	302
#define TOK_OFFSET	303
#define TOK_ADJUST	304

extern int InCurses;
static char log_buf[256];

/***************TEMPORARY************/
int objectXref[500];

char *execEnvItems[] = {        /* environment items with paths in them */
    "FIBSCRIPTS",
    "FIBFIELDS",
    NULL
};

char currentCommand[128];

/* this structure is used to map command names to function pointers */

struct Acommand
{
    char *name;
    void (*routine) ();
    int restrictions;
};

struct aToken
{
    char *name;
    int token;
};

char arrowXref[4] = {

    NORTHTOKEN,
    EASTTOKEN,
    SOUTHTOKEN,
    WESTTOKEN
};

/* prototypes */
void autoCalc (char *args);

int execStream (hydraClientData * clientData, int mask);
void execfile (char *args);
int commandCurrentlyRestricted (struct Acommand *command, FILE * cmdStream,
                                int deferFlag);
int name2ord (char *axisName);
int argPairToXref (char *args, xItem * theList);
double converthms (char *dec, int *hr, int *min, int *sec, int *other,
                   int *sign);
long secToSteps (double arcSec);
void flushStatus (void);
void abump (char *args);
void aload (char *args, int flag1);
void saveit (char *args);
void bload (char *args);
char *bdump (char *args);
void assign (char *args);
void applyRot (char *args);
void arrowmap (char *args);
int batchAstrometry (char *starFileName, int direction);
int convertFileFormat (char *starFileName);
void beginProg (char *args);
void brakes (char *args);
int bump (char *args);
void buttonmove (char *args, int dummy, int cmdInfo, int checkRestrictions);
void physmove (char *args, int dummy, int cmdInfo, int checkRestrictions);
void buttonview (char *args);
void calcRot (char *args);
void clearRot (char *args);
void configure (char *args);
void configOne (char *args);
void configMode (char *args);
void correctButton (char *args);
void cruiseIt (char *args);
int cruise (char *msg);
int detrot (char *args);
void detCenter (char *args);
void dropit (char *args);
void driveStatus (char *args);
void endProg (char *args);
void eraseProg (char *args);
void execProg (char *args);
void eccenmode (char *args);
void warpmode (char *args);
void thermmode (char *args);
void thermval (char *args);
void thisis (char *args);
void thisisRisky (char *args);
void flexmode (char *args);
int gox (char *args);
int goy (char *args);
int goxy (char *args);
void goTag (char *args);
void goz (char *args);
void coldstart (char *args);
void gripActivate (char *args);
void gripin (char *args);
void gripout (char *args);
void hangle (char *args);
void dangle (char *args);
void hist (char *args);
void homeMotors (char *args);
void initMotors (char *args);
void listButt (char *args);
void listDump (char *args);
void listLoc (char *args);
void listrot (char *args);
void listTag (char *args);
void lit (char *args);
void markIt (char *args);
void offset (char *args);
int offsetToXY (long dx, long dy, double vel, double accel);
void parkbutt (char *args, int dummy, int cmdInfo, int checkRestrictions);
void ping (char *args);
void preview (char *args);
void progDelay (char *args);
void purge (char *args);
void quit (char *args);
void restore (char *args);
void rotmode (char *args);
void rotval (char *args);
void buttonRecover (char *args);
void setrot (char *args);
void status (char *args);
void tag (char *args);
void tweak (char *args);
void unstow (char *args);
void setStats (char *args);
void view (char *args);
void waitSec (char *args);
void whamMotors (char *args);
void whereis (char *args);
void remoteXref (char *args);
void xparked (char *args);
void xstowed (char *args);
void zdb (char *args);
void zlight (char *args);
void thetaval (char *args);
void phival (char *args);
void setCable (char *args);
void syncCable (void);
void setOffset (char *args);
void graphCmd (char *args);
void complamp (char *args);
int lampShortMessage (int whichCable, int testMode);
void qtz (char *args);
int cameraSafetyInterlock (void);
void plate (char *args);
int plateFlat (void);
void led (char *args);
void screen (char *args);
int syncScreen (void);
int screenIsIn (void);
int screenLocked (void);
void tv (char *args);
int cameraVulnerable (void);
void hinge (char *args);
void power (char *args);
void rotator (char *args);
int setBlock (int flag);
void sysStatusDump (struct exposinfo *pExposure, FILE * statusFile);
void progtest (char *args);
void fakehw (char *args);
int releaseButtonAfterCycle (void);
int tokenize (char *args, int tokens[], struct aToken tokenTable[]);
void setup (char *args);
void setInputStreamMode (int mode);

int LastButtonmoveOK = TRUE;    /* Kludge of the highest order */
int LastViewOK = FALSE;         /* Kludge of the highest order */

int UserInterrupt = 0;          /* breaks out of scripts in ctrl-C */

/*  This is the complete command set.  If a command isn't in this list,
    then it isn't a valid command to the system.  Commands that do not
    parse arguments may call functions that are not in this file.  
    External functions used by the parser begin with an upper case letter.
    An example is the command "open" which calls Open() in motor.c
*/

struct Acommand FunctTable[] = {

    /* command      function    restrictions */

    {"abump", cardinalBump, (GRIP | RACK)},
    {"applyrot", applyRot, (NONE)},
    {"arrowmap", arrowmap, (NONE)},
    {"assign", assign, (PLATE | HINGE | ROTATE | GRIP | RACK)},
    {"beginprog", beginProg, (SIM | RACK)},
    {"brakes", brakes, (SIM | RACK)},
    {"bump", bump, (GRIP | RACK)},
    {"calcrot", calcRot, (NONE)},
    {"clearpoint", clearRot, (NONE)},
    {"close", Close, (SIM | RACK)},
    {"closewide", CloseWide, (SIM | RACK)},
    {"coldstart", coldstart, (NONE)},
    {"configmode", configMode, (NONE)},
    {"correct", correctButton, (PLATE | HINGE | ROTATE | GRIP | RACK)},
    {"dangle", dangle, (NONE)},
    /* { "detrot",     autoRotation,   (ROTATE | RACK)         }, */
    {"detcenter", detCenter, (ROTATE | RACK)},
    {"dinit", InitDrives, (SIM | RACK)},
    {"drivestats", driveStatus, (NONE | RACK)},
    {"dropit", dropit, (PLATE | GRIP | RACK)},
    {"concenmode", eccenmode, (NONE)},
    {"configure", configure, (HINGE | ROTATE | RACK)},
    {"config1", configOne, (HINGE | ROTATE | RACK)},
    {"cruise", cruiseIt, (GRIP | RACK)},
    {"endprog", endProg, (SIM | RACK)},
    {"eraseprog", eraseProg, (SIM | RACK)},
    {"exec", execfile, (NONE)},
    {"execprog", execProg, (SIM | RACK)},
    {"flexmode", flexmode, (NONE)},
    {"gox", gox, (GRIP | RACK)},
    {"goy", goy, (GRIP | RACK)},
    {"goxy", goxy, (GRIP | RACK)},
    {"gotag", goTag, (GRIP | RACK)},
    {"goz", goz, (GRIP | RACK)},
    {"gripactivate", gripActivate, (RACK)},
    {"gripin", gripin, (RACK)},
    {"gripout", gripout, (RACK)},
    {"griptest", gripTest, (RACK)},
    {"observe", gripOut, (RACK)},
    {"hangle", hangle, (NONE)},
    {"hist", hist, (NONE)},
    {"hold", Hold, (SIM | RACK)},
    {"home", homeMotors, (GRIP | RACK)},
    {"list", listDump, (NONE)},
    {"listrot", listrot, (NONE)},
    {"listtag", listTag, (NONE)},
    {"lit", lit, (SIM)},
    {"load", aload, (NONE)},
    {"mark", markIt, (NONE)},
    {"minit", initMotors, (SIM | RACK)},
    {"move", buttonmove, (DEFER | PLATE | HINGE | ROTATE | GRIP | RACK)},
    {"physmove", physmove, (DEFER | PLATE | HINGE | ROTATE | GRIP | RACK)},
    {"nessload", bload, (NONE)},
    {"offset", offset, (GRIP | RACK)},
    {"open", Open, (SIM | RACK)},
    {"openwide", OpenWide, (SIM | RACK)},
    {"park", parkbutt, (DEFER | PLATE | HINGE | ROTATE | GRIP | RACK)},
    {"ping", ping, (SIM)},
    {"preview", preview, (GRIP | RACK)},
    {"progdelay", progDelay, (SIM | RACK)},
    {"purge", purge, (NONE)},
    {"quit", quit, (NONE)},
    {"relax", Relax, (SIM)},
    {"restore", restore, (NONE)},
    {"rotmode", rotmode, (NONE)},
    {"rotval", rotval, (NONE)},
    {"save", saveit, (NONE)},
    {"setrot", setrot, (NONE)},
    {"status", status, (NONE)},
    {"tag", tag, (NONE)},
    {"thermmode", thermmode, (NONE)},
    {"thermval", thermval, (NONE)},
    {"thisis", thisis, (PLATE | HINGE | GRIP | RACK)},
    {"ThisIs", thisisRisky, (PLATE | HINGE | GRIP | RACK)},
    {"tweak", tweak, (PLATE | HINGE | ROTATE | GRIP | RACK)},
    {"unstow", unstow, (PLATE | HINGE | ROTATE | GRIP | RACK)},
    {"calcupdate", autoCalc, (RACK)},
    {"view", view, (GRIP | RACK)},
    {"wait", waitSec, (NONE)},
    {"warpmode", warpmode, (NONE)},
    {"wham", whamMotors, (SIM)},
    {"where", listLoc, (NONE)},
    {"whereis", whereis, (NONE)},
    {"xparked", xparked, (NONE)},
    {"xstowed", xstowed, (NONE)},
    {"zdb", zdb, (NONE)},
    {"zlight", zlight, (SIM | RACK)},

    /* graphics commands that get done
       in another program */

    {"update", graphCmd, (NONE)},
    {"targets", graphCmd, (NONE)},
    {"refresh", graphCmd, (NONE)},
    {"field", graphCmd, (NONE)},
    {"setblock", graphCmd, (NONE)},
    {"setstats", graphCmd, (NONE)},
    {"setcable", setCable, (NONE)},
    {"setoffset", setOffset, (NONE)},
    {"showfib", graphCmd, (NONE)},
    {"showobj", graphCmd, (NONE)},
    {"showscreen", graphCmd, (NONE)},
    {"showplate", graphCmd, (NONE)},
    {"showrot", graphCmd, (NONE)},
    {"showled", graphCmd, (NONE)},
    {"showcomp", graphCmd, (NONE)},
    {"showgrip", graphCmd, (NONE)},
    {"pushled", graphCmd, (NONE)},
    {"pushcomp", graphCmd, (NONE)},
    {"pushgrip", graphCmd, (NONE)},
    {"gfoo", graphCmd, (NONE)},

    /* commands that originate in the graphics
       and expect a response on the socket */

    {"?xref", remoteXref, (NONE)},

    /* std bus command words */

    {"complamp", complamp, (NONE)},
    {"qtz", qtz, (NONE)},
    {"plate", plate, (NONE)},
    {"led", led, (NONE)},
    {"screen", screen, (NONE)},
    {"tv", tv, (NONE)},
    {"power", power, (NONE)},
    {"hinge", hinge, (NONE)},

    /* high level telescope commands  (return ints for tcl) */

    {"rotator", rotator, (NONE)},

    /* high level wiyn words  (return ints for tcl) */

    {"setup", setup, (HINGE)},
    {"standard", wiynStdQuick, (HINGE)},
    {"Standard", wiynStdStar, (HINGE)},
    {"view", wiynView, (NONE)},
    {"tweak", wiynTweak, (NONE)},
    {"update", wiynUpdate, (NONE)},

    /* these are temporary */

    {"goguide", wiynGoGuide, (NONE)},

    {"pbp", fakehw, (NONE)},
    {"progtest", progtest, (NONE)},
    {"thetaval", thetaval, (NONE)},
    {"phival", phival, (NONE)}
};

struct aToken TokenTable[] = {

    /* setup command  Tokens */

    {"field", TOK_FIELD},
    {"flats", TOK_FLATS},
    {"comps", TOK_COMPS},
    {"quartz", TOK_QUARTZ},
    {"observe", TOK_OBSERVE},

    /* stdbus control Tokens */

    {"on", TOK_ON},
    {"off", TOK_OFF},
    {"in", TOK_IN},
    {"out", TOK_OUT},
    {"status", TOK_STATUS},
    {"test", TOK_TEST},
    {"init", TOK_INIT},
    {"flat", TOK_FLAT},
    {"warp", TOK_WARP},
    {"illum", TOK_ILLUM},
    {"indexers", TOK_INDEXERS},
    {"drives", TOK_DRIVES},
    {"cal", TOK_CAL},
    {"vacpower", TOK_VAC},
    {"valve", TOK_VALVE},

    /* stdbus control Tokens */

    {"tracking", TOK_TRACKING},
    {"position", TOK_POSITION},
    {"offset", TOK_OFFSET},
    {"adjust", TOK_ADJUST},

    {"", 0}
};

char *parkStatusNames[] = {
    "Extended",
    "Parked",
    "Stowed"
};

extern int getYesNo (FILE * devOut, FILE * devIn, char *prompt,
                     int defaultValue);
extern int hingeOpen (void);
extern void getCold (char *args);
extern int buttonAlreadyThere (struct fiber_dbase *fib, int deadZone);
extern void rotate (point *, double);
extern int GoToXY (long int x, long int y, double vel, double accel,
                   int system);
extern int GoToZ (long int z, double vel, double accel, int sync);
extern int HomeZ (void);
extern double radToDeg (double);
extern int bowOut (void);
extern int blindRelease (void);
extern int lampCableStatus (int *flags, int testMode);
extern int lampStatus (int *flags, int testMode);
extern int printLampStatus (int *flags);
extern int genericOnOffMessage (char *message, int status);
extern int lampStatusQZ (int *flags, int testMode);
extern int printLampStatusQZ (int *flags);
extern int getYesNoQuit (FILE * devOut, FILE * devIn, char *prompt,
                         int defaultValue);
extern int vacuumStatus (int *flags, int testMode);
extern int vacDiag (void);
extern int printVacStatus (int *flags);
extern int ledDiag (void);
extern int ledStatus (int *flags, int testMode);
extern int printLEDStatus (int *flags, int level);
extern int screenDiag (void);
extern int screenStatus (int *flags, int testMode);
extern int printScreenStatus (int *flags);
extern int lockoutStatus (int *flags, int testMode);
extern int tvStatusFops (int *flags);
extern int tvStatusGrip (int *flags);
extern int printTvStatus (char *tvName, int tvStatus);
extern int wiynPrepObserve (void);
int rotatorSetTracking (int mode);
int rotatorSetOffset (double offsetAngle);
int rotatorSetAbsAngle (double absoluteAngle);
int sendCableStuff (void);
void generateStats (int *objCount, int *skyCount, int *fopCount);
int sendIRAFstuff (void);
void adump (void);
int tellWhoeverCares (struct fiber_dbase *fib);
void hydraFileDump (struct starinfo *theStars[], struct exposinfo *pExposure,
                    FILE * infoFile, int flag);
int saveStatus (char *envName, struct templateItem template[]);
int rotatorNotZero (void);
void defTag (int id, struct plate_coords *coord);

/*
		execStream

   process commands from the specified input stream until
   EOF is encountered.  Commands are delimited by a newline.
   fields within commands are delimited by spaces.  The first
   field is the command itself.  A search is made of the
   structure of known commands, and if a match is found,
   execution transfers to the corresponding function for
   processing of the arguments (if any).
*/
execStream (hydraClientData * clientData, int mask)
{

    int noExit = TRUE;          /* flag not to exit */
    int i;

    char buff[256];             /* command line */
    char command[64];           /* command word */
    char *cmdPtr;
    char args[256];             /* command args */
    char *token;

    char tempPrompt[64];

    int commandFound;

    int checkRestrictions;

    /* first make sure that no one else can do input stream events until
       this command is finished!! */

    if ((clientData->stream == StdinInfo.stream) ||
        (clientData->stream == GraphinInfo.stream))
    {

        /* fputs( "***turning off input ****", stderr ); */
        setInputStreamMode (OFF);
    }

    if (UserInterrupt)
    {

        noExit = FALSE;

        if (clientData->stream == stdin)
        {
            UserInterrupt = FALSE;
        }
    }

    if (setjmp (Env))
    {
        goto exit;
    }

    while (noExit)
    {

        checkRestrictions = TRUE;

        /* set up for a recovery to the next command in
           the event of an error */

        bzero (buff, sizeof (buff));
        bzero (command, sizeof (command));
        bzero (args, sizeof (args));

        (void) sprintf (tempPrompt, "\nFiborg %s >", VERSION);

        /* try to get a line, and return if EOF */

        if (fgets (buff, sizeof (buff), clientData->stream) == NULL)
        {

            /* end of stream, which means either a true eof, or
               some other client of the program has disconnected.
               Either way, close off the channel */

            /* This really should all be in a separate function that 
               handles any sort of disconnect based on the info
               in "clientData"... */

            /* fputs( "EOF in execStream\n", stderr ); */

            /* get rid of any file handler */

            /* point output to somewhere safe */

            close (fileno (clientData->stream));

            fclose (clientData->stream);

            /* the following is a huge hack because it assumes that there is 
               only one gui present */

            if (clientData->stream == graphCmds)
            {
                clientData->stream = nullFile;
                graphCmds = nullFile;
            }

            goto exit;
        }

        buff[strlen (buff) - 1] = '\0';

        /*  handle empty lines */

        if (strlen (buff) == 0)
        {
            if (clientData->loopFlag == ONE_PASS)
            {

                if (!supress)
                    (void) fputs (tempPrompt, stderr);

                goto exit;
            }
            else
                continue;
        }

        /* tokenize */
        if (token = strtok (buff, " "))
        {
            strcpy (command, token);
        }
        else
        {
            continue;
        }
        /* cat all args */
        while (token = strtok (0, " "))
        {
            strcat (args, token);
            strcat (args, " ");
        }
        /* get rid of last white space */
        args[strlen (args) - 1] = '\0';

        cmdPtr = command;

        /* always log the command line */

        sprintf (log_buf, "%s %s %s \n", tempPrompt, command, args);
        log_putsQ (log_buf);

        /* if no prompt and not configuring, show the input string to
           show the progress of scripts that are executing */

        if (supress && !Configuring)
            (void) fprintf (stderr, "%s %s \n", command, args);

        if (*cmdPtr == '~')
        {
            cmdPtr++;
            checkRestrictions = FALSE;
        }

        /* Loop thru command list looking for match and call the 
           appropriate functions from the table */

        commandFound = FALSE;

        for (i = 0; i < (sizeof (FunctTable) / sizeof (FunctTable[0])); i++)
        {

            if (strcmp (cmdPtr, FunctTable[i].name) == 0)
            {

                commandFound = TRUE;

                strcpy (currentCommand, cmdPtr);

                if (!checkRestrictions)
                {

                    /* execute the function immediately */

                    (void) FunctTable[i].routine (args, 0,
                                                  (int) &FunctTable[i], FALSE,
                                                  0, 0);

                    break;
                }
                else
                {               /* safety check time! */

                    switch (commandCurrentlyRestricted
                            (&FunctTable[i], clientData->stream, DEFER))
                    {

                        case SAFE:
                            (void) FunctTable[i].routine (args, 0,
                                                          (int)
                                                          &FunctTable[i],
                                                          FALSE, 0, 0);
                            break;

                        case DEFERED:
                            (void) FunctTable[i].routine (args, 0,
                                                          (int)
                                                          &FunctTable[i],
                                                          TRUE, 0, 0);
                            break;

                        case UNSAFE:

                            /* Command isn't safe.  Exit command and any script executing */

                            noExit = FALSE;
                            break;

                        default:
                            log_fputs ("Unknown state in safety checks\n",
                                       stderr);
                            break;
                    }
                }
                break;
            }
        }                       /* end of look-it-up loop */

        if (!commandFound && (*cmdPtr != '#'))
        {
            /* log_fputs( "**UNKNOWN COMMAND**  try again...\n", stderr ); */
            if (clientData->stream == stdin)
            {
                log_fputs ("**UNKNOWN COMMAND (on stdin)**  try again...\n",
                           stderr);
            }
            else
            {
                log_fputs ("**UNKNOWN COMMAND (on other)**  try again...\n",
                           stderr);

            }
        }

        if (clientData->loopFlag)
        {

            if (!supress)
                (void) fputs (tempPrompt, stderr);

            goto exit;
        }

        if (UserInterrupt)
        {

            noExit = FALSE;

            /* if a setup is in progress, leave this flag alone to 
               prevent execution of post-setup activities.  "setup"
               will clear it later */

            if (clientData->stream == stdin)
            {
                UserInterrupt = FALSE;
            }
        }

        while (Tk_DoOneEvent (TK_DONT_WAIT) != 0)
        {
            /* next line is just for testing */
            /*fputs( "Tick...\n", stderr ); */
        }
    }

  exit:

    /* turn commands from stdin and the gui back on as tk events */

    if ((clientData->stream == StdinInfo.stream) ||
        (clientData->stream == GraphinInfo.stream))
    {

        /* fputs( "***Turning input back on***\n", stderr ); */
        setInputStreamMode (ON);
    }

    return 0;
}

/*
		execfile

   Transfer to the specified file as the source of the
   command stream.  This is fully recursive.
*/

void execfile (char *args)
{
    int temp;
    char *cmdPath;

    hydraClientData streamInfo;

    if (args[0] == '/')
    {

        /*
           (void)strcpy( cmdFileName, args );
         */
        cmdPath = args;
    }
    else
    {
        cmdPath = findFile (args, ExecEnvItems);
    }

    if (cmdPath == NULL)
    {
        fprintf (stderr, "ERROR: couldn't find file: \"%s\"\n", args);
        return;
    }

    temp = supress;             /* save the suppression flag status */

    streamInfo.stream = fopen (cmdPath, "r");

    if (streamInfo.stream == NULL)
        log_fputs ("Command sequence file not found.\n", stderr);
    else
    {

        supress = TRUE;
        streamInfo.loopFlag = MULTI_PASS;

        execStream (&streamInfo, 0);
    }

    /* now done for us in exec stream */

    supress = temp;             /* restore the suppression flag */
}

/*
		commandCurrentlyRestricted

   Check the restriction flags of for the specified command
   and return true if command can not be executed at this time.
*/

int commandCurrentlyRestricted (struct Acommand *command, FILE * cmdStream,
                                int deferFlag)
{
    int restricted = FALSE;
    int testBit;

#ifdef HYDRA_II_TEST_MODE
    return SAFE;
#endif
// HYDRA_II_TEST_MODE

    /* first be sure we're not deferring the test */

    if ((deferFlag == DEFER) && (command->restrictions & 0x01))
    {
        /* fputs( "Defering safety check\n", stderr ); */
        return (DEFERED);
    }

    /* now loop through the next 8 bits */

    for (testBit = 2; testBit <= 0x80; testBit *= 2)
    {

        switch (command->restrictions & testBit)
        {

            case ROTATE:

                if (!faith && !noGWC)
                {

                    if (0 /*rotatorNotInitialized() */ )
                    {
                        fputs
                            ("WARNING:  Instrument Rotator has not been initialized.\n",
                             stderr);
                        fputs ("Please verify rotator position visually...\n",
                               stderr);
                        if (!getYesNo
                            (stderr, stdin, "Is the rotator at zero?", TRUE))
                        {
                            restricted = TRUE;
                        }

                    }

                    if (rotatorNotZero ())
                    {
                        fputs
                            ("WARNING:  Instrument Rotator not at zero! --command ignored.\n",
                             stderr);
                        fputs ("Please zero rotator and try again...\n",
                               stderr);
                        restricted = TRUE;
                    }
                }

                break;

            case HINGE:

                if (!faith && hingeOpen ())
                {
                    fputs
                        ("WARNING:  Focal plane hinge is open! --command ignored.\n",
                         stderr);
                    fputs ("Please close hinge and try again...\n", stderr);
                    restricted = TRUE;
                }

                break;

            case PLATE:

                if (!faith && !plateFlat ())
                {
                    fputs
                        ("WARNING:  Focal plane is warped! --command ignored.\n",
                         stderr);
                    fputs ("Please flatten plate and try again...\n", stderr);
                    restricted = TRUE;
                }

                break;

            case RACK:

                if (!faith && (cmdStream == stdin))
                {
                    /*fputs( "running cold test\n", stderr ); */
                    if (SystemIsCold (&masterTimer))
                    {
                        getCold (0);
                        fputs
                            ("System is now ready for use. --try command again.\n",
                             stderr);

                        restricted = TRUE;
                    }
                }

                break;

            case GRIP:

                if (GripperParked)
                {
                    restricted = TRUE;
                    sprintf (log_buf,
                             "SORRY: The command \"%s\" is not valid with the gripper out...\n",
                             command->name);
                    log_fputs (log_buf, stderr);
                }

                break;

            case SIM:

                if (faith)
                {
                    sprintf (log_buf,
                             "SORRY: The command \"%s\" is not valid in the simulator...\n",
                             command->name);
                    log_fputs (log_buf, stderr);

                    restricted = TRUE;
                }

                break;

            case NONE:
            default:

                break;
        }
    }

    if (restricted)
        return (UNSAFE);
    else
        return (SAFE);

}

/*
		name2ord

   Convert a  character axis name to an axis ordinal.
   Any unrecognized name cause a "null ordinal" to be
   returned.
*/

name2ord (char *axisName)
{
    int ordinal;

    *axisName |= 040;

    switch (axisName[0])
    {

        case 'x':
            ordinal = X_ord;
            break;
        case 'y':
            ordinal = Y_ord;
            break;
        case 'z':
            ordinal = Z_ord;
            break;
        default:
            ordinal = NULL_ORDINAL;
            sprintf (log_buf, "ERROR:  Unknown axis:\"%s\"\n", axisName);
            log_fputs (log_buf, stderr);
            break;
    }

    return (ordinal);
}

/*
		argPairToXref

   take a string containing a number and a type 
   specifier, and set three integers to the
   object, fiber, and slit identification numbers.
   Flag un-doable items with  a negative value,
*/

int argPairToXref (char *args, xItem * theList)
{

    int atTarget;

    char arg1[32];
    char arg2[32];

    int intArg;
    char *type;

    int internalID;

    int argCount = 0;

    theList->fiber = VOIDENTRY;
    theList->slit = VOIDENTRY;
    theList->object = VOIDENTRY;

    if ((argCount = sscanf (args, "%s %s", arg1, arg2)) < 1)
    {
        log_fputs ("Missing argument(s)\n", stderr);
        return (ERROR);
    }

    if (argCount == 1)
    {

        if (isalpha (arg1[0]))
        {

            /* a probe is being referenced by name */

            type = PROBE;
        }
        else
        {

            /* a button is being referenced by number */

            if (sscanf (arg1, "%d", &intArg) < 1)
            {
                log_fputs ("Bad numeric identifier\n", stderr);
                return (ERROR);
            }
            type = DEFAULT_VIEW_TYPE;
        }
    }

    if (argCount >= 2)
    {

        /* figure out which argument is which */

        if (isalpha (arg1[0]))
        {

            /* first arg is the type */

            type = arg1;

            /* now see if the second one is numeric */

            if (sscanf (arg2, "%d", &intArg) < 1)
            {
                log_fputs ("Bad numeric identifier\n", stderr);
                return (ERROR);
            }
        }
        else
        {
            if (isalpha (arg2[0]))
            {

                /* second arg is the type */

                type = arg2;

                /* now see if the first one is numeric */

                if (sscanf (arg1, "%d", &intArg) < 1)
                {
                    log_fputs ("Bad numeric identifier\n", stderr);
                    return (ERROR);
                }
            }
            else
            {
                log_fputs ("Bad/missing type\n", stderr);
                return (ERROR);
            }
        }
    }

    switch (type[0])
    {

        case 'f':

            atTarget = xRefFiber (intArg, theList, Stars, FiBase);

            if (theList->fiber == VOIDENTRY)
            {

                log_fputs ("Warning, bad fiber id number\n", stderr);
                return (ERROR);
            }
            break;

        case 'o':

            switch (userToInternalID (Stars, intArg, &internalID))
            {

                case NONE_FOUND:

                    log_fputs ("Unknown/bad object id number\n", stderr);
                    break;

                case MULTI_FOUND:

                    sprintf (log_buf,
                             "Warning:  %d has two entries!  First one has been referenced\n",
                             intArg);
                    log_fputs (log_buf, stderr);

                 /*FALLTHRU*/ case ONE_FOUND:

                    atTarget =
                        xRefObject (internalID, theList, Stars, FiBase);

                    if (theList->object == VOIDENTRY)
                    {

                        log_fputs ("Unknown/bad object id number\n", stderr);
                        return (ERROR);
                    }

                    break;

                default:
                    break;
            }

            break;

        case 's':

            atTarget = xRefSlit (intArg, theList, Stars, FiBase);

            if (theList->slit == VOIDENTRY)
            {

                log_fputs ("Warning, bad slit id number\n", stderr);
                return (ERROR);
            }

            break;

        case 'P':

            atTarget = xRefProbe (arg1[0], theList, Stars, FiBase);

            if (theList->fiber == VOIDENTRY)
            {
                log_fputs ("Warning, bad probe name\n", stderr);
                return (ERROR);
            }

            break;

        default:
            log_fputs
                ("Bad type--object, fiber, slit, or probe name only...\n",
                 stderr);
            return (ERROR);
            break;
    }

    return (atTarget);
}

/* 
		converthms

   Convert a string in the form hh:mm:ss to decimal
   hours (or degrees).

   adapted from the MPG function library.
*/

double converthms (char *dec, int *hr, int *min, int *sec, int *other,
                   int *sign)
                /* what for ? */
{
    float hrTemp = 0.0;
    float minTemp = 0.0;
    float secTemp = 0.0;
    double angle = 0.0;
    double tmpAngle = 0.0;

    *sign = 1;

    /* skip leading space in string */

    while ((*dec == ' ') || (*dec == '\t'))
        dec++;

    /* set the sign flag */

    switch (*dec)
    {

        case '-':

            *sign = -1;
        /*FALLTHRU*/ case '+':

            /* skip leading '+' and '-' */
            dec++;
    }

    if (sscanf (dec, "%f:%f:%f", &hrTemp, &minTemp, &secTemp) >= 1)
    {

        /* WE HAVE AN ANGLE (EITHER DEGREES OR HOURS) WHO CARES WHICH ? */
        angle = (((secTemp / 60.) + minTemp) / 60. + hrTemp);

        tmpAngle = angle;
    }

    else
    {

        /* UNSUCCESSFUL PARSING */
        log_fputs (" time/angle parsing failed\n", stderr);
    }

    /* crack the angle into h:m:s components */

    *hr = (int) tmpAngle;

    tmpAngle = (tmpAngle - (double) *hr) * 60.;
    *min = (int) tmpAngle;

    /* sec should be a 'float' argument */

    tmpAngle = (tmpAngle - (double) *min) * 60.;
    *sec = (int) tmpAngle;

    tmpAngle = (tmpAngle - (double) *sec) * 1000.;

    *other = (int) tmpAngle;    /* milliseconds */

    return (angle * (double) *sign);
}

long secToSteps (double arcSec)
{
    return ((long) (((arcSec / WIYN_SCALE) / 25.4) * MOTOR_RESOLUTION));
}

void flushStatus (void)
{

    if (faith)
    {
        if (saveStatus (SIMSTATUS, statusList) != 0)
        {
            log_fputs ("WARNING:  System status not saved to disk\n", stderr);
        }
    }
    else
    {
        if (saveStatus (STATUSFILE, statusList) != 0)
        {
            log_fputs ("WARNING:  System status not saved to disk\n", stderr);
        }
    }
}

/****************** argument parsing functions ************************/

void abump (char *args)
{
    double magnitude;
    long numSteps;
    char direction;

    char tmpString[32];

    if (sscanf (args, "%lf %c", &magnitude, &direction) < 2)
        labtyd (0, "Missing argument.");

    else
    {
        numSteps = secToSteps (magnitude);
        sprintf (tmpString, "%ld %c\n", numSteps, direction);
        bump (tmpString);
    }
}

void aload (char *args, int flag1)
{
    FILE *starFile;
    char starFileName[64];
    int i;

    /* split up the args */

    sscanf (args, "%s", starFileName);

    /* starFileName is now the fileName */

    /* clean up any old file */

    for (i = 0; i < MAXSTARS; i++)
    {
        if (Stars[i] != NULL)
        {
            free ((char *) Stars[i]);
            Stars[i] = NULL;
        }
    }

    /*  "tweaks" loose meaning if a new file has been loaded */

    for (i = 0; i < MAXFIBER; i++)
    {
        FiBase[i].tweaked = FALSE;
    }

    batchAstrometry (args, SKY_TO_PLATE);       /* done by a separate program */

    starFile = openCoordFile (starFileName, "r");

    if (starFile == NULL)
    {
        log_fputs ("Hydra data file not found.\n", stderr);
    }

    else
    {
        if (LoadHydraFile (starFile, Stars, &Exposure) == FILE_OK)
        {

            strcpy (Exposure.fileName, starFileName);

#ifdef IGNORE
            switch (Exposure.cable[0])
            {

                case 'R':
                case 'r':
                    cableType = RED;
                    break;

                case 'P':
                case 'p':
                    cableType = RED;
                    break;

                case 'B':
                case 'b':
                    cableType = BLUE;
                    break;

                default:
                    break;
            }

            if (flag1 != NO_CABLE_CHECK)
            {

                /* default, so check the cable */

                if (cableType != CableStatus)
                {
                    log_fputs
                        ("\n WARNING:  File is not intended for the current fiber cable!\n",
                         stderr);
                    return;
                }
            }

#endif

            if (flag1 != NO_CABLE_CHECK)
            {

                /* default, so check the cable */

                if (getCableIdentifier (Exposure.cable) != CurrentCable)
                {

                    log_fputs
                        ("\n WARNING:  File is not intended for the current fiber cable!\n",
                         stderr);
                    return;
                }
            }

            NoFieldLoaded = FALSE;      /* allows "save"s to be done */

            /* tell the graphics program what field is loaded */

            if (graphic)
            {
                fprintf (graphCmds, "field %s\n", starFileName);
                fputs ("update\n", graphCmds);
                fflush (graphCmds);
            }
        }
        else
        {

            sprintf (log_buf,
                     "ERROR: File \"%s\"\nNot readable as Hydra file\n",
                     starFileName);
            log_fputs (log_buf, stderr);

            /* clean it up */
            for (i = 0; i < MAXSTARS; i++)
            {
                if (Stars[i] != NULL)
                {
                    free ((char *) Stars[i]);
                    Stars[i] = NULL;
                }
            }
        }
    }

    fclose (starFile);
}

void saveit (char *args)
{
    FILE *starFile;
    char starFileName[64];
    char *tmpName;
    char *token;

    if (NoFieldLoaded)
    {
        log_fputs ("no field has been loaded, so no save possible\n", stderr);
        return;
    }

    /* first arg, if available, is file name */
    if (token = strtok (args, " "))
    {
        strcpy (starFileName, token);
    }
    else
    {
      namePrompt:
        tmpName = getUserInput (stderr, stdin, CHAR, NO_DEFAULT,
                                "\nEnter new name for hydra file.", NULL,
                                NULL, NULL, NULL);

        if (tmpName == NULL)
        {
            log_fputs ("You must specify a filename!\n", stderr);
            goto namePrompt;
        }

        /* dump the newline from the end of the string */

        tmpName[strlen (tmpName) - 1] = '\0';

        strcpy (starFileName, tmpName);

    }

    /* Correct the Star and fiber Dbases for changes that have been
       made by tweaking, leaving fibers extended, or having assignments
       fail. */

    updateDbases ();

    starFile = openCoordFile (starFileName, "w");

    if (starFile == NULL)
    {
        log_fputs ("dump file not openable.\n", stderr);
    }
    else
    {

        /* kludge alert!!!
           This un-does the instrument mis-alignment correction
           so that the original value in the field center entry
           gets written back out as it exists in the original input
           file. */

        FieldRotationalOffset -= (ROTATION_CORRECTION * (M_PI / 180.0));

        hydraFileDump (Stars, &Exposure, starFile, 1);

        /* now we have to restore it! */

        FieldRotationalOffset += (ROTATION_CORRECTION * (M_PI / 180.0));

        /* end kludge */
    }

    fclose (starFile);

    batchAstrometry (starFileName, PLATE_TO_SKY);       /* done by a separate program */
}

void bload (char *args)
{

    convertFileFormat (args);   /* generates call to separate prog */

    /* now we can process it like a normal file */

    aload (args, 0);
}

 /*ARGSUSED*/ char *bdump (char *args)
{
    static char fileName[255];

    FILE *dumpFile;

    fileName[0] = '\0';
    strcpy (fileName, Exposure.fileName);

    strcat (fileName, ".dump");

    dumpFile = fopen (fileName, "w+");

    if (dumpFile == NULL)
        labtyd (0, "Can't open file for coordinate dump.");
    else
    {
        hydraFileDump (Stars, &Exposure, dumpFile, 0);
        fclose (dumpFile);
    }

    return (fileName);
}

void assign (char *args)
{
    int object;
    int button;
    int internalID;
    long x;
    long y;
    char cmdString[32];
    int i;

    if (sscanf (args, " %d %d", &object, &button) < 2)
        labtyd (0, "Missing argument.");
    else
    {

        switch (userToInternalID (Stars, object, &internalID))
        {

            case NONE_FOUND:

                log_fputs ("Unknown/bad object id number\n", stderr);
                return;

            case MULTI_FOUND:

                sprintf (log_buf,
                         "Warning:  %d has two entries!  First one has been referenced\n",
                         object);
                log_fputs (log_buf, stderr);

                break;

            default:
                break;
        }

        if ((internalID > MAXSTARS) || (Stars[internalID] == NULL))
        {
            labtyd (0, "Requested object does not exist.");
        }
        else
        {
            if (button >= MAXFIBER)
            {
                labtyd (0, "Requested button does not exist.");
            }
            else
            {

                x = (long) Stars[internalID]->specEncoderCoord.x;
                y = (long) Stars[internalID]->specEncoderCoord.y;

                fprintf (stderr,
                         "Assigning obj. #%d \"%s\" to button %d\n",
                         Stars[internalID]->IDnumber,
                         Stars[internalID]->name, button);

                sprintf (cmdString, "%d %ld %ld\n", button, x, y);
                buttonmove (cmdString, 0, 0, 0);

                /* clear any old assignments for this button */

                for (i = 0; i < MAXSTARS; i++)
                {

                    if (Stars[i] != NULL)
                    {

                        /* first look for other uses of this button and clear them */
                        if (Stars[i]->button == button)
                        {

                            FiBase[Stars[i]->button].assignedObject = NULL;
                            Stars[i]->button = COORD_OK;
                        }
                    }
                }

                /* now clear other uses of this object */

                FiBase[Stars[internalID]->button].assignedObject = NULL;

                /* record that this is now the button associated
                   with the object in the database */

                Stars[internalID]->button = button;
                FiBase[button].assignedObject = Stars[internalID];

                /* if we just assigned it, it can't be tweaked... */

                FiBase[button].tweaked = FALSE;

                /* update the scoreboard */

                setStats ("");

            }
        }
    }
}

/*
		applyRot

   Apply a new rotation angle to the system.
*/

 /*ARGSUSED*/ void applyRot (char *args)
{
    RotData.aveAngle = RotData.workingAngle;
    Rotflag = TRUE;
    flushStatus ();
}

void arrowmap (char *args)
{
    if (sscanf (args, " %c %c %c %c",
                &arrowXref[0], &arrowXref[1], &arrowXref[2],
                &arrowXref[3]) < 4)
        labtyd (0, "Missing argument or bad arg to arrowmap.");
}

batchAstrometry (char *starFileName, int direction)
{
    char *binPath;
    char *progName;
    char pathAndProgram[64];
    char completeCommand[128];

    int childPID;

    /* make a system call to calculate the coords for a 
       specified file */

    /* in case we don't find the environment... */

    pathAndProgram[0] = '\n';

    /* get the path to the binary */

    if ((binPath = getenv (FIBBIN)) != NULL)
    {
        (void) strcpy (pathAndProgram, binPath);
        (void) strcat (pathAndProgram, "/");
    }

    /* get the name of the program to run */

    if ((progName = getenv (ASTRO_PROG)) != NULL)
    {

        (void) strcat (pathAndProgram, progName);

        /* but the whole command together */

        if (direction == SKY_TO_PLATE)
            sprintf (completeCommand, "%s %s\n", pathAndProgram,
                     starFileName);
        else
            sprintf (completeCommand, "%s %s -R\n", pathAndProgram,
                     starFileName);

        if ((childPID = fork ()) == 0)
        {

            /* this is the child */

            system (completeCommand);

            exit (0);

        }
        else
        {

            /* this is still the parent */

            while (wait4 (childPID, NULL, WNOHANG, NULL) == 0)
            {

                while (Tk_DoOneEvent (TK_DONT_WAIT) != 0)
                {
                    /* next line is just for testing */
                    /* fputs( "Astrometry Tick...\n", stderr ); */
                }
            }

            fputs ("Astrometry Finished...\n", stderr);
        }

        return (OK);
    }
    else
    {
        log_fputs ("ERROR: astrometry program name not in environment\n",
                   stderr);
        return (ERROR);
    }
}

convertFileFormat (char *starFileName)
{
    char *binPath;
    char *progName;
    char pathAndProgram[64];
    char completeCommand[128];

    /* make a system call to calculate the coords for a 
       specified file */

    /* in case we don't find the environment... */

    pathAndProgram[0] = '\n';

    /* get the path to the binary */

    if ((binPath = getenv (FIBBIN)) != NULL)
    {
        (void) strcpy (pathAndProgram, binPath);
        (void) strcat (pathAndProgram, "/");
    }

    /* get the name of the program to run */

    if ((progName = getenv (FORMAT_PROG)) != NULL)
    {

        (void) strcat (pathAndProgram, progName);

        /* but the whole command together */

        sprintf (completeCommand, "%s %s\n", pathAndProgram, starFileName);

        return (system (completeCommand));
    }

    log_fputs ("ERROR: conversion program name not in environment\n", stderr);

    return (ERROR);
}

/*
		beginProg
   
   This function causes the Indexer to begin accepting
   commands as a download instead of executing them
   immediately.  The argument list is parsed for both
   an axis id and a sequence number.  The number of
   sequences available on a given unit depends on its
   type.
*/

void beginProg (char *args)
{
    char axis;
    int ordinal;
    int seqID;

    if (sscanf (args, "%c %d", &axis, &seqID) >= 2)
    {

        ordinal = name2ord (&axis);

        if (ordinal != NULL_ORDINAL)
            BeginProgram (ordinal, seqID);
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		brakes

   Turn the brakes on or off based on the argument
*/

void brakes (char *args)
{
    int flag;

    if (sscanf (args, "%d", &flag) > 0)
        (void) Brakes (flag);
    else
        log_fputs (BADARG, stderr);
}

/*
		bump
   
   Nudge the gripper by the specified amount in the specified
   direction.
*/

bump (char *args)
{
    int error = 0;
    int xyError = 0;
    long x = 0;
    long y = 0;
    long dn;
    char direction;

    /* Prepare to run at limits if no arguments */

    double vel = BUMP_VEL;
    double accel = BUMP_ACCEL;

    /* break off the direction character and the magnitude of the move */

    if (sscanf (args, "%ld %c", &dn, &direction) >= 2)
    {

        direction |= 040;

        switch (direction)
        {

            case 'n':
                y = dn * NORTH;
                break;

            case 's':
                y = dn * -(NORTH);
                break;

            case 'e':
                x = dn * EAST;
                break;

            case 'w':
                x = dn * -(EAST);
                break;

            default:
                error = TRUE;
                log_fputs ("Invalid direction\n", stderr);
                break;
        }

        if (!error)
        {

            if ((xyError = offsetToXY (x, y, vel, accel)) != 0)
            {
                log_fputs ("ERROR: Can't bump beyond limits...\n", stderr);
                return (xyError);
            }
        }
        else
        {
            return (XY_FAIL);
        }
    }

    else
    {
        log_fputs (BADARG, stderr);
        return (XY_FAIL);
    }

    return (0);
}

void buttonmove (char *args, int dummy, int cmdInfo, int checkRestrictions)
{
    xItem theList;
    int atTarget;

    int fiber;
    int count;

    struct aPoint target;
    struct fiber_dbase *fib;

    char buttonName[8];

    int deadZone = DEFAULT_DEAD_ZONE;

    count = sscanf (args, "%d %lf %lf %d",
                    &fiber, &target.x, &target.y, &deadZone);

    atTarget = xRefFiber (fiber, &theList, Stars, FiBase);

    if (theList.fiber == VOIDENTRY)
    {
        log_fputs ("Warning, bad fiber id number\n", stderr);
        return;
    }

    if ((count >= 2) && (fiber >= 0) && (fiber < MAXFIBER))
    {

        /* don't do locked or broken fibers */

        if (fiberBroken (fiber))
        {

            sprintf (log_buf,
                     "WARNING:  fiber %d is broken.  --leaving it parked.\n",
                     fiber);
            log_fputs (log_buf, stderr);

            return;
        }

        if (fiberLocked (fiber))
        {

            sprintf (log_buf,
                     "WARNING:  fiber %d is locked.  --leaving it parked.\n",
                     fiber);
            log_fputs (log_buf, stderr);

            return;
        }

        if (unresolved ())
        {

            if (FiberInTrouble.fiber != NULL)
            {
                sprintf (log_buf,
                         "ERROR: Can't move button %d with %d in transit!\n",
                         fiber, FiberInTrouble.fiber->fiber_id);
                log_fputs (log_buf, stderr);
            }
            else
            {
                sprintf (log_buf,
                         "ERROR: Can't move button %d with unknown button in gripper!\n",
                         fiber);
                log_fputs (log_buf, stderr);
            }

            LastButtonmoveOK = FALSE;
        }

        else
        {

            fib = &FiBase[fiber];

            if (xyValidForButton (fib, &target))
            {

                if (buttonAlreadyThere (fib, deadZone))
                {               /* we must do the move... */
                    /* fprintf( stderr, "fiber %d is already there... \n", fiber ); */

                    /* notify the gwc world if it's a fops
                       even if it didn't move */

                    if (fiberIsFop (fiber))
                    {
                        tellWhoeverCares (fib);
                    }

                    return;
                }

                if (checkRestrictions)
                {

                    /* fputs( "checking restrictions in move function\n", stderr ); */

                    if (commandCurrentlyRestricted (cmdInfo, NULL, NO_DEFER)
                        == UNSAFE)
                    {
                        sprintf (log_buf, "Unsafe to move fiber %d\n",
                                 theList.fiber);
                        log_fputs (log_buf, stderr);
                        return;
                    }
                    else
                    {
                        /* fprintf( stderr, "move of #%d not restricted\n", fiber ); */
                    }
                }

                if (fiberInCable (fib->fiber_id, 'Z'))
                {

                    sprintf (log_buf,
                             "SORRY, fiber #%d doesn't physically exist...\n",
                             fib->fiber_id);
                    log_fputs (log_buf, stderr);
                    return;
                }

                MoveButt (fib, deadZone);

                recordNewLocation (fib, UPDATE_FILE);
                LastButtonmoveOK = TRUE;
                FiBase[fiber].updated = TRUE;

                if (graphic)
                {
                    fputs (" update \n", graphCmds);
                    fflush (graphCmds);
                    if (Configuring == FALSE)
                    {
                        setStats ("");
                    }
                }

                /* notify the gwc world it's moved */

                if (fiberIsFop (fiber))
                {
                    tellWhoeverCares (fib);
                }

            }

            else
            {
                sprintf (log_buf,
                         "Invalid move requested for button #%d\n", fiber);
                log_fputs (log_buf, stderr);

                if (Configuring)
                {

                    /* leaving a button in place could prevent later
                       buttons from being placed! --so park it */

                    /* fprintf( stderr, "Parking button %d\n", fiber); */

                    sprintf (buttonName, "%d", fiber);
                    parkbutt (buttonName, 0, 0, 0);

                }
                LastButtonmoveOK = FALSE;
            }
        }
    }

    else
    {
        log_fputs (BADBUTT, stderr);
        LastButtonmoveOK = FALSE;
    }
}

void physmove (char *args, int dummy, int cmdInfo, int checkRestrictions)
{
    int fiber;
    int count;

    xItem theList;
    int atTarget;

    char buttonName[8];

    struct aPoint target;
    struct fiber_dbase *fib;

    int deadZone = DEFAULT_DEAD_ZONE;

    count = sscanf (args, "%d %lf %lf %d",
                    &fiber, &target.x, &target.y, &deadZone);

    atTarget = xRefFiber (fiber, &theList, Stars, FiBase);

    if (theList.fiber == VOIDENTRY)
    {
        log_fputs ("Warning, bad fiber id number\n", stderr);
        return;
    }

    if ((count >= 2) && (fiber >= 0) && (fiber < MAXFIBER))
    {

        /* don't do locked or broken fibers */

        if (fiberBroken (fiber))
        {

            sprintf (log_buf,
                     "WARNING:  fiber %d is broken.  --leaving it parked.\n",
                     fiber);
            log_fputs (log_buf, stderr);

            return;
        }

        if (fiberLocked (fiber))
        {

            sprintf (log_buf,
                     "WARNING:  fiber %d is locked.  --leaving it parked.\n",
                     fiber);
            log_fputs (log_buf, stderr);

            return;
        }

        if (unresolved ())
        {

            sprintf (log_buf,
                     "ERROR: Can't move button %d with %d in transit!\n",
                     fiber, FiberInTransit.fiber->fiber_id);
            log_fputs (log_buf, stderr);

            LastButtonmoveOK = FALSE;
        }

        else
        {

            fib = &FiBase[fiber];

            if (xyValidPlate (fib, &target))
            {

                if (buttonAlreadyThere (fib, deadZone))
                {               /* we must do the move... */
                    /* fprintf( stderr, "fiber %d is already there... \n", fiber ); */
                    return;
                }

                if (checkRestrictions)
                {

                    /* fputs( "checking restrictions in move function\n", stderr ); */

                    if (commandCurrentlyRestricted (cmdInfo, NULL, NO_DEFER)
                        == UNSAFE)
                    {
                        sprintf (log_buf, "Unsafe to move fiber %d\n",
                                 theList.fiber);
                        log_fputs (log_buf, stderr);
                        return;
                    }
                    else
                    {
                        /* fprintf( stderr, "move of #%d not restricted\n", fiber ); */
                    }
                }

                MoveButt (fib, deadZone);

                recordNewLocation (fib, UPDATE_FILE);
                LastButtonmoveOK = TRUE;

                if (atTarget)
                {

                    if (Stars[theList.object] != NULL)
                        fprintf (stderr,
                                 "Object #%d  \"%s\" is now unassigned.\n",
                                 Stars[theList.object]->IDnumber,
                                 Stars[theList.object]->name);
                }

                if (graphic)
                {
                    fputs (" update \n", graphCmds);
                    fflush (graphCmds);
                    if (Configuring == FALSE)
                    {
                        setStats ("");
                    }
                }

                /* notify the gwc world it's moved */

                if (fiberIsFop (fiber))
                {
                    tellWhoeverCares (fib);
                }
            }

            else
            {
                sprintf (log_buf,
                         "Invalid move requested for button #%d\n", fiber);
                log_fputs (log_buf, stderr);

                if (Configuring)
                {

                    /* leaving a button in place could prevent later
                       buttons from being placed! --so park it */

                    fprintf (stderr, "Parking button %d\n", fiber);

                    sprintf (buttonName, "%d", fiber);
                    parkbutt (buttonName, 0, 0, 0);
                }

                LastButtonmoveOK = FALSE;
            }
        }
    }

    else
    {
        log_fputs (BADBUTT, stderr);
        LastButtonmoveOK = FALSE;
    }
}

/*
	 buttonview

   view a specified button.
*/

void buttonview (char *args)
{
    struct fiber_dbase *fib;
    int button;
    char flags[8];
    int viewIt = FALSE;
    int i;

    flags[0] = 'a';
    flags[1] = '\0';

    if ((sscanf (args, "%d %3s", &button, flags) >= 1) &&
        (button >= 0) && (button < MAXFIBER))
    {

        fib = &FiBase[button];

        for (i = 0; ((flags[i] != '\n') && (i <= 2) && !viewIt); i++)
        {

            switch (flags[i])
            {

                case 'a':
                    viewIt |= TRUE;
                    break;

                case 'd':
                    if (fib->parkedStatus == EXTENDED)
                        viewIt |= TRUE;
                    break;

                case 'p':
                    if (fib->parkedStatus == PARKED)
                        viewIt |= TRUE;
                    break;

                case 's':
                    if (fib->parkedStatus == STOWED)
                        viewIt |= TRUE;
                    break;

                default:
                    sprintf (log_buf,
                             "WARNING: unknown flag '%c' in \"view\" \n",
                             flags[i]);
                    log_fputs (log_buf, stderr);
                    break;
            }
        }

        /* see if an unresolved move is pending and test range if a
           button is present */

        if (!unresolved ())
        {

            if (viewIt)
            {

                ViewButt (fib);

                sprintf (log_buf, "This fiber, #%d, is %s \n",
                         fib->fiber_id, parkStatusNames[fib->parkedStatus]);
                log_fputs (log_buf, stderr);

                currentButton = button;
            }
        }

        else
        {
            sprintf (log_buf,
                     "ERROR: Can't view button %d with %d in gripper!\n",
                     button, FiberInTransit.fiber->fiber_id);
            log_fputs (log_buf, stderr);
        }
    }
    else
        log_fputs (BADBUTT, stderr);
}

/*
		calcRot

   Re-calculate the rotation angle from the table of points.
*/

 /*ARGSUSED*/ void calcRot (char *args)
{
    calcRotation (&RotData);
}

/*
		clearRot

   Clear a rotation point.
*/

void clearRot (char *args)
{
    int pt;

    if ((sscanf (args, "%d", &pt) < 1))
        log_fputs ("Missing or bad point #\n", stderr);

    else
    {
        if (pt > 0)
            clearRotationPt (pt, &RotData);
        else
            initRotation (&RotData);
    }
}

 /*ARGSUSED*/ void configure (char *args)
{
    char *cmdPath;
    char tmpFileName[128];
    char preConfigCmd[128];

    int childPID;

    int argCount;

    char arg1[32];
    char arg2[32];
    char arg3[32];
    char flags[8];
    char eStr[4];

    char *field;
    char *threshold;

    *arg1 = '\0';
    *arg2 = '\0';
    *arg3 = '\0';
    *flags = '\0';
    *eStr = '\0';

    /* break up the arguments */

    /* this might get overwritten by args */

    field = Exposure.fileName;
    threshold = eStr;

    /* fprintf( stderr, " configure args: %s \n", args ); */
    argCount = sscanf (args, "%s %s %s", arg1, arg2, arg3);

    /* see if there are any flags and copy them */

    if ((argCount >= 1) && (*arg1 == '-'))
        strcpy (flags, arg1);

    /* one arg means either flags or a filename */

    if (argCount == 1)
    {
        if (*arg1 != '-')
            field = arg1;
    }

    /* two args means either a) flags & filename  b) flags & threshold */

    if (argCount == 2)
    {

        if (*flags == '-')
        {

            if (rindex (flags, 'u') != NULL)
            {

                /* arg is a threshold */

                threshold = arg2;
            }
            else
            {

                /* arg is a field name */

                field = arg2;
            }
        }
        else
        {
            /* error */
            log_fputs ("ERROR:  Bad args to \"configure\".\n", stderr);
        }
    }

    /* three args means flags & filename & threshold */

    if (argCount == 3)
    {

        if ((*flags == '-') && (rindex (flags, 'u') != NULL))
        {

            field = arg2;
            threshold = arg3;
        }
        else
        {

            /* error */
            log_fputs ("ERROR:  Bad args to \"configure\".\n", stderr);
        }
    }

    if (faith)
    {                           /* simulator */
        if (*flags != '-')
            strcpy (flags, "-s");
        else
            strcat (flags, "s");
    }

    /* bring in gripper if required */

    if (GripperParked)
    {

        log_fputs ("ATTENTION:  Gripper was disabled--re-enabling...\n\n",
                   stderr);
        gripin (0);
    }

    if ((cmdPath = getenv (FIBFIELDS)) != NULL)
    {

        tmpFileName[0] = '\0';

        (void) strcpy (tmpFileName, cmdPath);
        (void) strcat (tmpFileName, "/");
        (void) strcat (tmpFileName, ASSIGN_TEMPLATE);
    }
    else
    {
        labtyd (0, "Cant find Data path in environment");
    }

    /* create the temporary and unique file name  then open */

    mkstemp (tmpFileName);

    /* generate the command */

    sprintf (preConfigCmd, "${%s}/%s  %s  %s %s  >  %s\n",
             FIBBIN, PRE_CONFIG_SCRIPT, flags, field, threshold, tmpFileName);

    if (verbose)
    {
        fputs ("preconfig: ", stderr);
        fputs (preConfigCmd, stderr);
    }

    if ((childPID = fork ()) == 0)
    {

        /* this is the child */

        system (preConfigCmd);

        exit (0);

    }
    else
    {

        /* this is still the parent */

        while (wait4 (childPID, NULL, WNOHANG, NULL) == 0)
        {

            while (Tk_DoOneEvent (TK_DONT_WAIT) != 0)
            {
                /* next line is just for testing */
                /* fputs( "Transition Tick...\n", stderr ); */
            }
        }

        fputs ("Transition Finished...\n", stderr);
    }

    /* now configure the system */

    Configuring = TRUE;         /* causes un-doable moves to park button */

    execfile (tmpFileName);

    Configuring = FALSE;

    unlink (tmpFileName);

#ifdef IGNORE

    /* now see which ones got assigned and build the cross reference */

    for (i = 0; i < MAXSTARS; i++)
    {
        if ((Stars[i] != NULL) && (Stars[i]->button >= 0))
        {

            if ((Stars[i]->objectType == 'S') ||
                (Stars[i]->objectType == 's'))
            {

                tolerance = SKY_DEAD_ZONE;
            }
            else
            {
                tolerance = Sloppiness;
            }

            if (buttonAtTarget (i, (tolerance + DEFAULT_DEAD_ZONE)))
            {

                objectXref[i] = Stars[i]->button;

                /* if it got there, it's could  no longer 
                   possibly  be longer "tweaked", so... */

                FiBase[Stars[i]->button].tweaked = FALSE;

            }

        }
    }
#endif

    if (graphic)
    {
        fputs (" refresh \n", graphCmds);
        fputs (" update \n", graphCmds);
        fflush (graphCmds);
        setStats ("");
    }
}

/*
		configOne

   This function is much like configure() except that
   it takes as an argument a fiber and a coordinate
   which it passes on to the transition program.

*/

 /*ARGSUSED*/ void configOne (char *args)
{
    char *cmdPath;
    char tmpFileName[128];
    char preConfigCmd[128];

    char arg1[32];
    char arg2[32];
    char arg3[32];
    char flags[8];
    char eStr[4];

    *arg1 = '\0';
    *arg2 = '\0';
    *arg3 = '\0';
    *flags = '\0';
    *eStr = '\0';

    /*
       Create flags for transition program.

       We *always* want to leave unused fibers alone.

       The "S" says that there is no target file, but
       a singer fiber to be positioned which is specified
       on the command line.
     */

    strcpy (flags, "-pS");

    if (faith)
    {                           /* simulator */
        strcat (flags, "s");
    }

    /* break up the arguments */

    /* fprintf( stderr, " configure args: %s \n", args ); */

    /* bring in gripper if required */

    if (GripperParked)
    {

        log_fputs ("ATTENTION:  Gripper was disabled--re-enabling...\n\n",
                   stderr);
        gripin (0);
    }

    if ((cmdPath = getenv (FIBFIELDS)) != NULL)
    {

        tmpFileName[0] = '\0';

        (void) strcpy (tmpFileName, cmdPath);
        (void) strcat (tmpFileName, "/");
        (void) strcat (tmpFileName, ASSIGN_TEMPLATE);
    }
    else
    {
        labtyd (0, "Cant find Data path in environment");
    }

    /* create the temporary and unique file name  then open */

    mkstemp (tmpFileName);

    /* generate the command */

    sprintf (preConfigCmd, "${%s}/%s  %s  %s  >  %s\n",
             FIBBIN, PRE_CONFIG_SCRIPT, flags, args, tmpFileName);

    if (verbose)
    {
        fputs ("preconfig: ", stderr);
        fputs (preConfigCmd, stderr);
    }

    system (preConfigCmd);

    /* now configure the system */

    Configuring = TRUE;         /* causes un-doable moves to park button */

    execfile (tmpFileName);

    Configuring = FALSE;

    unlink (tmpFileName);

    if (graphic)
    {
        fputs (" refresh \n", graphCmds);
        fputs (" update \n", graphCmds);
        fflush (graphCmds);
        setStats ("");
    }
}

/*
		configMode

   Turn configuration flag on or off.
   This flag is used to speed up certain commands by
   bypassing work that only need be done after the
   system is in a static state.
*/

void configMode (char *args)
{
    int temp;

    if ((sscanf (args, "%d", &temp) > 0) && ((temp == 1) || (temp == 0)))
    {

        Configuring = temp;
    }

    else
        log_fputs (BADARG, stderr);
}

/*
		correctButton
   
   Bring the "current" button to the current gripper location.
   The current button is the last one that was moved or viewed.
   In some cases, there may not be a current button.  This routine
   is used to tweak button positions to corrected coordinates
   interactively.
*/

 /*ARGSUSED*/ void correctButton (char *args)
{
    struct aPoint target;
    struct fiber_dbase *fib;

    target.x = location_now.x;
    target.y = location_now.y;

    if (rememberedButton != VOIDBUTTON)
    {

        fib = &FiBase[rememberedButton];

        if (unresolved ())
        {

            sprintf (log_buf, "ERROR: Can't tweak with a button. \n");
            log_fputs (log_buf, stderr);
        }

        else
        {

            /* un-apply the appropriate flexure corrections to get back to
               sky coordinates */

            coordCorrect (&target, &PhysicalEnvironment, INVERT);

            if (Rotflag)
            {

                /* un-rotate to get physical coords */

                rotate (&target, -(RotData.aveAngle));
            }

            if (xyValidForButton (fib, &target))
            {

                MoveButt (fib, FORCE_MOVE);
                recordNewLocation (fib, UPDATE_FILE);
                fib->tweaked = TRUE;
            }
            else
                log_fputs ("ERROR: Bad target for button in gripper!\n",
                           stderr);
        }
    }
    else
    {
        log_fputs ("ERROR:  Don't know what button to correct!\n", stderr);
    }
}

 /*ARGSUSED*/ void cruiseIt (char *args)
{
    (void) cruise ("");
}

#ifdef __CRUISE__

 /*ARGSUSED*/ cruise (msg)
char *msg;

{
    int key = 0;
    char tmpMag[3];
    int bumpError = 0;
    int magnitude = SMALLNUDGE;
    int keyOrd = VOID_ORD;

    char tmpString[32];

    int bailOut = 0;            /* flag giving type of exit from loop */

    /* bring in gripper if required */

    if (GripperParked)
    {

        log_fputs ("ATTENTION:  Gripper is out!  Can't do this!...\n\n",
                   stderr);
        return -1;
    }

    /* prevent escapes to the shell */

    signal (SIGTSTP, SIG_IGN);

    tmpMag[1] = '\0';

/*
   getbegyx( win, minY, minX );
   getmaxyx( win, maxY, maxX );
*/

    InCurses = TRUE;

    /* now setup curses */

    refresh ();
    wclear (win);
    /* cbreak(); */
    noecho ();
    nodelay (win, TRUE);
    /*halfdelay(3 ); */
    notimeout (win, FALSE);
    keypad (win, TRUE);

    mvwaddstr (win, 10, 10, msg);

    mvwaddstr (win, 12, 13, "Use HJKL (like vi) or arrows to move gripper");
    mvwaddstr (win, 14, 14, "Press RETURN when done. ('E' to void activity)");

    /*  loop until escape character received */

    while ((key != RETURN) && (key != NEWLINE) && (bailOut == 0))
    {

        /*fgets( key, 2, stdin ); */

        while ((key = wgetch (win)) == ERR)
        {
            while (Tk_DoOneEvent (TK_DONT_WAIT) != 0)
            {
                /* next line is just for testing */
                /* fputs( "Cruise idle Tick...\n", stderr ); */
            }
        }

        switch (key)
        {

            case 'J':
                magnitude = BIGNUDGE;
                keyOrd = DOWN_ORD;
                break;

            case 'j':
            case KEY_DOWN:
                keyOrd = DOWN_ORD;
                break;

            case 'K':
                magnitude = BIGNUDGE;
                keyOrd = UP_ORD;
                break;

            case 'k':
            case KEY_UP:
                keyOrd = UP_ORD;
                break;

            case 'L':
                magnitude = BIGNUDGE;
                keyOrd = RIGHT_ORD;
                break;

            case 'l':
            case KEY_RIGHT:
                keyOrd = RIGHT_ORD;
                break;

            case 'H':
                magnitude = BIGNUDGE;
                keyOrd = LEFT_ORD;
                break;

            case 'h':
            case KEY_LEFT:
                keyOrd = LEFT_ORD;
                break;

            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
                tmpMag[0] = (char) key;
                sscanf (tmpMag, "%d", &magnitude);
                magnitude = pow (2.0, (double) magnitude) * SMALLNUDGE;;
                keyOrd = VOID_ORD;
                break;

            case '0':
                magnitude = pow (2.0, 11.0) * SMALLNUDGE;;
                keyOrd = VOID_ORD;
                break;

            case 'e':
            case 'E':
                bailOut = TRUE;
                break;

            default:
                keyOrd = VOID_ORD;

                /* fputs( "Tock...\n", stderr ); */

                while (Tk_DoOneEvent (TK_DONT_WAIT) != 0)
                {
                    /* next line is just for testing */
                    /* fputs( "Default cruise Tick...\n", stderr ); */
                }

                break;
        }

        if (keyOrd != VOID_ORD)
        {
            sprintf (tmpString, "%d %c\n", magnitude, arrowXref[keyOrd]);
            if (bump (tmpString) == XY_LIM)
            {
                log_fputs ("WARNING:  Can't cruise past X,Y limits!\n",
                           stderr);
            }
            keyOrd = VOID_ORD;
            magnitude = SMALLNUDGE;
        }
    }

    endwin ();

    InCurses = FALSE;

    /* re-enable escapes from the program */

    signal (SIGTSTP, SIG_DFL);

    if (bumpError == XY_LIM)
    {
        log_fputs ("ERROR:  Attempt to cruise past X,Y limits! \n", stderr);
    }

    return (bailOut);
}

#else

 /*ARGSUSED*/ cruise (char *msg)
{
    int key = 0;
    char tmpMag[3];
    int magnitude = SMALLNUDGE;
    int keyOrd = VOID_ORD;

    char tmpString[32];

    int bailOut = 0;            /* flag giving type of exit from loop */

    tmpMag[1] = '\0';

    /*  loop until escape character received */

    while ((key != RETURN) && (key != NEWLINE) && !(bailOut))
    {

        key = fgetc (stdin);

        switch (key)
        {

            case 'J':
                magnitude = BIGNUDGE;
                keyOrd = DOWN_ORD;
                break;

            case 'j':
                keyOrd = DOWN_ORD;
                break;

            case 'K':
                magnitude = BIGNUDGE;
                keyOrd = UP_ORD;
                break;

            case 'k':
                keyOrd = UP_ORD;
                break;

            case 'L':
                magnitude = BIGNUDGE;
                keyOrd = RIGHT_ORD;
                break;

            case 'l':
                keyOrd = RIGHT_ORD;
                break;

            case 'H':
                magnitude = BIGNUDGE;
                keyOrd = LEFT_ORD;
                break;

            case 'h':
                keyOrd = LEFT_ORD;
                break;

            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
                tmpMag[0] = (char) key;
                sscanf (tmpMag, "%d", &magnitude);
                magnitude *= SMALLNUDGE;
                keyOrd = VOID_ORD;
                break;

            case 'e':
            case 'E':
                bailOut = TRUE;
                break;

            default:
                keyOrd = VOID_ORD;
                break;
        }

        if (keyOrd != VOID_ORD)
        {
            sprintf (tmpString, "%d %c\n", magnitude, arrowXref[keyOrd]);
            if (bump (tmpString) != 0)
            {
                bailOut = TRUE;
            }
            magnitude = SMALLNUDGE;
        }
    }

    return (bailOut);
}

#endif

/*
		detrot

   Command the fiborg program to determine the rotation
   angle of the specified object
*/

detrot (char *args)
{
    int object;
    int internalID;
    long x;
    long y;

    char argString[32];
    char cruiseMsg[32];

    if (sscanf (args, " %d", &object) < 1)
    {
        log_fputs ("Missing argument.", stderr);
        return (FALSE);
    }
    else
    {

        switch (userToInternalID (Stars, object, &internalID))
        {

            case NONE_FOUND:

                log_fputs ("Unknown/bad object id number\n", stderr);
                return (FALSE);

            case MULTI_FOUND:

                sprintf (log_buf,
                         "Warning:  %d has two entries!  First one has been referenced\n",
                         object);
                log_fputs (log_buf, stderr);

                break;

            default:
                break;
        }

        if ((internalID > MAXSTARS) || (Stars[internalID] == NULL))
        {
            log_fputs ("Requested object does not exist.", stderr);
            return (FALSE);
        }

        else
        {

            x = (long) Stars[internalID]->camEncoderCoord.x;
            y = (long) Stars[internalID]->camEncoderCoord.y;

            /* got to where the star is */

            sprintf (argString, "%ld %ld\n", x, y);

            if (goxy (argString) != 0)
            {
                return (FALSE);
            }

            /* set the tag for the calculated location */

            sprintf (argString, "%d %ld %ld\n", CALCTAG, x, y);
            tag (argString);

            /* interactively move to actual location of star */

            sprintf (cruiseMsg,
                     "Viewing: obj# %d \"%-15s\"  Center on this star",
                     object, Stars[internalID]->name);

            if (cruise (cruiseMsg) == 0)
            {

                /* set the tag for the observed position */

                sprintf (argString, "%d\n", OBSTAG);
                tag (argString);

                /* add the point to the rotation list */

                sprintf (argString, "%s\n", Stars[internalID]->name);
                setrot (argString);

                /* calculate the new rotation angle */

                calcRot (NULL);

                return (TRUE);
            }
            else
            {

                log_fputs ("Detrot abandoned: No change to rotation. \n",
                           stderr);
                return (FALSE);
            }
        }
    }
}

/*
		detCenter

   Compute a new plate center the "easy" way!
*/

void detCenter (char *args)
{
    long startX;
    long startY;

    long oldX;
    long oldY;

    long newX;
    long newY;

    char cruiseMsg[32];

    /* save the old values just in case... */

    oldX = OffsetXY.x;
    oldY = OffsetXY.y;

    startX = location_now.x;
    startY = location_now.y;

    sprintf (cruiseMsg, "Center Gripper on reference point");

    if (cruise (cruiseMsg) == 0)
    {

        newX = OffsetXY.x + (location_now.x - startX);
        newY = OffsetXY.y + (location_now.y - startY);

        sprintf (log_buf, "Old offset values were: %6ld, %6ld\n", OffsetXY.x,
                 OffsetXY.y);
        log_fputs (log_buf, stderr);
        sprintf (log_buf, "NEW offset values  are: %6ld, %6ld\n", newX, newY);
        log_fputs (log_buf, stderr);

        if (getYesNo (stderr, stdin, "Record These Values?", FALSE))
        {

            OffsetXY.x = newX;
            OffsetXY.y = newY;

            flushStatus ();

            log_fputs
                ("New plate center recorded--Homing stages to verify... \n",
                 stderr);

            if (!faith)
            {

                goxy ("-15000 26500");

                HomeMotors (X_ord);
                HomeMotors (Y_ord);

                /* go back to where we started to see if it helped */
                GoToXY (startX, startY, BIG_NUMBER, BIG_NUMBER, PHYSICAL);
            }

            if (!getYesNo (stderr, stdin, "Does this look right?", TRUE))
            {

                OffsetXY.x = oldX;
                OffsetXY.y = oldY;

                flushStatus ();
                log_fputs ("Old plate center restored... \n", stderr);
            }

        }
        else
        {
            log_fputs ("detcenter abandoned: No change to plate center. \n",
                       stderr);
        }

    }
    else
    {

        log_fputs ("detcenter abandoned: No change to plate center. \n",
                   stderr);
    }

}

/*
		dropit
   
   Put down a button (if there is one...) at the current
   location.  if the fiber in transit structure is not
   empty, calculate and record new information for that fiber.
*/

 /*ARGSUSED*/ void dropit (char *args)
{
#ifndef HYDRA_II_TEST_MODE
    if (!buttonPresent ())
    {
        sprintf (log_buf, "Why drop a button that is not there?\n");
        log_fputs (log_buf, stderr);
    }
    else
#endif
// HYDRA_II_TEST_MODE
        releaseHere ();
}

/*
		driveStatus
   
   Get a status report from the specified axis.  Note that
   the X axis has two drives!

*/

void driveStatus (char *args)
{
    char axis;
    int ordinal;

    int dummy[64];              /* drive status array--needed as an arg */

    if (sscanf (args, "%c", &axis) >= 1)
    {

        ordinal = name2ord (&axis);

        switch (ordinal)
        {

            case X_ord:

                sprintf (log_buf, "\n\nChecking first X drive\n\n");
                log_fputs (log_buf, stderr);

                (void) GetDriveStatus (XDRIVE2, dummy, &masterTimer);

                log_fputs ("\n", stderr);

                sprintf (log_buf, "\n\nChecking second X drive\n\n");
                log_fputs (log_buf, stderr);

                (void) GetDriveStatus (XDRIVE1, dummy, &masterTimer);

                log_fputs ("\n", stderr);

                break;

            case Y_ord:

                sprintf (log_buf, "\n\nChecking Y drive\n\n");
                log_fputs (log_buf, stderr);

                (void) GetDriveStatus (YDRIVE1, dummy, &masterTimer);

                log_fputs ("\n", stderr);

                break;

            case Z_ord:
                sprintf (log_buf, "Drive stats not avail. for Z axis.\n");
                log_fputs (log_buf, stderr);
                break;

            default:
                break;
        }
    }

    else
        log_fputs (BADARG, stderr);
}

/*
		endProg
   
   Signal the indexers to stop interpreting commands as part
   of a downloaded program.
*/

void endProg (char *args)
{
    char axis;
    int ordinal;

    if (sscanf (args, "%c", &axis) >= 1)
    {

        ordinal = name2ord (&axis);

        if (ordinal != NULL_ORDINAL)
            EndProgram (ordinal);
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		eraseProg
   
   Signal an indexer to erase a the specified downloaded program.
*/

void eraseProg (char *args)
{
    char axis;
    int ordinal;
    int seqID;

    if (sscanf (args, "%c %d", &axis, &seqID) >= 2)
    {

        ordinal = name2ord (&axis);

        if (ordinal != NULL_ORDINAL)
            EraseProgram (ordinal, seqID);
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		execProg

   signal the specified indexer to begin execution of a
   downloaded sequence of instructions.
*/

void execProg (char *args)
{
    char axis;
    int ordinal;
    int seqID;

    if (sscanf (args, "%c %d", &axis, &seqID) >= 2)
    {

        ordinal = name2ord (&axis);

        if (ordinal != NULL_ORDINAL)
            ExecProgram (ordinal, seqID);
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		eccenmode

   Turn the button eccentricity correction on or off 
*/

void eccenmode (char *args)
{
    int temp;                   /* so we can check the value before overwriting old one */

    if ((sscanf (args, "%d", &temp) > 0) && ((temp == 1) || (temp == 0)))
    {

        EccenFlag = temp;
        flushStatus ();
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		warpmode

   Turn the stage characteristic curve mapping on or off.
*/

void warpmode (char *args)
{
    int temp;                   /* so we can check the value before overwriting old one */

    if ((sscanf (args, "%d", &temp) > 0) && ((temp == 1) || (temp == 0)))
    {

        warpFlag = temp;
        flushStatus ();
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		thermmode

   Turn the thermal expansion correction on or off.
*/

void thermmode (char *args)
{
    int temp;                   /* so we can check the value before overwriting old one */

    if ((sscanf (args, "%d", &temp) > 0) && ((temp == 1) || (temp == 0)))
    {

        thermFlag = temp;
        flushStatus ();
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		thermval

   Set the global variable that holds the temperature of the instrument
   This value is used in flexure calculations.  Value is deg. F.
*/

void thermval (char *args)
{
    double temp;

    if ((sscanf (args, "%lf", &temp) > 0) &&
        ((temp < 100.0) && (temp > -20.0)))
    {

        PhysicalEnvironment.temp = temp;
        flushStatus ();
    }
    else
        log_fputs ("ERROR:  Temperature is too large/small! \n", stderr);
}

/*
	 thisis

   Force the software to update a buttons location to be the
   same as the gripper. --including a button in the gripper...
*/

void thisis (char *args)
{
    int button;
    struct fiber_dbase *fib;
    struct aPoint target;
    struct flex_geometry *originalGeometry;
    int locationOK;

    int minButton = 0;
    int maxButton = 199;

    if (buttonPresent ())
    {

        if ((FiberInTransit.fiber == NULL) && (FiberInTrouble.fiber == NULL))
        {

            /* the software has no idea what button is in the gripper */

            log_fputs ("CAUTION:  There is no record of a recent problem\n",
                       stderr);
            log_fputs ("          or of a move in progress.\n", stderr);
            log_fputs
                ("\n        You need to manually identify this fiber.\n",
                 stderr);

            if (!getYesNo
                (stderr, stdin, "Are you prepared to do this?", FALSE))
            {

                /* user bailed out */

                log_fputs
                    ("Exiting \"thisis\".  Identify fiber in gripper and try again.\n",
                     stderr);

                return;
            }
            else
            {

                button =
                    *(int *) getUserInput (stderr, stdin, INT, NO_DEFAULT,
                                           "What button is this? [ 0 to 199 ] ",
                                           NULL, &minButton, &maxButton,
                                           NULL);

                sprintf (log_buf,
                         "setting fiber in transit and fiber in trouble to %d\n",
                         button);
                log_fputs (log_buf, stderr);

                FiberInTransit.fiber = &FiBase[button];
                FiberInTransit.state = IN_TRANSIT;

                FiberInTrouble.fiber = &FiBase[button];
                FiberInTransit.state = CANT_DROP;

                return;

            }
        }

        else
        {                       /* we should already know which one we're holding */

            if ((FiberInTrouble.fiber != NULL) &&
                (FiberInTrouble.state == CANT_DROP))
            {

                sprintf (log_buf,
                         "I assume this is fiber %d  ( the one we couldn't drop...)",
                         FiberInTrouble.fiber->fiber_id);
                log_fputs (log_buf, stderr);

                return;
            }
            if ((FiberInTransit.fiber != NULL) &&
                (FiberInTrouble.fiber == NULL))
            {

                sprintf (log_buf,
                         "I assume this is fiber %d  (fiber in transit at the moment )\n",
                         FiberInTrouble.fiber->fiber_id);
                log_fputs (log_buf, stderr);

                log_fputs
                    ("But there is no record of a failure to drop it!\n",
                     stderr);

                return;
            }
        }
    }
    else
    {

        /* be sure this command makes sense */

        if (FiberInTrouble.fiber == NULL)
        {
            log_fputs
                ("Sorry, there is no record of a failure to grab a fiber!\n",
                 stderr);
            return;
        }

        sprintf (log_buf,
                 "fiber %d is the \"fiber in trouble\"\n",
                 FiberInTrouble.fiber->fiber_id);
        log_fputs (log_buf, stderr);

        fib = FiberInTrouble.fiber;

        /* convert to proper type of structure */

        target.x = location_now.x;
        target.y = location_now.y;

        /* un-apply the flexure corrections to get plate coords */

        coordCorrect (&target, &PhysicalEnvironment, INVERT);

        if (prepNewLocation (fib, &target))
            labtyd (SYSTEM_CALL, "Couldn't prep. new location in database");

        /*
           replace the link to the current geometry with that of the  target
           for the collision testing.  This is done because the collision detection 
           asks, "Do these fibers collide?" *not* "Will they collide?"
         */

        originalGeometry = fib->CurGeometry;
        fib->CurGeometry = fib->TrialGeometry;

        locationOK = thisIsOK (FiBase, fib);

        fib->CurGeometry = originalGeometry;

        if (!locationOK)
        {                       /* this location is no good */

#ifdef LOG
            log_fputs ("ERROR:  The software thinks another button is already \
 at this location.\nCHECK SYSTEM VISUALLY BEFORE PROCEEDING!\n",
                       stderr);

#else
            log_fputs ("ERROR:  The software thinks another button is already \
 at this location.\nCHECK SYSTEM VISUALLY BEFORE PROCEEDING!\n",
                       stderr);
#endif

            restoreOldLocation ();

        }

        else
        {
            recordNewLocation (fib, UPDATE_FILE);
        }

    }
}

/* This version of "thisis" has NO error checking. */

void thisisRisky (char *args)
{
    int button;
    struct fiber_dbase *fib;
    struct aPoint target;

    if ((sscanf (args, "%d", &button) == 1) &&
        (button >= 0) && (button < MAXFIBER))
    {

        fib = &FiBase[button];

        /* convert to proper type of structure */

        target.x = location_now.x;
        target.y = location_now.y;

        /* un-apply the flexure corrections to get plate coords */

        coordCorrect (&target, &PhysicalEnvironment, INVERT);

        if (prepNewLocation (fib, &target))
            labtyd (SYSTEM_CALL, "Couldn't prep. new location in database");

        recordNewLocation (fib, UPDATE_FILE);

        if (buttonPresent ())
        {

            /* set the fiber in transit and fiber in trouble */
            sprintf (log_buf,
                     "\nThe fiber in the jaws is being set to #%d\n", button);
            log_fputs (log_buf, stderr);

            log_fputs
                ("And it's coordinate on the plate is being updated to the\n",
                 stderr);
            log_fputs ("position currently under the gripper. \n", stderr);

            FiberInTransit.fiber = fib;
            FiberInTransit.state = IN_TRANSIT;

            FiberInTrouble.fiber = fib;
            FiberInTrouble.state = CANT_DROP;
        }

    }
    else
    {
        log_fputs ("ERROR:  No such button exists!\n", stderr);
    }
}

/*
		flexmode

   Turn the gravitational flexure mapping on or off.
*/

void flexmode (char *args)
{
    int temp;                   /* so we can check the value before overwriting old one */

    if ((sscanf (args, "%d", &temp) > 0) && ((temp == 1) || (temp == 0)))
    {

        flexFlag = temp;
        flushStatus ();
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		gox

   Move the gripper to the specified position in x
   leaving the y position the same.
   Accept 1-4 arguments.  Used defaults for the
   one's that aren't supplied in the command line.
*/

gox (char *args)
{
    struct aPoint target;

    /* Prepare to run at limits if no arguments.
       Do this by sending a very large number that will
       be reduced by the limit checking routines to the
       true maximum value.
     */

    double vel = BIG_NUMBER;
    double accel = BIG_NUMBER;

    int system;
    char syschar = '\0';        /* temp storage for coord system flag */

    target.x = location_now.x;
    target.y = location_now.y;

    /* un-apply the appropriate flexure corrections to get back to
       sky coordinates */

    coordCorrect (&target, &PhysicalEnvironment, INVERT);

    if (Rotflag)
    {

        /* un-rotate to get physical coords */

        rotate (&target, -(RotData.aveAngle));
    }

    /* void the current button since we are moving! */

    currentButton = VOIDBUTTON;

    /* scan for arguments */

    if (sscanf (args, "%lf %lf %lf %c", &target.x, &vel, &accel, &syschar) >=
        1)
    {
        switch (syschar)
        {

            case 'p':
                system = PHYSICAL;
                break;
            case 'v':
                system = VIRTUAL;
                break;
            default:
                system = Rotflag;       /* assume the current mode if */
                break;
        }

        /* see if an unresolved move is pending and test range if a
           button is present */

        if (unresolved ()
            && !xyValidForButton (FiberInTransit.fiber, &target))
        {

            log_fputs ("ERROR: Bad target for button in gripper!\n", stderr);
            return (XY_LIM);
        }
        else
        {

            /* do the move */

            if (GoToXY ((long) target.x, (long) target.y, vel, accel, system))
            {

                log_fputs ("ERROR: target is beyond X-Y limits.\n", stderr);
                return (XY_LIM);
            }
        }

    }

    else
    {
        log_fputs (BADARG, stderr);
        return (XY_LIM);
    }
}

/*
		goy

   Move the gripper to the specified position in y
   leaving the x position the same.
   Accept 1-4 arguments.  Used defaults for the
   one's that aren't supplied in the command line.
*/

goy (char *args)
{
    struct aPoint target;

    /* Prepare to run at limits if no arguments.
       Do this by sending a very large number that will
       be reduced by the limit checking routines to the
       true maximum value.
     */

    double vel = BIG_NUMBER;
    double accel = BIG_NUMBER;

    int system;
    char syschar = '\0';        /* temp storage for coord system flag */

    target.x = location_now.x;
    target.y = location_now.y;

    /* un-apply the appropriate flexure corrections to get back to
       sky coordinates */

    coordCorrect (&target, &PhysicalEnvironment, INVERT);

    if (Rotflag)
    {

        /* un-rotate to get physical coords */

        rotate (&target, -(RotData.aveAngle));
    }

    /* void the current button since we are moving! */

    currentButton = VOIDBUTTON;

    /* do goy command */

    if (sscanf (args, "%lf %lf %lf %c", &target.y, &vel, &accel, &syschar) >=
        1)
    {

        switch (syschar)
        {

            case 'p':
                system = PHYSICAL;
                break;
            case 'v':
                system = VIRTUAL;
                break;
            default:
                system = Rotflag;       /* assume the current mode if */
                break;
        }

        /* see if an unresolved move is pending and test range if a
           button is present */

        if (unresolved ()
            && !xyValidForButton (FiberInTransit.fiber, &target))
        {

            log_fputs ("ERROR: Bad target for button in gripper!\n", stderr);
            return (XY_LIM);
        }
        else
        {
            /* do the move */

            if (GoToXY ((long) target.x, (long) target.y, vel, accel, system))
            {

                log_fputs ("ERROR: target is beyond X-Y limits.\n", stderr);
                return (XY_LIM);
            }
        }

    }

    else
    {
        log_fputs (BADARG, stderr);
        return (XY_LIM);
    }
}

/*
		goxy

   Move the gripper to the specified position.
   Accept 2-5 arguments.  Used defaults for the
   one's that aren't supplied in the command line.
*/

goxy (char *args)
{
    struct aPoint target;

    /* Prepare to run at limits if no arguments.
       Do this by sending a very large number that will
       be reduced by the limit checking routines to the
       true maximum value.
     */

    double vel = BIG_NUMBER;
    double accel = BIG_NUMBER;

    int system;
    char syschar = '\0';        /* temp storage for coord system flag */

    /* void the current button since we are moving! */

    currentButton = VOIDBUTTON;

    /* do goxy command */

    if (sscanf (args, "%lf %lf %lf %lf %c", &target.x, &target.y,
                &vel, &accel, &syschar) >= 2)
    {

        switch (syschar)
        {

            case 'p':
                system = PHYSICAL;
                break;
            case 'v':
                system = VIRTUAL;
                break;
            default:
                system = Rotflag;       /* assume the current mode if */
                break;
        }

        /* see if an unresolved move is pending and test range if a
           button is present */

        if (unresolved ()
            && !xyValidForButton (FiberInTransit.fiber, &target))
        {

            log_fputs ("ERROR: Bad target for button in gripper!\n", stderr);
            return (XY_LIM);
        }
        else
        {

            /* do the move */
            if (GoToXY ((long) target.x, (long) target.y, vel, accel, system))
            {

                log_fputs ("ERROR: target is beyond X-Y limits.\n", stderr);
                return (XY_LIM);
            }
        }
    }
    else
    {
        log_fputs (BADARG, stderr);
        return (XY_LIM);
    }
}

/*
		goTag

   Go to the location recorded in the specified tag.
*/

void goTag (char *args)
{
    int tagID;

    struct aPoint newLoc;

    double vel = BIG_NUMBER;
    double accel = BIG_NUMBER;

    if (sscanf (args, "%d", &tagID) > 0)
    {

        if ((tagID < 0) || (tagID > MAXTAGS))
        {
            log_fputs ("Invalid tag #\n", stderr);
        }
        else
        {

            newLoc.x = (double) Tags[tagID].x;
            newLoc.y = (double) Tags[tagID].y;

            /* see if an unresolved move is pending and test range if a
               button is present */

            if (!unresolved ())
            {

                GoToXY ((long) newLoc.x, (long) newLoc.y, vel, accel,
                        PHYSICAL);
            }

            else
            {
                if (xyValidForButton (FiberInTransit.fiber, &newLoc))

                    GoToXY ((long) newLoc.x, (long) newLoc.y, vel, accel,
                            PHYSICAL);

                else
                    log_fputs ("ERROR: Bad target for button in gripper!\n",
                               stderr);
            }
        }
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		goz
   Move the Z axis to the specified position.
   Accept 1-3 arguments.  Use defaults for missing
   arguments.
*/

void goz (char *args)
{
    long z;

#ifdef HYDRA_II
    if (sscanf (args, "%ld", &z) == 1)
        move_z (z);
#else
    /* Prepare to run  with defaults if no arguments */

    double vel = BIG_NUMBER;
    double accel = BIG_NUMBER;

    int syncFlag = TRUE;        /* do a position request to sync at end of move */

    /* do goz command */

    if (sscanf (args, "%ld %lf %lf %d", &z, &vel, &accel, &syncFlag) > 0)
        GoToZ (z, vel, accel, syncFlag);
#endif
// HYDRA_II

    else
        log_fputs (BADARG, stderr);
}

/*
		coldstart
   Start up the system from a totally unknown state.
*/

void coldstart (char *args)
{
    /* clear any gripout that was active */
    GripperParked = FALSE;
    flushStatus ();

    /* make sure stuff controlled by std bus is ready.
       i.e. plate flat, etc... */

    if (!faith)
    {

        log_fputs ("Turning on gripper LEDs.\n", stderr);
        led ("illum 8");
        /* led( "on" ); */

        log_fputs ("Flattening focal plane.\n", stderr);
        plate ("init");

        log_fputs ("Making sure complamps are off.\n", stderr);
        complamp ("off");

        log_fputs ("Checking screen status.\n", stderr);
        screen ("status");
        syncScreen ();
        (void) syncCable ();

    }

    if (faith && !PromptHw)
    {
        log_fputs ("System Initialized\n", stderr);
        return;
    }
    else
    {                           /* initialize the hardware */

        if (faith)
            log_fputs ("Z axis initialized\n", stderr);
        else
            execfile (Z_BASIC_INIT);

        if (buttonPresent ())
            buttonRecover (0);
        else
        {
            if (faith)
                log_fputs ("XY initialized\n", stderr);
            else
                execfile (XY_BASIC_INIT);
        }
    }
}

/*
		gripActivate

   Turn on the gripper LEDs and bring gripper
   to center, executing a gripin if necessary.
*/

 /*ARGSUSED*/ void gripActivate (char *args)
{

    /* turn on the gripper led */

    /* led( "on" ); */

    if (faith)
    {

        GripperParked = FALSE;

        goxy ("0 0");

        /* fiborgStatusDump(stderr); */
        /* printLocation(); */
    }

    else
    {
        if (GripperParked)
        {
            log_fputs ("ATTENTION:  Gripper is powered down.\n ", stderr);
            log_fputs ("            Energizing and homing stages.\n", stderr);
            gripIn ();
        }
        else
        {
            goxy ("0 0");
        }
    }
}

/*
		gripin

   power up and bring in the gripper.
*/

 /*ARGSUSED*/ void gripin (char *args)
{
    fprintf (graphCmds, "pushgrip in\n");
    fflush (graphCmds);

    log_fputs ("BRINGING IN GRIPPER.  (please wait).\n ", stderr);

    if (faith)
    {

        GripperParked = FALSE;
        goxy ("0 0");
    }

    else
    {

        /* led( "on" ); */
        gripIn ();
    }

    flushStatus ();

}

/*
		gripout

   move the gripper to its stowed position.
*/

 /*ARGSUSED*/ void gripout (char *args)
{

    fprintf (graphCmds, "pushgrip out\n");
    fflush (graphCmds);

    log_fputs ("STOWING GRIPPER.  (please wait).\n ", stderr);

    if (!faith && buttonPresent ())
    {
        if (FiberInTrouble.fiber != NULL)
        {
            sprintf (log_buf,
                     "ERROR:  It would be very bad to \"gripout\" with fiber #%d in the jaws..\n",
                     FiberInTrouble.fiber->fiber_id);
            log_fputs (log_buf, stderr);
        }
        else
        {
            log_fputs
                ("ERROR:  It would be very bad to park gripper with an unknown fiber in the jaws..\n",
                 stderr);
        }
    }
    else
    {
        gripOut ();
        flushStatus ();
    }

    led ("off");
}

/*
		hangle

   Set the global variable that holds the hour angle.
   This value is used in flexure calculations.
*/

void hangle (char *args)
{
    int foo;                    /* separate components that we won't use here */
    int sign;
    double tempHa;

    tempHa = converthms (args, &foo, &foo, &foo, &foo, &sign);
    PhysicalEnvironment.ha = (tempHa / 12) * 180.0;
    flushStatus ();
}

/*
		dangle

   Set the global variable that holds the declination angle.
   This value is used in flexure calculations.
*/

void dangle (char *args)
{
    int foo;                    /* separate components that we won't use here */
    int sign;
    double tempDec;

    tempDec = converthms (args, &foo, &foo, &foo, &foo, &sign);
    PhysicalEnvironment.dec = tempDec;
    flushStatus ();
}

/*
		hist

   Open a window and show the system history.
*/

 /*ARGSUSED*/ void hist (char *args)
{
    char commandString[128];

    sprintf (commandString, "sh ${%s}/%s&", FIBBIN, HISTORY_SCRIPT);

    system (commandString);

}

/*
		homeMotors

   Home the specified axis and reset the zero point.
*/

void homeMotors (char *args)
{
    int axis = name2ord (args);

    if (axis >= 0)
        if (axis != Z_ord)
            HomeMotors (axis);  /* axis is x or y */
        else
            HomeZ ();           /* axis is Z */

    else
        log_fputs (BADARG, stderr);
}

/*
		initMotors

   Send the commands to initialize the motors.
   (cold start )
*/

void initMotors (char *args)
{
    int axis = name2ord (args);

    if (axis >= 0)
        InitMotors (axis);
    else
        log_fputs (BADARG, stderr);
}

/*
	 listButt

   List the presumed location of a specified button.
*/

void listButt (char *args)
{
    int button;

    if ((sscanf (args, "%d", &button) == 1) &&
        (button >= 0) && (button < MAXFIBER))
    {
        (void) sprintf (log_buf,
                        "Button #%d should be at x = %ld, y = %ld which is %s\n",
                        button, FiBase[button].CurPos.x,
                        FiBase[button].CurPos.y,
                        parkStatusNames[(FiBase[button].parkedStatus)]);

        log_fputs (log_buf, stderr);
    }
    else

        log_fputs (BADARG, stderr);
}

/*
		listDump

   Open a window and show the processed nessie file.
*/

void listDump (char *args)
{
    char *dumpName;

    char commandString[512];

    switch (*args)
    {

        case 'i':
            adump ();           /* creates .iraf file */
            sendIRAFstuff ();   /* gwc call */
            break;

        case 'c':
            dumpName = bdump ((char *) NULL);
            sprintf (commandString, "sh ${%s}/%s %s&", FIBBIN, DUMP_SCRIPT,
                     dumpName);
            system (commandString);
            break;

        default:
            log_fputs ("Unknown arg to \"list\"\n", stderr);
            break;
    }
}

/*
		listLoc

   Poll the indexers for their current locations
   and print the responses.
*/

 /*ARGSUSED*/ void listLoc (char *args)
{
    printLocation ();
    if (!faith)
    {
        gripStatus ();
    }
}

/*
		listrot

   Print the current rotation angle.
*/

void listrot (char *args)
{
    char mode = 's';

    sscanf (args, "%c", &mode);

    switch (mode)
    {

        case 'a':

            (void) sprintf (log_buf, "Rotation angle = %f\n",
                            radToDeg (RotData.aveAngle));
            log_fputs (log_buf, stderr);
            break;

        case 'w':

            (void) sprintf (log_buf, "New rotation value = %f\n",
                            radToDeg (RotData.workingAngle));
            log_fputs (log_buf, stderr);
            break;

        case 'p':

            listRotationPts (&RotData);
            break;

        case 's':

            listRotationPrs (&RotData);
            break;

        default:
            log_fputs ("Unknown argument to 'listrot'\n", stderr);
            break;
    }
}

/*
		listTag

   Print the contents of a specified tag.
*/

void listTag (char *args)
{
    int id;

    if ((sscanf (args, "%d", &id) < 1))
        log_fputs ("Missing argument\n", stderr);

    else
    {

        if ((id >= 0) && (id < MAXTAGS))
            printTag (id);
        else
            log_fputs ("Invalid tag #\n", stderr);
    }
}

/*
		lit

   "Literal" command. This command sends its argument list directly to
   the indexers without any processing.
*/

void lit (char *args)
{
    /* void the current button since we might be moving! */

    currentButton = VOIDBUTTON;

    send_it (args);
}

/*
		markIt

   Mark the current button as the button to be remembered.
*/

void markIt (char *args)
{
    view (args);

    if (currentButton < 0)
        labtyd (1, "WARNING:  Don't know what button to mark!\n");
    else
    {
        rememberedButton = currentButton;
    }
}

/*
		offset

   Incrementally offset from the current position.
   Accept 2 to 4 arguments.  Substitute default values
   for missing arguments.
*/

void offset (char *args)
{
    long dx, dy;

    /* Prepare to run at limits if no arguments */

    double vel = BUMP_VEL;
    double accel = BUMP_ACCEL;

    /* void the current button since we are moving! */

    currentButton = VOIDBUTTON;

    if (sscanf (args, "%ld %ld %lf %lf", &dx, &dy, &vel, &accel) >= 2)
        offsetToXY (dx, dy, vel, accel);
    else
        log_fputs (BADARG, stderr);
}

/*
		offsetToXY

   Take an x,y offset, convert it to an absolute x,y.
   and go there.
*/

offsetToXY (long int dx, long int dy, double vel, double accel)
{
    struct aPoint target;

    int system;                 /* virtual or physical */

    target.x = location_now.x;
    target.y = location_now.y;

    /* un-apply the appropriate flexure corrections to get back to
       sky coordinates */

    coordCorrect (&target, &PhysicalEnvironment, INVERT);

    if (Rotflag)
    {

        /* un-rotate to get physical coords */

        rotate (&target, -(RotData.aveAngle));
        system = VIRTUAL;
    }

    else
        system = PHYSICAL;

    target.x += (double) dx;
    target.y += (double) dy;

    /* see if an unresolved move is pending and test range if a
       button is present */

    if (!unresolved ())
    {

        return (GoToXY ((long) target.x, (long) target.y, vel, accel, system)
                != 0);
    }

    else
    {

        /* there is a ton of stuff to do to validate a bump
           for a button in the jaws */

        if (xyValidForButton (FiberInTransit.fiber, &target))
        {

            return (GoToXY
                    ((long) target.x, (long) target.y, vel, accel, system));
        }

        else
        {
            log_fputs ("ERROR: Bad target for button in gripper!\n", stderr);
            return (1);
        }
    }
}

/*
		parkbutt

   Move a button to its park position.
*/

void parkbutt (char *args, int dummy, int cmdInfo, int checkRestrictions)
{
    xItem theList;
    int atTarget;

    char arg1[32];

    sscanf (args, " %s", arg1);

    atTarget = argPairToXref (arg1, &theList);

    /* no point in continuing if we have no fiber ID'd... */

    if (theList.fiber == VOIDENTRY)
        return;

    if (FiBase[theList.fiber].parkedStatus == PARKED)
    {
        /* sprintf (log_buf, "Fiber %d is already parked...\n", theList.fiber ); */
        /*log_fputs (log_buf, stderr); */
        return;
    }

    if (checkRestrictions)
    {

        /* fputs( "checking restrictions in park function\n", stderr ); */

        if (commandCurrentlyRestricted (cmdInfo, NULL, NO_DEFER) == UNSAFE)
        {
            sprintf (log_buf, "Unsafe to park fiber %d\n", theList.fiber);
            log_fputs (log_buf, stderr);
            return;
        }
        else
        {
            /* fputs( "defered park not restricted\n", stderr ); */
        }
    }

    if (buttonPresent ())
    {

        if ((FiberInTrouble.fiber != NULL) &&
            (theList.fiber != FiberInTrouble.fiber->fiber_id))
        {

            sprintf (log_buf,
                     "SORRY.. You can't park fiber %d with fiber %d in the gripper\n",
                     theList.fiber, FiberInTrouble.fiber->fiber_id);
            log_fputs (log_buf, stderr);

            return;
        }
    }

    /* if required, point out that it's now unassigned */

    if (atTarget)
    {
        sprintf (log_buf, "Object #%d  \"%s\" will now be unassigned.\n",
                 Stars[theList.object]->IDnumber,
                 Stars[theList.object]->name);
        log_fputs (log_buf, stderr);
    }

    /* now try to park it */

    ParkButt (theList.fiber);
    currentButton = theList.fiber;

    if (graphic)
    {
        fputs (" update \n", graphCmds);
        fflush (graphCmds);
        if (Configuring == FALSE)
        {
            setStats ("");
        }
    }

    /* notify the gwc world it's moved */

    if (fiberIsFop (theList.fiber))
    {
        tellWhoeverCares (&FiBase[theList.fiber]);
    }
}

void ping (char *args)
{
    Ping ();
}

void preview (char *args)
{
    int i;
    int first;
    int last;
    int time;
    long x;
    long y;

    char argString[32];

    if (sscanf (args, " %d %d %d", &first, &last, &time) < 3)
        labtyd (0, "Missing argument.");
    else
    {
        if ((first < 1) || (first > MAXSTARS) ||
            (last < 2) || (last > MAXSTARS) || (first >= last) || time < 1)
            labtyd (0, "Arguments do not make sense...");

        else
        {

            for (i = first; i <= last; i++)
            {
                if (Stars[i] != NULL)
                {

                    x = (long) Stars[i]->specEncoderCoord.x;
                    y = (long) Stars[i]->specEncoderCoord.y;

                    sprintf (argString, "%ld %ld\n", x, y);
                    goxy (argString);

                    sprintf (argString, "%d \n", time);
                    waitSec (argString);
                }
            }
        }
    }
}

/*
		progDelay

   Command the specified indexer to execute a delay before
   processing any additional commands.  
*/

void progDelay (char *args)
{
    char axis;
    int ordinal;
    double time;

    if (sscanf (args, "%c %lf", &axis, &time) >= 2)
    {

        ordinal = name2ord (&axis);

        if (ordinal != NULL_ORDINAL)
            ProgDelay (ordinal, time);
    }

    else
        log_fputs (BADARG, stderr);
}

/*
		purge

   Close, compress, and re-open the position archive.
*/

void purge (char *args)
{

    char *archiveFileName;

    char commandString[128];

    if (faith)
        archiveFileName = getenv (SIMFILE);
    else
        archiveFileName = getenv (DATAFILE);

    if (verbose)
    {
        sprintf (log_buf, "using fiber database: %s \n", archiveFileName);
        log_fputs (log_buf, stderr);
    }

    /* bail out if environment is bad. */

    if (archiveFileName == NULL)
    {
        log_fputs
            ("Couldn't find name of archive in user environment--purge aborted\n",
             stderr);

        return;
    }

    closeArchive (positionArchive);

    if (faith)
    {                           /*simulator is running */
        sprintf (commandString, "sh ${%s}/%s ${%s}",
                 FIBBIN, COMPRESS_SCRIPT, SIMFILE);
    }
    else
    {                           /* positioner is running */
        sprintf (commandString, "sh ${%s}/%s ${%s}",
                 FIBBIN, COMPRESS_SCRIPT, DATAFILE);
    }

    system (commandString);

    if ((positionArchive = openArchive (archiveFileName, READ_WRITE)) == NULL)
    {

        log_fputs
            ("FATAL ERROR: Could not open button archive after purge. EXITING!\n",
             stderr);

        bowOut ();
    }

    if (graphic)
    {
        fprintf (graphCmds, "restore\n");
        fflush (graphCmds);
    }

}

/*
		quit

   Exit the program. Send a SIGHUP to the other processes
   in the process group.
*/

 /*ARGSUSED*/ void quit (char *args)
{
    char commandString[128];

    log_fputs ("Turning off complamps\n", stderr);
    complamp ("off");

    if (!faith &&
        getYesNo (stderr, stdin, "Do you wish to turn OFF the rack?", TRUE))
    {

        log_fputs ("Turning off drives\n", stderr);
        stdACdrives (STD_ord, OFF);

        sleep (2);

        log_fputs ("Turning off indexers\n", stderr);
        stdACindexers (STD_ord, OFF);

        sleep (2);

        log_fputs ("Turning off calibration rack\n", stderr);
        stdACcalib (STD_ord, OFF);

        sleep (2);

        log_fputs ("Turning off pier power\n", stderr);
        stdACpier (STD_ord, OFF);

        sleep (2);

        log_fputs ("Turning off vacuum AC valve\n", stderr);
        stdACvalve (STD_ord, OFF);

        sleep (2);
    }

    fprintf (graphCmds, "quit\n");
    fflush (graphCmds);

    closeArchive (positionArchive);

    if (faith)
    {                           /*simulator is running */
        sprintf (commandString, "sh ${%s}/%s ${%s}",
                 FIBBIN, COMPRESS_SCRIPT, SIMFILE);
    }
    else
    {                           /* positioner is running */
        sprintf (commandString, "sh ${%s}/%s ${%s}",
                 FIBBIN, COMPRESS_SCRIPT, DATAFILE);
    }

    system (commandString);

    log_fputs ("Sending kill to grapics \n", stderr);

    sleep (5);

    if (!faith)
    {
        log_fputs ("Unlinking program lock.\n", stderr);
        unlink (LOCK_PATH);
    }

    /* we send this signal, so we need to ignore it */

    signal (SIGHUP, SIG_IGN);

    log_fputs ("Sending killpg to other processes \n", stderr);

    if (killpg (getpgrp (), SIGHUP))
    {
        log_fputs ("WARNING: couldn't kill other processes\n", stderr);
    }

    log_fputs ("killpg sent to other processes \n", stderr);

    unlink (SOCK_NAME);

    (void) bowOut ();

}

/*
		restore

   Restore the internal database of button locations
   from the disk archive file.
*/

 /*ARGSUSED*/ void restore (char *args)
{
    /* void the current button since we are changing the dbase. */

    currentButton = VOIDBUTTON;

    restoreFromArchive (positionArchive, FiBase, ALL);
}

/*
		rotmode

   Turn coordinate rotation on or off.
*/

void rotmode (char *args)
{
    int temp;

    if ((sscanf (args, "%d", &temp) > 0) && ((temp == 1) || (temp == 0)))
    {

        Rotflag = temp;
        flushStatus ();
    }

    else
        log_fputs (BADARG, stderr);
}

/*
		rotval

   Set the coordinate system rotation angle.  (radians!)
*/

void rotval (char *args)
{
    double temp;

    if ((sscanf (args, "%lf", &temp) > 0) &&
        ((temp < 90.0) && (temp > -90.0)))
    {

        RotData.aveAngle = (temp / 360.0) * TWOPI;
        flushStatus ();
    }

    else
        log_fputs ("ERROR:  Angle is too large/small! \n", stderr);
}

/*
		buttonRecover

   Deal  with a button in the jaws at power-up. 

*/

void buttonRecover (char *args)
{
    int button = -1;

    struct fiber_dbase *fib;

    int minButton = 0;
    int maxButton = 199;

    int hardwareCold = 0;

    struct aPoint parkVector;
    struct aPoint parkPoint;

    struct itimerval coldTimer;

    coldTimer.it_value.tv_sec = COLD_TIME;
    coldTimer.it_value.tv_usec = 0;
    coldTimer.it_interval.tv_sec = 0;
    coldTimer.it_interval.tv_usec = 0;

    if (faith)
    {
        log_fputs ("Real hardware would initialize Z now \n", stderr);
    }
    else
    {
        execfile (Z_BASIC_INIT);
    }

#ifndef HYDRA_II_TEST_MODE
    if (!buttonPresent ())
    {
        log_fputs ("The system can't find a button in the jaws... \n",
                   stderr);
        log_fputs ("So this error will be cleared. \n", stderr);

        FiberInTransit.fiber = NULL;
        FiberInTransit.id = -1;
        FiberInTransit.state = NO_TROUBLE;

        flushStatus ();
    }
    else
#endif
// HYDRA_II_TEST_MODE
    {                           /* there *is* a button */

        log_fputs ("WARNING: There IS a button in the gripper!\n", stderr);
        log_fputs
            ("You may continue the recovery process or exit the program.\n",
             stderr);

        if (getYesNo (stderr, stdin,
                      "\nAttempt to release ( or park if possible ) this button?",
                      TRUE))
        {

            /* attempt the recovery */

            log_fputs ("Attempting the recovery\n", stderr);

            /* identify the button if not identified already */

            if (FiberInTrouble.fiber == NULL)
            {
                /* user must identify */
                log_fputs
                    ("ATTENTION:  The software has no clue which button this is!\n",
                     stderr);
                log_fputs
                    ("You will need to identify it. If you can't, the only option \n",
                     stderr);
                log_fputs
                    ("will be to drop it here until you you can identify it. \n",
                     stderr);
                log_fputs
                    ("CAUTION: Do NOT guess.  Hardware damage could result!!\n",
                     stderr);
                if (getYesNo
                    (stderr, stdin, "\nDo you know which button this is?",
                     TRUE))
                {

                    button =
                        *(int *) getUserInput (stderr, stdin, INT, NO_DEFAULT,
                                               "What button is this? [ 0 to 199 ] ",
                                               NULL, &minButton, &maxButton,
                                               NULL);

                    sprintf (log_buf,
                             "setting fiber in transit and fiber in trouble to %d\n",
                             button);
                    log_fputs (log_buf, stderr);

                    FiberInTransit.fiber = &FiBase[button];
                    FiberInTransit.state = IN_TRANSIT;

                    FiberInTrouble.fiber = &FiBase[button];
                    FiberInTransit.state = CANT_DROP;

                }
                else
                {
                    labtyd (1, "Exiting button-in-gripper recovery.");
                }
            }
            else
            {
                button = FiberInTrouble.fiber->fiber_id;
            }

            /* check for rack cycle */

            if (PromptHw)
            {                   /* allows user to create errors */
                if (getYesNo (stderr, stdin, "Has the rack cycled?", TRUE))
                    hardwareCold = TRUE;
                else
                    hardwareCold = FALSE;
            }
            else
            {
                hardwareCold = SystemIsCold (&coldTimer);
            }

            if (!hardwareCold)
            {
                /* hardware is warm, sync software, and park it. */

                log_fputs ("Rack power has not been cycled.\n", stderr);
                log_fputs
                    ("Assuming indexers know valid location of gripper.\n",
                     stderr);

                if (faith)
                {
                    log_fputs
                        ("With real hardware, Sync to indexers would happen here\n",
                         stderr);
                }
                else
                {
                    syncToIndexers (&coldTimer);
                }

                printLocation ();

                if (FiberInTrouble.fiber != NULL)
                {
                    ParkButt (FiberInTrouble.fiber->fiber_id);
                }
                else
                {
                    log_fputs
                        ("Since this button is un-identified, and automatic collision prevention \n",
                         stderr);
                    log_fputs
                        ("is impossible, you must visually determine if the gripper \n",
                         stderr);
                    log_fputs
                        ("is over a save location for dropping the button.\n",
                         stderr);

                    log_fputs
                        ("Call for help if you don't understand this!! \n",
                         stderr);

                    if (getYesNo
                        (stderr, stdin, "Is the gripper over a safe area?",
                         FALSE))
                    {
                        /* drop it */

                        if (blindRelease () != 0)
                        {
                            log_fputs
                                ("WARNING:  This problem requires expert intervention.  Call for help.\n",
                                 stderr);
                            sleep (5);
                            labtyd (1, "can't clear button from gripper");
                        }
                        else
                        {
                            log_fputs
                                ("ATTENTION:  The unknown button was left at the location listed below\n",
                                 stderr);
                            log_fputs
                                ("Write the plate coordinate down!!! You will need it once you have\n",
                                 stderr);
                            log_fputs ("identified the button.\n", stderr);

                            printLocation ();
                        }
                    }
                    else
                    {
                        log_fputs ("Then please call for help!!!\n", stderr);
                        labtyd (1, "can't clear button from gripper");
                    }
                }

            }
            else
            {                   /* hardware was cold */

                fib = FiberInTrouble.fiber;     /* because fiber in trouble disappears */

                log_fputs ("The electronics rack HAS been power cycled.\n",
                           stderr);

                if ((LastXY.x != VOID_COORD) && (LastXY.y != VOID_COORD))
                {
                    /* we managed to save the current gripper coord before the
                       rack was cycled */

                    log_fputs
                        ("Current gripper coord was recorded before failure.\n",
                         stderr);
                    sprintf (log_buf, "It would seem we are at: %ld, %ld\n",
                             LastXY.x, LastXY.y);
                    log_fputs (log_buf, stderr);

                    parkVector.x = fib->ParkPos.x - LastXY.x;
                    parkVector.y = fib->ParkPos.y - LastXY.y;

                    parkPoint.x = fib->ParkPos.x;
                    parkPoint.y = fib->ParkPos.y;

                    sprintf (log_buf, "Park vector is: %d, %d\n",
                             (int) parkVector.x, (int) parkVector.y);
                    log_fputs (log_buf, stderr);

                    if (prepNewLocation (fib, &parkPoint))
                        labtyd (SYSTEM_CALL,
                                "Couldn't prep. new location in database");

                    DisposeButt (fib, &parkVector);

                    setLocation (fib, &fib->ParkPos, UPDATE_FILE);

                    fib->parkedStatus = PARKED;

                }
                else
                {               /* the current gripper coord is unknown */

                    /* exec "drop it here" script, and try to find out
                       where it was left. */

                    if (getYesNo (stderr, stdin,
                                  "\nAre you trained to carry out this recovery process?",
                                  TRUE))
                    {
                        releaseButtonAfterCycle ();

                    }
                    else
                    {
                        log_fputs
                            ("Do NOTHING for now and call an authorized person for help!!! \n",
                             stderr);
                        labtyd (1, "Exiting button-in-gripper recovery.");
                    }
                }

            }
        }
        else
        {
            labtyd (1, "Exiting button-in-gripper recovery.");
        }
    }
}

/* 
		setrot

   record a rotation point.  The code assumes that the
   two tags have been set to the appropriate values.
   If no name is given, a default name is recorded.
*/

void setrot (char *args)
{
    char name[64];

    if (sscanf (args, "%s", name) > 0)
        addRotationPt (&RotData, name, &Tags[STARTAG], &Tags[MEASTAG]);
    else
        addRotationPt (&RotData, DEFAULTNAME, &Tags[STARTAG], &Tags[MEASTAG]);
}

/*
		status

   Print a summary of the system status.
*/

 /*ARGSUSED*/ void status (char *args)
{
    sysStatusDump (&Exposure, stderr);
    fiborgStatusDump (stderr);
}

/*
		tag

   Tag the specified location for future reference
*/

void tag (char *args)
{
    int tagID;

    struct plate_coords pt;

    int count;

    count = sscanf (args, "%d %ld %ld", &tagID, &pt.x, &pt.y);

    switch (count)
    {

        case 0:                /* no arguments present */

            log_fputs ("Invalid or missing tag #\n", stderr);
            break;

        case 1:                /* only the tag id is specified */

            if ((tagID >= 0) && (tagID < MAXTAGS))
                tagCoord (tagID);
            else
                log_fputs ("Invalid tag #\n", stderr);
            break;

        case 2:                /* tag and one coordinate */

            log_fputs ("Incorrect number of arguments\n", stderr);
            break;

        case 3:                /* tag and two coordinates */

            if ((tagID < 0) || (tagID >= MAXTAGS))
            {
                log_fputs ("Invalid tag #", stderr);
                break;
            }
            else
                defTag (tagID, &pt);
            break;

        default:               /* more than three arguments */

            log_fputs ("Too many arguments-command ignored.\n", stderr);
            break;
    }
}

void tweak (char *args)
{
    markIt (args);

    if (LastViewOK)
    {
        if (cruise ("center on star") == 0)
        {
            correctButton (NULL);
        }
        else
        {
            log_fputs ("tweak abandoned: Button not moved. \n", stderr);
        }
    }
    else
    {
        log_fputs ("Bad argument(s) to tweak. \n", stderr);
    }

}

/*
		unstow

   Move a button from its stow position to its park position.
*/

void unstow (char *args)
{
    int id;

    if ((sscanf (args, "%d", &id) > 0) && (id >= 0) && (id < MAXFIBER))
    {
        Unstow (id);
        currentButton = id;
    }
    else
        log_fputs (BADBUTT, stderr);
}

/*
		updateStats

   Compute the number of assigned fibers and send report to
   the graphics window.
*/

void setStats (char *args)
{
    int objCount;
    int skyCount;
    int fopCount;

    generateStats (&objCount, &skyCount, &fopCount);

    if (graphic)
    {

        fprintf (graphCmds, "setstats %d %d %d\n",
                 objCount, skyCount, fopCount);

        fflush (graphCmds);
    }
}

void view (char *args)
{
    xItem theList;
    int atTarget;

    int x;
    int y;

    char objectString[128];
    char fiberString[32];
    char slitString[32];
    char argString[32];

    atTarget = argPairToXref (args, &theList);

    /* deal with a button present */

    if (!faith && buttonPresent ())
    {
        if (FiberInTrouble.fiber != NULL)
        {
            if (FiberInTrouble.fiber->fiber_id == theList.fiber)
            {
                sprintf (log_buf,
                         "ATTENTION:  Gripper is currently holding button #%d\n",
                         theList.fiber);
                log_fputs (log_buf, stderr);
                log_fputs ("So you are already viewing it...\n", stderr);
            }
            else
            {
                sprintf (log_buf,
                         "WARNING:  Impossible to view button #%d with #%d in the gripper!\n",
                         theList.fiber, FiberInTrouble.fiber->fiber_id);
                log_fputs (log_buf, stderr);
            }
        }
        else
        {
            sprintf (log_buf,
                     "WARNING:  Impossible to go view button #%d with an unknown button in the gripper!\n",
                     theList.fiber);
            log_fputs (log_buf, stderr);
        }
        return;
    }

    if (theList.fiber != VOIDENTRY)
    {

        /* there is a button that can be referenced */
        sprintf (argString, "%d\n", theList.fiber);
        buttonview (argString);

        if (FiberInTrouble.fiber != NULL)
        {
            if (FiberInTrouble.fiber->fiber_id == theList.fiber)
            {
                log_fputs ("This is the \"fiber in trouble\"\n", stderr);
            }
        }
    }
    else
    {

        /* see if we have a object that is valid */
        if (theList.object != VOIDENTRY)
        {

            if ((theList.object > MAXSTARS)
                || (Stars[theList.object] == NULL))
            {
                log_fputs ("Requested object does not exist.\n", stderr);
                LastViewOK = FALSE;
                return;
            }

            else
            {
                x = (long) Stars[theList.object]->specEncoderCoord.x;
                y = (long) Stars[theList.object]->specEncoderCoord.y;

                sprintf (argString, "%d %d\n", x, y);
                goxy (argString);
            }
        }
        else
        {
            log_fputs ("view arguments don't lead anywhere\n", stderr);
            LastViewOK = FALSE;
            return;
        }
    }

    /* if we get here, we went somewhere, so print a message */

    if (theList.slit != VOIDENTRY)
    {

        if (theList.fiber != VOIDENTRY)
            sprintf (fiberString, "fiber: %d ", theList.fiber);
        else
            sprintf (fiberString, "unassigned &");

        if (fiberIsFop (theList.fiber))
        {
            sprintf (slitString, "Probe %c ", slitToProbeID (theList.slit));
        }
        else
        {
            sprintf (slitString, "at slit pos: %d ", theList.slit);
        }
    }
    else
    {
        sprintf (fiberString, "No fiber attached; ");
        sprintf (slitString, "unreferenced to slit");
    }

    if (theList.object != VOIDENTRY)
    {
        sprintf (objectString, "Viewing obj. #%d \"%s\"  ",
                 Stars[theList.object]->IDnumber,
                 Stars[theList.object]->name);
    }
    else
    {
        sprintf (objectString, "Viewing unassigned  ");
    }

    sprintf (log_buf, "%s%s%s\n", objectString, fiberString, slitString);
    log_fputs (log_buf, stderr);

    if ((theList.fiber != VOIDENTRY) && fiberBroken (theList.fiber))
    {
        log_fputs ("This fiber is broken!\n", stderr);
    }

    LastViewOK = TRUE;
}

/*
		waitSec

   Suspend software execution for the time specified in the argument
   in seconds.  This does not pause the indexers!
*/

void waitSec (char *args)
{
    unsigned time;
    char dummy;

    if (sscanf (args, "%u", &time) > 0)
    {
        if (time <= 0)
        {
            log_fputs ("HALTED:  Press <return> to continue\n", stderr);
            dummy = fgetc (stdin);
            log_fputs ("CONTINUING\n", stderr);
        }
        else
        {
            sleep (time);
        }
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		whamMotors

   Execute the commands that guarantee obtaining control of a
   (possibly) runaway indexer.  After executing this sequence,
   an initialization will always (barring hardware failure)
   restore control of a hung or lost axis.
*/

void whamMotors (char *args)
{
    int axis;

    axis = name2ord (args);

    if (axis >= 0)
        WhamMotors (axis);
    else
        log_fputs (BADARG, stderr);
}

void whereis (char *args)
{
    xItem theList;
    int atTarget;

    int x;
    int y;

    char objectString[128];
    char fiberString[32];
    char slitString[32];

    atTarget = argPairToXref (args, &theList);

    if ((theList.object == VOIDENTRY) && (theList.fiber == VOIDENTRY))
    {
        log_fputs ("whereis arguments don't lead anywhere\n", stderr);
        return;
    }

    /* fprintf( stderr, "xref:  fiber= %d  slit= %d  object=%d\n", theList.fiber, theList.slit, theList.object ); */

    /* if we get here, we went somewhere, so print a message */

    if (theList.fiber != VOIDENTRY)
        sprintf (fiberString, "fiber: %d ", theList.fiber);
    else
        sprintf (fiberString, "unassigned &");

    if ((theList.slit != VOIDENTRY) &&
        (theList.fiber != VOIDENTRY) &&
        (fiberInCable (theList.fiber, CurrentCable)))
    {

        if (fiberIsFop (theList.fiber))
        {
            sprintf (slitString, "Probe %c ", slitToProbeID (theList.slit));
        }
        else
        {
            sprintf (slitString, "at slit pos: %d ", theList.slit);
        }
    }
    else
    {
        sprintf (slitString, "NOT IN CURRENT CABLE");
    }

    if ((atTarget) && (theList.object != VOIDENTRY))
    {
        sprintf (objectString, "Viewing obj. #%d \"%s\"  ",
                 Stars[theList.object]->IDnumber,
                 Stars[theList.object]->name);
    }
    else
    {
        sprintf (objectString, "Unassigned  ");
    }

    sprintf (log_buf, "%s%s%s\n", objectString, fiberString, slitString);
    log_fputs (log_buf, stderr);

    /* see if we have a object that is valid */

    if ((theList.object != VOIDENTRY) && (theList.fiber == VOIDENTRY))
    {

        if ((theList.object > MAXSTARS) || (Stars[theList.object] == NULL))
        {
            log_fputs ("Requested object does not exist.\n", stderr);
            return;
        }

        else
        {
            x = (long) Stars[theList.object]->specEncoderCoord.x;
            y = (long) Stars[theList.object]->specEncoderCoord.y;

            sprintf (log_buf, "That object is at %d %d\n", x, y);
            log_fputs (log_buf, stderr);
        }

        if (graphic)
        {

            fprintf (graphCmds, "showobj %d\n", theList.object);
            fflush (graphCmds);
        }

    }

    if ((theList.fiber != VOIDENTRY) && (theList.fiber < MAXFIBER))
    {

        if ((FiberInTrouble.fiber != NULL) &&
            (FiberInTrouble.fiber->fiber_id == theList.fiber))
        {

            if (buttonPresent ())
            {
                sprintf (log_buf, "Button #%d is in the Gripper!!!!\n",
                         theList.fiber);
                log_fputs (log_buf, stderr);
                printLocation ();
            }

            else
            {

                (void) sprintf (log_buf,
                                "Button #%d is in trouble.\n", theList.fiber);
                log_fputs (log_buf, stderr);

                (void) sprintf (log_buf,
                                "It was last known to be at x = %ld, y = %ld\n",
                                FiBase[theList.fiber].CurPos.x,
                                FiBase[theList.fiber].CurPos.y);
                log_fputs (log_buf, stderr);

            }
        }
        else
        {
            (void) sprintf (log_buf,
                            "Button #%d is at x = %ld, y = %ld which is %s\n",
                            theList.fiber, FiBase[theList.fiber].CurPos.x,
                            FiBase[theList.fiber].CurPos.y,
                            parkStatusNames[(FiBase[theList.fiber].
                                             parkedStatus)]);
            log_fputs (log_buf, stderr);
        }

        if (fiberBroken (theList.fiber))
        {
            log_fputs ("This fiber is broken!\n", stderr);
        }

        if (graphic)
        {

            fprintf (graphCmds, "showfib %d\n", theList.fiber);
            fflush (graphCmds);
        }
    }
}

void remoteXref (char *args)
{
    xItem theList;
    int atTarget;

    atTarget = argPairToXref (args, &theList);

    if (graphic)
    {

        fprintf (graphCmds, "setxr %d %d %d \n", theList.fiber,
                 theList.object, theList.slit);
        fflush (graphCmds);
    }

}

/*
	 xparked

   Force the software to update a buttons location to be the
   parked position 
*/

void xparked (char *args)
{
    int button;

    if ((sscanf (args, "%d", &button) == 1) &&
        (button >= 0) && (button < MAXFIBER))
    {

        /* set to parked position */

        setLocation (&FiBase[button], &FiBase[button].ParkPos, UPDATE_FILE);
        FiBase[button].parkedStatus = PARKED;

    }
    else
        log_fputs (BADBUTT, stderr);
}

/*
	 xstowed

   Force the software to update a buttons location to be the
   stowed position 
*/

void xstowed (char *args)
{
    int button;

    if ((sscanf (args, "%d", &button) == 1) &&
        (button >= 0) && (button < MAXFIBER))
    {

        /* set to parked position */

        if ((FiberInTrouble.fiber != NULL) &&
            (FiberInTrouble.fiber->fiber_id == button))
        {

            FiberInTrouble.fiber = NULL;
        }

        setLocation (&FiBase[button], &FiBase[button].StowPos, UPDATE_FILE);
        FiBase[button].parkedStatus = STOWED;
    }
    else
        log_fputs (BADBUTT, stderr);
}

/*
		zdb
   Zero the internal database.  An argument of 0
   specifies that the buttons will be assumed to be in
   their park positions.  An argument of 1 specifies
   the stow position.
*/

void zdb (char *args)
{
    int flag;

    if (sscanf (args, "%d", &flag) >= 1)
    {
        setDbase (flag);
        currentButton = VOIDBUTTON;
        if (graphic)
        {
            fputs (" update \n", graphCmds);
            fflush (graphCmds);
        }
    }
    else
        log_fputs (BADARG, stderr);
}

/*
		zlight

   Turn the light in the z axis opto-interrupter on or off.
   This also affects the brakes.
*/

void zlight (char *args)
{
    int flag;

    if (sscanf (args, "%d", &flag) > 0)
        (void) Zlight (flag);
    else
        log_fputs (BADARG, stderr);
}

/* These two functions are temporary functions for setting
the value of variables used in gravity flexure calculations */

void thetaval (char *args)
{
    double temp;

    if ((sscanf (args, "%lf", &temp) > 0) &&
        ((temp < 90.0) && (temp > -90.0)))
        PhysicalEnvironment.theta = temp;

    else
        log_fputs ("ERROR:  Angle is too large/small! \n", stderr);
}

void phival (char *args)
{
    double temp;

    if ((sscanf (args, "%lf", &temp) > 0) &&
        ((temp < 90.0) && (temp > -90.0)))
        PhysicalEnvironment.phi = temp;

    else
        log_fputs ("ERROR:  Angle is too large/small! \n", stderr);
}

void setCable (char *args)
{
    char cableName[32];
    char cablePrompt[255];

    char tmpName[255];
    char tmpNameClean[255];

    int exit = FALSE;

    sprintf (cablePrompt, "\nEnter cable name [%s]", getCableList ());

    strcpy (cableName, getNameForCable (CurrentCable));

    while (!exit)
    {

        if (args[0] == '\0')
        {
            strcpy (tmpName, getUserInput (stderr, stdin, CHAR, SHOW_DEFAULT,
                                           cablePrompt, cableName, NULL, NULL,
                                           NULL));
        }
        else
        {
            strcpy (tmpName, args);
        }

        /* clean up any trash at end */

        sscanf (tmpName, "%s", tmpNameClean);

        exit = ((CurrentCable = getCableIdentifier (tmpNameClean)) != -99);

        if (exit)
        {
            continue;
        }
        else
        {
            sprintf (log_buf, " Sorry, only \"%s\" are acceptable...\n",
                     getCableList ());
            log_fputs (log_buf, stderr);
            args[0] = '\0';
        }

    }

    flushStatus ();             /* makes sure that defaults get written back out */

    sendCableStuff ();

    if (graphic)
    {

        fprintf (graphCmds, "setcable %s \n", tmpNameClean);
        fflush (graphCmds);
    }
}

void syncCable (void)
{
    char cableName[32];

    strcpy (cableName, getNameForCable (CurrentCable));

    fprintf (graphCmds, "setcable %s \n", cableName);
    fflush (graphCmds);
}

void setOffset (char *args)
{

    double angle;

    if (sscanf (args, "%lf", &angle) > 0)
    {

        FieldRotationalOffset = angle * DEG2RAD;
        log_fputs ("Angle set.\n", stderr);
        log_fputs ("NOTE:  Loading a field will override this value!\n",
                   stderr);
    }
    else
    {
        log_fputs ("ERROR:  Bad value.  --try again...\n", stderr);
    }
}

void graphCmd (char *args)
{
    /* send the command and it's arguments to the other window */

    if (graphic)
    {

        fprintf (graphCmds, "%s %s\n", currentCommand, args);
        fflush (graphCmds);
    }
}

/****************** Std Bus user words  **************************/

void complamp (char *args)
{
    int tokens[5];
    int response[12];
    int cableBits[12];
    int printStatus = FALSE;
    int testMode = LAMP_NULL;
    int whichCable;

    tokenize (args, tokens, TokenTable);

    if (faith)
    {

        switch (tokens[0])
        {

            case TOK_ON:
                fprintf (graphCmds, "showcomp 1 1 1 1 1 1 1 1\n");
                fprintf (graphCmds, "pushcomp on\n");
                fflush (graphCmds);
                break;

            case TOK_OFF:
                fprintf (graphCmds, "showcomp 0 0 0 0 0 0 0 0\n");
                fprintf (graphCmds, "pushcomp off\n");
                fflush (graphCmds);
                break;

            default:
                break;

        }
        return;
    }

    /* find out which cable is connected */

    if (lampCableStatus (cableBits, TH_CONNECTED) == TRUE)
    {
        whichCable = TH_CONNECTED;
    }
    else
    {
        if (lampCableStatus (cableBits, CU_CONNECTED) == TRUE)
        {
            whichCable = CU_CONNECTED;
        }
        else
        {
            log_fputs ("ERROR: No calibration lamp cable installed!\n",
                       stderr);
            return;
        }
    }

    switch (tokens[0])
    {

        case TOK_ON:

            cameraSafetyInterlock ();

            if (screenLocked ())
            {
                log_fputs ("SORRY: Lamps disabled by screen safety switch!\n",
                           stderr);
                testMode = LAMP_NULL;
            }
            else
            {

                switch (whichCable)
                {

                    case TH_CONNECTED:
                        stdTHlamps (STD_ord, 1);
                        break;

                    case CU_CONNECTED:
                        stdCUlamps (STD_ord, 1);
                        break;

                    default:
                        break;
                }

                fprintf (graphCmds, "pushcomp on\n");
                fflush (graphCmds);

                testMode = LAMP_ON;
            }
            break;

        case TOK_OFF:

            if (screenLocked ())
            {
                log_fputs ("WARNING: Screen safety switch is active!\n",
                           stderr);
                log_fputs ("         You won't be able to turn the lamps\n",
                           stderr);
                log_fputs ("         back on until you reset the switch...\n",
                           stderr);
            }

            switch (whichCable)
            {

                case TH_CONNECTED:
                    stdTHlamps (STD_ord, 0);
                    break;

                case CU_CONNECTED:
                    stdCUlamps (STD_ord, 0);
                    break;

                default:
                    break;
            }

            fprintf (graphCmds, "pushcomp off\n");
            fflush (graphCmds);

            testMode = LAMP_OFF;
            break;

        case TOK_STATUS:

            switch (whichCable)
            {

                case TH_CONNECTED:
                    log_fputs ("TH cable is installed\n", stderr);
                    break;

                case CU_CONNECTED:
                    log_fputs ("CU cable is installed\n", stderr);
                    break;

                default:
                    break;
            }

            printStatus = TRUE;
            break;

        case TOK_TEST:
            fputs ("lamptest\n", stderr);
            break;

        default:
            log_fputs ("ERROR:  Bad argument\n", stderr);
            break;

    }

    if (!faith)
    {
        if (lampStatus (response, testMode) == LAMP_OK)
        {
            if (!printStatus)
                lampShortMessage (whichCable, testMode);
        }
        else
        {
            log_fputs ("WARNING: Lamps are not consistent\n", stderr);
            printStatus = TRUE;
        }

        if (printStatus == TRUE)
            printLampStatus (response);

        fprintf (graphCmds, "showcomp %d %d %d %d %d %d %d %d\n",
                 !response[LAMP1], !response[LAMP2], !response[LAMP3],
                 !response[LAMP4], !response[LAMP5], !response[LAMP6],
                 !response[LAMP7], !response[LAMP8]);

        fflush (graphCmds);
    }
}

lampShortMessage (int whichCable, int testMode)
{
    switch (whichCable)
    {

        case TH_CONNECTED:
            genericOnOffMessage ("TH lamp set", testMode);
            break;

        case CU_CONNECTED:
            genericOnOffMessage ("CU lamp set", testMode);
            break;

        default:
            break;
    }
}

void qtz (char *args)
{
    int tokens[5];
    int response[12];
    int printStatus = FALSE;
    int testMode = LAMP_NULL;

    if (faith)
        return;

    tokenize (args, tokens, TokenTable);

    switch (tokens[0])
    {

        case TOK_ON:

            cameraSafetyInterlock ();

            stdQZlamps (STD_ord, 1);
            testMode = LAMP_ON;
            break;

        case TOK_OFF:
            stdQZlamps (STD_ord, 0);
            testMode = LAMP_OFF;
            break;

        case TOK_STATUS:
            printStatus = TRUE;
            break;

        case TOK_TEST:
            fputs ("lamptest\n", stderr);
            break;

        default:
            log_fputs ("ERROR:  Bad argument\n", stderr);
            break;

    }

    if (lampStatusQZ (response, testMode) != LAMP_OK)
    {

        log_fputs ("WARNING: QZ lamp status is not consistent\n", stderr);
        printStatus = TRUE;
    }

    if (printStatus == TRUE)
        printLampStatusQZ (response);

}

int cameraSafetyInterlock (void)
{
    int userResponse;

    /* make sure the high voltage is off first... */

    log_fputs ("Checking Gripper and FOPS camera(s)\n", stderr);

    while (cameraVulnerable ())
    {

        log_fputs ("Please have OT turn down high voltage on camera(s)\n",
                   stderr);

        userResponse =
            getYesNoQuit (stderr, stdin, "\n  Are the cameras off?", FALSE);

        switch (userResponse)
        {

            case QUIT:
                labtyd (0, "ATTENTION:  abandoning current command!");
                break;

            case TRUE:
            case FALSE:
            default:
                break;
        }
    }
}

void plate (char *args)
{
    int tokens[5];
    int response[12];
    int printStatus = FALSE;
    int testMode = VAC_NULL;

    tokenize (args, tokens, TokenTable);

    if (faith)
    {

        switch (tokens[0])
        {

            case TOK_FLAT:
                fprintf (graphCmds, "showplate flat\n");
                fflush (graphCmds);
                break;

            case TOK_WARP:
                fprintf (graphCmds, "showplate warp\n");
                fflush (graphCmds);
                break;

            default:
                break;

        }
        return;
    }

    vacuumStatus (response, VAC_NULL);

    switch (tokens[0])
    {

        case TOK_FLAT:

            if (response[FP_VALVE] != VAC_AMBIENT)
            {

                fprintf (graphCmds, "showplate flat\n");
                fflush (graphCmds);

#ifdef HYDRA_II
#else
                stdVacSwitch (STD_ord, 1);
#endif
// HYDRA_II
                sleep (7);      /* allows pressures to adjust */
            }

            testMode = VAC_AMBIENT;
            break;

        case TOK_WARP:

            if (response[FP_VALVE] != VAC_ON)
            {

                fprintf (graphCmds, "showplate warp\n");
                fflush (graphCmds);

#ifdef HYDRA_II
#else
                stdVacSwitch (STD_ord, 0);
#endif
// HYDRA_II
                sleep (12);     /* allows pressures to adjust */
            }

            testMode = VAC_ON;
            break;

        case TOK_VAC:
            if (tokens[1] == TOK_ON)
                stdACpier (STD_ord, 1);
            else
                stdACpier (STD_ord, 0);
            break;

        case TOK_STATUS:
            printStatus = TRUE;
            break;

        case TOK_INIT:
            /* regardless of it's current position, send it to
               the flat position to insure that the ac is on
               to the sensor switch */

            fprintf (graphCmds, "showplate flat\n");
            fflush (graphCmds);

            stdVacSwitch (STD_ord, 1);
            testMode = VAC_NULL;

            break;

        case TOK_TEST:
            vacDiag ();
            break;

        default:
            log_fputs ("ERROR:  Bad argument\n", stderr);
            break;

    }

    if (!faith)
    {
        if (vacuumStatus (response, testMode) != VAC_OK)
        {

            log_fputs ("WARNING: Vacuum test points are not consistent\n",
                       stderr);
            printStatus = TRUE;
        }

#ifdef ignore

        else
        {                       /* we got where we planned to go */

            /* now decide how to display the plate in the graphics */

            switch (tokens[0])
            {

                case TOK_FLAT:
                    fprintf (graphCmds, "showplate flat\n");
                    fflush (graphCmds);
                    break;

                case TOK_WARP:
                    fprintf (graphCmds, "showplate warp");
                    fflush (graphCmds);
                    break;

                default:
                    break;

            }
        }

#endif

        if (printStatus == TRUE)
            printVacStatus (response);
    }

}

/* handy status function for the plate */

int plateFlat (void)
{
    int response[12];

    (void) vacuumStatus (response, VAC_NULL);

    if (response[FP_VAC] == VAC_AMBIENT)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }

}

void led (char *args)
{
    int tokens[5];
    int response[12];
    int printStatus = FALSE;
    int testMode = LAMP_NULL;

    static int level;

    tokenize (args, tokens, TokenTable);

    if (faith)
    {
        switch (tokens[0])
        {

            case TOK_ON:
                fprintf (graphCmds, "showled on\n");
                fprintf (graphCmds, "pushled on\n");
                fflush (graphCmds);
                break;

            case TOK_OFF:
                fprintf (graphCmds, "showled off\n");
                fprintf (graphCmds, "pushled off\n");
                fflush (graphCmds);
                break;

            default:
                break;
        }

        return;
    }

    switch (tokens[0])
    {

        case TOK_ILLUM:

            if ((tokens[1] == TOK_LITERAL) &&
                (sscanf (tokens[2], "%d", &level) >= 1) &&
                (level > 0) && (level < 9))
            {

                /* fprintf( stderr, " level is %d \n", level ); */
            }
            else
            {
                log_fputs ("ERROR: bad value to 'led illum'\n", stderr);
            }

            break;

        case TOK_ON:
#ifdef HYDRA_II
            mc_exe_cmd ("leds on");
#else
            stdGripLED (STD_ord, level);
#endif
// HYDRA_II
            testMode = LAMP_ON;
            fprintf (graphCmds, "showled on\n");
            fprintf (graphCmds, "pushled on\n");
            fflush (graphCmds);
            break;

        case TOK_OFF:
#ifdef HYDRA_II
            mc_exe_cmd ("leds off");
#else
            stdGripLED (STD_ord, 0);
#endif
// HYDRA_II
            testMode = LAMP_OFF;
            fprintf (graphCmds, "showled off\n");
            fprintf (graphCmds, "pushled off\n");
            fflush (graphCmds);
            break;

        case TOK_STATUS:
            fputs ("led status\n", stderr);
            printStatus = TRUE;
            break;

        case TOK_TEST:

            ledDiag ();
            break;

        default:
            log_fputs ("ERROR:  Bad argument\n", stderr);
            break;
    }

#ifndef HYDRA_II
    usleep (500000);

    if (!faith)
    {
        if (ledStatus (response, testMode) != LAMP_OK)
        {

            log_fputs ("WARNING: LED status is not consistent\n", stderr);
            printStatus = TRUE;
        }

        if (printStatus == TRUE)
            printLEDStatus (response, level);
    }
#endif
// HYDRA_II
}

void screen (char *args)
{
    int tokens[5];
    int response[12];
    int printStatus = FALSE;
    int testMode = LAMP_NULL;

    tokenize (args, tokens, TokenTable);

    if (faith)
    {                           /* special case for the simulator */

        switch (tokens[0])
        {

            case TOK_IN:

                if (graphic)
                    fprintf (graphCmds, "showscreen in\n");
                fflush (graphCmds);
                break;

            case TOK_OUT:

                if (graphic)
                    fprintf (graphCmds, "showscreen out\n");
                fflush (graphCmds);
                break;

            default:
                log_fputs ("SORRY, not valid in Simulator...\n", stderr);
                break;

        }

        return;
    }

    switch (tokens[0])
    {

        case TOK_IN:

            if (screenLocked ())
            {

                log_fputs ("SORRY: Screen disabled by safety switch!\n",
                           stderr);
            }
            else
            {

                if (graphic)
                {
                    /* fprintf( graphCmds, "showscreen in\n" ); */
                }

                stdScreen (STD_ord, 0);
                sleep (3);

                testMode = SCREEN_IN;
            }
            break;

        case TOK_OUT:

            if (screenLocked ())
            {
                log_fputs ("SORRY: Screen disabled by safety switch!\n",
                           stderr);
            }
            else
            {

                if (graphic)
                {
                    /* fprintf( graphCmds, "showscreen out\n" ); */
                }

                stdScreen (STD_ord, 1);
                sleep (3);

                testMode = SCREEN_OUT;
            }
            break;

        case TOK_STATUS:
            printStatus = TRUE;
            break;

        case TOK_TEST:
            screenDiag ();
            break;

        default:
            log_fputs ("ERROR:  Bad argument\n", stderr);
            break;

    }

    if (!faith)
    {
        if (screenStatus (response, testMode) != SCREEN_OK)
        {

            if (graphic)
                fprintf (graphCmds, "showscreen out\n");
            fflush (graphCmds);

            log_fputs ("WARNING: Screen status inconsistent with command\n",
                       stderr);
            printStatus = TRUE;
        }
        else
        {

            if (testMode == SCREEN_OUT)
            {
                fprintf (graphCmds, "showscreen out\n");
                fflush (graphCmds);
            }
            if (testMode == SCREEN_IN)
            {
                fprintf (graphCmds, "showscreen in\n");
                fflush (graphCmds);
            }
        }

        if (printStatus == TRUE)
            printScreenStatus (response);
    }
}

syncScreen (void)
{
    if (screenIsIn ())
    {
        fprintf (graphCmds, "showscreen in\n");
        fflush (graphCmds);
    }
    else
    {
        fprintf (graphCmds, "showscreen out\n");
        fflush (graphCmds);
    }
}

int screenIsIn (void)
{
    int flags[12];

    if (faith)
        return (FALSE);
    else
        return (screenStatus (flags, SCREEN_IN) == SCREEN_OK);
}

screenLocked (void)
{
    int flags[12];

    return (lockoutStatus (flags, LOCKOUT_OFF) == LOCKOUT_OK);
}

void tv (char *args)
{
    int responseFops[12];
    int responseGrip[12];
    int statusCodeFops;
    int statusCodeGrip;

    if (faith)
    {
        log_fputs ("SORRY, Can't test tv's in simulator\n", stderr);
        return;
    }

    statusCodeFops = tvStatusFops (responseFops);
    statusCodeGrip = tvStatusGrip (responseGrip);

    printTvStatus ("FOPS", statusCodeFops);
    printTvStatus ("Gripper", statusCodeGrip);
}

cameraVulnerable (void)
{

    int responseFops[12];
    int responseGrip[12];
    int statusCodeFops;
    int statusCodeGrip;

    int vulnerable = FALSE;

    if (faith)
    {
        return (FALSE);
    }

    statusCodeFops = tvStatusFops (responseFops);
    statusCodeGrip = tvStatusGrip (responseGrip);

    if (statusCodeGrip == TV_HIVOLTAGE_ON)
    {
        log_fputs ("GRIPPER TV IS ON\n", stderr);
        vulnerable = TRUE;
    }

    if (statusCodeFops == TV_HIVOLTAGE_ON)
    {
        log_fputs ("FOPS TV IS ON\n", stderr);
        vulnerable = TRUE;
    }

    return (vulnerable);
}

void hinge (char *args)
{
    if (faith)
    {
        log_fputs ("SORRY, Can't test hinge in simulator\n", stderr);
        return;
    }

    if (hingeOpen ())
    {
        log_fputs ("ATTENTION:  The focal plane hinge is OPEN!!\n", stderr);
    }
    else
    {
        log_fputs ("The focal plane hinge is CLOSED.\n", stderr);
    }
}

void power (char *args)
{
    int tokens[5];
    int onOffMode = 0;

    if (faith)
        return;

    tokenize (args, tokens, TokenTable);

    switch (tokens[1])
    {

        case TOK_ON:
            onOffMode = 1;
            break;

        case TOK_OFF:
            onOffMode = 0;
            break;

        default:
            log_fputs
                ("ERROR:  Bad third argument.  \"on\" or \"off\" only!\n",
                 stderr);
            return;
            break;
    }

    switch (tokens[0])
    {

        case TOK_INDEXERS:
            stdACindexers (STD_ord, onOffMode);
            break;

        case TOK_DRIVES:
            stdACdrives (STD_ord, onOffMode);
            break;

        case TOK_CAL:
            stdACcalib (STD_ord, onOffMode);
            break;

        case TOK_VAC:
            stdACpier (STD_ord, onOffMode);
            break;

        case TOK_VALVE:
            stdACvalve (STD_ord, onOffMode);
            break;

        case TOK_STATUS:
            log_fputs ("SORRY: Status Not supported for `AC' command.\n",
                       stderr);
            break;

        case TOK_TEST:
            log_fputs ("SORRY: No test mode in `AC' command.\n", stderr);
            break;

        default:
            log_fputs ("ERROR:  Bad second argument\n", stderr);
            break;

    }
}

void rotator (char *args)
{
    int tokens[5];
    int count;
    double value;
    double oldValue;

    tokenize (args, tokens, TokenTable);

/*
   if( faith ){
      return;
   }
*/

    switch (tokens[0])
    {

        case TOK_TRACKING:

            switch (tokens[1])
            {

                case TOK_ON:
                    rotatorSetTracking (1);     /* arg is boolean --too lazy to write an include file */
                    break;

                case TOK_OFF:
                    rotatorSetTracking (0);
                    break;

                default:
                    log_fputs
                        ("ERROR:  Bad argument to \"rotator tracking\".\n",
                         stderr);
                    break;
            }

            break;

        case TOK_POSITION:

            switch (tokens[1])
            {

                case TOK_LITERAL:

                    count = sscanf (tokens[2], "%lf", &value);

                    if (count >= 1)
                    {
                        rotatorSetAbsAngle (value);
                    }
                    else
                    {
                        log_fputs
                            ("ERROR:  Bad value argument to \"rotator position\".\n",
                             stderr);
                    }

                    break;

                default:
                    log_fputs
                        ("ERROR:  Bad argument to \"rotator position\".\n",
                         stderr);
                    break;
            }

            break;

        case TOK_OFFSET:

            switch (tokens[1])
            {

                case TOK_LITERAL:

                    count = sscanf (tokens[2], "%lf", &value);

                    if (count >= 1)
                    {
                        rotatorSetOffset (value);
                    }
                    else
                    {
                        log_fputs
                            ("ERROR:  Bad value argument to \"rotator offset\".\n",
                             stderr);
                    }

                    break;

                default:
                    log_fputs
                        ("ERROR:  Bad argument to \"rotator offset\".\n",
                         stderr);
                    break;
            }

            break;

        case TOK_ADJUST:

            switch (tokens[1])
            {

                case TOK_LITERAL:

                    count = sscanf (tokens[2], "%lf", &value);

                    if (count >= 1)
                    {

                        oldValue = CurrentRotatorOffset * (180.0 / M_PI);

                        sprintf (log_buf, "oldvalue = %f;  newvalue = %f\n",
                                 oldValue, (oldValue + value));
                        log_fputs (log_buf, stderr);

                        rotatorSetOffset (oldValue + value);
                    }
                    else
                    {
                        log_fputs
                            ("ERROR:  Bad value argument to \"rotator adjust\".\n",
                             stderr);
                    }

                    break;

                default:
                    log_fputs
                        ("ERROR:  Bad argument to \"rotator adjust\".\n",
                         stderr);
                    break;
            }

            break;

        default:
            log_fputs ("ERROR:  Bad argument to \"rotator\".\n", stderr);
            break;
    }
}

/*********************** end Std Bus commands *******************/

setBlock (int flag)
{
    /* send the command and it's arguments to the other window */

    if (graphic)
    {

        fprintf (graphCmds, "setblock %d\n", flag);
        fflush (graphCmds);
    }
}

void sysStatusDump (struct exposinfo *pExposure, FILE * statusFile)
{
    sprintf (log_buf, "Current file name : %s\n", pExposure->fileName);
    log_fputs (log_buf, statusFile);

    sprintf (log_buf, "Field name        : %s\n", pExposure->expName);
    log_fputs (log_buf, statusFile);

    sprintf (log_buf, "Equinox(IN)       : %10.2f\n", pExposure->epoch);
    log_fputs (log_buf, statusFile);

    sprintf (log_buf, "Current epoch     : %10.2f\n", pExposure->current);
    log_fputs (log_buf, statusFile);

    sprintf (log_buf, "Mid-exposure ST   : %10.2f\n", pExposure->st);
    log_fputs (log_buf, statusFile);

    sprintf (log_buf, "Exp. Time (hr)    : %10.2f\n", pExposure->exptime);
    log_fputs (log_buf, statusFile);

    sprintf (log_buf, "Spectrograph wavelength : %10.2f\n", pExposure->wave);
    log_fputs (log_buf, statusFile);
}

/******************************************************************/

/*		Special tests, etc.			*/

 /*ARGSUSED*/ void progtest (char *args)
{
    labtyd (MISSED_BUTTON, args);
}

 /*ARGSUSED*/ void fakehw (char *args)
{

    if (sscanf (args, "%d", &PromptHw) < 1)
        fputs ("bad arg to pbp\n", stderr);

}

releaseButtonAfterCycle (void)
{

    struct plate_coords limitCold;
    struct plate_coords limitWarm;

    struct plate_coords buttonCoord;

    struct itimerval coldTimer;

    coldTimer.it_value.tv_sec = 60;     /* should be long enough... */
    coldTimer.it_value.tv_usec = 0;
    coldTimer.it_interval.tv_sec = 0;
    coldTimer.it_interval.tv_usec = 0;

    /* verify safety in dropping */

    log_fputs ("In order to continue, you must visually determine \n",
               stderr);
    log_fputs ("that the gripper is over a save location to drop the\n",
               stderr);
    log_fputs ("button. \n", stderr);

    log_fputs ("Call for help if you don't understand this!! \n", stderr);

    if (getYesNo (stderr, stdin, "Is the gripper over a safe area?", FALSE))
    {

        /* initialize Z */

        /* execfile( Z_BASIC_INIT ); */

        /* call special 3 attempt drop */

        if (blindRelease () != 0)
        {
            log_fputs ("Button still stuck...\n", stderr);
            log_fputs ("Please call for help!!!\n", stderr);
            labtyd (1, "can't clear button from gripper");
        }
        else
        {

            log_fputs
                ("Driving to limits to establish a vector to this location.\n",
                 stderr);

            /* call "drive to corner script" */

            if (faith)
            {
                log_fputs ("Real hardware would drive to limit now \n",
                           stderr);
            }
            else
            {
                execfile (XY_TO_LIMIT);
            }

            /* read encoder coordinate */

            syncToIndexers (&coldTimer);

            limitCold = location_now;

            /* now home things */

            if (faith)
            {
                log_fputs ("Real hardware would homeXY now \n", stderr);
            }
            else
            {
                execfile (XY_BASIC_INIT);
            }

            /* and drive to the same limit */

            if (faith)
            {
                log_fputs ("Real hardware would drive to limit now \n",
                           stderr);
            }
            else
            {
                execfile (XY_TO_LIMIT);
            }

            syncToIndexers (&coldTimer);

            limitWarm = location_now;

            /* compute and set location of dropped button. */

            buttonCoord.x = limitWarm.x - limitCold.x;
            buttonCoord.y = limitWarm.y - limitCold.y;

            sprintf (log_buf, "Button was left at %d, %d\n",
                     (int) buttonCoord.x, (int) buttonCoord.y);
            log_fputs (log_buf, stderr);

            if (FiberInTrouble.fiber == NULL)
            {
                log_fputs
                    ("Write this coordinate down!!! You will need it once you have\n",
                     stderr);
                log_fputs ("identified the button. (to find it again.)\n",
                           stderr);
            }
            else
            {
                setLocation (&FiBase[FiberInTrouble.fiber->fiber_id],
                             &buttonCoord, UPDATE_FILE);

                log_fputs
                    ("software should now be able to locate this button.\n",
                     stderr);
                if (faith)
                {
                    log_fputs ("Real hardware would homeXY now \n", stderr);
                }
                else
                {
                    execfile (XY_BASIC_INIT);
                }

            }

        }
    }
    else
    {
        log_fputs ("Please call for help!!!\n", stderr);
        labtyd (1, "Exiting button-in-gripper recovery.");
    }
}

tokenize (char *args, int *tokens, struct aToken *tokenTable)
{
    char currentArg[32];

    int currentToken = 0;
    int i;
    int noMatch = TRUE;

    while (isspace (*args))
        args++;

    while (sscanf (args, " %s", currentArg) > 0)
    {

        noMatch = TRUE;

        for (i = 0; (tokenTable[i].token != 0); i++)
        {

            if (strcmp (currentArg, tokenTable[i].name) == 0)
            {

                tokens[currentToken] = tokenTable[i].token;
                currentToken++;
                noMatch = FALSE;
                break;

            }
        }

        if (noMatch)
        {
            tokens[currentToken++] = TOK_LITERAL;
            tokens[currentToken++] = strstr (args, currentArg);
        }

        while (isspace (*args))
            args++;

        args += strlen (currentArg);

    }

    tokens[currentToken] = NOTOKEN;
}

/*
	setup

   This is the new setup system for wiyn.  It takes the word "setup"
   followed by text arguments to determine how the system is to be
   configured.
*/

void setup (char *args)
{
    int tokens[5];

    tokenize (args, tokens, TokenTable);

    switch (tokens[0])
    {

        case TOK_FIELD:        /*** Configure a field ***/

            wiynSetup ();

            break;

        case TOK_FLATS:        /*** Set up for dome flats ***/

            fputs ("flats\n", stderr);
            wiynFlats (0);
            break;

        case TOK_COMPS:        /*** Set up for Comparison images ***/

            wiynComps (0);
            break;

        case TOK_QUARTZ:       /*** Set up for quartz flat field ***/

            fputs ("quartz\n", stderr);
            wiynQuartz ();
            break;

        case TOK_OBSERVE:      /*** Make sure everything is ready for exposure on sky ***/

            listDump ("i");
            wiynPrepObserve ();
            break;

        default:
            log_fputs ("ERROR:  I don't know how to set that up!\n", stderr);
            fputs ("Options are... \n\n", stderr);
            fputs ("   \"setup field\" \n", stderr);
            fputs ("   \"setup observe\" \n", stderr);
            fputs ("   \"setup flats\" \n", stderr);
            fputs ("   \"setup comps\" \n", stderr);
            fputs ("   \"setup quartz\" \n", stderr);
            return;
            break;
    }
}

void setInputStreamMode (int mode)
{
    int handlerMode;
    int fd;

    if (mode == ON)
    {

        handlerMode = TK_READABLE;
        setBlock (0);           /* tell gui we're available */
    }
    else
    {

        handlerMode = 0;
        setBlock (1);           /* tell gui we're busy */
    }

    fd = fileno (StdinInfo.stream);
    Tk_DeleteFileHandler (fd);
    Tk_CreateFileHandler (fd, handlerMode, execStream,
                          (ClientData) & StdinInfo);

    if (GraphinInfo.stream != nullFile)
    {
        fd = fileno (GraphinInfo.stream);
        Tk_DeleteFileHandler (fd);
        Tk_CreateFileHandler (fd, handlerMode, execStream,
                              (ClientData) & GraphinInfo);
    }

}
