
/****************************************************************

		prompter.c

   A set of routines that takes a some pointers and 
   prompts the user for input, while informing him
   what he'll get as a default.

****************************************************************/
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#include "prompter.h"
#include "tk.h"
#include "log.h"
#include "sla.h"

static char log_buf[256];

char *tkfgets (char *s, int n, FILE * stream);
extern double converthms (char *dec, int *hr, int *min, int *sec, int *other,
                          int *sign);
extern long int ulimit (int, ...);

#ifdef IGNORE
main ()
{
    int default1 = 1234;
    double default2 = 5.678;
    char *default3 = "fooooo";
    double default4 = (3.14159 / 2);
    double default5 = ((30.5 / 180) * 3.14159);

    char *result1;
    char *result2;
    char *result3;
    char *result4;
    char *result5;

/*
   result1 = getUserInput( stderr, stdin, INT,    "enter an integer  ", (char*)&default1 );
   result2 = getUserInput( stderr, stdin, DOUBLE, "enter a float", (char*)&default2 );
   result3 = getUserInput( stderr, stdin, CHAR,    "enter a string", default3 );
   result4 = getUserInput( stderr, stdin, TIME,    "enter a time", (char*)&default4 );
   result5 = getUserInput( stderr, stdin, ANGLE,    "enter an angle", (char*)&default5 );

   fprintf( stdout, "%d\n%f\n%s\n%f\n%f\n", *(int*)result1, *(double*)result2, result3, 
      *(double*)result4, *(double*)result5 );
*/

    setup (NULL);

}
#endif

int getYesNo (FILE * devOut, FILE * devIn, char *prompt, int defaultValue)
{

    char newPrompt[255];

    char *responseString;
    char *defaultString;
    int defaultFlag;
    int validResponse = FALSE;

    strcpy (newPrompt, prompt);
    strcat (newPrompt, " [ Yes or No ]? ");

    switch (defaultValue)
    {

        case DEFAULT_NO:

            defaultFlag = TRUE;
            defaultString = "No";
            break;

        case DEFAULT_YES:

            defaultFlag = TRUE;
            defaultString = "Yes";
            break;

        case FORCE_RESPONSE:

            defaultFlag = FALSE;
            defaultString = NULL;
            break;

        default:
            break;
    }

    while (!validResponse)
    {

        /* get a string from the user */

        responseString =
            getUserInput (devOut, devIn, CHAR, defaultFlag, newPrompt,
                          defaultString, NULL, NULL, NULL);

        /* strip any white space */

        while ((*responseString == ' ') || (*responseString == '\t'))
            responseString++;

        switch (responseString[0])
        {

            case 'y':
            case 'Y':
                return (TRUE);
                break;

            case 'N':
            case 'n':
                return (FALSE);
                break;

            case '\n':
                if (defaultFlag == TRUE)
                    return (defaultValue);
                break;

            default:
                break;
        }
        log_fputs (" Sorry.  Only a yes/no answer is acceptable\n", devOut);
    }
 /*NOTREACHED*/}

int getYesNoQuit (FILE * devOut, FILE * devIn, char *prompt, int defaultValue)
{

    char newPrompt[255];

    char *responseString;
    char *defaultString;
    int defaultFlag;
    int validResponse = FALSE;

    strcpy (newPrompt, prompt);
    strcat (newPrompt, " [ Yes, No, Quit ]? ");

    switch (defaultValue)
    {

        case DEFAULT_NO:

            defaultFlag = TRUE;
            defaultString = "No";
            break;

        case DEFAULT_YES:

            defaultFlag = TRUE;
            defaultString = "Yes";
            break;

        case FORCE_RESPONSE:

            defaultFlag = FALSE;
            defaultString = NULL;
            break;

        default:
            break;
    }

    while (!validResponse)
    {

        /* get a string from the user */

        responseString =
            getUserInput (devOut, devIn, CHAR, defaultFlag, newPrompt,
                          defaultString, NULL, NULL, NULL);

        /* strip any white space */

        while ((*responseString == ' ') || (*responseString == '\t'))
            responseString++;

        switch (responseString[0])
        {

            case 'y':
            case 'Y':
                return (TRUE);
                break;

            case 'N':
            case 'n':
                return (FALSE);
                break;

            case 'Q':
            case 'q':
                return (QUIT);
                break;

            case '\n':
                if (defaultFlag == TRUE)
                    return (defaultValue);
                break;

            default:
                break;
        }
        log_fputs (" Sorry.  Only a yes/no/quit answer is acceptable\n",
                   devOut);
    }
 /*NOTREACHED*/}

char getMenuSelection (char **menuItems, FILE * devIn, FILE * devOut)
{
    int item = 1;
    int responseOK = FALSE;
    char *response;
    char lastItem;
    char *defaultItem;

    /* print a stock message that the following is a menu */

    log_fputs ("\n", devOut);
    log_fputs (menuItems[0], devOut);
    log_fputs ("\nSelect from the following:\n\n", devOut);

    /* print all the items, identified by an incrementing
       character starting with "a" */

    while (menuItems[item] != NULL)
    {

        lastItem = 'a' + item - 1;

        sprintf (log_buf, "\t%c.  %s\n", lastItem, menuItems[item]);
        log_fputs (log_buf, devOut);

        item++;
    }

    /* the defaultItem choice is stored after the null at the end
       of the list */

    defaultItem = menuItems[++item];

    /* prompt until a valid character is entered */

    while (responseOK == FALSE)
    {

        response = getUserInput (devOut, devIn, CHAR, SHOW_DEFAULT,
                                 "", defaultItem, NULL, NULL, NULL);

        if ((*response >= 'a') && (*response <= lastItem))
        {
            responseOK = TRUE;
        }
    }

    return (*response);
}

char *getUserInput (FILE * devOut, FILE * devIn, int type, int defaultFlag,
                    char *prompt, char *defaultValue, char *min, char *max,
                    char *prec)
{

    char defaultString[32];
    static char rawInput[255];
    char *result;

    static int userInt;
    static double userDouble;

    /* stuff for xx:xx:xx conversions */

    int angle[4];
    int sign;
    char angleSign;

    int foo;                    /*dummy variable */

    /* set up the string for the default value */

    if (defaultFlag == SHOW_DEFAULT)
    {

        switch (type)
        {

            case INT:
                sprintf (defaultString, "(%*d)>", *(int *) prec,
                         *(int *) defaultValue);
                break;

            case DOUBLE:
                sprintf (defaultString, "(%.*f)>", *(int *) prec,
                         *(double *) defaultValue);
                break;

            case CHAR:
                sprintf (defaultString, "(%s)>", defaultValue);
                break;

            case TIME:

                /* convert decimal value into x:x:x type field */
                /* the variable "angle" is mis-named for this use
                   it should be called "time" */

                sla_dr2tf (*(int *) prec, *(double *) defaultValue,
                           &angleSign, angle);

                if (angleSign == '+')
                    angleSign = ' ';

                sprintf (defaultString, "(%c%2d:%02d:%02d.%-*d)>",
                         angleSign, angle[0], angle[1], angle[2],
                         *(int *) prec, angle[3]);
                break;

            case ANGLE:

                /* convert decimal value into x:x:x type field */

                sla_cr2af (*(int *) prec, *(double *) defaultValue,
                           &angleSign, angle);

                if (angleSign == '+')
                    angleSign = ' ';

                sprintf (defaultString, "(%c%2d:%02d:%02d.%-*d)>",
                         angleSign, angle[0], angle[1], angle[2],
                         *(int *) prec, angle[3]);
                break;

            default:
                sprintf (defaultString, "(no default)>");
                break;
        }
    }
    else
    {
        strcpy (defaultString, ">");
    }

    /* print the prompt with the default to be used if no user entry */

  promptPoint:

    sprintf (log_buf, "%s\n%s ", prompt, defaultString);
    log_fputs (log_buf, devOut);

    /* get the user input */

    if (tkfgets (rawInput, 254, devIn) == NULL)
    {
        sprintf (log_buf, "Error reading input.  --using default value\n");
        log_fputs (log_buf, devOut);
        return ((char *) defaultValue);
    }

    /* if user hit return, then bail now with the default value */

    if (rawInput[0] == '\n')
    {

        if (NO_DEFAULT)
        {
            log_fputs
                ("SORRY, no default available. You MUST enter something...\n",
                 devOut);
            goto promptPoint;
        }
        else
        {
            return (defaultValue);
        }
    }

    log_putsQ (rawInput);

    /* otherwise, see if the data is any good... */

    switch (type)
    {

        case INT:

            if (sscanf (rawInput, "%d", &userInt) < 1)
            {
                log_fputs ("Value not an integer--try again.\n", devOut);
                goto promptPoint;
            }

            if ((userInt < *(int *) min) || (userInt > *(int *) max))
            {
                log_fputs ("Value out of range--try again.\n", devOut);
                goto promptPoint;
            }

            result = (char *) &userInt;

            break;

        case DOUBLE:

            if (sscanf (rawInput, "%lf", &userDouble) < 1)
            {
                log_fputs ("Value not a number--try again.\n", devOut);
                goto promptPoint;
            }

            if ((userDouble < *(double *) min)
                || (userDouble > *(double *) max))
            {
                log_fputs ("Value out of range--try again.\n", devOut);
                goto promptPoint;
            }

            result = (char *) &userDouble;

            break;

        case TIME:
            if (sscanf (rawInput, "%d", &foo) < 1)
            {
                log_fputs ("Value not a number--try again.\n", devOut);
                goto promptPoint;
            }

            userDouble = converthms (rawInput, &foo, &foo, &foo, &foo, &sign);
            userDouble *= (M_PI / 12);

            if ((userDouble < *(double *) min)
                || (userDouble > *(double *) max))
            {
                log_fputs ("Value out of range--try again.\n", devOut);
                goto promptPoint;
            }

            result = (char *) &userDouble;

            break;

        case ANGLE:

            if (sscanf (rawInput, "%d", &foo) < 1)
            {
                log_fputs ("Value not a number--try again.\n", devOut);
                goto promptPoint;
            }

            userDouble = converthms (rawInput, &foo, &foo, &foo, &foo, &sign);
            userDouble *= (M_PI / 180);

            if ((userDouble < *(double *) min)
                || (userDouble > *(double *) max))
            {
                log_fputs ("Value out of range--try again.\n", devOut);
                goto promptPoint;
            }

            result = (char *) &userDouble;

            break;

        case CHAR:

            result = rawInput;
            break;

        default:
            break;
    }

    return (result);
}

char *tkfgets (char *s, int n, FILE * stream)
{
    int fdi, selectWidth;
    fd_set selectMask;
    struct timeval timeout;

    fdi = fileno (stream);
    selectWidth = fdi + 1;

    /* set up the timer for two hertz. */
    timeout.tv_sec = 0;
    timeout.tv_usec = 500000;

    /* wait for something to happen */
    while (1)
    {
        FD_ZERO (&selectMask);
        FD_SET (fdi, &selectMask);

        if (select (selectWidth, &selectMask, NULL, NULL, &timeout) < 0)
        {
            perror ("select");
            return NULL;
        }

        if (FD_ISSET (fdi, &selectMask))
        {
            return (fgets (s, n, stream));
        }

        while (Tk_DoOneEvent (TK_DONT_WAIT) != 0);

    }
}
