#include <stdio.h>
#include <math.h>
#include <string.h>

#include "miscMath.h"
#include "plate.h"
#include "motor.h"
#include "translate.h"
#include "flex.h"
#include "archive.h"
#include "fiborg.h"
#include "kaboom.h"
#include "buttons.h"
#include "coordConvert.h"
#include "nesslib.h"
#include "nessIO.h"
#include "xref.h"

int uniqueID (void);

void updateDbases (void)
{
    struct fiber_dbase *fib;
    struct starinfo *theStar;
    struct starinfo *newStar;

    point newCoord;
    xItem theList;

    int onTarget;

    int i;
    int j;

    for (i = 0; i < MAXFIBER; i++)
    {

        /* get the cross-references for this fibers */

        onTarget = xRefFiber (i, &theList, Stars, FiBase);

        /* fprintf( stderr, " %d %d %d\n",  i, theList.object, onTarget ); */

        /* create a couple pointers to make life simple */

        fib = &FiBase[i];

        if (theList.object != VOIDENTRY)
            theStar = Stars[theList.object];
        else
            theStar = NULL;

        /* now figure out the status of each fiber */

        if (fib->parkedStatus == EXTENDED)
        {

            /* fprintf( stderr, "Fiber %d is extended\n", i ); */

            /* translate button's coords back into a sky position */

            buttonToSky (fib, &newCoord);

            /* now we can figure out what type this is... */

            if ((theList.object == VOIDENTRY) || !onTarget)
            {

                /* this is a new position */

                fprintf (stderr, "Fiber %d is a new sky\n", i);

                /* first, be sure any old reference to this fiber is null */

                for (j = 0; j < MAXSTARS; j++)
                {
                    if ((Stars[j] != NULL) && (Stars[j]->button == i))
                    {
                        Stars[j]->button = COORD_OK;
                        FiBase[i].assignedObject = NULL;
                    }
                }

                /* it is a random sky so create a new object */

                newStar = createObject (Stars);

                /* fill in the data */

                newStar->button = i;
                FiBase[i].assignedObject = newStar;
                newStar->objectType = 'R';      /* random Sky */
                newStar->IDnumber = uniqueID ();
                strcpy (newStar->name, "Random Sky");

                newStar->camEncoderCoord = newCoord;
                newStar->specEncoderCoord = newCoord;
            }

            else
            {                   /* there's an object associated with this fiber */

                if (fib->tweaked)
                {
                    /* the fiber has been manually tweaked */

                    theStar->camEncoderCoord = newCoord;
                    theStar->specEncoderCoord = newCoord;

                    if (theStar->objectType == 'F')
                    {
                        /* Fops get flagged separately from ordinary tweaks */
                        theStar->objectType = 'B';
                    }
                    else
                    {
                        theStar->objectType = 'T';
                    }
                }
                else
                {
                    /* fprintf( stderr, "Fiber %d is assigned to %s, ptr= %d\n", i, theStar->name, theStar->button ); */
                }
            }
        }

        else
        {                       /* parked or stowed */

            /* if a fiber is not positioned on the prescribed object,
               keep the object alive, but flag it as unused. */

            if (theStar != NULL)
            {
                /* flag object as unused but ok */
                theStar->button = COORD_OK;
            }
            FiBase[i].assignedObject = NULL;
        }

    }
}

uniqueID (void)
{
    int i;
    int j;

    int unique;

    for (i = 9999; i > 9800; i--)
    {

        unique = TRUE;

        for (j = 0; j < MAXSTARS; j++)
        {
            if ((Stars[j] != NULL) && (Stars[j]->IDnumber == i))
            {
                unique = FALSE;
                break;
            }
        }

        if (unique)
            return (i);
    }

    return (9999);
}
