
/* 
		setupWiyn.c

   Works to do high level user interface for setting up fields,
   tweaking, and taking calibration frames

   Note:  Comments with "***" at the beginning are Sam's original
   specifications.

   Included functions:

      wiynSetup()
      wiynCalibrate()
      wiynView()
      wiynFlats()
      wiynTweak()
      wiynObserve()

*/

#include <stdio.h>
#include "prompter.h"
#include "log.h"

extern void screen (char *args);
extern void led (char *args);
extern int cameraSafetyInterlock (void);
extern void complamp (char *args);
extern void gripout (char *args);
extern void qtz (char *args);
extern int getYesNo (FILE * devOut, FILE * devIn, char *prompt,
                     int defaultValue);

foowiynSetup (char *args)
{

/*** SETUP Command: */

/*** The setup command should be modified to perform the following */

/*** (NOTE, these items should be allowed to have some flexibility in */

/*** their order of operation when possible.  Also, if possible, some */

/*** items should be operated in parallel, for example, rotation of */

/*** the instrument while deflecting the plate): */

/*** -)  Accept the target file name. */

/*** -)  Obtain the current epoch, air mass, and target position from */

    /*** the telescope computer. */

/*** -)  Obtain temperature and possibly humidity and pressure from */

    /*** (unknown?) computer. */

/*** -)  Perform the astrometry. */

/*** -)  Configure the field. */

   /*** -  Rotate the instrument rotator to the "setup" position and wait */

       /*** for verification that it is ready. */

   /*** -  Unwarp the focal plate via control of the vacuum valve and wait */

       /*** for verification. */

   /*** -  Verify that the stages are homed and ready to configure. */

   /*** -  Control illumination and power to the gripper LEDs. */

   /*** -  Configure the fibers. */

   /*** -  Allow examination of fiber probes (though stars will not be in */

       /*** the appropriate places). */

   /*** -  Deflect the focal plane via control of the vacuum system and wait */

       /*** for verification that it is ready. */

   /*** -  Allow the instrument rotator to rotate to its tracking position.  */

}

wiynComps (char *args)
{
    /* close screen */

    log_fputs ("Bringing in calibration screen.\n", stderr);

    screen ("in");

    log_fputs ("Making sure gripper LED are off.\n", stderr);

    led ("off");

    /* make sure camera's are safe... */

    cameraSafetyInterlock ();

    /* turn on complamps */

    log_fputs ("Turning on comparison lamps.\n", stderr);

    complamp ("on");

    /* griper out of way */

    log_fputs ("Making sure gripper is out of the way.\n", stderr);
    gripout ("");

    /* tell 'em were done */

    log_fputs ("\nYOU MAY NOW BEGIN TAKING COMPS.\n", stderr);

    log_fputs
        ("\nPlease use \"setup observe \" or \"complamp off\" when finished.\n",
         stderr);
}

wiynFlats (char *args)
{

   /*** -  Allow flat field observations of dome white spot. */

   /*** -  Verify that TV cameras are off or 0 volts and LEDs are off. */

    log_fputs ("Making sure gripper LED are off.\n", stderr);

    led ("off");

    cameraSafetyInterlock ();

    /* turn off complamps */

    log_fputs ("Making sure comparison sources are off.\n", stderr);

    complamp ("off");
    qtz ("off");

    /* open screen */

    log_fputs ("Making sure screen is out of the way.\n", stderr);

    screen ("out");

    /* griper out of way */

    log_fputs ("Making sure gripper is out of the way.\n", stderr);
    gripout ("");

    /* position telescope */

    log_fputs
        ("ATTENTION:  Please have OT set to white spot and turn on lamps.\n",
         stderr);

    while (!getYesNo
           (stderr, stdin, "\n  Is telescope in position with lamps on?",
            FALSE))
    {
        /* work takes place in control structure */

        /* tell 'em were done */

        log_fputs ("\nYOU MAY NOW BEGIN TAKING FLATS.\n", stderr);
    }
}

foowiynView (char *args)
{

/*** -)  Allow viewing of target objects. */

   /*** -  Verify that telescope is pointing in appropriate position. */

   /*** -  Allow telescope focusing and guiding initiation. */

   /*** -  Allow/control insertion of the FBI. */

   /*** -  View targets/fibers. */

}

wiynTweak (char *args)
{

/*** -)  Allow tweaking of misplaced fibers. */

   /*** -  Routine that allows cruise to center up targets on gripper TV */

      /*** and stores positions into a tweak array. */

   /*** -  Rotate instrument to setup position. */

   /*** -  Tweak fibers according to offsets stored in tweak array. */

   /*** -  Rotate instrument back to target, reinitiate guiding, etc. */

   /*** -  View all tweaked fibers. */

}

wiynObserve (char *args)
{

/*** -)  Observe */

   /*** -  Verify that gripper is out of way. */

   /*** -  Verify that calibration lamps and LEDs are off. */

   /*** -  Verify that guiding is initiated or optionally not used. */

   /*** -  Optional control of CCD and Bench Spectrograph to obtain data. */

/***  */

}

wiynUpdate (char *args)
{

/*** -)  Allow objects only tweak to update positions for atmospheric dispersion */

    /*** changes.  Only tweak those objects exceeding a user defined threshold. */

}

void wiynQuartz (void)
{
}

/*** -)  Allow flat field and calibration exposures to also be taken after */

    /*** observations are complete. */

/*** STANDARD STAR OBSERVATIONS Command: */

/***  */

/*** This command allows fibers to be placed at field center for the observation */

/*** of standard stars. */

/***  */

/*** -)  Rotate instrument to setup position.  */

/***  */

/*** -)  Move selected fiber into field center and FOPS to a selected guide */

    /*** star position.  The guide star position may have to be determined via */

    /*** the gripper TV camera in order to account for any errors in the coordinate */

    /*** offsets between the target star and the guide star. */

/***  */

/*** -)  Optional control of CCD and Bench Spectrograph to obtain data. */

/***  */

/*** -)  Allow several fibers to be consecutively placed in center after */

    /***    exposures are complete for previous fiber. */
