#include <stdio.h>
#include <sys/time.h>
#include <setjmp.h>
#include <signal.h>
#include <unistd.h>

#include "errors.h"
#include "death.h"
#include "miscMath.h"
#include "motor.h"
#include "kaboom.h"
#include "nesslib.h"
#include "flex.h"
#include "translate.h"
#include "detrot.h"
#include "xref.h"
#include "fiborg.h"
#include "reports.h"
#include "stdBus.h"
#include "log.h"

static char log_buf[256];

int TvBitsToCode (int *flags);
int printBits (char *msgString, int *flags);
int genericOnOffMessage (char *message, int status);
int tvMessage (char *tvName, char *tvItem, int status);
int vacMessage (char *message, int status);
int screenMessage (char *message, int status);
int hingeMessage (char *message, int status);

int stdLampStatus (int axis_ord);
int stdLampConnected (int axis_ord);
int stdVacStatus (int axis_ord);
int stdGripLEDstatus (int axis_ord);
int stdScreenStatus (int axis_ord);
int stdScreenLockout (int axis_ord);
int stdTvStatus (int axis_ord);
int stdHingeStatus (int axis_ord);
int stdGripLED (int axis_ord, int arg);
int stdVacSwitch (int axis_ord, int arg);
int stdScreen (int axis_ord, int arg);

lampStatus (int *flags, int testMode)
{
    stdLampStatus (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);

    switch (testMode)
    {

        case LAMP_ON:

            if ((flags[LAMP1] == LAMP_OFF) ||
                (flags[LAMP2] == LAMP_OFF) ||
                (flags[LAMP3] == LAMP_OFF) ||
                (flags[LAMP4] == LAMP_OFF) ||
                (flags[LAMP5] == LAMP_OFF) ||
                (flags[LAMP6] == LAMP_OFF) ||
                (flags[LAMP7] == LAMP_OFF) || (flags[LAMP8] == LAMP_OFF))
            {

                return LAMP_ERROR;
            }
            break;

        case LAMP_OFF:

            if ((flags[LAMP1] == LAMP_ON) ||
                (flags[LAMP2] == LAMP_ON) ||
                (flags[LAMP3] == LAMP_ON) ||
                (flags[LAMP4] == LAMP_ON) ||
                (flags[LAMP5] == LAMP_ON) ||
                (flags[LAMP6] == LAMP_ON) ||
                (flags[LAMP7] == LAMP_ON) || (flags[LAMP8] == LAMP_ON))
            {

                return LAMP_ERROR;
            }
            break;

        case LAMP_NULL:
            break;

        default:
            fputs ("Unknown test mode in lampStatus\n", stderr);
            break;
    }

    return LAMP_OK;
}

lampCableStatus (int *flags, int testMode)
{
    stdLampConnected (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);

    switch (testMode)
    {

        case TH_CONNECTED:

            if (flags[TH_CABLE] == CABLE_CONNECTED)
            {

                return TRUE;
            }
            break;

        case CU_CONNECTED:

            if (flags[CU_CABLE] == CABLE_CONNECTED)
            {

                return TRUE;
            }
            break;

        default:
            fputs ("Unknown test mode in lampCableStatus\n", stderr);
            break;
    }

    return CABLE_ERROR;         /* no cable connected */
}

lampStatusQZ (int *flags, int testMode)
{
    stdLampStatus (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);

    switch (testMode)
    {

        case LAMP_ON:

            if (flags[QZ1] == LAMP_OFF)
            {

                return LAMP_ERROR;
            }
            break;

        case LAMP_OFF:

            if (flags[QZ1] == LAMP_ON)
            {

                return LAMP_ERROR;
            }
            break;

        case LAMP_NULL:
            break;

        default:
            fputs ("Unknown test mode in lampStatusQZ\n", stderr);
            break;
    }

    return LAMP_OK;
}

vacuumStatus (int *flags, int testMode)
{
    /* make sure the ac power to the valve is on */

    /* stdACvalve( STD_ord, 1 ); */

    usleep (300000);

    stdVacStatus (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);

    switch (testMode)
    {

        case VAC_ON:

            if ((flags[PIER_VAC] == VAC_AMBIENT) ||
                (flags[FP_VAC] == VAC_AMBIENT) ||
                (flags[FP_VALVE] == VAC_AMBIENT))
            {

                return VAC_ERROR;
            }
            break;

        case VAC_AMBIENT:

            if ((flags[PIER_VAC] == VAC_AMBIENT) ||
                (flags[FP_VAC] == VAC_ON) || (flags[FP_VALVE] == VAC_ON))
            {

                return VAC_ERROR;
            }
            break;

        case VAC_NULL:
            break;

        default:
            fputs ("Unknown test mode in vacuumStatus\n", stderr);
            break;
    }

    return VAC_OK;

}

ledStatus (int *flags, int testMode)
{
    stdGripLEDstatus (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);

    switch (testMode)
    {

        case LAMP_ON:

            if (flags[LED_POWER] == LAMP_OFF)
            {

                return LAMP_ERROR;
            }
            break;

        case LAMP_OFF:

            if (flags[LED_POWER] == LAMP_ON)
            {

                return LAMP_ERROR;
            }
            break;

        case LAMP_NULL:
            break;

        default:
            fputs ("Unknown test mode in ledStatus\n", stderr);
            break;
    }

    return LAMP_OK;
}

screenStatus (int *flags, int testMode)
{
    stdScreenStatus (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);

    switch (testMode)
    {

        case SCREEN_IN:

            if ((flags[SCREEN_LIM1] == 0) || (flags[SCREEN_LIM2] == 1))
            {

                return SCREEN_ERR;
            }
            break;

        case SCREEN_OUT:

            if ((flags[SCREEN_LIM2] == 0) || (flags[SCREEN_LIM1] == 1))
            {

                return SCREEN_ERR;
            }
            break;

        case SCREEN_NULL:
            break;

        default:
            fputs ("Unknown test mode in screenStatus\n", stderr);
            break;
    }

    return SCREEN_OK;
}

lockoutStatus (int *flags, int testMode)
{
    stdScreenLockout (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);

    switch (testMode)
    {

        case LOCKOUT_ON:

            if (flags[LOCKOUT_BIT] == 1)
            {

                return LOCKOUT_ERR;
            }
            break;

        case LOCKOUT_OFF:

            if (flags[LOCKOUT_BIT] == 0)
            {

                return LOCKOUT_ERR;
            }
            break;

        case LOCKOUT_NULL:
            break;

        default:
            fputs ("Unknown test mode in lockoutStatus\n", stderr);
            break;
    }

    return LOCKOUT_OK;
}

/* 
		tvStatusFops

   This function gets the status of the fops TV,
   and returns a code indicating what is turned on.
*/

tvStatusFops (int *flags)
{
    int statusCode;

    stdTvStatus (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);
    statusCode = TvBitsToCode (&flags[FOPS_BIT_1]);

    return (statusCode);
}

/* 
		tvStatusGrip

   This function gets the status of the gripper TV,
   and returns a code indicating what is turned on.
*/

tvStatusGrip (int *flags)
{
    int statusCode;

    stdTvStatus (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);
    statusCode = TvBitsToCode (&flags[GRIP_BIT_1]);

    return (statusCode);
}

/*
		TvBitsToCode

   This function takes andy's funky bit field and returns a code
   that indicates to what degree the specified tv is turned on.
   Note that the argument is a pointer to a segment of the array
   that is returned by the tv status command.  Only the first
   two bits of this array are used, and the calling functions should
   insure that they are meaningful bits.
*/

int TvBitsToCode (int *flags)
{

    /* one of these statements will always be true */

    if (flags[0] && flags[1])
        return (TV_POWER_OFF);

    if (flags[0] && !flags[1])
        return (TV_POWER_ON);

    if (!flags[0] && !flags[1])
        return (TV_INTENSITY_ON);

    if (!flags[0] && flags[1])
        return (TV_HIVOLTAGE_ON);

}

hingeStatus (int *flags, int testMode)
{
    stdHingeStatus (STD_ord);
    GetTriggers (STD_ord, flags, &masterTimer);

    switch (testMode)
    {

        case HINGE_OPEN:

            if (flags[HINGE_BIT] == 0)
            {

                return HINGE_ERR;
            }
            break;

        case HINGE_CLOSED:

            if (flags[HINGE_BIT] == 1)
            {

                return HINGE_ERR;
            }
            break;

        case HINGE_NULL:
            break;

        default:
            fputs ("Unknown test mode in hingeStatus\n", stderr);
            break;
    }

    return HINGE_OK;
}

int hingeOpen (void)
{
    int flags[12];

    return (hingeStatus (flags, HINGE_OPEN) == HINGE_OK);
}

printLampStatus (int *flags)
{
    printBits ("Lamp status bits", flags);

    genericOnOffMessage ("lamp # 1", flags[LAMP1]);
    genericOnOffMessage ("lamp # 2", flags[LAMP2]);
    genericOnOffMessage ("lamp # 3", flags[LAMP3]);
    genericOnOffMessage ("lamp # 4", flags[LAMP4]);
    genericOnOffMessage ("lamp # 5", flags[LAMP5]);
    genericOnOffMessage ("lamp # 6", flags[LAMP6]);
    genericOnOffMessage ("lamp # 7", flags[LAMP7]);
    genericOnOffMessage ("lamp # 8", flags[LAMP8]);
}

printLampStatusQZ (int *flags)
{
    printBits ("QTZ bits", flags);
    genericOnOffMessage ("QTZ lamp ", flags[QZ1]);
}

printLEDStatus (int *flags, int level)
{
    printBits ("grip LED bits", flags);
    genericOnOffMessage ("Gripper LED", flags[LED_POWER]);
    sprintf (log_buf, "Level is set to %d\n", level);
    log_fputs (log_buf, stderr);
}

printTvStatus (char *tvName, int tvStatus)
{
    tvMessage (tvName, "power", (tvStatus >= 1));
    tvMessage (tvName, "intensifier", (tvStatus >= 2));
    tvMessage (tvName, "high voltage", (tvStatus >= 3));
}

tvMessage (char *tvName, char *tvItem, int status)
{

    sprintf (log_buf, "%s TV %s is ", tvName, tvItem);
    log_fputs (log_buf, stderr);

    if (status)
        log_fputs ("ON\n", stderr);
    else
        log_fputs ("OFF\n", stderr);
}

genericOnOffMessage (char *message, int status)
{

    sprintf (log_buf, "%s is ", message);
    log_fputs (log_buf, stderr);

    if (status == LAMP_ON)
        log_fputs ("ON\n", stderr);
    else
        log_fputs ("OFF\n", stderr);
}

printVacStatus (int *flags)
{
    printBits ("vacuum bits", flags);
    vacMessage ("pier switch senses", flags[PIER_VAC]);
    vacMessage ("FP switch senses  ", flags[FP_VAC]);
    vacMessage ("FP valve is set to", flags[FP_VALVE]);
}

vacMessage (char *message, int status)
{

    sprintf (log_buf, "%s  ", message);
    log_fputs (log_buf, stderr);

    if (status == VAC_ON)
        log_fputs ("VACUUM\n", stderr);
    else
        log_fputs ("AMBIENT\n", stderr);
}

printScreenStatus (int *flags)
{
    printBits ("screen bits", flags);
    screenMessage ("Screen out  sensor is", flags[SCREEN_LIM1]);
    screenMessage ("Screen in   sensor is", flags[SCREEN_LIM2]);
}

screenMessage (char *message, int status)
{

    sprintf (log_buf, "%s  ", message);
    log_fputs (log_buf, stderr);

    if (status == VAC_ON)
        log_fputs ("ENGAGED\n", stderr);
    else
        log_fputs ("NOT ENGAGED\n", stderr);
}

printHingeStatus (int *flags)
{
    printBits ("hinge bits", flags);
    hingeMessage ("Hinge is ", flags[SCREEN_LIM1]);
}

hingeMessage (char *message, int status)
{

    sprintf (log_buf, "%s  ", message);
    log_fputs (log_buf, stderr);

    if (status == HINGE_OPEN)
        log_fputs ("OPEN\n", stderr);
    else
        log_fputs ("CLOSED\n", stderr);
}

printBits (char *msgString, int *flags)
{
    char bitString[16];
    int i;

    for (i = 0; i <= 10; i++)
    {

        if (flags[i] == 0)
            bitString[i] = '0';
        else
            bitString[i] = '1';
    }

    bitString[i] = '\0';

    fprintf (stderr, "%s: %s\n", msgString, bitString);
}

ledDiag (void)
{
    int onFlags[12];
    int offFlags[12];

    log_fputs ("Insuring that LED is off.\n", stderr);

    stdGripLED (STD_ord, 0);
    sleep (2);

    stdGripLED (STD_ord, 8);

    log_fputs ("Turning LED on.\n", stderr);

    stdGripLED (STD_ord, 1);
    sleep (2);

    stdGripLEDstatus (STD_ord);
    GetTriggers (STD_ord, onFlags, &masterTimer);

    printLEDStatus (onFlags, 8);

    if (onFlags[LED_POWER] == LAMP_OFF)
    {
        log_fputs ("\nERROR:  Status bit shows LED not on.\n", stderr);
    }

    log_fputs ("\nTurning LED off.\n", stderr);

    stdGripLED (STD_ord, 0);
    sleep (2);

    stdGripLEDstatus (STD_ord);
    GetTriggers (STD_ord, offFlags, &masterTimer);

    printLEDStatus (offFlags, 0);

    if (offFlags[LED_POWER] == LAMP_ON)
    {
        log_fputs ("\nERROR:  Status bit shows LED is on.\n", stderr);
    }
}

vacDiag (void)
{
    int flatFlags[12];
    int warpFlags[12];

    /* start with the plate flat */

    log_fputs ("Starting test. \nFlattening plate. \n\n", stderr);

    stdVacSwitch (STD_ord, 1);

    sleep (7);                  /* allows pressures to adjust */

    /* get the status bits */

    stdVacStatus (STD_ord);
    GetTriggers (STD_ord, flatFlags, &masterTimer);

    /* print the raw bits and their interpretation */

    log_fputs ("\n\nWith Plate flat:\n\n", stderr);
    printVacStatus (flatFlags);

    /* validate each flag for this position */

    if (flatFlags[PIER_VAC] == VAC_AMBIENT)
    {
        log_fputs ("ERROR: Pier vacuum is not present.\n", stderr);
    }

    if (flatFlags[FP_VAC] == VAC_ON)
    {
        log_fputs ("ERROR: Plate pressure sensor reports vacuum.\n", stderr);
    }

    if (flatFlags[FP_VALVE] == VAC_ON)
    {
        log_fputs ("ERROR: Vacuum valve in wrong position.\n", stderr);
    }

    log_fputs ("\nWarping plate. \n\n", stderr);

    /* warp the plate */

    stdVacSwitch (STD_ord, 0);
    sleep (12);                 /* allows pressures to adjust */

    /* get the status bits */

    stdVacStatus (STD_ord);
    GetTriggers (STD_ord, warpFlags, &masterTimer);

    /* print the raw bits and their interpretation */

    log_fputs ("\nWith Plate warped:\n\n", stderr);
    printVacStatus (warpFlags);

    /* validate each flag for this position */

    if (warpFlags[PIER_VAC] == VAC_AMBIENT)
    {
        log_fputs ("ERROR: Pier vacuum is not present.\n", stderr);
    }

    if (warpFlags[FP_VAC] == VAC_AMBIENT)
    {
        log_fputs ("ERROR: Plate pressure sensor reports no vacuum.\n",
                   stderr);
    }

    if (warpFlags[FP_VALVE] == VAC_AMBIENT)
    {
        log_fputs ("ERROR: Vacuum valve in wrong position.\n", stderr);
    }

    log_fputs ("\nReturning  plate to flat. \n\n", stderr);

    stdVacSwitch (STD_ord, 1);
    sleep (8);                  /* allows pressures to adjust */
}

screenDiag (void)
{

    int inFlags[12];
    int outFlags[12];

    log_fputs ("Insuring that screen is out.\n", stderr);

    stdScreen (STD_ord, 1);

    sleep (5);

    log_fputs ("Moving screen in.\n", stderr);

    stdScreen (STD_ord, 0);
    sleep (5);

    stdScreenStatus (STD_ord);
    GetTriggers (STD_ord, inFlags, &masterTimer);

    printScreenStatus (inFlags);

    if (inFlags[SCREEN_LIM1] == 0)
    {
        log_fputs ("ERROR:  Screen \"in\" limit switch is not enabled.\n",
                   stderr);
    }

    if (inFlags[SCREEN_LIM2] == 1)
    {
        log_fputs ("ERROR:  Screen \"out\" limit switch is enabled.\n",
                   stderr);
    }

    log_fputs ("Moving screen out.\n", stderr);

    stdScreen (STD_ord, 1);
    sleep (5);

    stdScreenStatus (STD_ord);
    GetTriggers (STD_ord, outFlags, &masterTimer);

    printScreenStatus (outFlags);

    if (outFlags[SCREEN_LIM2] == 0)
    {
        log_fputs ("ERROR:  Screen \"in\" limit switch is not enabled.\n",
                   stderr);
    }

    if (outFlags[SCREEN_LIM1] == 1)
    {
        log_fputs ("ERROR:  Screen \"out\" limit switch is enabled.\n",
                   stderr);
    }

}
