
/***************************************************************

		stopwatch.c

   A set of functions for  taking high resolution time
   measurements of the execution of functions. And
   assigning names to them.

***************************************************************/

#include <sys/time.h>
#include <stdio.h>
#include <string.h>

#include "stopwatch.h"
#include "log.h"

/* statics */

struct eventItem eventList[200];

int nextFreeEvent;

/* local functions */

double elapsedTime (struct eventItem *event1, struct eventItem *event2);

/* functions */

void clearEvents (void)
{
    nextFreeEvent = 0;
}

void event (char *name)
{
    (void) gettimeofday (&eventList[nextFreeEvent].eventTime,
                         (struct timezone *) NULL);
    strcpy (eventList[nextFreeEvent++].eventName, name);
}

void listEvents (FILE * stream)
{
    char log_buf[256];
    double eventDuration;

    int i = 1;                  /* event 0 is start time */

    while (i < nextFreeEvent)
    {

        eventDuration = elapsedTime (&eventList[i - 1], &eventList[i]);

#ifdef LOG
        sprintf (log_buf, " event: \"%31s\" took %9.3f seconds \n",
                 eventList[i].eventName, eventDuration);
        log_fputs (log_buf, stream);
#else
        sprintf (log_buf, " event: \"%31s\" took %9.3f seconds \n",
                 eventList[i].eventName, eventDuration);
        log_fputs (log_buf, stream);
#endif

        i++;

    }

    eventDuration = elapsedTime (&eventList[0], &eventList[i - 1]);

#ifdef LOG
    sprintf (log_buf, " total time was: %9.3f seconds \n", eventDuration);
    log_fputs (log_buf, stream);

#else
    fprintf (stream, " total time was: %9.3f seconds \n", eventDuration);
#endif

}

double elapsedTime (struct eventItem *event1, struct eventItem *event2)
{

    return (((double) event2->eventTime.tv_sec +
             ((double) event2->eventTime.tv_usec * 1e-6)) -
            ((double) event1->eventTime.tv_sec +
             ((double) event1->eventTime.tv_usec * 1e-6)));
}

/*   for testing only  */

/*

main()

{
   char dummy;

   clearEvents();

   event( "start" );
   dummy = getchar();
   event( "middle" );
   dummy = getchar();
   event( "end" );

   listEvents( stdout );
}

*/
