c       program bench to estimate interactively the configuration
c       and performance of hydra and the bench spectrograph on the
c       wiyn telescope
c
c	The following mods made Aug 29, 1995 by Sam Barden
c		Include 400@4.2 grating
c		Modify all grating names for consistency
c		Tweak blaze angle of 316@7.0
c		Name detector T2KC with proper read noise and gain
c		Renamed Red Lens to BSC for Bench Spectrograph Camera
c		Added delimiter line of ******* to output file
c       The following mods made Sept. 11, 1995:
c               slit image given in angstroms as well as pixels and mm
c               renamed "resolution" to "resolving power
c       The following mods made 15 Sept. 1995 by CAP:
c               changed calculation of wlower and wupper to
c               improve accuracy (I hope!)
c
c       if saved, results are written to the file wiyn.setup
c
c       enter collimator choice, grating parameters, detector parameters,
c       observing parameters, etc.
c
c       alpha   = angle of incidence (collimator to grating normal) in degrees
c       beta    = angle of diffraction (camera to grating normal) in degrees
c       camcol  = full angle between camera and collimator in degrees
c       half    = half angle between camera and collimator in degrees
c
c       lobs    = wavelength of interest for observations (microns)
c       llobs   = wavelength in Angstroms
c       lblaze  = wavelength of blaze peak at chosen cam-col angle (microns)
c       wave    = central wavelength in Angstroms
c
c       thetab  = blaze angle in degrees
c       grooves = grating grooves/mm
c       sigma   = grating spacing (microns)
c       mbar    = order number at blaze peak
c       mlam    = order number at a specific wavelength
c       gstep   = step size for adjusting grating rotation to place
c                 desired central wavelength on the detector
c       bstep   = size of big step for adjusting grating angle to
c                 desired wavelength
c       sstep   = size of small step for adjusting grating angle
c
c       atel    = 3.5  (diameter of telescope in meters)
c       rtel    = 6.3  (focal ratio of telescope)
c       scale   = 10.  (plate scale (arc-sec/mm at telescope))
c
c       dslit   = slit width in mm
c       aslit   = slit width in arc-seconds
C
c       rcol    = 6.0  (fiber output focal ratio (= col focal ratio))
c       acol    = 150  (diameter of collimator in mm)
c       fcol    =      (focal length of collimator)
c       fcam    =      (focal length of camera in mm)
c       acam    =      (diameter of camera for unvignetted beam)
c       rcam    =      (focal ratio of camera for unvignetted beam)
c
c       pixel   = pixel size in microns
c       pmm     = pixel size in mm
c       ccdsize = detector width in mm
c       pixnum  = number of pixels
c
c       wslit   = slit image in mm
c       wpix    = slit image in pixels
c       wang    = slit image in angstroms
c       res     = resolution (delta lambda of slit image / lambda)
c       disp    = dispersion (A/mm)
c       pdisp   = dispersion (A/pixel)
c       dmspec  = demagnification in the dispersion direction
c       dmspat  = demagnification perpendicular to the dispersion
c
c
      character*10 ccdname,gname,camname,fibnam
      real*4 mag, mlam, lambda, lblaze, lobs, llobs
      common llobs,mag,dqe,pixels,cres,asky,etime,rnoise,gain,
     *         atel,seff,backg,flash,sky,ext,sig,sn
c     
c       now set the assumed telescope and fiber parameters
c
      sec = 2.06265e5
      atel = 3.5
      xatel = atel * 1000.
      scale = 10. 
      rtel = 6.3
      rcol = 6.0 
      bstep = 0.2
      sstep = 0.001
c   
c     now enter required parameters
c
      write(6,160)
 160  format(//' STARTING WIYN/BENCH SETUP PROGRAM',//)
c
c       enter observing parameters
c
      write(unit=6, fmt=132) 
  132 format(' enter desired central wavelength in Angs.... ')
      read(unit=5, fmt=103) llobs  
      lobs = llobs / 10000.
c     write (6,152)
 152  format(' enter magnitude at the above wavelength...  ')
c     read (5,103) mag
      mag = 16.0
c     write (6,153)
 153  format(' enter seeing value....   ')
c     read (5,103) seeing
      seeing = 1.0
c     write (6,154)
 154  format(' select fiber (1=Red Cable, 2=Blue Cable)...   ')
c     read (5,169) islit
      fibnam="Red Cable"
      dslit = 0.2
c     if(islit.eq.2) dslit = 0.3
      if(llobs.lt.4500.) dslit=0.31
      if(dslit.eq.0.31) fibnam="Blue Cable"
      aslit = (dslit) * scale
c     write (6,155)
 155  format(' enter exposure time in seconds...   ')
c     read (5,103) etime
      etime = 600.
c
c
c       camera parameters
c
      write(unit=6, fmt=119)
  119 format(' enter camera selection (1=BSC or 2=Simmons)... ')
      read(unit=5, fmt=169) icam
      fcam = 285.
      if(icam.eq.2) fcam=381.
      if(icam.eq.1) camname = "BSC"
      if(icam.eq.2) camname = "Simmons "
c	
c       enter grating parameters
c
      write(6,604)
 604  format(//' the available WIYN gratings are:'//,
     * '  1 = 316@7.0    316 l/mm,  7450A first order  ',/
     * '  2 = 400@4.2    400 l/mm,  5000A first order  ',/
     * '  3 = 600@13.9   600 l/mm,  7700A first order  ',/
     * '  4 = 860@30.9   860 l/mm, 11500A first order  ',/
     * '  5 = 1200@28.7 1200 l/mm,  7700A first order  ',/
     * '  6 = 316@63.4   316 l/mm,  63.4 degree echelle',/
     * '  7 = other       ',//
     * ' enter the number of the grating....   ')
      read(5,169) igrat
 169  format(i2)
      if(igrat.eq.7) go to 605
      call wiyngrat(igrat,gname,grooves,thetab,icol)
      camcol = 30.
      if(igrat.eq.6) camcol=13.
      go to 607
c
c     ask for details about any unspecified grating
c
 605  continue
      write(6,606)
 606  format(' enter grating name...  ')
      read(5,603) gname
      write(unit=6, fmt=120) 
  120 format(35h enter grooves/mm for grating....  )
      read(unit=5, fmt=103) grooves
  122 format(i3)
      write(unit=6, fmt=115) 
  115 format(32h enter grating blaze angle....  )
      read(unit=5, fmt=103) thetab
  103 format(f10.0)
c     write(unit=6, fmt=106) 
c 106 format(' which collimator (long=1, short=2)?   ')
c     read(unit=5, fmt=159) icol
 159  format(i2)
c     get the camera collimator angle
      write(unit=6, fmt=118)
  118 format(' enter camera-collimator angle in degrees',/,
     *'    (30 deg. for low res., 13 for echelle)....  ')
      read(unit=5, fmt=103) camcol
 607  continue
      half = camcol / 2.
c     now finish up grating stuff
      sigma = 1000. / grooves
      acol=150
      fcol = 1023.6
c     if(icol.eq.2) fcol = 6860.
c     if(icol.eq.2) acol = 221
c
c       enter detector parameters
c
c     write (6,170)
 170  format(' select the ccd from the following list:'/,
     *'  1 = f3ka    4 = t2kb     7 = s2ka  '/,
     *'  2 = f3kb    5 = t2kc     8 = ti5   '/,
     *'  3 = t2ka    6 = t1ka     9 = other '//,
     *' enter number of ccd....  ')
c     read(5,159) iccd
      iccd = 5
      if(iccd.eq.9) go to 600
      call ccdpars(iccd,ccdname,pixnum,pixel)
      go to 9
c
c     enter ccd parameters if not on list
c
 600  continue
      write(6,602)
 602  format(' enter name of ccd....  ')
      read(5,603) ccdname
 603  format(a10)
      write(unit=6, fmt=130) 
  130 format(41h enter number of pixels on detector....  )
      read(unit=5, fmt=103) pixnum
      write(unit=6, fmt=108) 
  108 format(34h enter pixel size in microns....  )
      read(unit=5, fmt=103) pixel
      write (6,150)
150   format(' enter the read noise in electrons... ')
      read (5,103) rnoise
      write(6,157)
157   format(' enter detector gain (e-/adu)... ')
      read(5,103) gain
      write (6,151)
151   format(' enter the detector efficiency (i.e. 0.5)... ')
      read (5,103) dqe
c
c
9     continue
c     recompute initial order information
      alpha = thetab + half
      beta = thetab - half
      mlam = (sigma / lobs)*(sin(alpha * 0.0174533)+
     &sin(beta * 0.0174533))
      mbar = nint(mlam)
c
   10 continue
c
      alpha = thetab + half
      beta = thetab - half
      mlam = (sigma / lobs)*(sin(alpha * 0.0174533)+
     &sin(beta * 0.0174533))
      pmm = pixel / 1000.
      ccdsize = pixnum * pmm
      bstep = 0.2
      sstep = 0.003
      if(thetab.lt.10.) bstep = .05
      if(thetab.lt.10.) sstep = .0005
c
c       calculate order and maximum on-blaze wavelength
c
      lblaze = (sigma / mbar)*(sin(alpha * 0.0174533)+
     &sin(beta * 0.0174533))
      dlam = abs(lblaze-lobs)
c     write (unit=6, fmt=301) mbar
c     write (unit=6, fmt=302) lblaze
      gstep = bstep
      if(dlam/lobs.lt.0.01) gstep=sstep 
      ncycle = 0
      if(lobs.lt.lblaze) go to 502
c
c       the case for wavelength greater than the blaze wavelength
c
  503 alpha = alpha + gstep
      beta = beta + gstep
      lambda = (sigma / mbar) * (sin(alpha * 0.0174533) + 
     &sin(beta * 0.0174533))
      dlam = abs(lambda-lobs)
      if(dlam/lobs.lt.0.01) gstep=sstep 
c     write(unit=6, fmt=302) lambda,dlam,gstep
      ncycle = ncycle + 1
      if(ncycle.gt.99999) stop
      if(lambda.lt.lobs) go to 503
      go to 520
c       
c       the case for wavelength less than the blaze wavelength
c
  502 alpha = alpha - gstep
      beta = beta - gstep
      lambda = (sigma / mbar) * (sin(alpha * 0.0174533) + 
     &sin(beta * 0.0174533))
      dlam = abs(lambda-lobs)
      if(dlam/lobs.lt.0.01) gstep=sstep 
c     write(unit=6, fmt=302) lambda,dlam,gstep
      ncycle = ncycle + 1
      if(ncycle.gt.99999) stop
      if(lambda.gt.lobs) go to 502
c
  520 continue
c
c     write(unit=6, fmt=301) ncycle
      gamma = (sin(alpha * 0.0174533)+sin(beta * 0.0174533))/
     &cos(alpha * 0.0174533)
      ranam = cos(alpha * 0.0174533) / cos(beta * 0.0174533)
      aslit = dslit * scale
      acam = acol / ranam
      rcam = fcam / acam
      wslit = ranam * dslit * (fcam/fcol)
c     res = (sec/aslit) * gamma * acol / xatel
      wslitm = 1000. * wslit
      wpix = wslit / pmm
      dmspat = fcol / fcam
      dmspec = dmspat / ranam
c
c       calculate spectrum parameters on detector
c
      wave = lambda * 10000.
      disp = 10000. * sigma * cos(beta * 0.0174533) / ( fcam * mbar )
      pdisp = disp * pixel / 1000.
      res = wave / (pdisp * wpix)
      if(wpix.le.2.) res = wave / (pdisp * 2.)
      wang = pdisp*wpix
c     wlower = (wave - ((ccdsize / 2.) * disp))
c     wupper = (wave + ((ccdsize / 2.) * disp))
      dbeta = atan(ccdsize/(2.*fcam))
      wlower = 10000.*(sigma/mbar)*(sin(alpha * 0.0174533)+
     &sin(beta * 0.0174533 - dbeta))
      wupper = 10000.*(sigma/mbar)*(sin(alpha * 0.0174533)+
     &sin(beta * 0.0174533 + dbeta))
      if(wlower.lt.2000) wlower=0.0000
c
c
 25   continue
c     computed expected counts and s/n ratio per pixel
c     with the following assumptions
c       mag = magnitude of object at given wavelength
c       dqe = qe of detector at the given wavelength
c       pixels = the number of pixels per resolution element
c       cres = the resolution of the spectrum in Angstroms
c       asky = the slit area in arc seconds
c       eff = overall efficiency of telescope + spectrograph
c           (not counting slit loses or detector)
c       backg = background count rate = 0. for CCD's at the feed
c       preflash level = 0.0 for feed detectors (tek and ford)
c       assume typical sky at full moon for worst case
c       let extinction = 0.2 for typical yellow/red extinction
c
c     set basic assumptions - assume an efficiency of 5%
      eff = 0.05
c
c     reduce efficiency for echelle grating to account for filter
c     and overfill and vignetting
      if(igrat.eq.6) eff = 0.6 * eff
c
c     reduce efficiency if grating used off-blaze
c     (daryl doesn't like this so it's commented out!)
c     eff = eff * (1. - abs(mlam-mbar))
c
      flash = 0.
      backg = 0.
      sky = 19.
      ext = 0.2
      cres = disp * wslit
      asky = aslit*seeing
      pixels = dslit/dmspec * seeing/scale/dmspat * (1000./pixel)**2
c
c     correct for slit losses (very simple assumptions....  the
c     star image is a uniformly illuminated square the size of
c     the seeing disk!
      seff = eff
      if(seeing.gt.aslit) seff = eff*(aslit/seeing)**2
c
      call counts
c
c     now convert from per resolution element to per pixel
c      in the extracted spectrum
c
       sigel = sig/wpix
       sigadu = sigel / gain
       sn = sn / sqrt(wpix)
c
c	now write out results
c
  200 format(1h )
      write(unit=6, fmt=201) 
  201 format(//49h SET-UP FOR THE WIYN/BENCH SPECTROGRAPH (ON-AXIS)/)
c
      write(unit=6, fmt=401) ccdname
      write(unit=6, fmt=402) acol,pixnum,pixel
      write(unit=6, fmt=403) fcol,rnoise,gain
      write(unit=6, fmt=404) rcol,ccdsize
      write(unit=6, fmt=200)
      write(unit=6, fmt=405) gname
      write(unit=6, fmt=406) grooves,camcol
      write(unit=6, fmt=407) thetab,alpha
      write(unit=6, fmt=408) mbar,mlam,beta
      write(unit=6, fmt=200)
      write(unit=6, fmt=410) camname,fibnam
      write(unit=6, fmt=411) fcam,dslit
      write(unit=6, fmt=412) rcam,aslit
      write(unit=6, fmt=413) acam
      write(unit=6, fmt=200)
      write(unit=6, fmt=420)
      write(unit=6, fmt=414) wlower,disp
      write(unit=6, fmt=415) wave,pdisp
      write(unit=6, fmt=416) wupper,res
      write(unit=6, fmt=200)
      write(unit=6, fmt=417) wslit
      write(unit=6, fmt=418) dmspec, wpix
      write(unit=6, fmt=419) dmspat, wang
      write(unit=6, fmt=200)
      write(unit=6, fmt=451)
      write(unit=6, fmt=452) mag,sigel
      write(unit=6, fmt=453) etime,sigadu
      write(unit=6, fmt=454) seeing,sn
      write(unit=2, fmt=455) 
      write(unit=2, fmt=456) 
 401  format(' COLLIMATOR      ',10x,5x,'DETECTOR (',a4,')')
 402  format(' diameter (mm):  ',f10.0,5x,'pixel number, size:',2f6.0)
 403  format(' focal length (mm): ',f7.1,5x,'rdnoise, gain:',4x,2f7.1)
 404  format(' focal ratio:    ',f10.2,5x,'overall size (mm):',f7.1)
 405  format(' GRATING (',a10,')',11x,'CONFIGURATION')
 406  format(' lines per mm    ',f10.2,5x,'camera-coll angle:',f6.0)
 407  format(' blaze angle:    ',f10.2,5x,'use angle (alpha):',f9.3)
 408  format(' selected order  ',i3,'(',f6.2,')',4x,'beta:   ',f19.3)
 409  format('                 ',15x,'camera distance (in):',f6.1)
 410  format(' CAMERA (',a8,     ')  ',12x,'FIBER DIAM.  (',a10,')')
 411  format(' focal length (mm): ',f7.1,5x,'in mm:         ',f12.3)
 412  format(' focal ratio:    ',f10.2,5x,'in arc-sec     ',f11.2)
 413  format(' mono. beam diam. (mm):',f5.0)
 420  format(' SPECTRAL PARAMETERS')
 414  format(' lower lambda:   ',f10.0,5x,'dispersion (A/mm):',f9.3)
 415  format(' central lambda: ',f10.0,5x,'dispersion (A/pix):',f8.3)
 416  format(' upper lambda:   ',f10.0,5x,'resolving power:   ',f9.0)
 417  format(' DEMAGNIFICATION ',15x,     'SLIT IMAGE (in mm): ',
     &f8.3)
 418  format(' spectral:       ',f10.2,5x,'SLIT IMAGE (in pix):',
     &f8.3)
 419  format(' spatial:        ',f10.2,5x,'SLIT IMAGE (in ang):',
     &f8.2)
 451  format(' ESTIMATED OBSERVING AND PERFORMANCE PARAMETERS')
 452  format(' magnitude:      ',f10.2,5x,'summed e-/pix:   ',f10.0)
 453  format(' exp. time (sec):',f10.0,5x,'summed adu/pix:  ',f10.0)
 454  format(' seeing (arc-sec):',f9.1,5x,'s/n per summed pix:',f8.0)
 455  format(' ')
 456  format(' ******************************************************')
 421  format(////////)
c
      write(unit=6, fmt=200) 
c
 400  continue
c
c	now offer to change values
c
      write(unit=6, fmt=200) 
      write(unit=6, fmt=300) 
  300 format(
     &' 0 = save to wiyn.setup   5 = exposure time     ',/,
     &' 1 = quit (no save)       6 = fiber diam. (mm)  ',/,
     &' 2 = wavelength           7 = grating           ',/,
     &' 3 = magnitude            8 = camera/config     ',/,
     $' 4 = seeing               9 = detector          ',//,
     &'  enter 0-9 to change the parameter listed above: ')
      read(unit=5, fmt=301) i
      if(i.eq.0) go to 1
  301 format(i10)
  302 format(5f12.6)
      go to (99, 20, 30, 40, 50, 60, 70, 80, 90), i
c     
c     change wavelength
   20 write(unit=6, fmt=132) 
      read(unit=5, fmt=103) llobs  
      lobs = llobs/10000.
      dslit = 0.2
      if(llobs.lt.4500.) dslit=0.31
      fibnam='Red Cable'
      if(dslit.eq.0.31) fibnam='Blue Cable'
      go to 9
c    
c     change magnitude
   30 write (unit=6, fmt=152)
      read(5,103) mag
      go to 25
c
c     change seeing 
   40 write(6,153)
      read(5,103) seeing
      go to 25
c
c     change exposure time
   50 write(6,155)
      read(5,103) etime
      go to 25
c
c     change slit width
   60 write(6,154)
      read (5,169) islit
      fibnam="Red Cable"
      dslit = 0.2
      if(islit.eq.2) dslit = 0.31
      if(dslit.eq.0.31) fibnam="Blue Cable"
      go to 10
c
c     change grating
  70  write(6,613)
 613  format(//' the available WIYN gratings are:'//,
     * '  1 = 316@7.0    316 l/mm,  7450A first order  ',/
     * '  1 = 400@4.2    400 l/mm,  5000A first order  ',/
     * '  3 = 600@13.9   600 l/mm,  7700A first order  ',/
     * '  4 = 860@30.9   860 l/mm, 11500A first order  ',/
     * '  5 = 1200@28.7 1200 l/mm,  7700A first order  ',/
     * '  6 = 316@63.4   316 l/mm,  63.4 degree echelle',/
     * '  7 = other       ',/
     * '  8 = change order only '//
     * ' enter the number of the grating....   ')
      read(5,169) igrat
      camcol = 30.
      if(igrat.eq.6) camcol = 13.
      if(igrat.eq.7) go to 608
      if(igrat.eq.8) go to 611
      call wiyngrat(igrat,gname,grooves,thetab,icol)
      go to 62
c
c     ask for details about any unspecified grating
c
  608 continue 
      write(6,606)
      read(5,603) gname
      write(unit=6, fmt=120) 
      read(unit=5, fmt=103) grooves
      write(unit=6, fmt=115) 
      read(unit=5, fmt=103) thetab
c     write(unit=6, fmt=106) 
c     read(unit=5, fmt=159) icol
c     now finish up grating stuff
      write(unit=6, fmt=118)
      read(unit=5, fmt=103) camcol
 62   continue
      half = camcol / 2.
      sigma = 1000. / grooves
      go to 9
c
c     change grating order only
 611  write(unit=6, fmt=161)
      read(unit=5, fmt=162) mbar
  161 format(' enter desired order.... ')
  162 format(i5)
      go to 10
c
c     change camera
80    continue
      write(unit=6, fmt=119)
      read(unit=5, fmt=169) icam
      fcam = 285.
      if(icam.eq.2) fcam=381.
      if(icam.eq.1) camname = "BSC"
      if(icam.eq.2) camname = "Simmons "
      write(unit=6, fmt=145 )
  145 format(' enter camera-collimator angle in degrees',/,
     *'    (standard values are 30 deg. for low res.,',
     *'13 for echelle)....  ')
      read(unit=5, fmt=103) camcol
      go to 10
c
c     change detector
 90   continue
      write (6,170)
      read(5,159) iccd
      if(iccd.eq.8) go to 91
      call ccdpars(iccd,ccdname,pixnum,pixel)
      go to 10
c
c     enter ccd parameters if not on list
c
 91   continue
      write(6,602)
      read(5,603) ccdname
      write(unit=6, fmt=130) 
      read(unit=5, fmt=103) pixnum
      write(unit=6, fmt=108) 
      read(unit=5, fmt=103) pixel
      write (6,150)
      read (5,103) rnoise
      write(6,157)
      read(5,103) gain
      write (6,151)
      read (5,103) dqe
      go to 10
c
c	now write final results to a file wiyn.setup
c
    1 continue
c
      open(unit=2, file='wiyn.setup', status='unknown',
     &form='formatted',access='append') 
      write(unit=2, fmt=401) ccdname
      write(unit=2, fmt=402) acol,pixnum,pixel
      write(unit=2, fmt=403) fcol,rnoise,gain
      write(unit=2, fmt=404) rcol,ccdsize
      write(unit=2, fmt=200)
      write(unit=2, fmt=405) gname
      write(unit=2, fmt=406) grooves,camcol
      write(unit=2, fmt=407) thetab,alpha
      write(unit=2, fmt=408) mbar,mlam,beta
      write(unit=2, fmt=200)
      write(unit=2, fmt=410) camname,fibnam
      write(unit=2, fmt=411) fcam,dslit
      write(unit=2, fmt=412) rcam,aslit
      write(unit=2, fmt=413) acam
      write(unit=2, fmt=200)
      write(unit=2, fmt=420)
      write(unit=2, fmt=414) wlower,disp
      write(unit=2, fmt=415) wave,pdisp
      write(unit=2, fmt=416) wupper,res
      write(unit=2, fmt=200)
      write(unit=2, fmt=417) wslit
      write(unit=2, fmt=418) dmspec, wpix
      write(unit=2, fmt=419) dmspat, wang
      write(unit=2, fmt=200)
      write(unit=2, fmt=451)
      write(unit=2, fmt=452) mag,sigel
      write(unit=2, fmt=453) etime,sigadu
      write(unit=2, fmt=454) seeing,sn
      write(unit=2, fmt=455) 
      write(unit=2, fmt=456) 
  303 format(//'SETUP WRITTEN TO FILE SETUP.OUT')
      close(unit=2)
      go to 400
c
99    continue
      end
c
c
c
        subroutine counts
c       program to calculate counts detected vs. time for
c       observations out the coude feed telescope
c      
c       this version is optimized for the coude feed.
c       wave = wavelength in Angstroms
c       mag = magnitude of object at given wavelength
c       dqe = qe of detector at the given wavelength
c       pixels = the number of pixels per resolution element
c       res =  the resolution of the spectrum in Angstroms
c       asky = the slit area in arc seconds
c
        real*4 mag
        common wave,mag,dqe,pixels,res,asky,etime,rnoise,gain,
     *         tel,eff,backg,flash,sky,ext,sig,snstar
c
c
c       compute the count rates for source and sky.
c       calculation is for the zenith
c
        cstar=res*eff*dqe*tel**2*(4.0e10/wave)*10**(-1.*(mag+ext)/2.5)        
        csky=res*eff*dqe*asky*16.*(4.0e10/wave)*10**(-1.*sky/2.5)
c
        t=etime
        sig = cstar*etime
        snstar=cstar*t/sqrt(cstar*t+csky*t+pixels*rnoise**2
     *           +pixels*flash+pixels*t*backg)
c
      return
      end
c
      subroutine ccdpars(iccd,cname,pixnum,pixel)
      common wave,mag,dqe,pixels,res,asky,etime,rnoise,gain,
     *         tel,eff,backg,flash,sky,ext,sig,snstar
      character*10 ccdname(10),cname
      dimension ti5(4),t1ka(4),s2ka(4),t2ka(4),t2kb(4),
     *           t2kc(4),f3ka(4),f3kb(4)
      dimension dqwave(16),dqeti5(16),dqet1ka(16),
     *          dqes2ka(16),dqet2ka(16),dqet2kb(16),
     *          dqet2kc(16),dqef3ka(16),dqef3kb(16)
c
      data ccdname /'F3KA','F3KB','T2KA','T2KB','T2KC','T1KA','S2KA',
     * 'TI5','not valid', 'not valid' /
c     the following gives the number of pixels in the dispersion
c     direction, the pixel size, the read noise and the default
c     gain for each detector
      data f3ka/ 3072, 15, 8.0, 2.8/
      data f3kb/ 3072, 15, 8.0, 2.8/
      data t2ka/ 2048, 24, 4.0, 2.6/
      data t2kb/ 2048, 24, 4.0, 2.6/
      data t2kc/ 2048, 24, 4.3, 1.7/
      data t1ka/ 1024, 24, 3.5, 1.7/
      data s2ka/ 2048, 21, 3.0, 0.95/
      data ti5 /  800, 15,  12, 4.1/
      data dqwave /9500.,9000.,8500.,8000.,7500.,7000.,6500.,
     *             6000.,5500.,5000.,4500.,4050.,3800.,3650.,
     *             3340.,3200. /
      data dqef3ka/29.6,45.5,57.1,65.4,71.0,74.2,76.6,77.7,78.3,79.8,
     *             77.3,73.3,64.9,62.7,58.0,56.6 /
      data dqef3kb/29.6,45.5,57.1,65.4,71.0,74.2,76.6,77.7,78.3,79.8,
     *             77.3,73.3,64.9,62.7,58.0,56.6 /
      data dqet2ka/16.8,29.9,43.0,55.4,64.0,68.8,69.4,70.5,63.6,60.2,
     *             56.7,51.5,43.8,33.8,16.6,12.0 /
      data dqet2kb/18.1,33.1,46.4,60.3,73.2,79.9,82.5,81.8,78.9,76.5,
     *             70.6,64.7,56.2,49.8,22.2,16.0 /
      data dqet2kc/18.1,33.1,46.4,60.3,73.2,79.9,82.5,81.8,78.9,76.5,
     *             70.6,64.7,56.2,49.8,22.2,16.0 /
      data dqet1ka/20.7,34.9,47.6,65.6,75.2,81.5,82.2,80.3,74.2,60.0,
     *             49.1,39.3,30.7,25.9,14.0, 9.7 /
      data dqes2ka/10.3,21.0,31.9,42.0,43.8,47.5,50.3,40.0,33.3,21.9,
     *             17.6,16.7,14.0,15.3,15.2,15.0 /
      data dqeti5/ 15.5,24.1,33.6,41.4,50.0,57.4,63.8,68.8,70.6,67.2,
     *             62.5,56.3,51.5,47.9,51.7,52.3 /
c
      cname = ccdname (iccd)
      go to (10, 20, 30, 40, 50, 60, 70, 80) iccd
c     
c     constants for f3ka
  10  continue
      pixnum = f3ka(1)
      pixel = f3ka(2)
      rnoise = f3ka(3)
      gain = f3ka(4)
      call interp(wave,dqwave,dqef3ka,dqe)
      go to 100
c
c     constants for f3kb
  20  continue
      pixnum = f3kb(1)
      pixel = f3kb(2)
      rnoise = f3kb(3)
      gain = f3kb(4)
      call interp(wave,dqwave,dqef3kb,dqe)
      go to 100
c
c     constants for t2ka
  30  continue
      pixnum = t2ka(1)
      pixel = t2ka(2)
      rnoise = t2ka(3)
      gain = t2ka(4)
      call interp(wave,dqwave,dqet2ka,dqe)
      go to 100
c
c     constants for t2kb
  40  continue
      pixnum = t2kb(1)
      pixel = t2kb(2)
      rnoise = t2kb(3)
      gain = t2kb(4)
      call interp(wave,dqwave,dqet2kb,dqe)
      go to 100
c
c     constants for t2kc
  50  continue
      pixnum = t2kc(1)
      pixel = t2kc(2)
      rnoise = t2kc(3)
      gain = t2kc(4)
      call interp(wave,dqwave,dqet2kc,dqe)
      go to 100
c
c     constants for t1ka
  60  continue
      pixnum = t1ka(1)
      pixel = t1ka(2)
      rnoise = t1ka(3)
      gain = t1ka(4)
      call interp(wave,dqwave,dqet1ka,dqe)
      go to 100
c
c     constants for s2ka
  70  continue
      pixnum = s2ka(1)
      pixel = s2ka(2)
      rnoise = s2ka(3)
      gain = s2ka(4)
      call interp(wave,dqwave,dqes2ka,dqe)
      go to 100
c
c     constants for ti5
  80  continue
      pixnum = ti5(1)
      pixel = ti5(2)
      rnoise = ti5(3)
      gain = ti5(4)
      call interp(wave,dqwave,dqeti5,dqe)
      go to 100
c
 100  continue
c
      dqe=dqe/100.
      return
      end
c
      subroutine interp(xval,x,y,yval)
c
      dimension x(16),y(16)

      if(xval.gt.x(1)) go to 20
      do 10 i=2,16
      if(xval.gt.x(i)) go to 30
10    continue
      i=16
      go to 30
c
c     compute yval for xval greater than x(1)
20    continue
      yval = y(1) + (y(2)-y(1))*(xval-x(1)) / (x(2)-x(1))
      return
c
c     interpolate for xval values within range
30    ival = i
      yval = y(ival-1.) + (y(ival)-y(ival-1.))*
     *    (xval-x(ival-1.)) / (x(ival)-x(ival-1.))
c
      return
      end
c
c
      subroutine wiyngrat(igrat,gname,grooves,thetab,icol)
      character*10 gratings(6),gname
      dimension gpmm(6),angle(6),ncol(6)
      data gratings / '316@7.0','400@4.2','600@13.9','860@30.9',
     *                 '1200@28.7','echelle' /
      data gpmm / 316., 400., 600., 860., 1200., 316./
      data angle/7.0, 4.2, 13.9, 30.9, 28.7, 63.4 /
      data  ncol /1,1,1,1,1,1/
      gname = gratings(igrat)
      grooves = gpmm(igrat)
      thetab = angle(igrat)
      icol = ncol(igrat) 
      return
      end



