
/**
 *
 *     - - - - - -
 *      c t f 2 d
 *     - - - - - -
 *
 *
 *  convert hours, minutes, seconds to days
 *
 *  (single precision)
 *
 *
 *  given:
 *     ihour       int       hours
 *     imin        int       minutes
 *     sec         real      seconds
 *
 *  returned:
 *     days        real      interval in days
 *     j           int       status:  0 = ok
 *                                    1 = ihour outside range 0-23
 *                                    2 = imin outside range 0-59
 *                                    3 = sec outside range 0-59.999...
 *
 *  notes:
 *
 *     1)  The result is computed even if any of the range checks
 *         fail.
 *     2)  The sign must be dealt with outside this routine.
 *
 *
 *  p.t.wallace   starlink   november 1984
 *  r.j.wolff     noao       Feb 1988   transcribed from fortran to C
 *
 */

sla_ctf2d (int ihour, int imin, double sec, double *days, int *j)
                                         /* variables were REAL */
{

    /* seconds per day */
    /* define was REAL */
#define d2s  (double)86400.0

    /* preset status */
    *j = 0;

    /* validate sec, min, hour */
    if (sec < 0.0 || sec >= 60.0)
        *j = 3;
    if (imin < 0 || imin > 59)
        *j = 2;
    if (ihour < 0 || ihour > 23)
        *j = 1;

    /* compute interval */
    *days = (60.0 * (60.0 * (float) ihour + (float) imin) + sec) / d2s;
}
